/*
This source file is the copyright property of the University of Leeds
('The University').

Permission to use, copy, modify, and distribute this source file for
educational, research, and not-for-profit purposes, without fee and
without a signed licensing agreement, is hereby granted, provided that
the above copyright notice, this paragraph and the following three
paragraphs appear in all copies, modifications, and distributions.

In no event shall The University be liable to any party for direct,
indirect, special, incidental or consequential damages, including lost
profits, arising out of the use of this software and its documentation.

The software is provided without warranty. The University has no
obligation to provide maintenance, support, updates, enhancements, or
modifications.

This software was written by Adam Baumberg, Vision Group,
School of Computer Studies, University of Leeds, UK. The code and use
thereof should be attributed to the author where appropriate
(including demonstrations which rely on it's use).
*/

/*
 * MeanProfile.cc
 * 
 * class for calculating an updated mean profile
 *
 */


#include <MeanProfile.h>

#define SQUARE(x) ((x) * (x))

void MeanProfile::update(Profile* new_profile)
{
  no_processed++;
  if (no_processed > max_n) no_processed = max_n;

  origin += (new_profile->origin - origin) / no_processed;
  width += (new_profile->width - width) / no_processed;
  height += (new_profile->height - height) / no_processed;

  *this += (*new_profile - *this) / no_processed;
  Point2 d;
  for (int i = 0; i < NO_CONTROL_POINTS; i++)
    {
      for (int j = 0; j < NO_CONTROL_POINTS; j++)
	{
	  d = new_profile->spline[i] - new_profile->spline[j];
	  *mean_squares.get(i,j) += 
	    (d.length2() - *mean_squares.get(i,j)) 
	      / no_processed;
	  *mean_distances.get(i,j) +=
	    (fabs(d.length()) - *mean_distances.get(i,j))
	      / no_processed;	
	}
    }
}

void MeanProfile::update(ProfileSet& new_data_set)
{
  Profile reflected;
  for (item(Profile)* curr = new_data_set.first; curr != NULL; curr =
       curr->next)
    {
      update(curr->dat);
      //update(curr->dat->reflect(&reflected));
    }
      
}

void MeanProfile::reset_mean()
{
  no_processed = 0; 
  mean_squares.clear();
  mean_distances.clear();
  clear();
}    

