/*
This source file is the copyright property of the University of Leeds
('The University').

Permission to use, copy, modify, and distribute this source file for
educational, research, and not-for-profit purposes, without fee and
without a signed licensing agreement, is hereby granted, provided that
the above copyright notice, this paragraph and the following three
paragraphs appear in all copies, modifications, and distributions.

In no event shall The University be liable to any party for direct,
indirect, special, incidental or consequential damages, including lost
profits, arising out of the use of this software and its documentation.

The software is provided without warranty. The University has no
obligation to provide maintenance, support, updates, enhancements, or
modifications.

This software was written by Adam Baumberg, Vision Group,
School of Computer Studies, University of Leeds, UK. The code and use
thereof should be attributed to the author where appropriate
(including demonstrations which rely on it's use).
*/

/*
 * MeanProfile.h
 * 
 * class for calculating an updated mean profile
 *
 */

#include <NagMatrix.h>
#include <ProfileSet.h>
#include <values.h>

class MeanProfile : public Profile
{
public:
  
  int no_processed;
  int max_n;

  NagMatrix mean_squares;
  NagMatrix mean_distances;

  void reset_mean();

  MeanProfile(int max = MAXINT) : Profile(),
  mean_squares(NO_CONTROL_POINTS, NO_CONTROL_POINTS),
  mean_distances(NO_CONTROL_POINTS, NO_CONTROL_POINTS)
    {
      max_n = max;
      reset_mean();
    }

  //MeanProfile(istream& data_in);

  void calculate_mean(istream& data_in);
  void update(Profile* NewProfile);
  void update(Profile& NewProfile) { update(&NewProfile); }
  void update(ProfileSet& NewProfileSet);
};
