/*
This source file is the copyright property of the University of Leeds
('The University').

Permission to use, copy, modify, and distribute this source file for
educational, research, and not-for-profit purposes, without fee and
without a signed licensing agreement, is hereby granted, provided that
the above copyright notice, this paragraph and the following three
paragraphs appear in all copies, modifications, and distributions.

In no event shall The University be liable to any party for direct,
indirect, special, incidental or consequential damages, including lost
profits, arising out of the use of this software and its documentation.

The software is provided without warranty. The University has no
obligation to provide maintenance, support, updates, enhancements, or
modifications.

This software was written by Adam Baumberg, Vision Group,
School of Computer Studies, University of Leeds, UK. The code and use
thereof should be attributed to the author where appropriate
(including demonstrations which rely on it's use).
*/

/***************************************************************
 * C - C++ Header
 *
 * File : 	MultiPCA.h
 *
 * Module :	MultiPCA
 *
 * Author : 	A M Baumberg (CoMIR)
 *
 * Creation Date : Wed Jan  3 10:15:50 1996 
 *
 * Comments : 	A class for carrying out
 *		mixed model regression using EM method
 *
 ***************************************************************/


/* include files */

#ifndef __MULTI_PCA__
#define __MULTI_PCA__

#include <NagMatrix.h>
#include <NagVector.h>

class PCAclass;

// A cluster contains the information
// for one gaussian model
class Cluster
{
friend MultiPCA;
  
public:
  
  NagMatrix Covariance;
  NagMatrix Inverse_Covariance;
  NagVector mean;
  
  realno determinant;
  realno alpha_wght;
  
  Cluster();
  PCAclass* create_PCAclass();
  PCAclass* create_PCAclass(NagMatrix& Q, NagVector& offset);
  
};

class MultiPCA
{
public:
  
  int epoch_size;		// up to size of training set
  int data_size;		// length of each training vector
  int no_clusters;		// often denoted 'K'

  Cluster* cluster_set;		// array of Clusters

  realno small_variance;	// set to the a priori minimum variance
				// for a coordinate in input space

  realno min_value;
  realno max_value;
  realno variance;
  realno scalar_constant;

  NagMatrix distance_matrix;	// a metric on the clusters

public:
  

  MultiPCA(int datasize, int nclusters);
  
  void set_min_variance(realno v) { small_variance = v; }
  
  void initialise(realno min = -1, realno max = 1, realno min_var = 0,
		  realno max_var = 0 );	// randomly
  void initialise(NagMatrix& training_epoch);	// or using training
						// examples

  realno update(NagMatrix& training_epoch);

  void sort_models();
  void filter_models(realno min_items = 4.0);

  void build_distance_table(NagMatrix& training_data);
  void dump_pca_models(char* base_file_name);
  void dump_pca_models(char* base_file_name, NagMatrix& Q, NagVector& offset);
  int classify(NagVector& x_t);

private:

  realno prob_density(NagVector& x, int cluster);


};

  
#endif





