/*
This source file is the copyright property of the University of Leeds
('The University').

Permission to use, copy, modify, and distribute this source file for
educational, research, and not-for-profit purposes, without fee and
without a signed licensing agreement, is hereby granted, provided that
the above copyright notice, this paragraph and the following three
paragraphs appear in all copies, modifications, and distributions.

In no event shall The University be liable to any party for direct,
indirect, special, incidental or consequential damages, including lost
profits, arising out of the use of this software and its documentation.

The software is provided without warranty. The University has no
obligation to provide maintenance, support, updates, enhancements, or
modifications.

This software was written by Adam Baumberg, Vision Group,
School of Computer Studies, University of Leeds, UK. The code and use
thereof should be attributed to the author where appropriate
(including demonstrations which rely on it's use).
*/

/*
 * PCAclass
 * 
 * a class for carrying out Principal Component Analysis on
 * arbitrary data
 *
 * uses NagVector class as general data item
 */

#ifndef __PCACLASS_H__
#define __PCACLASS_H__

#include <values.h>
#include <fstream.h>
#include <NagMatrix.h>

// added 24/10/95
// minimum covariance entry
// threshold the covariance matrix to remove
// chance correlations
extern realno MIN_COV_ENTRY;


class PCAclass
{

public:

  int no_processed;
  int data_size;
  int max_n;

  NagMatrix covariance;
  NagMatrix change_covariance;
  
  NagVector mean;
  NagVector eigenvalues; // vector of eigenvalues
  NagMatrix eigenvectors; // matrix of eigenvectors
  NagVector weights; // set of weights

  NagVector noise_model;

// constructors etc

  PCAclass(int size, int max = MAXINT);
  PCAclass(char *fname, int max = MAXINT);

// data access functions

  realno get_eval(int i)
    { return eigenvalues[data_size - i]; }
  realno* get_evec(int i)
    { return &(eigenvectors[(data_size - i)
			    * data_size]) ; }
  realno get_noise_model(int i)
    { return noise_model[(data_size - i)]; }
  
  realno* cov(int i, int j) 
    { return (covariance.get(i,j)); }

// utility functions

  realno Hdot(NagVector& v1, NagVector& v2);  
  void reset_n() { no_processed = 0; }

  void process_cov(NagVector& new_data);
  void process_cov(NagVector& new_data, NagVector& new_change_data);
  void add_noise(realno noise_var, int nsp = 0);
  void get_noise_model();

  void process_mean(NagVector& new_data);
  //void process_all(NagVector& new_data);
  
  // copy upper triangle of covariance matrix
  // into lower triangle 
  void restore_covariance();

  void get_evals();
  void get_mode(int, realno, NagVector& result);

  void adjust_mode(int, realno, NagVector& result);
  void map_to_shape(int, NagVector& result, realno max_distance = 30);
  void map_to_model(int, NagVector& shape, NagVector& result);
  realno map_to_mode(int index, NagVector& shape);
};


ostream& operator<<(ostream&, PCAclass&);
istream& operator>>(istream&, PCAclass&);  

#endif


