/*
This source file is the copyright property of the University of Leeds
('The University').

Permission to use, copy, modify, and distribute this source file for
educational, research, and not-for-profit purposes, without fee and
without a signed licensing agreement, is hereby granted, provided that
the above copyright notice, this paragraph and the following three
paragraphs appear in all copies, modifications, and distributions.

In no event shall The University be liable to any party for direct,
indirect, special, incidental or consequential damages, including lost
profits, arising out of the use of this software and its documentation.

The software is provided without warranty. The University has no
obligation to provide maintenance, support, updates, enhancements, or
modifications.

This software was written by Adam Baumberg, Vision Group,
School of Computer Studies, University of Leeds, UK. The code and use
thereof should be attributed to the author where appropriate
(including demonstrations which rely on it's use).
*/

/*
 * ProfileSequence.h
 *
 * This class handles the ProfileSets  stored on file
 *
 */

#include <PCAclass.h>
#include <MeanProfile.h>

#include <stream.h>
#include <boolean.h>
#include <shape_constants.h>


class ProfileSequence
{
protected:

  char filename[100];
  NagVector weights;

public:

  int no_ctrl_pnts;
  MeanProfile* mean;
  PCAclass* pca;

  ProfileSequence(char* name, char* model_name = NULL);
  ~ProfileSequence();

  void calculate_mean();
  void do_analysis();
  void display_shapes(Point2 shape_origin);
  void display_mode(int& mode);
  void random_walk(int no_modes);
  void align(char* file_out = NULL);
  void track(char* file_out = NULL);
  void reflect(char* file_out = NULL);
  void halve(char* file_out = NULL);
  void edit_shapes(Point2 shape_origin, char* file_out_name = NULL);
  
  void setup_weights();
  bool align_profile(Profile& var);
  void center(char* file_out = NULL);
  void convert_to_matrix(NagMatrix& training_matrix, realno
			 use_fraction = 1.01);
  int count_profiles();

};
  
