/*
This source file is the copyright property of the University of Leeds
('The University').

Permission to use, copy, modify, and distribute this source file for
educational, research, and not-for-profit purposes, without fee and
without a signed licensing agreement, is hereby granted, provided that
the above copyright notice, this paragraph and the following three
paragraphs appear in all copies, modifications, and distributions.

In no event shall The University be liable to any party for direct,
indirect, special, incidental or consequential damages, including lost
profits, arising out of the use of this software and its documentation.

The software is provided without warranty. The University has no
obligation to provide maintenance, support, updates, enhancements, or
modifications.

This software was written by Adam Baumberg, Vision Group,
School of Computer Studies, University of Leeds, UK. The code and use
thereof should be attributed to the author where appropriate
(including demonstrations which rely on it's use).
*/

/*
 * Tracker.h
 * 
 * a module for tracking profiles from frame to frame
 *
 */

#include <ProfileSet.h>

typedef realno (DFunc) (Profile*, Profile*);

class Tracker
{
protected:
  
  // dist_fn is a function which calculates some
  // distance bewtween two Profiles
  
  DFunc* dist_fn;

  // far[i][j] will hold the distance from Profile i to Profile j
  realno far[8][8]; 

  realno near_threshold;
  ProfileSet* last_frame;
  ProfileSet* new_frame;
  int last_no;
  int new_no;
  int last_ref;

  
public:
 
  ProfileSet* get_new_frame()  { return new_frame; }
  ProfileSet* get_last_frame() { return last_frame; }

  Tracker(DFunc* dfn, realno near_thresh);
  Tracker();
  void process_frame(ProfileSet* next_frame);


private: /* static functions */
  void calculate_distances();
  realno find_best_match(int& from_indx, int& to_indx);
};
  
  
// Local Variables:
// compile-command: "csh -c \'cd ../PCA;make -k |& grep -v warning\'"
// End:
