/*
This source file is the copyright property of the University of Leeds
('The University').

Permission to use, copy, modify, and distribute this source file for
educational, research, and not-for-profit purposes, without fee and
without a signed licensing agreement, is hereby granted, provided that
the above copyright notice, this paragraph and the following three
paragraphs appear in all copies, modifications, and distributions.

In no event shall The University be liable to any party for direct,
indirect, special, incidental or consequential damages, including lost
profits, arising out of the use of this software and its documentation.

The software is provided without warranty. The University has no
obligation to provide maintenance, support, updates, enhancements, or
modifications.

This software was written by Adam Baumberg, Vision Group,
School of Computer Studies, University of Leeds, UK. The code and use
thereof should be attributed to the author where appropriate
(including demonstrations which rely on it's use).
*/

/* test for ProfileSequence class */

#include <ProfileSequence.h>
#include <gl.h>
#include <getopt.h>
#include <math.h>
#include <stdlib.h>

#define RADIANS_OF(x) ((x) * 3.14159265358979323846 / 180)
#define USAGE " sequencetest -a alignfile -t trackfile -o outputfile \
-p pca_file -s show_file -d direction_angle \n \
-e error_allowed -w waiting_time -x use_extended_model \n"

int NO_CONTROL_POINTS = 32;
extern int WAIT_TIME;
extern bool Profile::draw_arrow;
extern bool Profile::draw_colour;
main(int argc, char** argv)
{
  Profile::draw_colour = TRUE;
  FILTER_DIRECTION = Point2(0,0);
  COS_ANGLE_ERROR = -2.0;
  char inputfile[50];
  char outfile[50];
  char *outfile_pnt = &(outfile[0]);
  inputfile[0] = outfile[0] = 0;
  char mode = 'x';
  int c;
  bool tmp_sequence = FALSE;
  while ((c = getopt(argc, argv, "xa:t:o:p:s:d:e:w:")) != EOF)
    switch(c)
      {
      case 'a':
	if (optarg != NULL) sscanf(optarg, "%s", inputfile);
	mode = 'a';
	break;
      case 't':
	if (optarg != NULL) sscanf(optarg, "%s", inputfile);
	mode = 't';
	break;
      case 'o':
	if (optarg != NULL) sscanf(optarg, "%s", outfile);
	break;
      case 'p':
	if (optarg != NULL) sscanf(optarg, "%s", inputfile);
	mode = 'p';
	break;
      case 's':
	if (optarg != NULL) sscanf(optarg, "%s", inputfile);
	mode = 's';
	break;
      case 'd':
	{
	  float theta;
	  sscanf(optarg, "%f", &theta);
	  theta = RADIANS_OF(theta);
	  FILTER_DIRECTION = Point2(fcos(theta), fsin(theta));
	 break;
	}
      case 'e':
	{
	  float err;
	  sscanf(optarg, "%f", &err);
	  COS_ANGLE_ERROR = fcos(RADIANS_OF(err));
	  break;
	}
      case 'w':
	if (optarg != NULL) sscanf(optarg, "%d", &WAIT_TIME);
	break;
      case 'x':
	Profile::draw_arrow = TRUE;
	break;
      case '?':
	fprintf(stderr," usage - %s", USAGE);
      }

  char* in_ext = strrchr(inputfile, '.');
  if (in_ext != NULL && strncmp(in_ext,".Z",2) == 0)
    {
      char sys_command[200];
      sprintf(sys_command," zcat %s > /tmp/tmp_sequence.txt",
	      inputfile);
      system(sys_command);
      sprintf(inputfile,"/tmp/tmp_sequence.txt");
      tmp_sequence = TRUE;
    }

  if (outfile[0] == 0) outfile_pnt = NULL;
  ProfileSequence data(inputfile);
  switch (mode)
    {
    case 'a':
      data.align(outfile_pnt);
      break;
    case 't':
      data.track(outfile_pnt);
      break;
    case 's':
      {
	data.calculate_mean();
	prefsize(data.mean->width * 4 ,data.mean->height * 4);
#ifndef VOGL
	foreground();
#endif
	winopen(" profiles ");
	color(BLACK);
	clear();  
	Point2 centre(data.mean->width * 2,data.mean->height * 2);
	data.display_shapes(centre);
	break;
      }
    case 'p':
      {
        data.do_analysis();
	ofstream tmp_file("tmp");
	tmp_file << *(data.pca) << endl;
	tmp_file.close();
	prefsize(data.mean->width * 4 ,data.mean->height * 4);
	foreground();
	winopen(" PCA ");
	color(BLACK);
	clear();  
	Point2 centre(data.mean->width * 2,data.mean->height * 2);
	//data.display_shapes(centre);
	int indx = 0;
	do
	  {
	    color(BLACK);
	    clear();
	    cout << " enter mode to look at (1..) " << endl;
	    indx = -1;
	    cin >> indx;
	    cout << " looking at mode " << indx << endl;
	    if (indx > 0) data.display_mode(indx,1.2);
	    else if (indx < 0) data.random_walk(-indx);
	  } while (indx != 0);
	
	gexit();
      }
    }    
  if (tmp_sequence) system("rm /tmp/tmp_sequence.txt");

}

// Local Variables:
// compile-command: "make sequencetest"
// End:
