/*
This source file is the copyright property of the University of Leeds
('The University').

Permission to use, copy, modify, and distribute this source file for
educational, research, and not-for-profit purposes, without fee and
without a signed licensing agreement, is hereby granted, provided that
the above copyright notice, this paragraph and the following three
paragraphs appear in all copies, modifications, and distributions.

In no event shall The University be liable to any party for direct,
indirect, special, incidental or consequential damages, including lost
profits, arising out of the use of this software and its documentation.

The software is provided without warranty. The University has no
obligation to provide maintenance, support, updates, enhancements, or
modifications.

This software was written by Adam Baumberg, Vision Group,
School of Computer Studies, University of Leeds, UK. The code and use
thereof should be attributed to the author where appropriate
(including demonstrations which rely on it's use).
*/

/*
 * BWImage.h
 * 
 * a Gray8Image whith each pixel containing
 * either MARK or CLEAR
 *
 * and with added functionality
 *
 */


#ifndef __BW_IMAGE_H__
#define __BW_IMAGE_H__

#include <Gray8Image.h>

class BWImage : public Gray8Image

{
 public:

  BWImage(int w, int h) : Gray8Image(w,h) {}
  
  BWImage* compress(BWImage* r = NULL);
  BWImage* uncompress(BWImage* r = NULL);
  Image* copy(Image* r = NULL) 
    { 
      if (r == NULL) r = new BWImage(width,height);
      return (this->Gray8Image::copy(r));
    }

  void remove_small_regions(int minsize);
  long display(long window = NULLWIN);
  BWImage* fix_holes(int gapsize, BWImage* r = NULL);
  bool is_compressed() { return ((end_data - data) < width * height ); }
  Image* neighbour_order(int n, Image* res = NULL);
  Image* add_gaussian_noise(realno sd, realno& noise, Image* res);
};


#endif
