/*
This source file is the copyright property of the University of Leeds
('The University').

Permission to use, copy, modify, and distribute this source file for
educational, research, and not-for-profit purposes, without fee and
without a signed licensing agreement, is hereby granted, provided that
the above copyright notice, this paragraph and the following three
paragraphs appear in all copies, modifications, and distributions.

In no event shall The University be liable to any party for direct,
indirect, special, incidental or consequential damages, including lost
profits, arising out of the use of this software and its documentation.

The software is provided without warranty. The University has no
obligation to provide maintenance, support, updates, enhancements, or
modifications.

This software was written by Adam Baumberg, Vision Group,
School of Computer Studies, University of Leeds, UK. The code and use
thereof should be attributed to the author where appropriate
(including demonstrations which rely on it's use).
*/

/*
 * image.c++
 * general image class
 * not fully implemented yet
 *
 * author : A M Baumberg
 */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <Gray8Image.h>
#include <BWImage.h>
#include <kernel.h>
#include <RGB32Image.h>
#include <string.h>
#include <point2.h>

#ifndef MAX
#define MAX(a,b) (( (a) > (b)) ? (a) : (b))
#endif

#ifndef ABS
#define ABS(a)          (((a)<0) ? -(a) : (a))
#endif

#ifndef SGN
#define SGN(a)          (((a)<0) ? -1 : 1)
#endif

#ifdef gl_interface
#include <gl/gl.h>
#else
#include <X11/X.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xatom.h>
#endif


// use setenv DISPLAY_GRAYS ON
// to display all 256 graylevels (default OFF)
EnvParameter gray_mode_full("DISPLAY_GRAYS", FALSE);

Image* Gray8Image::copy(Image* r)
{
  if (r == NULL) r = (Image *) new Gray8Image(width, height);
  unsigned char *idat = data, *odat = r->get_data(), *edat = endofdata();
  for (;idat < edat; idat++)
    { *odat++ = *idat; }
  return (r);
}

Image* Gray8Image::resample(int xstep , int ystep , Image* resampled)
{
  int new_width = ((width / xstep) / 4) * 4;
  int new_height = height / ystep;
  if ((height % ystep) != 0) new_height++;
  if (resampled == NULL) resampled = (Image *)
	    new Gray8Image(new_width, new_height);
  unsigned char* idat;
  unsigned char* odat;
  int outy = 0;
  for (int y = 0; y < height; y += ystep)
    {
      idat = get_pixel(0,y);
      odat = resampled->get_pixel(0, outy);
      for (int x = 0; x < new_width; x ++)
	{
	  *odat = *idat;
	  odat++;
	  idat = idat + xstep;
	}
      outy++;
    }
  return (resampled);
}

  
void Gray8Image::clear_border()
{
  int w1 = width-1;
  int h1 = height-1;
  for (int y = 0; y < height; y++)
    {
      *get_pixel(0,y) = CLEAR;
      *get_pixel(w1,y) = CLEAR;
    }
  for (int x = 0; x < width; x++)
    {
      *get_pixel(x,0) = CLEAR;
      *get_pixel(x,h1) = CLEAR;
    }
}

Image* Gray8Image::convolve(Kernel k, Image* convolved)
{
  unsigned char *upnt, *pnt, *dpnt;
  unsigned char *out;
  int temp_total;
  int k11 = k.coeff[0][0], k12 = k.coeff[0][1], k13 = k.coeff[0][2]; 
  int k21 = k.coeff[1][0], k22 = k.coeff[1][1], k23 = k.coeff[1][2]; 
  int k31 = k.coeff[2][0], k32 = k.coeff[2][1], k33 = k.coeff[2][2];
  int ksum = abs(k11)+abs(k12)+abs(k13)
    +abs(k21)+abs(k22)+abs(k23)
      +abs(k31)+abs(k32)+abs(k33);
  if (convolved == NULL) convolved = new Gray8Image(width, height);
  convolved->clear_border();
  for (int y = 1; y < (height-1); y++)
    {
      upnt = get_pixel(0,y-1);
      pnt = get_pixel(0,y);
      dpnt = get_pixel(0,y+1);
      out = convolved->get_pixel(1,y);
      for (int x = 1; x < (width-1); x++)
	{
	  temp_total  = k11 * upnt[0] + k12 * upnt[1] + k13 * upnt[2];
	  temp_total += k21 * pnt[0] + k22 * pnt[1] + k23 * pnt[2];
	  temp_total += k31 * dpnt[0] + k32 * dpnt[1] + k33 * dpnt[2];
	  *(out++) = (unsigned char) abs(temp_total / ksum);
	  upnt++; pnt++; dpnt++;
	}
    }
  return (convolved);
}

// realno Gray8Image::pix_convolve(NagMatrix& k, int x, int y)
// {
//   int hcols = (k.no_columns() - 1) / 2;
//   int hrows = (k.no_rows() - 1) / 2;
//   if (check_pixel(x + hcols, y + hrows) == 0)
//     return (0);
//   if (check_pixel(x - hcols, y - hrows) == 0)
//     return (0);
//   realno total = 0.0;
//   int i,j;
//   x -= hcols;
//   y -= hrows;
//   for (i = 0;i < k.no_columns(); i++)
//     for (j = 0; j < k.no_rows(); j++)
//       total += k.read(i,j) * *get_pixel(i+x,j+y);

//   return (total);
// }

// Image* Gray8Image::mconvolve(NagMatrix& k, Image* convolved)
// {
//   if (convolved == NULL) convolved = new Gray8Image(width, height);
//   realno tmp;
//   for (int x = 0; x < width; x++)
//     for (int y = 0; y < height; y++)
//       {
// 	tmp = fabs(pix_convolve(k,x,y));
// 	if (tmp > 255) tmp = 255;
// 	*convolved->get_pixel(x,y) = (unsigned char) tmp;
//       }

//   return (convolved);
// }

static int compar_char(const void* c1, const void* c2)
{
  return (int) (*((unsigned char *) c1) - *((unsigned char *) c2));
}

Image* Gray8Image::neighbour_order(int n, Image* ordered)
{
  if (ordered == NULL) ordered = new Gray8Image (width, height);
  ordered->clear_border();
  unsigned char table[9];
  unsigned char *upnt, *pnt, *dpnt;  
  unsigned char *out;
  int ntake1 = n - 1;
  for (int y = 1; y < height-1; y++)
    {
      upnt = get_pixel(0,y-1);
      pnt = get_pixel(0,y);
      dpnt = get_pixel(0,y+1);  
      out = ordered->get_pixel(1,y);
      for (int x = 1; x < width-1; x++)
	{
	  table[0] = upnt[0]; table[1] = upnt[1]; table[2] = upnt[2];
	  table[3] = pnt[0]; table[4] = pnt[1]; table[5] = pnt[2];
	  table[6] = dpnt[0]; table[7] = dpnt[1]; table[8] = dpnt[2];

	  qsort(table, 9, sizeof(unsigned char), &compar_char);
	  *out = table[ntake1];
	  upnt++; pnt++; dpnt++;out++;
	}
    }
  return (ordered);
}

Image* Gray8Image::minimum(Image* res)
{
  if (res == NULL)
    res = new Gray8Image(width, height);
  res->clear_border();
  unsigned char *upnt, *pnt, *dpnt, min, tmp;  
  unsigned char *out;
  for (int y = 1; y < height-1; y++)
    {
      upnt = get_pixel(0,y-1);
      pnt = get_pixel(0,y);
      dpnt = get_pixel(0,y+1);  
      out = res->get_pixel(1,y);
      for (int x = 1; x < width-1; x++)
	{
	  min = *upnt;
	  if (min > (tmp = *(upnt + 1))) min = tmp;
	  if (min > (tmp = *(upnt + 2))) min = tmp;
	  if (min > (tmp = *(pnt))) min = tmp;
	  if (min > (tmp = *(pnt + 1))) min = tmp;
	  if (min > (tmp = *(pnt + 2))) min = tmp;
	  if (min > (tmp = *(dpnt))) min = tmp;
	  if (min > (tmp = *(dpnt + 1))) min = tmp;
	  if (min > (tmp = *(dpnt + 2))) min = tmp;
	  
	  *out++ = min;
	  upnt++; pnt++; dpnt++;
	}
    }
  return (res);
}

Image* Gray8Image::maximum(Image* res)
{
  if (res == NULL)
    res = new Gray8Image(width, height);
  res->clear_border();
  unsigned char *upnt, *pnt, *dpnt, max, tmp;  
  unsigned char *out;
  for (int y = 1; y < height-1; y++)
    {
      upnt = get_pixel(0,y-1);
      pnt = get_pixel(0,y);
      dpnt = get_pixel(0,y+1);  
      out = res->get_pixel(1,y);
      for (int x = 1; x < width-1; x++)
	{
	  max = *upnt;
	  if (max < (tmp = *(upnt + 1))) max = tmp;
	  if (max < (tmp = *(upnt + 2))) max = tmp;
	  if (max < (tmp = *(pnt))) max = tmp;
	  if (max < (tmp = *(pnt + 1))) max = tmp;
	  if (max < (tmp = *(pnt + 2))) max = tmp;
	  if (max < (tmp = *(dpnt))) max = tmp;
	  if (max < (tmp = *(dpnt + 1))) max = tmp;
	  if (max < (tmp = *(dpnt + 2))) max = tmp;
	  
	  *out++ = max;
	  upnt++; pnt++; dpnt++;
	}
    }
  return (res);
}


static inline  unsigned char get_mid (unsigned char& a, 
				      unsigned char& b, unsigned char& c)
{
  if (a < b)
    {
      if (b <= c) return (b);
      else
	{
	  if (c <= a) return (a);
	  else return (c);
	}
    }
  else 
    {
      if (a <= c) return (a);
      else 
	{
	  if (b >= c) return (b);
	  else return (c);
	}
    }
}


Image* Gray8Image::fmed(Image* res)
{
  if (res == NULL)
    res = new Gray8Image(width, height);
  res->clear_border();
  unsigned char *upnt, *pnt, *dpnt, mid1, mid2, mid3, tmp;  
  unsigned char *out;
  for (int y = 1; y < height-1; y++)
    {
      upnt = get_pixel(0,y-1);
      pnt = get_pixel(0,y);
      dpnt = get_pixel(0,y+1);  
      out = res->get_pixel(1,y);
      for (int x = 1; x < width-1; x++)
	{
	  mid1 = get_mid(upnt[0],upnt[1],upnt[2]);
	  mid2 = get_mid(pnt[0],pnt[1],pnt[2]);
	  mid3 = get_mid(dpnt[0],dpnt[1],dpnt[2]);
	  *out++ = get_mid(mid1, mid2, mid3);

	  upnt++; pnt++; dpnt++;
	}
    }
  return (res);
}


Image* Gray8Image::sobel(int threshold, Image* hsobel)
{
  int sqthresh = threshold * threshold;
  int hpix, vpix;
  hsobel = convolve(SOBELH_KER, hsobel);
  Image* vsobel = convolve(SOBELV_KER);
  unsigned char *hdat, *vdat, *idat, *enddat;
  enddat = endofdata();
  hdat = hsobel->get_data();
  vdat = vsobel->get_data();
  for (idat = data; idat < enddat; idat++)
    {
      hpix = (int) *hdat;
      vpix = (int) *vdat;
      if ((hpix * hpix + vpix * vpix) > sqthresh) *hdat = MARK;
      else *hdat = CLEAR;
      hdat++; vdat++;
    }
  delete vsobel;
  return (hsobel);
}

Image* Gray8Image::sobel(Image* hsobel)
{
  int hpix, vpix;
  hsobel = convolve(Kernel(1,0,-1,2,0,-2,1,0,-1,TRUE), hsobel);
  Image* vsobel = convolve(Kernel(1,0,-1,2,0,-2,1,0,-1,FALSE));
  unsigned char *hdat, *vdat, *idat, *enddat;
  enddat = endofdata();
  hdat = hsobel->get_data();
  vdat = vsobel->get_data();
  for (idat = data; idat < enddat; idat++)
    {
      hpix = (int) *hdat;
      vpix = (int) *vdat;
      *hdat = (unsigned char) (MAX(hpix,vpix));
      hdat++; vdat++;
    }
  delete vsobel;
  return (hsobel);
}
      
Image* Gray8Image::threshold(int thresh, Image* outimg, int* no_marked)
{
  if (outimg == NULL)  outimg = new BWImage(width, height);
  int spare_marked;
  if (no_marked == NULL) no_marked = &spare_marked;
  *no_marked = 0;
  unsigned char *idat = data, *odat = outimg->get_data();
  unsigned char* enddat = endofdata();
  for (;idat<enddat;idat++)
    {
      if (*idat > thresh) 
	{
	  *odat = MARK;
	  (*no_marked)++;
	}
      else *odat = CLEAR;
      odat ++;
    }
  return (outimg);
}


Image* Gray8Image::extract_subimage(int xmin, int xmax, int ymin, int ymax
				    ,Image* subimg)
{
  check_pixel(xmin,ymin);
  check_pixel(xmax,ymax);
  if ((check_pixel(xmin,ymin) == NULL) || 
      (check_pixel(xmax,ymax) == NULL) ||
      xmin > xmax || ymin > ymax)
    {
      fprintf (stderr, " bad parameters to extract_subimage ");
      exit(1);
    }
  int newwidth = xmax - xmin + 1;
  if (subimg == NULL) subimg = new Gray8Image(newwidth, ymax - ymin + 1);
  unsigned char* indat,  *outdat;
  for (int y = ymin; y <= ymax; y++)
    {
      indat = get_pixel(xmin,y);
      outdat = subimg->get_pixel(0,y-ymin);
      for (int x = xmin; x <= xmax; x++)
	{
	  *outdat++ = *indat;
	  *indat++ = CLEAR;
	}
    }
  return (subimg);
}

/*void Gray8Image::paste_subimage(int xmin, int xmax, int ymin, int ymax
				    ,Image* subimg)
{
  check_pixel(xmin,ymin);
  check_pixel(xmax,ymax);
  if (xmin > xmax || ymin > ymax)
    {
      fprintf (stderr, " bad parameters to paste_subimage ");
      exit(1);
    }
  unsigned char* indat,  *outdat;
  for (int y = ymin; y <= ymax; y++)
    {
      outdat = get_pixel(xmin,y);
      indat = subimg->get_pixel(0,y-ymin);
      for (int x = xmin; x <= xmax; x++)
	{
	  *outdat++ = *indat++;
	}
    }
}*/

Image* Gray8Image::fcombine(Image* image2, int (*func) (void*, void*),
			    Image* res)
{
  if (res == NULL) res = new Gray8Image(width, height);
  unsigned char* idat1 = data, *idat2 = image2->get_data();
  unsigned char* odat = res->get_data(), *enddat = endofdata();
  for (;idat1 < enddat; idat1++)
    *odat++ = (*func) ((void*) (idat1), (void*) (idat2++));
  return (res);
}
      

Image* Gray8Image::simple_difference(Image* image2, realno thresh, 
				     int& no_marked, Image* res)
{
  no_marked = 0;
  int threshold = (int) thresh;
  // if thresh < 0 don't threshold, just subtract
  if (thresh < 0) 
    {
      no_marked = 0;
      return (difference(image2, res));
    }
  if (image2->get_width() != width || image2->get_height() != height)
    {
      fprintf(stderr, " can't difference different sized images !");
      exit(1);
    }
  if (res == NULL) res = new BWImage(width, height);
  unsigned char *idat1 = data,  *idat2 = image2->get_data()
    , *odat = res->get_data(), *enddat = endofdata();
  for (;idat1 < enddat; idat1++)
    {
      if (abs(*idat1 - *idat2) > threshold) 
	{
	  *odat = MARK;
	  no_marked++;
	}
      else *odat = CLEAR;
      idat2++;odat++;
    }
  return(res);
}

#include <Gray8diff.cc>


Image* Gray8Image::difference(Image* image2, Image* res)
{
  if (image2->get_width() != width || image2->get_height() != height)
    {
      fprintf(stderr, " can't difference different sized images !");
      exit(1);
    }
  if (res == NULL) res = new Gray8Image(width, height);
  unsigned char *idat1 = data,  *idat2 = image2->get_data()
    , *odat = res->get_data(), *enddat = endofdata();
  for (;idat1 < enddat; idat1++)
    {
      *odat++ =  abs (*idat1 - *idat2);
      idat2++;
    }
  return(res);
}


/*
Image* Gray8Image::flip_vertically(Image *res)
{
  if (res == NULL) res = new Gray8Image(width, height);
  unsigned char* idat, *odat;
  int y1 = height - 1;
  for (int y = 0; y < height; y++)
    {
      odat = res->get_pixel(0,y);
      idat = get_pixel(0,y1--); 
      for (int x = 0; x < width; x++)
	{
	  *odat++ = *idat++;
	}
    }
  return (res);
}
*/

Image* Gray8Image::half_blur(Image* res)
{
  int new_width = width  / 2;
  int new_height = height / 2;
  if (res == NULL) res = new Gray8Image(new_width, new_height);
  unsigned char* odat = res->get_data();
  for (int y = 0; y < new_height; y++)
    {
      unsigned char* in_even = get_pixel(0,y * 2);
      unsigned char* in_odd = get_pixel(0,y * 2 + 1);
      for (int x = 0; x < new_width; x++)
	{
	  int curr_pix = *in_even++ + *in_odd++ + 
	    *in_even++ + *in_odd++;
	  *odat++ = (curr_pix >> 2);
	}
    }
  return (res);
}
	  
Image* Gray8Image::blur(Image* res)
{      
  unsigned char *upnt, *pnt, *dpnt;
  unsigned char *out;
  int temp_total;
  if (res  == NULL) res = new Gray8Image(width, height);

  for (int y = 1; y < height-1; y++)
    {
      upnt = get_pixel(0,y-1);
      pnt = get_pixel(0,y);
      dpnt = get_pixel(0,y+1);
      out = res->get_pixel(1,y);
      for (int x = 1; x < width; x++)
	{
	  temp_total  = upnt[0] + upnt[1] + upnt[2] 
	    + pnt[0] + pnt[2]
	      + dpnt[0] + dpnt[1] + dpnt[2];
	  *(out++) = (unsigned char) (temp_total >> 3);
	  upnt++; pnt++; dpnt++;
	}
    }
  return (res);
}

Image* Gray8Image::mask(Image* mask, Image* res)
{
  if (mask == NULL) return (NULL);
  if (res == NULL) res = new Gray8Image(width, height);
  if ((mask->get_width() != width) || (mask->get_height() != height))
    {
      for (int y = 0; y < height; y++)
	for (int x = 0; x < width; x++)
	  {
	    unsigned char* mask_pix = mask->check_pixel(x,y);
	    if ((mask_pix != NULL) && (*mask_pix != 0))
	      *res->get_pixel(x,y) = *get_pixel(x,y);
	  }
    }
  else 
    {
      unsigned char* rdat = res->get_data();
      unsigned char* dat1 = data;
      unsigned char* dat2 = mask->get_data();
      unsigned char* edat = endofdata();
      while (dat1 < edat)
	*rdat++ = *dat1++ & *dat2++;
      return (res);
    }
}
	




#ifdef gl_interface 
// GL interface
long Gray8Image::display(long awindow)
{
  int width4 = width;
  if ((width % 4) != 0)
    width4 = 4 * ((width / 4) + 1);
  if (awindow != NULLWIN) glwin = awindow;
  if (glwin == NULLWIN)
    {
      int zoom = image_class_zoom.value;
      char wintitle [100];
      sprintf(wintitle ," GRAY8 image %d by %d ", width, height);
      prefsize(width4 * zoom, height * zoom);
      foreground();
      if (window_title == NULL) glwin = winopen(wintitle);
      else glwin = winopen(window_title);

      winset(glwin);
      reshapeviewport();
      prefsize(width4 * zoom, height * zoom);
      if (zoom  != 1)
	{
	  rectzoom((float) zoom, (float) zoom);
	  viewport(0, width4 * zoom - 1, 0, height * zoom - 1);
	  ortho2(-0.5, width4 + 0.5, -0.5, height + 0.5);
	}
      winconstraints();
      pixmode(PM_SIZE, 8);
      if (access_mode == IA_TOP_TO_BOTTOM) pixmode(PM_TTOB, 1);
      cmode();
      if (gray_mode_full.is_off())
	{
	  pixmode(PM_SHIFT,-1);
	  pixmode(PM_ADD24,128);
	  for(int k = 0; k < 128; k++)
	    mapcolor(k+128, 2 * k,2 * k,2 * k);
	}
      
      else  for(int k = 0; k < 256; k++) mapcolor(k,k,k,k);
      gflush();
      gconfig();
    }
  if (width != width4)
    {
      Gray8Image tmp_image(width4,height);
      tmp_image.clear();
      tmp_image.paste_subimage(0,width-1,0,height-1, this);
      winset(glwin);
      lrectwrite(0,0,width4-1,height-1, (unsigned long*)
		 tmp_image.get_data());
    }
  else
    {
      winset(glwin);
      lrectwrite(0,0,width-1,height-1,(unsigned long *) data);
    }
  return(glwin);
}

#else
// X interface


static Colormap cmap = 0;
int no_using_cmap = 0;
static unsigned char pixel;
//static unsigned long plane_masks[8];
//static unsigned int no_planes = 8;
static unsigned long reverse_cmap[256];
static int no_gray_cells;

static void setup_graymap(Display* mydisplay, int screen, Window mywindow)
{
  cmap = DefaultColormap(mydisplay, screen);
  char* image_mode = getenv("IMAGE_MODE");
  no_gray_cells = 256;
  int min_cells = 256;
  if ((image_mode == NULL) || strcasecmp(image_mode,"HALF") == 0)
    {
      no_gray_cells = 128;
      min_cells = 16;
    }

  Status res = 0;
  unsigned long pix_vals[256];
  int no_cells = no_gray_cells;

  while ((res == 0) && (no_cells >= min_cells))
    {
      
      res = XAllocColorCells(mydisplay, cmap, False, NULL,
			     0, pix_vals, no_cells);
      no_cells /= 2;
    }

  if (res == 0)
    {
      cmap = XCreateColormap(mydisplay, mywindow,
			     DefaultVisual(mydisplay, screen),
			     AllocNone);
      no_cells = no_gray_cells;
      while ((res == 0) && (no_cells >= 8))
	{
	  res = XAllocColorCells(mydisplay, cmap, True, NULL,
				 0, pix_vals, no_cells);

	  no_cells /= 2;
	}
      
      if (res == 0) exit(1);
    }
  
  no_cells *= 2;
      
  XColor* gray = (XColor *) new XColor [no_cells];
  for (int cell = 0; cell < no_cells; cell++)
    {
      int intensity = (int) ((cell  * (long) 65536 ) / (no_cells));
      gray[cell].red = gray[cell].green = gray[cell].blue =
	intensity;
      gray[cell].pixel = pix_vals[cell];
      gray[cell].flags = DoRed | DoGreen | DoBlue;
    }
      
  for (int i = 0; i < 256; i++)
    reverse_cmap[i] = pix_vals[(i * no_cells) / 256];

  XStoreColors(mydisplay, cmap, gray, no_cells);
  delete [no_cells] gray;
  pixel = (unsigned char) pix_vals[0];
}

void Gray8Image::virtual_display()
{
  if (mydisplay == NULL)
    {
      mydisplay = XOpenDisplay("");
      int screen = DefaultScreen(mydisplay);
      mypixmap = XCreatePixmap(mydisplay,
			       DefaultRootWindow(mydisplay), width,
			       height, 8);
      myimage = XCreateImage( mydisplay, DefaultVisual(mydisplay, screen),
			     8, ZPixmap, 0, None, width, height,
			     8, 0);
      mygc = XCreateGC (mydisplay, mypixmap, 0, 0);

      if ((no_gray_cells == 256) && (access_mode == IA_TOP_TO_BOTTOM))
	myimage->data = (char *) data;
      else
      myimage->data = new char[width * height];
    }
  if ((char*) data != myimage->data)
    {
      Gray8Image tmp_img(width, height, (unsigned char*) myimage->data);
      flip_vertically(&tmp_img);
    }
  XPutImage(mydisplay, mypixmap, mygc, myimage, 0,0,0,0,width,
	    height);
}
  
long Gray8Image::display(long dummy)
{ 
  char *image_mode = getenv("IMAGE_DISPLAY");
  if ((image_mode != NULL) && strcasecmp(image_mode,"OFF") == 0)
    {
      virtual_display();
      return (0);
    }


  if (mydisplay == NULL)
    {
      char wintitle [40];
      char* wintitle_pnt = (char*) wintitle;
      sprintf(wintitle ," gray image %d by %d ", width, height);      
      if (window_title != NULL) wintitle_pnt = window_title;
      XSizeHints myhints;
      myhints.width = width;
      myhints.height = height;
      myhints.flags = PSize;
      mydisplay = XOpenDisplay("");
      int screen = DefaultScreen(mydisplay);
      unsigned long white = WhitePixel(mydisplay, screen);
      unsigned long black = BlackPixel(mydisplay, screen);
      mywindow = XCreateSimpleWindow (mydisplay, DefaultRootWindow(mydisplay)
				      ,100,100,width,height,5,black, white);
      mypixmap = XCreatePixmap(mydisplay,
			       DefaultRootWindow(mydisplay), width,
			       height, 8);
      XSetStandardProperties(mydisplay, mywindow, wintitle_pnt,
			     "image", None,
			     NULL, 0, &myhints);
      XSetWindowBackgroundPixmap(mydisplay, mywindow, mypixmap);
      XMapRaised(mydisplay, mywindow);
      myimage = XCreateImage( mydisplay, DefaultVisual(mydisplay, screen),
			     8, ZPixmap, 0, None, width, height,
			     8, 0);
      if (no_using_cmap == 0) setup_graymap(mydisplay, screen, mywindow);
      no_using_cmap++;
      XSetWindowColormap(mydisplay, mywindow, cmap);
//       XSetWindowAttributes xswa;
//       xswa.backing_pixel = (unsigned long) pixel;
//       xswa.backing_planes = plane_masks[0] | plane_masks[1] | plane_masks[2]
// 	| plane_masks[3] | plane_masks[4] | plane_masks[5] | plane_masks[6]
// 	  | plane_masks[7];
//       XChangeWindowAttributes(mydisplay, mywindow, CWBackingPlanes |
// 			      CWBackingPixel, &xswa); 
      //mygc = XCreateGC (mydisplay, mywindow, 0, 0);
      mygc = XCreateGC (mydisplay, mypixmap, 0, 0);
      /*if ((shift == 0) && (offset == 0))
	myimage->data = (char *) data;
      else*/

      if ((no_gray_cells == 256) && (access_mode == IA_TOP_TO_BOTTOM))
	myimage->data = (char*) data;
      else
	myimage->data = new char[width * height];
    }
  if ((char*) data != myimage->data)

    {
      int x,y;
      unsigned char *idat, *odat = (unsigned char*) myimage->data;
      for (y = height-1; y>=0; y--)
	{
	  idat = get_pixel(0,y);
	  for (x = 0; x < width; x++)
	    *odat++ = reverse_cmap[*idat++];
	}
    }
  

  //XPutImage(mydisplay, mywindow, mygc, myimage, 0,0,0,0,width,

  //height);
  XPutImage(mydisplay, mypixmap, mygc, myimage, 0,0,0,0,width,
	    height);
  XClearArea(mydisplay, mywindow, 0,0,0,0, True);
  return(0);
}

#endif


Image* Gray8Image::diff_stats(Image* image2, realno& mean,
			       realno& variance, Image* res)
{
  if (image2->get_width() != width || image2->get_height() != height)
    {
      fprintf(stderr, " can't difference different sized images !");
      exit(1);
    }
  if (res == NULL) res = new Gray8Image(width, height);
  unsigned char *idat1 = data,  *idat2 = image2->get_data()
    , *odat = res->get_data(), *enddat = endofdata();
  int diff, no_pixels = enddat - data;
  realno total_diff = 0 , total_square = 0;
  for (;idat1 < enddat; idat1++)
    {
      diff = (*idat1 - *idat2);
      *odat++ =  abs (diff);
      total_diff += diff;
      total_square += (diff * diff);
      idat2++;
    }
  mean = (total_diff / no_pixels);
  variance = (total_square / no_pixels) - (mean * mean);
  return(res);
}

Image* Gray8Image::image_blend(Image* i2, int a, int b, Image* res)
{
  // if trying to blend a Grayscale and colour image
  if ((i2 != NULL) && (i2->imgtype() == RGB32))
    return (i2->image_blend(this,b,a,res));

  if (res == NULL) res = new Gray8Image(width, height);
  unsigned char *idat1 = data, *idat2 = i2->get_data(),
    *odat = res->get_data(),  *enddat = endofdata();
  int pix;
  int aplusb = a + b;
  for (;idat1 < enddat; idat1++)
    {
      pix = (a * *idat1) + (b * *idat2);
      *odat++ = (pix / aplusb);
      idat2++;
    }
  return (res);
}

Image* Gray8Image::map_intensities(PntGrayMap gmap, Image* res)
{
  if (res == NULL) res = new Gray8Image(width, height);
  unsigned char *idat = data, *odat = res->get_data(),
    *enddat = endofdata();
  for (;idat < enddat; idat++)
    *odat++ = gmap[*idat];

  return (res);
}

Image* Gray8Image::b_w_compress(Image* res)
{
  if (res == NULL) res = new Gray8Image(width, height);
  unsigned char* idat = data, *odat = res->get_data();
  unsigned char flag;
  int counter;
  while (idat < end_data)
    {
      flag = *idat;
      counter = -1;
      while ((idat < end_data) && (*idat == flag) && counter < 127)
	{
	  counter++;
	  idat++;
	}
      if (flag == MARK) counter = counter | 128;
      *odat++ = counter;
    }
  res->set_enddata(odat);
  return (res);
}

Image* Gray8Image::b_w_uncompress(Image* res)
{
  if (res == NULL) res = new Gray8Image(width, height);
  unsigned char* idat = data, *odat = res->get_data();
  unsigned char* edat = res->endofdata();
  unsigned char flag;
  int counter;
  while (odat < edat)
    {
      counter = (*idat & 127) + 1;
      if ((*idat & 128) != 0) flag = MARK;
      else flag = CLEAR;
      while (counter > 0  && odat < edat)
	{
	  counter--;
	  *odat++ = flag;
	}
      idat++;
    }
  return (res);
}

realno Gray8Image::get_marked_fraction()
{
  int total_marked  = 0;
  for (unsigned char* idat = data; idat < end_data; idat ++)
    {
      if (*idat == MARK) total_marked++;
    }
  return ((realno) total_marked / (end_data - data));
}

void Gray8Image::save_pm(char* filename)
{
  // save as portable gray map (binary mode)
  FILE* outfile = NULL;
  if (filename == NULL) outfile = stdout;
  else if ((outfile = fopen(filename,"w")) == NULL)
    {
      fprintf(stderr, " Gray8Image: couldn't open file %s", filename);
      exit(1);
    }
  fprintf(outfile,"P5\n%d %d\n255\n", width, height);
  if (get_access_mode() == IA_TOP_TO_BOTTOM)
    fwrite(data,1, width*height, outfile);
  else
    {
      for (int i = height-1; i >=0; i--)
	fwrite(get_pixel(0,i),1,width,outfile);
    }
	  
  if (filename != NULL) fclose(outfile);
}

Gray8Image::Gray8Image(char* pgmfile)
{
  FILE* infile = NULL;
  if ((infile = fopen(pgmfile,"r")) == NULL)
    {
      fprintf(stderr, " Gray8Image: couldn't open file %s", pgmfile);
      exit(1);
    }
  char c1,c2;
  fscanf(infile,"%c%c", &c1, &c2);
  if (c1 != 'P' || c2 != '5')
    {
      fprintf(stderr," file not pgm in Gray8Image::Gray8Image \n");
      exit(1);
    }
  char aline[100];
  fgets(aline, 99, infile);
  while (aline[0] == '#' || aline[0] == '\n') fgets(aline, 99, infile);
  sscanf(aline,"%d %d\n", &width, &height);
  fscanf(infile,"%d\n", &bytes_per_pixel);
  if (bytes_per_pixel != 255)
    {
      fprintf(stderr," bad pgm format \n");
      exit(1);
    }

  bytes_per_pixel = 1;
  line_length = width * bytes_per_pixel;
  if (((width * bytes_per_pixel) % 4) == 0)
    data = (unsigned char*) new long[(width * height *
				      bytes_per_pixel) / 4];
  else
    data = new unsigned char[width * height * bytes_per_pixel];
  end_data = (data + width * height * bytes_per_pixel);

  fread(data,1, width*height, infile);
  if (access_mode != IA_TOP_TO_BOTTOM) flip_vertically(this);
#ifdef gl_interface
  glwin = NULLWIN;
#else
  mydisplay = NULL;
#endif
  Gray8_draw_color = 255;
}

Image* Gray8Image::to_rgb(Image* res)
{
  if (res == NULL) res = new RGB32Image(width, height);
  unsigned char* idat = data;
  //unsigned char* odat = res->get_data();
  RGB32pixel* odat = (RGB32pixel*) res->get_data();
  unsigned char* edat = endofdata();
  //RGB8pixel* pix;
  unsigned char glevel;
  for (;idat < edat; idat ++)
    {
      if ((*idat * 1.164) > 255) glevel = 255;
      else glevel = (*idat * 1.164);
      //glevel = glevel >> 5;
      //pix = (RGB32pixel*) odat;
      //pix->blue = (glevel / 2);
      odat->blue = (glevel);
      odat->green = (glevel);
      odat->red = (glevel); 
      odat++;
    }
  return (res);
}

void Gray8Image::draw_rectangle(int xmin, int xmax, int ymin, int
				ymax, int val)
{
  if (xmin < 0) xmin = 0;
  if (ymin < 0) ymin = 0;
  if (xmax >= width) xmax = (width-1);
  if (ymax >= height) ymax = (height-1);
  int nbytes = (xmax - xmin + 1) * bytes_per_pixel; 
  for (int y = ymin; y <= ymax; y++)
    memset(get_pixel(xmin,y),val, nbytes);
}

// written by ajh
// optimised by amb
Image* Gray8Image::transform(int centre_x, int centre_y, realno angle,
			     realno scale, Image* res)
{
  if (res == NULL) res = copy_type();
  unsigned char *dest;
  int x_from, y_from;
  realno cosa = cos(angle) / scale;
  realno sina = sin(angle) / scale;
  int& dw = res->get_width();
  int& dh = res->get_height();
  int dw2 = dw / 2;
  int dh2 = dh / 2;
  for (int y = 0; y < dh; ++y) 
    {
      dest = res->get_pixel(0,y);
      for (int x = 0; x < dw; ++x) 
	{
	  int x1 = x - dw2;
	  int y1 = y - dh2;

	  x_from = (int) (centre_x + x1 * cosa + y1 * sina);
	  y_from = (int) (centre_y + y1 * cosa - x1 * sina);

	  if ((x_from < 0) || (y_from < 0) || (x_from >= width)
	      || (y_from >= height))
	    *dest++ = 0;
	  else *dest++ = *get_pixel(x_from, y_from);
	}
    }
  return (res);
}


Gray8Image::~Gray8Image()
{
#ifndef gl_interface
  if (mydisplay != NULL)
    {
      XFreeGC(mydisplay, mygc);
      XFreePixmap(mydisplay, mypixmap);
      if (mywindow != NULLWIN)
	{
	  XDestroyWindow(mydisplay, mywindow);
	  no_using_cmap--;
	  if (no_using_cmap == 0)
	    {
	      XFreeColormap(mydisplay, cmap);
	      //fprintf(stderr," Freeing colormap \n");
	      cmap = 0;
	      no_using_cmap = 0;
	    }
	    
	}
      if (myimage->data != (char *) data) delete [] (myimage->data);
      myimage->data = None;
      XFree((char *) myimage);
      mydisplay = NULL;
    }
#endif
}


void Gray8Image::median_img_array(Image** img_array, int asize)
{

  if (img_array[0]->imgtype() != GRAY8)
    {
      fprintf(stderr," bad call to Gray8Image::median_img_array \n");
	fflush(stderr);
      abort();
    }
  
  unsigned char* table = new unsigned char[asize];
  int cpos = (asize + 1) / 2;
  int i,j,k;
  for (i = 0; i < width; i++)
    for (j = 0; j < height; j++)
      {
	for (k = 0; k < asize; k++)
	  table[k] = *(img_array[k]->check_pixel(i,j));
	
	qsort(table, asize, sizeof(unsigned char), &compar_char);
	*get_pixel(i,j) = table[cpos];
      }
  delete [] table;
}


RGB32Image* Gray8Image::rgb_read_image(RGB32Image* res)
{

#ifdef gl_interface
  
  long xsize, ysize;
  winset(glwin);
  getsize(&xsize, &ysize);

  if (res == NULL) res = new RGB32Image(xsize, ysize);

  // read colour mapped pixel data
  //Image* tmp_img = new Gray8Image(xsize, ysize);

  Colorindex* image_data = new Colorindex[xsize * ysize];
  rectread(0,0,xsize-1,ysize-1, image_data);

  // unmap the data
  Colorindex* idat = image_data;
  Colorindex* edat = &image_data[xsize * ysize];
  RGB32pixel* odat = (RGB32pixel*) res->get_data();

  short red_val, green_val, blue_val;

  for (;idat < edat; idat++)
    {
      
      getmcolor(*idat, &red_val, &green_val, &blue_val);
      odat->alpha = 0;
      odat->red = red_val;
      odat->green = green_val;
      odat->blue = blue_val;
      odat++;
    }

  delete [] image_data;
  return (res);
#else

  // to be implemented !
  fprintf(stderr,"Gray8Image::rgb_read_image not implemented in X !");
	fflush(stderr);
  abort();
  return (NULL);

#endif
}



void Gray8Image::set_draw_colour(unsigned char red, unsigned char green,
				       unsigned char blue)
{
  realno col = 0.299 * red + 0.587 * green + 0.114 * blue;
  Gray8_draw_color = (unsigned char) (col + 0.5);
}

void Gray8Image::set_draw_colour(unsigned char grey_value)
{
  Gray8_draw_color = grey_value;
}

void Gray8Image::draw_horizontal(int y, int xl, int xr)
{
  unsigned char* pixleft = check_pixel(xl, y);
  unsigned char* pixright = check_pixel(xr,y);
  if ((pixright == NULL) || (pixleft == NULL)) return;
  memset(pixleft, Gray8_draw_color, pixright - pixleft + 1);
}

void Gray8Image::plot_pixel(int x, int y)
{
  unsigned char *pnt = check_pixel(x,y);
  if (pnt != NULL) *pnt = Gray8_draw_color;
}

Image* Gray8Image::to_rgb(RGB32pixel* color_map, Image* res)
{
  if (res == NULL) res = new RGB32Image(width, height);
  RGB32pixel* pnt_out = (RGB32pixel*) res->get_data();
  unsigned char* pnt_in = data;
  while (pnt_in < end_data)
    *pnt_out++ = color_map[*pnt_in++];
  return (res);
}
  
