/*
This source file is the copyright property of the University of Leeds
('The University').

Permission to use, copy, modify, and distribute this source file for
educational, research, and not-for-profit purposes, without fee and
without a signed licensing agreement, is hereby granted, provided that
the above copyright notice, this paragraph and the following three
paragraphs appear in all copies, modifications, and distributions.

In no event shall The University be liable to any party for direct,
indirect, special, incidental or consequential damages, including lost
profits, arising out of the use of this software and its documentation.

The software is provided without warranty. The University has no
obligation to provide maintenance, support, updates, enhancements, or
modifications.

This software was written by Adam Baumberg, Vision Group,
School of Computer Studies, University of Leeds, UK. The code and use
thereof should be attributed to the author where appropriate
(including demonstrations which rely on it's use).
*/

/*
 * image.h
 * general image class
 * not fully implemented yet
 *
 * author : A M Baumberg
 */

#ifndef __GRAY8IMAGE_H__
#define __GRAY8IMAGE_H__

#include <image.h>
#include <math.h>
#include <stdlib.h>
#include <GrayPixel.h>
#include <RGB32Image.h>


// use setenv DISPLAY_GRAYS ON
// to display all 256 graylevels (default OFF)

class Gray8Image : public Image
{
protected:

  unsigned char Gray8_draw_color;

public:
  
  Gray8Image(int w, int h, unsigned char *img_data = NULL) 
    : Image(w, h, 1, img_data) { Gray8_draw_color = 255; }
  Gray8Image(char *pgmfile);
  void virtual_display();
  ~Gray8Image();

  /* general image functions as in image.h */

  ImageType imgtype() { return (GRAY8); }
  Image* copy (Image* r = NULL);
  Image* copy_type(int w = 0, int h = 0)
    { 
      if (w == 0) w = width;
      if (h == 0) h = height;
      return (new Gray8Image(w, h));
    }
  Image* virtual_copy()
  { return new Gray8Image(width, height, data); }

  BasePixel* get_base_pixel(int x, int y)
    { return new GrayPixel(*get_pixel(x,y)); }
  Image* resample (int xstep, int ystep, Image* r = NULL);
  RGB32Image* rgb_read_image(RGB32Image* res = NULL);

  Image* convolve (Kernel k, Image* r = NULL);

  Image* fcombine(Image* image2, int (*func) (void*, void* ), Image* r
		  = NULL);
  Image* neighbour_order(int n, Image* r = NULL);
  Image* minimum(Image* r = NULL);
  Image* maximum(Image* r = NULL);
  Image* fmed (Image* r = NULL);
  Image* sobel(int threshold, Image* r = NULL);
  Image* sobel(Image* r = NULL);
  Image* threshold (int threshold, Image* r = NULL, int* no_marked = NULL);

  Image* blur(Image*r = NULL);
  Image* half_blur( Image* r = NULL);
  Image* mask(Image* mask, Image* r = NULL);

  Image* extract_subimage(int xlo, int xhi, int ylo, int yhi,
			   Image* r = NULL);

  Image* difference(Image* i2, realno threshold, Image* r = NULL);
  Image* difference(Image* i2, Image* r = NULL);
  Image* simple_difference(Image* image2, realno threshold, 
			   int& no_marked, Image* r = NULL);

  //Image* flip_vertically(Image* r = NULL);
  long display(long glw = NULLWIN);

  Image* diff_stats(Image* i2, realno& mean, realno& variance, 
		    Image* r = NULL);

  Image* image_blend(Image* i2, int a, int b, Image* r = NULL);
  Image* map_intensities(PntGrayMap gmap, Image* r = NULL);

  void clear_border();
  void draw_rectangle(int xmin, int xmax, int ymin, int ymax, int val); 
  Image* transform(int centre_x, int centre_y, realno angle,
		   realno scale, Image* res = NULL);


 /* extra Gray8  specific functions */
  

  /* compress a black & white image as returned from difference */
  Image* b_w_compress(Image* r = NULL);
  Image* b_w_uncompress(Image* r = NULL);
  
  /* return fraction of marked pixels in image*/
  realno get_marked_fraction();

  /* save as ppm */
  void save_pm(char* filename = NULL);
  
  /* convert to RGB32 */
  Image* to_rgb(Image* r);
  Image* to_rgb(RGB32pixel* color_map, Image* res);

  void median_img_array(Image** img_array, int array_size);

  void set_draw_colour(unsigned char red, unsigned char green,
		       unsigned char blue);
  void set_draw_colour(unsigned char grey_value);
  
  void draw_horizontal(int y, int xl, int xr);

  void plot_pixel(int x, int y);

};

#endif
