/*
This source file is the copyright property of the University of Leeds
('The University').

Permission to use, copy, modify, and distribute this source file for
educational, research, and not-for-profit purposes, without fee and
without a signed licensing agreement, is hereby granted, provided that
the above copyright notice, this paragraph and the following three
paragraphs appear in all copies, modifications, and distributions.

In no event shall The University be liable to any party for direct,
indirect, special, incidental or consequential damages, including lost
profits, arising out of the use of this software and its documentation.

The software is provided without warranty. The University has no
obligation to provide maintenance, support, updates, enhancements, or
modifications.

This software was written by Adam Baumberg, Vision Group,
School of Computer Studies, University of Leeds, UK. The code and use
thereof should be attributed to the author where appropriate
(including demonstrations which rely on it's use).
*/

/*
 * YUVdiff211.cc
 *
 * quick differencing using subsampling
 * to speed things up for video differencing
 *
 * - AMB 6/4/93
 *
 */


static const int BOX_SIZE = 8;

static inline void get_detail(unsigned char* g1, unsigned char* g2,
			      unsigned char* r, int& width)
{
  int lpx,lpy;
  unsigned char* tg1, *tg2, *tr;
  for (lpy = 0; lpy < BOX_SIZE; lpy++)
    {
      tg1 = g1; tg2 = g2;
      tr = r;
      for (lpx = 0; lpx < BOX_SIZE; lpx++)
	*r++ = abs(*g2++ - *g1++);

      g1 = tg1 + width; g2 = tg2 + width;
      r = tr + width;
    }
}

      
Image* Gray8Image::difference(Image* image2, realno threshold, Image* res)
{
  if (this == NULL || image2 == NULL) return (NULL);
  if (image2->imgtype() != GRAY8)
    {
      fprintf(stderr, " can't difference different image types! ");
      exit(1);
    }
  if (res == NULL) res = new Gray8Image(width, height);
  res->clear(CLEAR);
  unsigned char *g1 = data;
  unsigned char *g2 = image2->get_data();

  unsigned char *tg1, *tg2, *trdat;
  unsigned char* edat = endofdata();
  unsigned char* rdat = res->get_data();
  unsigned char *rdat1, *rdat2, *rdat3;

  int up = BOX_SIZE * width;
  int left = BOX_SIZE;

  int upleft = left + up;
  int upleft2 = upleft / 2;
  int upright = up - left;

  g1 += (upleft2) * 3; g2 += (upleft2) * 3;
  rdat += (upleft2) * 3;

  int lx, ly;
  int lymax = height / BOX_SIZE;
  int lxmax = width / BOX_SIZE; 

  int interest_threshold = (threshold);

  int adj1 = upleft2, adj2 = left+adj1, adj3 = up+adj1, adj4 = upleft+adj1
    , adj5 = adj1-left, adj6 = adj1-up, adj7 = adj1-upleft
      , adj8 = adj1-upright, adj9 = adj1+upright;

  Image* flag_img = new Image(lxmax, lymax);
  lxmax -=2 ;lymax -=2;
  flag_img->clear(FALSE);
  for (ly = 1; ly < lymax; ly++)
    {
      tg1 = g1; tg2 = g2; 
      trdat = rdat;
      for (lx = 1; lx < lxmax; lx++)
	{
	  *rdat = abs (*g2 - *g1);
	  if (*rdat >= interest_threshold)
	    {
	      // this square
	      if (!*(flag_img->get_pixel(lx,ly)))
		{
		  get_detail(g1-adj1,g2-adj1, rdat-adj1, width);
		  *(flag_img->get_pixel(lx,ly)) = TRUE;
		}
	      // left
	      if (!*(flag_img->get_pixel(lx-1,ly)))
		{
		  get_detail(g1-adj2,g2-adj2, rdat-adj2, width);
		  *(flag_img->get_pixel(lx-1,ly)) = TRUE;
		}
	      // up
	      if (!*(flag_img->get_pixel(lx,ly-1)))
		{
		  get_detail(g1-adj3,g2-adj3, rdat-adj3, width);
		  *(flag_img->get_pixel(lx,ly-1)) = TRUE;
		}
	      // up-left
	      if (!*(flag_img->get_pixel(lx-1,ly-1)))
		{
		  get_detail(g1-adj4,g2-adj4, rdat-adj4, width);
		  *(flag_img->get_pixel(lx-1,ly-1)) = TRUE;
		}
	      // right 
	      if (!*(flag_img->get_pixel(lx+1,ly)))
		{
		  get_detail(g1-adj5,g2-adj5, rdat-adj5, width);
		  *(flag_img->get_pixel(lx+1,ly)) = TRUE;
		}
	      // down
	      if (!*(flag_img->get_pixel(lx,ly+1)))
		{
		  get_detail(g1-adj6,g2-adj6, rdat-adj6, width);
		  *(flag_img->get_pixel(lx,ly+1)) = TRUE;
		}
	      // down-right
	      if (!*(flag_img->get_pixel(lx+1,ly+1)))
		{
		  get_detail(g1-adj7,g2-adj7, rdat-adj7, width);
		  *(flag_img->get_pixel(lx+1,ly+1)) = TRUE;
		}
	      // down-left
	      if (!*(flag_img->get_pixel(lx-1,ly+1)))
		{
		  get_detail(g1-adj8,g2-adj8, rdat-adj8, width);
		  *(flag_img->get_pixel(lx-1,ly+1)) = TRUE;
		}
	      // up-right
	      if (!*(flag_img->get_pixel(lx+1,ly-1)))
		{
		  get_detail(g1-adj9,g2-adj9, rdat-adj9, width);
		  *(flag_img->get_pixel(lx+1,ly-1)) = TRUE;
		}
	    } 
	  g1 += left; g2 += left;
	  rdat += left;
	}
      g1 = tg1 + up; g2 = tg2 + up;
      rdat = trdat + up;

    }
  delete (flag_img);
  return (res);
}
