/*
This source file is the copyright property of the University of Leeds
('The University').

Permission to use, copy, modify, and distribute this source file for
educational, research, and not-for-profit purposes, without fee and
without a signed licensing agreement, is hereby granted, provided that
the above copyright notice, this paragraph and the following three
paragraphs appear in all copies, modifications, and distributions.

In no event shall The University be liable to any party for direct,
indirect, special, incidental or consequential damages, including lost
profits, arising out of the use of this software and its documentation.

The software is provided without warranty. The University has no
obligation to provide maintenance, support, updates, enhancements, or
modifications.

This software was written by Adam Baumberg, Vision Group,
School of Computer Studies, University of Leeds, UK. The code and use
thereof should be attributed to the author where appropriate
(including demonstrations which rely on it's use).
*/

/***************************************************************
 * C - C++ Header
 *
 * File : 	GrayPixel.h
 *
 * Module :	GrayPixel
 *
 * Author : 	A M Baumberg (CoMIR)
 *
 * Creation Date : Thu Aug 18 12:47:49 1994 
 *
 * Comments : 	fGrayPixel is a real-valued implementation
 *
 ***************************************************************/


/* include files */

#ifndef __GRAY_PIXEL_H__
#define __GRAY_PIXEL_H__

#include <math.h>
#include <base_pixel.h>

class GrayPixel;

class fGrayPixel : public BasePixel
{

public:

  realno val;
  friend GrayPixel;


  PixelType pix_type() { return (fGRAY_PIX); }

  fGrayPixel() { val = 0; }
  fGrayPixel(realno c) { val = c; }

  BasePixel* add(BasePixel* pix2, BasePixel* res = NULL)
    {
      if (res == NULL) 
	res =  (new fGrayPixel(val + pix2->get_field(0)));
      else res->set_field(0, val + pix2->get_field(0));
      return (res);
    }
  BasePixel* subtract(BasePixel* pix2, BasePixel* res = NULL)
    {
      if (res == NULL) 
	res = (new fGrayPixel(val - pix2->get_field(0)));
      else res->set_field(0, val - pix2->get_field(0));
      return (res);
    }

  realno dot(BasePixel* pix2)
    { return ( val * pix2->get_field(0)); }
  BasePixel* imult(int a, BasePixel* res = NULL)
    {
      if (res == NULL)
	res = (new fGrayPixel(a * val)); 
      else res->set_field(0, a * val);
      return (res);
    }

  BasePixel* fmult(realno a, BasePixel* res = NULL)
    { 
      if (res == NULL)
	res = (new fGrayPixel(a * val));
      else res->set_field(0, a * val);
      return (res);
    }

  BasePixel* idiv(int a, BasePixel* res = NULL)
    {
      if (res == NULL)
	res = (new fGrayPixel(val / a));
      else res->set_field(0, val / a);
      return (res);
    }
      
  BasePixel* fdiv(realno a, BasePixel* res = NULL)
    {
      if (res == NULL)
	res = (new fGrayPixel(val / a));
      else res->set_field(0, val / a);
      return (res);
    }

  BasePixel* mult(BasePixel* pix2, BasePixel* res = NULL)
    {
      if (res == NULL)
	res = new fGrayPixel(val * pix2->get_field(0));
      else res->set_field(0, val * pix2->get_field(0));
      return (res);
    }

  BasePixel* div(BasePixel* pix2, BasePixel* res = NULL)
    {
      if (res == NULL)
	res = new fGrayPixel(val / pix2->get_field(0));
      else res->set_field(0, val / pix2->get_field(0));
      return (res);
    }
  
  BasePixel* copy_sign(BasePixel* res = NULL)
    {
      if (res == NULL)
	res = new fGrayPixel(copysign(1.0, val));
      else res->set_field(0, copysign(1.0, val));
      return (res);
    }

  BasePixel* abs_pix(BasePixel* res = NULL)
    {
      if (res == NULL)
	res = new fGrayPixel(fabs(val));
      else res->set_field(0, fabs(val));
      return (res);
    }

  BasePixel* fbinary_op(BasePixel* pix2, realno (*bin_fn) (realno,  realno),
			BasePixel* res = NULL)
    {
      if (res == NULL) res = new fGrayPixel(bin_fn(val, pix2->get_field(0)));
      else res->set_field(0, bin_fn(val, pix2->get_field(0)));
      return (res);
    }

  BasePixel* funitary_op(realno (*uni_fn) (realno), BasePixel* res =
			 NULL)
    {
      if (res == NULL) res = new fGrayPixel(uni_fn(val));
      else res->set_field(0, uni_fn(val));
      return (res);
    }

  realno length2() 
    { return (val * val); }
  realno length()
    { return (fabs(val)); }
  int ilength2()
    { return (val * val); }
  int ilength()
    { return abs(val); }
  
  realno get_field(int /*i*/) 
    { return (val); }
      
  int get_ifield(int /*i*/)
    { return (val); }

  void set_field(int /*i*/, realno x) { val = x; }
  void set_ifield(int /*i*/, int x) { val = x; }


  BasePixel* get_elt(int n)
    { return &(((fGrayPixel*)this)[n]); }
  BasePixel* copy_type(realno n = 0)
    { return new fGrayPixel(n); }
  
  BasePixel* clone(BasePixel* res = NULL)
    { if (res == NULL) res = new fGrayPixel(val); 
      else res->set_field(0,val);
      return (res); }
};

  
  

class GrayPixel : public  BasePixel
{

public:

  int val;

  PixelType pix_type() { return (GRAY_PIX); }

  GrayPixel() { val = 0; }
  GrayPixel(int c) { val = c; }
  BasePixel* add(BasePixel* pix2, BasePixel* res = NULL)
    {
      if (res == NULL) 
	res =  (new GrayPixel(val + pix2->get_ifield(0)));
      else res->set_ifield(0,val + pix2->get_ifield(0));
      return (res);
    }
  BasePixel* subtract(BasePixel* pix2, BasePixel* res = NULL)
    {
      if (res == NULL) 
	res = (new GrayPixel(val - pix2->get_ifield(0)));
      else res->set_ifield(0,val - pix2->get_ifield(0));
      return (res);
    }

  realno dot(BasePixel* pix2)
    { return ( val * pix2->get_ifield(0)); }
  BasePixel* imult(int a, BasePixel* res = NULL)
    {
      if (res == NULL)
	res = (new GrayPixel(a * val)); 
      else res->set_ifield(0, a * val);
      return (res);
    }

  BasePixel* fmult(realno a, BasePixel* res = NULL)
    { 
      if (res == NULL)
	res = (new fGrayPixel(a * val));
      else res->set_ifield(0, a * val);
      return (res);
    }

  BasePixel* idiv(int a, BasePixel* res = NULL)
    {
      if (res == NULL)
	res = (new GrayPixel(val / a));
      else res->set_ifield(0, val / a);
      return (res);
    }
      
  BasePixel* fdiv(realno a, BasePixel* res = NULL)
    {
      if (res == NULL)
	res = (new fGrayPixel(val / a));
      else res->set_ifield(0, val / a);
      return (res);
    }

  BasePixel* mult(BasePixel* pix2, BasePixel* res = NULL)
    {
      if (res == NULL)
	res = new GrayPixel(val * pix2->get_ifield(0));
      else res->set_ifield(0,val * pix2->get_ifield(0));
      return (res);
    }

  BasePixel* div(BasePixel* pix2, BasePixel* res = NULL)
    {
      if (res == NULL)
	res = new GrayPixel(val / pix2->get_ifield(0));
      else res->set_ifield(0,val / pix2->get_ifield(0));
      return (res);
    }

  BasePixel* copy_sign(BasePixel* res = NULL)
    {
      int rval = 0;
      if (val > 0) rval = 1;
      else if (val < 0) rval = -1;
      if (res == NULL)
	res = new GrayPixel(rval);
      else res->set_field(0,rval);
      return (res);
    } 

  BasePixel* abs_pix(BasePixel* res = NULL)
    {
      if (res == NULL)
	res = new GrayPixel(abs(val));
      else res->set_ifield(0,abs(val));
      return (res);
    }

  BasePixel* fbinary_op(BasePixel* pix2, realno (*bin_fn) (realno,  realno),
			BasePixel* res = NULL)
    {
      if (res == NULL) res = new fGrayPixel(bin_fn(val, pix2->get_field(0)));
      else res->set_field(0, bin_fn(val, pix2->get_field(0)));
      return (res);
    }

  BasePixel* funitary_op(realno (*uni_fn) (realno), BasePixel* res =
			 NULL)
    {
      if (res == NULL) res = new fGrayPixel(uni_fn(val));
      else res->set_field(0, uni_fn(val));
      return (res);
    }

  realno length2() 
    { return (val * val); }
  realno length()
    { return (fabs(val)); }
  int ilength2()
    { return (val * val); }
  int ilength()
    { return abs(val); }
  
  realno get_field(int /*i*/) 
    { return (val); }
      
  int get_ifield(int /*i*/)
    { return (val); }

  void set_field(int /*i*/, realno x) { val = x; }
  void set_ifield(int /*i*/, int x) { val = x; }

  BasePixel* get_elt(int n)
    { return &(((GrayPixel*)this)[n]); }
  BasePixel* copy_type(realno n = 0)
    { return new GrayPixel(n); }
  
  BasePixel* clone(BasePixel* res = NULL)
    { if (res == NULL) res = new GrayPixel(val); 
      else res->set_ifield(0,val);
      return (res); }   
};



#endif
