/*
This source file is the copyright property of the University of Leeds
('The University').

Permission to use, copy, modify, and distribute this source file for
educational, research, and not-for-profit purposes, without fee and
without a signed licensing agreement, is hereby granted, provided that
the above copyright notice, this paragraph and the following three
paragraphs appear in all copies, modifications, and distributions.

In no event shall The University be liable to any party for direct,
indirect, special, incidental or consequential damages, including lost
profits, arising out of the use of this software and its documentation.

The software is provided without warranty. The University has no
obligation to provide maintenance, support, updates, enhancements, or
modifications.

This software was written by Adam Baumberg, Vision Group,
School of Computer Studies, University of Leeds, UK. The code and use
thereof should be attributed to the author where appropriate
(including demonstrations which rely on it's use).
*/

/*
 *	Movie.h -- Public include file for movies.
 */
#ifndef __MOVIE_H__
#define __MOVIE_H__

typedef struct MovieHeader {
    long	movie_id;        /* magic number for movie  'MOVI' */
    short	version;         /* Currently 2 */
    short	frameWidth;      /* X size of the movie frames. */
    short	frameHeight;     /* Y size of the movie frames. */
    double	framesPerSec;    /* Frame rate of the movie. */
    short	movieType;       /* SOUND or SILENT - see #define below */
    short 	showType;        /* MOVIE or SLIDE_SHOW - see #define below */
    long	frameCount;      /* Number of frames in the movie */
    long 	image_format;	 /* See #defines below */ 
    long	imageXsize;      /* x size of libil image */
    long	imageYsize;      /* y size of libil image */
    long	imageCsize;      /* c size (number of channels) of libil image*/
    long	imageDatatype;   /* ilType of data in libil image */
    long	loop_mode;       /* For movieplayer use. See #defines below */
    long	audioSampleRate; /* Sample rate of the sound track */
    long	audioSampleSize; /* Size of audio samples in the sound track */
    long	audioSampleFormat;/* Format of the data in the audio samples */
    long	audioNumChannels; /* Number of channels in the sound track */
    long	audioCompression; /* Audio compression type used */
    long	default_volume;   /* For movieplayer use, but not used */
    char	title[128];       /* Title of the movie */
    char	comments[128];    /* Comments about the movie, or whatever */
    char	field_extensions[256]; /* Space for additional features. */
} MovieHeader ;

typedef struct DirectoryEntry {
    long	location;       /* absolute file position of start of audio  */
    long	asize;		/* byte size of audio frame */
    long	vsize;		/* bytes size of video frame */
    long        displayTime;    /* for slide shows seconds to display image */
    long	frame_flags;	/* DSM's idea. Currently unused. */
} DirectoryEntry ;

#define MOVIE_ID		'MOVI' 
#define MOVIE_ID_OLD		0x00010400	
#define MOVIE_VERSION_1		1
#define MOVIE_VERSION_2		2
#define MOVIE           	1   /* For MovieHeader.showType */ 
#define SLIDE_SHOW      	2   /* For MovieHeader.showType */
#define SOUND           	1   /* For MovieHeader.movieType */
#define SILENT          	2   /* For MovieHeader.movieType */
#define IMAGE_FORMAT_MVC1	1   /* CL_MVC1 compressed image data */
#define IMAGE_FORMAT_FIT	2   /* Uncompressed image data */
#define IMAGE_FORMAT_CMAP_RLE	3   /* Not implemented */
#define IMAGE_FORMAT_MPEG	4   /* MPEG compressed movie. */
#define IMAGE_FORMAT_8RGB       5   /* Indigo 8 bit RGB image data. */
#define IMAGE_FORMAT_UNCOMPRESSED   8
#define IMAGE_FORMAT_GRAYSCALE	    16

#define LOOP_NONE		0   /* For MovieHeader.loop_mode */
#define LOOP_CONTINUOUSLY	1   /* For MovieHeader.loop_mode */
#define LOOP_SWINGING		2   /* For MovieHeader.loop_mode */

#define FRAME_FLAG_ISKEY	1

#endif
