/*
This source file is the copyright property of the University of Leeds
('The University').

Permission to use, copy, modify, and distribute this source file for
educational, research, and not-for-profit purposes, without fee and
without a signed licensing agreement, is hereby granted, provided that
the above copyright notice, this paragraph and the following three
paragraphs appear in all copies, modifications, and distributions.

In no event shall The University be liable to any party for direct,
indirect, special, incidental or consequential damages, including lost
profits, arising out of the use of this software and its documentation.

The software is provided without warranty. The University has no
obligation to provide maintenance, support, updates, enhancements, or
modifications.

This software was written by Adam Baumberg, Vision Group,
School of Computer Studies, University of Leeds, UK. The code and use
thereof should be attributed to the author where appropriate
(including demonstrations which rely on it's use).
*/

/***************************************************************
 * C++ source
 *
 * File : 	MpegStream.cc
 *
 * Module :	MpegStream
 *
 * Author : 	A M Baumberg (CoMIR)
 *
 * Creation Date : Wed Jun 12 16:49:32 1996 
 *
 * Comments : 	
 *
 ***************************************************************/


/* include files */
extern "C"
{
#include <mpeg.h>
}

#include <MpegStream.h>

MpegStream::MpegStream(char *filename, bool colour)
{
  mpeg_info = new ImageDesc;

  mpeg_file = fopen(filename, "r");
  if (!mpeg_file)
    {
      fprintf(stderr,": can't open mpeg file %s \n", filename);
      exit(1);
    }

  SetMPEGOption(MPEG_DITHER, colour ? (int) FULL_COLOR_DITHER :
		(int) GRAY_DITHER);

  
  if (!OpenMPEG (mpeg_file, mpeg_info))
   {
      fprintf (stderr, "OpenMPEG on %s failed\n", filename);
      exit (1);
   }

  current = NULL;
  frame_number = 0;
}

MpegStream::~MpegStream()
{
  fclose(mpeg_file);
  delete (current);
  delete mpeg_info;
}

Image* MpegStream::get_next()
{

  if (current == NULL)
    {
      if (mpeg_info->Depth == 8)
	current = new Gray8Image(mpeg_info->Width, mpeg_info->Height);
	  
      else 
	{
	  if (mpeg_info->Depth == 24)
	    current = new RGB32Image(mpeg_info->Width,
				     mpeg_info->Height);
	  else
	    {
	      // error !
		  fprintf(stderr,"Abort in MpegStream::get_next\n");
	fflush(stderr);
	      abort();
	    }
	}
      if (current->get_access_mode() != IA_TOP_TO_BOTTOM)
	{
	  fprintf(stderr," Warning: flipping images \n");
	  fprintf(stderr," use setenv IMAGE_VIDEO ON \n");
	}
    }
	
  frame_number++;
  if (GetMPEGFrame((char*) current->get_data()))
    {
      // mpeg_lib returns image addressed top to bottom
      if (current->get_access_mode() != IA_TOP_TO_BOTTOM)
	current->flip_vertically(current);


     return (current);
    }

  // otherwise
  frame_number--;
  delete (current);
  return (current = NULL);

}


void MpegStream::reset_movie()
{
  RewindMPEG(mpeg_file, mpeg_info);
  frame_number = 0;
}


void MpegStream::goto_frame(int n)
{
  if (!SkipMPEGFrames(n - frame_number))
    {
      fprintf(stderr,"Can not find MPEG frame %d \n", n);
      exit(1);
    }
  ImageSource::frame_number = n;
}
      
