/*
This source file is the copyright property of the University of Leeds
('The University').

Permission to use, copy, modify, and distribute this source file for
educational, research, and not-for-profit purposes, without fee and
without a signed licensing agreement, is hereby granted, provided that
the above copyright notice, this paragraph and the following three
paragraphs appear in all copies, modifications, and distributions.

In no event shall The University be liable to any party for direct,
indirect, special, incidental or consequential damages, including lost
profits, arising out of the use of this software and its documentation.

The software is provided without warranty. The University has no
obligation to provide maintenance, support, updates, enhancements, or
modifications.

This software was written by Adam Baumberg, Vision Group,
School of Computer Studies, University of Leeds, UK. The code and use
thereof should be attributed to the author where appropriate
(including demonstrations which rely on it's use).
*/

/***************************************************************
 * C - C++ Header
 *
 * File : 	PgmSource.h
 *
 * Module :	PgmSource
 *
 * Author : 	A M Baumberg (CoMIR)
 *
 * Creation Date : Wed Jun 29 20:55:39 1994 
 *
 * Comments : 	
 *
 ***************************************************************/


#ifndef __PGM_SOURCE_H__
#define __PGM_SOURCE_H__

#include <imgsrc.h>
#include <stdio.h>
#include <strings.h>

// utility function to read a pnm
Image* read_pnm(FILE *infile, Image* res = NULL);
Image* read_pnm(char* filename, Image* res = NULL);


// PgmSource :
// an imgsrc interface to a set of pnm files

class PgmSource : public ImageSource
{
public:

  char* file_prefix;
  char* file_suffix;
  char format_string[20];
  int no_digits;

  int step;
  int start_frame;
  int curr_frame; // same as frame_number !


public:

  PgmSource(char* prefix, char* suffix, int start_frame_no,
	    int ndigits)
    : ImageSource(NULL,0) 
      {
	file_prefix = new char[strlen(prefix)+1];
	file_suffix = new char[strlen(suffix)+1];
	strcpy(file_prefix, prefix);
	strcpy(file_suffix, suffix);
	sprintf(format_string,"%%s%%.%dd%%s",ndigits);
	no_digits = ndigits;
	curr_frame = start_frame = frame_number = start_frame_no;
      }
  
  PgmSource(char* initial_frame);

  ~PgmSource()
    { delete [] file_prefix; delete [] file_suffix;  }

  void restart()
    { frame_number = curr_frame = start_frame; }

  /* goto_frame now uses absolute frame numbering */
  void goto_frame(int n);
  
  Image* get_next();

  int get_no_frames();

};

#endif


