/*
This source file is the copyright property of the University of Leeds
('The University').

Permission to use, copy, modify, and distribute this source file for
educational, research, and not-for-profit purposes, without fee and
without a signed licensing agreement, is hereby granted, provided that
the above copyright notice, this paragraph and the following three
paragraphs appear in all copies, modifications, and distributions.

In no event shall The University be liable to any party for direct,
indirect, special, incidental or consequential damages, including lost
profits, arising out of the use of this software and its documentation.

The software is provided without warranty. The University has no
obligation to provide maintenance, support, updates, enhancements, or
modifications.

This software was written by Adam Baumberg, Vision Group,
School of Computer Studies, University of Leeds, UK. The code and use
thereof should be attributed to the author where appropriate
(including demonstrations which rely on it's use).
*/

/***************************************************************
 * C - C++ Header
 *
 * File : 	PnmStream.h
 *
 * Module :	PnmStream
 *
 * Author : 	A M Baumberg (CoMIR)
 *
 * Creation Date : Thu Jun 13 10:21:20 1996 
 *
 * Comments : 	connect to a Stream of PNM's 
 *
 ***************************************************************/


/* include files */

#ifndef __PNM_STREAM_H__
#define __PNM_STREAM_H__

#include <imgsrc.h>

// PnmStream :
// Passed a unix command that will generate a stream of
// pnm's to stdout OR simply an input stream containing pnm's
// This class connects to the stream to generate
// images accessed using the get_next() function

// note the get_next() method is implemented
// in "PgmSource.cc"

class PnmStream : public ImageSource
{
protected:

  FILE *input_stream;
  
public:
  
  PnmStream(char* command = NULL) : ImageSource(NULL,0) 
    {
      char mode = 'r';
      if (command != NULL) input_stream = popen(command, &mode);
      else input_stream = NULL;
    }
  
  PnmStream(FILE* stream_in) : ImageSource(NULL, 0)
  { input_stream = stream_in; }

  void reopen_stream(char* command)
    {
      char mode = 'r';
      if (input_stream != NULL) pclose(input_stream);
      input_stream = popen(command, &mode);
    }

  ~PnmStream()
    { pclose(input_stream); }

  Image* get_next();

};



#endif
