/*
This source file is the copyright property of the University of Leeds
('The University').

Permission to use, copy, modify, and distribute this source file for
educational, research, and not-for-profit purposes, without fee and
without a signed licensing agreement, is hereby granted, provided that
the above copyright notice, this paragraph and the following three
paragraphs appear in all copies, modifications, and distributions.

In no event shall The University be liable to any party for direct,
indirect, special, incidental or consequential damages, including lost
profits, arising out of the use of this software and its documentation.

The software is provided without warranty. The University has no
obligation to provide maintenance, support, updates, enhancements, or
modifications.

This software was written by Adam Baumberg, Vision Group,
School of Computer Studies, University of Leeds, UK. The code and use
thereof should be attributed to the author where appropriate
(including demonstrations which rely on it's use).
*/

/*
 * image.h
 * general image class
 * not fully implemented yet
 *
 * author : A M Baumberg
 */

#ifndef __RGB32IMAGE_H__
#define __RGB32IMAGE_H__

#include <RGBpixel.h>
#include <image.h>
#include <math.h>

struct RGB32pixel
{
public:

  char alpha;
  char blue;
  char green;
  char red;
};

class Gray8Image;


class RGB32Image : public Image
{
protected:
 
  RGB32pixel RGB32_draw_color;

public:
  
  RGB32Image(int w, int h, unsigned char* img_data = NULL)
    : Image(w, h, 4, img_data) { RGB32_draw_color.alpha = 
				   RGB32_draw_color.red = 
				   RGB32_draw_color.green =
				   RGB32_draw_color.blue = 255; }
  ImageType imgtype() { return (RGB32); }
  Image* copy (Image* r = NULL);
  Image* copy_type(int w = 0, int h = 0) 
    { 
      if (w == 0) w = width;
      if (h == 0) h = height;
      return (new RGB32Image(w, h));
    }
  
  Image* virtual_copy()
  { return new RGB32Image(width, height, data);  }

  BasePixel* get_base_pixel(int x, int y);
  long display(long glw = NULLWIN);
  RGB32Image* rgb_read_image(RGB32Image* res);
  void save_pm(char* fname);
  Image* difference(Image* image2, realno threshold, Image* res =
		    NULL);
  Image* simple_difference(Image* image2, realno thresh, 
			   int & no_marked, Image* res =  NULL);
  Image* resample(int xstep , int ystep , Image* resampled = NULL);
  int measure_contrast(void* p1, void* p2);
  Image* to_rgb(Image* /*res*/ = NULL) { return (this); }
  Image* convolve(Kernel k, Image* convolved = NULL);
  Image* image_blend(Image* i2, int a, int n, Image* res = NULL);
  Image* map_intensities(PntGrayMap gmap, Image* res = NULL);
  
  void median_img_array(Image** img_array, int asize);
  
  Image* sobel(int threshold, Image* res = NULL);
  Image* sobel(Image* hsobel = NULL);
  Image* half_blur(Image* res = NULL);
  Image* transform(int centre_x, int centre_y, realno angle,
		   realno scale, Image* res = NULL);

  // grayscale_using_lookup added by Tony Heap (ajh)
  Gray8Image* grayscale_using_lookup(char* lookup, Gray8Image* dest = NULL);

  // drawing routines

  void set_draw_colour(unsigned char red, unsigned char green,
		       unsigned char blue);
  void set_draw_colour(unsigned char grey_value);
  
  void draw_horizontal(int y, int xl, int xr);

  void plot_pixel(int x, int y);
  
};
// This has been superceded by read_pnm in PgmSource.h
// RGB32Image* read_ppm(char *fname);


#endif
