/*
This source file is the copyright property of the University of Leeds
('The University').

Permission to use, copy, modify, and distribute this source file for
educational, research, and not-for-profit purposes, without fee and
without a signed licensing agreement, is hereby granted, provided that
the above copyright notice, this paragraph and the following three
paragraphs appear in all copies, modifications, and distributions.

In no event shall The University be liable to any party for direct,
indirect, special, incidental or consequential damages, including lost
profits, arising out of the use of this software and its documentation.

The software is provided without warranty. The University has no
obligation to provide maintenance, support, updates, enhancements, or
modifications.

This software was written by Adam Baumberg, Vision Group,
School of Computer Studies, University of Leeds, UK. The code and use
thereof should be attributed to the author where appropriate
(including demonstrations which rely on it's use).
*/

/*
 * image.c++
 * general image class
 * not fully implemented yet
 *
 * author : A M Baumberg
 */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <RGB8Image.h>
#include <kernel.h>
#include <string.h>


#define MAX(a,b) (( (a) > (b)) ? (a) : (b))

#ifdef gl_interface
#include <gl/gl.h>
#else
#include <X11/X.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xatom.h>
#endif

const RGB8pixel RED_MASK_PIXEL = { 7, 0, 0 };
const RGB8pixel BLUE_MASK_PIXEL = { 0, 3, 0 };
const RGB8pixel GREEN_MASK_PIXEL = { 0, 0, 7};
const unsigned char& RED_MASK = *((char*) &RED_MASK_PIXEL);
const unsigned char& BLUE_MASK = *((char*) &BLUE_MASK_PIXEL);
const unsigned char& GREEN_MASK = *((char*) &GREEN_MASK_PIXEL);


class cdiff_table 
{
 public:
  unsigned char* data;
  cdiff_table(realno t);
};


cdiff_table::cdiff_table(realno threshold)
{
  fprintf (stderr, " creating table ... \n");
  data = new unsigned char[256*256];
  for (int c1 = 0; c1 < 256; c1++)

    {
      for (int c2 = 0; c2 < 256; c2++)
	{
	  unsigned char cc1 = c1;
	  unsigned char cc2 = c2;
	  RGB8pixel* pc1 = (RGB8pixel*) &cc1;
	  RGB8pixel* pc2 = (RGB8pixel*) &cc2;
	  
	  realno rdiff = (pc1->red ) - (pc2->red);
	  realno gdiff = (pc1->green ) - (pc2->green);
	  realno bdiff = (pc1->blue) - (pc2->blue );
	  realno temp = (rdiff * rdiff) + (gdiff * gdiff) + (bdiff * bdiff) * (128 / threshold);
	  if (temp > 255) temp = 255;
	  data[c1 * 256 + c2] = (unsigned char) temp;
	}
    }
  fprintf(stderr, " finished! \n");
}

static cdiff_table* table = NULL;

Image* RGB8Image::copy(Image* r)
{
  if (r == NULL) r = (Image *) new RGB8Image(width, height);
  unsigned char *idat = data, *odat = r->get_data(), *edat = endofdata();
  for (;idat < edat; idat++)
    { *odat++ = *idat; }
  return (r);
}

BasePixel* RGB8Image::get_base_pixel(int x, int y)
{
  RGB8pixel* tmp = (RGB8pixel*) get_pixel(x,y);
  return new RGBpixel(tmp->red, tmp->green, tmp->blue);
}



Image* RGB8Image::difference(Image* image2, realno threshold, Image* res)
{
  if (image2->get_width() != width || image2->get_height() != height)
    {
      fprintf(stderr, " can't difference different sized images !");
      exit(1);
    }
  if (table == NULL) table = new cdiff_table(threshold);
  if (res == NULL) res = new Gray8Image(width, height);
  unsigned char *idat1 = data,  
   *idat2 =  image2->get_data(),
   *enddat =  endofdata();
  unsigned char* odat = res->get_data();
  for (;idat1 < enddat; idat1++)
    {
      int i1 = *idat1;
      int i2 = *idat2;
      *odat = table->data[i1 * 256 + i2];
      idat2++;odat++;
    }
  return(res);
}

class colour_table
{

public:

  unsigned char data[256];
  colour_table();
};

colour_table::colour_table()
{
  RGB8pixel* pval;
  unsigned char y;
  for (int x = 0; x < 256; x++)
    {
      y = (char) x;
      pval = (RGB8pixel*) &y;
      data[x] = 0.3 * ((pval->red) << 5) +
	0.59 * ((pval->green) << 5) +
	  0.11 * ((pval->blue) << 6);
    }
}

colour_table ctab = colour_table();

Image* RGB8Image::to_gray(Image* res)
{
  if (res == NULL) res = new Gray8Image(width, height);
  unsigned char *idat = data,  
   *enddat =  endofdata();
  unsigned char* odat = res->get_data();
  unsigned char* ctd = ctab.data;
  for (;idat < enddat; idat++)
    {
      //*odat++ = ctd[*idat];
      *odat++ = *idat;
    }
  return(res);
}

int RGB8Image::measure_contrast(void* p1, void* p2)
{
  RGB8pixel* pix1 = (RGB8pixel*) p1;
  RGB8pixel* pix2 = (RGB8pixel*) p2;
  if ((pix1 == NULL) || (pix2 == NULL)) return (0);
  return (abs(pix1->red - pix2->red) + abs(pix1->green - pix2->green)
	  + abs(pix1->blue - pix2->blue));
}


// NOTE LACK OF FUNCTIONALITY 

#ifdef gl_interface
// GL interface


long RGB8Image::display(long awindow)
{
  if (awindow != NULLWIN) glwin = awindow;
  if (glwin == NULLWIN)
    {
      int zoom = image_class_zoom.value;
      char wintitle [40];
      sprintf(wintitle ," RGB8 image %d by %d ", width, height);
      prefsize(width * zoom, height * zoom);
      foreground();
      glwin = winopen(wintitle);
      winset(glwin);
      reshapeviewport();
      prefsize(width * zoom, height * zoom);
      if (zoom  != 1)
	{
	  rectzoom((float) zoom, (float) zoom);
	  viewport(0, width * zoom - 1, 0, height * zoom - 1);
	  ortho2(-0.5, width + 0.5, -0.5, height + 0.5);
	}
      winconstraints();
      pixmode(PM_SIZE, 8);
      //RGBmode();
      //cpack(0x00030707);
      cmode();
      if (access_mode == IA_TOP_TO_BOTTOM)
	{
	  pixmode(PM_TTOB, 1);
	  //ortho2(-0.5, width + 0.5, height + 0.5, -0.5);
	}
      for (unsigned char k = 0;k < 255; k++)
	{
	  RGB8pixel* pix = (RGB8pixel*) &k;
	  mapcolor(k, ((int) (pix->red)) << 5, ((int) (pix->green)) <<
		   5, ((int) (pix->blue)) << 6);
	}
      gflush();
      gconfig();
    }
  winset(glwin);
  lrectwrite(0,0,width-1,height-1,(unsigned long *) data);
  return(glwin);
}


#else
// X interface


static Colormap cmap = 0;
static unsigned char pixel;
static unsigned long plane_masks[8];
static unsigned int no_planes = 8;
static int shift;


static void setup_rgbcolmap(Display* mydisplay, int screen, Window mywindow)
{
  cmap = DefaultColormap(mydisplay, screen);
  Status res = 0;
  unsigned long pix;
  for (no_planes = 8; no_planes >4 && res == 0; no_planes--)
    {
      res = XAllocColorCells(mydisplay, cmap, True, plane_masks,
				   no_planes, &pix, 1);
    }
  if (res == 0)
    {
      cmap = XCreateColormap(mydisplay, mywindow,
			     DefaultVisual(mydisplay, screen),
			     AllocNone);
      res = XAllocColorCells(mydisplay, cmap, True, plane_masks,
				(no_planes = 8) , &pix, 1);
      if (res == 0) exit(1);
    }
  int no_cells = 1 << no_planes;
  XColor* rgbcol = (XColor *) new XColor [no_cells];
  for (shift = 0; plane_masks[0] != (1 << shift); shift++);
  for (int cell = 0; cell < no_cells; cell++)
    {
      int intensity = (int) ((cell * (long) 255 ) / (no_cells-1));
      rgbcol[cell].red = ((intensity >> 5) & 7) * 256;
      rgbcol[cell].green = ((intensity >> 2) & 7) * 256;
      rgbcol[cell].blue = (intensity & 3) * 256;
      rgbcol[cell].pixel = (cell << shift) | pix;
      rgbcol[cell].flags = DoRed | DoGreen | DoBlue;
    }
  XStoreColors(mydisplay, cmap, rgbcol, no_cells);
  delete [no_cells] rgbcol;
  pixel = (unsigned char) pix;
}
      
long RGB8Image::display(long dummy)
{
  if (mydisplay == NULL)
    {
      char wintitle [40];
      sprintf(wintitle ," RGB8 image %d by %d ", width, height);      
      XSizeHints myhints;
      myhints.width = width;
      myhints.height = height;
      myhints.flags = PSize;
      mydisplay = XOpenDisplay("");
      int screen = DefaultScreen(mydisplay);
      unsigned long white = WhitePixel(mydisplay, screen);
      unsigned long black = BlackPixel(mydisplay, screen);
      mywindow = XCreateSimpleWindow (mydisplay, DefaultRootWindow(mydisplay)
				      ,100,100,width,height,5,black, white);
      XSetStandardProperties(mydisplay, mywindow, wintitle, "image", None,
			     NULL, 0, &myhints);
      
      XMapRaised(mydisplay, mywindow);
      myimage = XCreateImage( mydisplay, DefaultVisual(mydisplay, screen),
			     8, ZPixmap, 0, None, width, height,
			     8, 0);
      if (cmap == 0) setup_rgbcolmap(mydisplay, screen, mywindow);
      XSetWindowColormap(mydisplay, mywindow, cmap);
      XSetWindowAttributes xswa;
      xswa.backing_pixel = (unsigned long) pixel;
      xswa.backing_planes = plane_masks[0] | plane_masks[1] | plane_masks[2]
	| plane_masks[3] | plane_masks[4] | plane_masks[5] | plane_masks[6]
	  | plane_masks[7];
      XChangeWindowAttributes(mydisplay, mywindow, CWBackingPlanes |
			      CWBackingPixel, &xswa); 
      mygc = XCreateGC (mydisplay, mywindow, 0, 0);
      if (shift == 0 && pixel == 0 && no_planes ==  8) 
	myimage->data = (char *) data;
      else myimage->data = new char[width * height];
    }
  if ((char*) data != myimage->data)
    {
      int rel_shift = shift + (no_planes - 8);
      int x,y;
      unsigned char *idat, *odat = (unsigned char*) myimage->data;
      if (rel_shift < 0) 
	{
	  rel_shift = -rel_shift;
	  for (y = height-1; y>=0; y--)
	    {
	      idat = get_pixel(0,y);
	      for (x = 0; x < width; x++)
		  *odat++ = (*idat++ >> rel_shift) | pixel;
	    }
	}
      else
	{
	  for (y = height-1; y>=0; y--)
	    {
	      idat = get_pixel(0,y);
	      for (x = 0; x < width; x++)
		  *odat++ = (*idat++ << rel_shift) | pixel;
	    }
	}
    }
  XPutImage(mydisplay, mywindow, mygc, myimage, 0,0,0,0,width, height);
  return (0);
}

#endif
