/*
This source file is the copyright property of the University of Leeds
('The University').

Permission to use, copy, modify, and distribute this source file for
educational, research, and not-for-profit purposes, without fee and
without a signed licensing agreement, is hereby granted, provided that
the above copyright notice, this paragraph and the following three
paragraphs appear in all copies, modifications, and distributions.

In no event shall The University be liable to any party for direct,
indirect, special, incidental or consequential damages, including lost
profits, arising out of the use of this software and its documentation.

The software is provided without warranty. The University has no
obligation to provide maintenance, support, updates, enhancements, or
modifications.

This software was written by Adam Baumberg, Vision Group,
School of Computer Studies, University of Leeds, UK. The code and use
thereof should be attributed to the author where appropriate
(including demonstrations which rely on it's use).
*/

/***************************************************************
 * C - C++ Header
 *
 * File : 	RGBpixel.h
 *
 * Module :	RGBpixel
 *
 * Author : 	A M Baumberg (CoMIR)
 *
 * Creation Date : Wed Jun 29 14:18:56 1994 
 *
 * Comments : 	
 *
 ***************************************************************/


/* include files */

#ifndef __RGB_PIXEL_H__
#define __RGB_PIXEL_H__

#include <math.h>
#include <base_pixel.h>

class RGBpixel;

class fRGBpixel : public BasePixel
{
  friend RGBpixel;

public:
  realno rgb[3];

  realno& blue;
  realno& green;
  realno& red;

  PixelType pix_type() { return (fRGB_PIX); }

  fRGBpixel() : red(rgb[0]), green(rgb[1]), blue(rgb[2])
    { red = green = blue = 0 ; }
  fRGBpixel(realno r, realno g, realno b) :
  red(rgb[0]), green(rgb[1]), blue(rgb[2])
    { red = r; green = g; blue = b; }
  BasePixel* add(BasePixel* pix2, BasePixel* res = NULL)
    { 
      if (res == NULL)
	res = new fRGBpixel(red + pix2->get_field(0),
			   green + pix2->get_field(1),
			   blue + pix2->get_field(2));
      else
	{
	  res->set_field(0, red + pix2->get_field(0));
	  res->set_field(1, green + pix2->get_field(1));
	  res->set_field(2, blue + pix2->get_field(2));
	}
      return (res);
    }
  

  BasePixel* subtract(BasePixel* pix2, BasePixel* res = NULL)
    { 
      if (res == NULL)
	res = new fRGBpixel(red - pix2->get_field(0),
			   green - pix2->get_field(1),
			   blue - pix2->get_field(2));
      else 
	{
	  res->set_field(0, red - pix2->get_field(0));
	  res->set_field(1, green - pix2->get_field(1));
	  res->set_field(2, blue - pix2->get_field(2));
	}
      return (res);
    }

  realno dot(BasePixel* pix2)
    { return (blue * pix2->get_field(2)
	      + green * pix2->get_field(1) 
	      + red * pix2->get_field(0)); }

  BasePixel* imult(int a, BasePixel* res = NULL)
    { if (res == NULL)
	res = new fRGBpixel(a * red, a * green, a * blue);
      else
	{
	  res->set_field(0, a * red);
	  res->set_field(1, a * green);
	  res->set_field(2, a * blue);
	}
      return (res);
    }
     
  BasePixel* fmult(realno a, BasePixel* res = NULL)
    {
      if (res == NULL)
	res = new fRGBpixel(a * red, a * green, a * blue);
      else
	{
	  res->set_field(0, a * red);
	  res->set_field(1, a * green);
	  res->set_field(2, a * blue);
	}
      return (res);
    } 

  BasePixel* idiv(int a, BasePixel* res = NULL)
    { 
      if (res == NULL)
	res = new fRGBpixel(red / a, green / a, blue / a);
      else
	{
	  res->set_field(0, red / a);
	  res->set_field(1, green / a);
	  res->set_field(2, blue / a);
	}
      return (res);
    } 
  BasePixel* fdiv(realno a, BasePixel* res = NULL)
    { 
      if (res == NULL)
	res = new fRGBpixel(red / a, green / a, blue / a);
      else
	{
	  res->set_field(0, red / a);
	  res->set_field(1, green / a);
	  res->set_field(2, blue / a);
	}
      return (res);
    }  
  
  BasePixel* mult(BasePixel* pix2, BasePixel* res = NULL)
    {
      if (res == NULL)
	res = new fRGBpixel(red * pix2->get_field(0),
			     green * pix2->get_field(1),
			     blue * pix2->get_field(2));

      else 
	{
	  res->set_field(0, red * pix2->get_field(0));
	  res->set_field(1, green * pix2->get_field(1));
	  res->set_field(2, blue * pix2->get_field(2));
	}
      return (res);
    }

  BasePixel* div(BasePixel* pix2, BasePixel* res = NULL)
    {
      if (res == NULL)
	res = new fRGBpixel(red / pix2->get_field(0),
			    green / pix2->get_field(1),
			    blue / pix2->get_field(2));

      else 
	{
	  res->set_field(0, red / pix2->get_field(0));
	  res->set_field(1, green / pix2->get_field(1));
	  res->set_field(2, blue / pix2->get_field(2));
	}
      return (res);
    }
  
  BasePixel* copy_sign(BasePixel* res = NULL)
    {
      realno rval1 = 0, rval2 = 0, rval3 = 0;
      if (red > 0) rval1 = 1;
      else if (red < 0) rval1 = -1;
      if (green > 0) rval2 = 1;
      else if (green < 0) rval2 = -1;
      if (blue > 0) rval3 = 1;
      else if (blue < 0) rval3 = -1;

      if (res == NULL)
	res = new fRGBpixel(rval1, rval2, rval3);
      else 
	{
	  res->set_field(0, rval1);
	  res->set_field(1, rval2);
	  res->set_field(2, rval3);
	}
      return (res);
    } 

  BasePixel* abs_pix(BasePixel* res = NULL)
    {
      if (res == NULL)
	res = new fRGBpixel(fabs(red), fabs(green), fabs(blue));
      else 
	{
	  res->set_field(0, fabs(red));
	  res->set_field(1, fabs(green));
	  res->set_field(2, fabs(blue));
	 }
      return (res);
    }

  BasePixel* fbinary_op(BasePixel* pix2, realno (*bin_fn) (realno,  realno),
			BasePixel* res = NULL)
    {
      if (res == NULL) res = 
	new fRGBpixel(bin_fn(red, pix2->get_field(0)),
		      bin_fn(green, pix2->get_field(1)),
		      bin_fn(blue, pix2->get_field(2)));
      res->set_field(0, bin_fn(red, pix2->get_field(0)));
      res->set_field(1, bin_fn(green, pix2->get_field(1)));
      res->set_field(2, bin_fn(blue, pix2->get_field(2)));
      return (res);
    }

  BasePixel* funitary_op(realno (*uni_fn) (realno), BasePixel* res = NULL)
    {
      if (res == NULL) res = 
	new fRGBpixel(uni_fn(red), uni_fn(green), uni_fn(blue));
      else
	{
	  res->set_field(0, uni_fn(red));
	  res->set_field(1, uni_fn(green));
	  res->set_field(2, uni_fn(blue));
	}
      return (res);
    }
  
  realno length2() 
    { return (red * red + green * green + blue * blue); }
  realno length()
    { return sqrt(length2()); }
  int ilength2()
    { return (red * red + green * green + blue * blue); }
  int ilength()
    { return (fabs(red) + fabs(green) + fabs(blue)) ; }

  realno get_field(int i)
    { return rgb[i]; }
  int get_ifield(int i)
    { return rgb[i]; }
  void set_field(int i, realno x)
    { rgb[i] = x; }
  void set_ifield(int i, int x)
    { rgb[i] = x; }

      
  BasePixel* get_elt(int n)
    { return &(((fRGBpixel*)this)[n]); }
  BasePixel* copy_type(realno n = 0)
    { return new fRGBpixel(n,n,n); }
 
  BasePixel* clone(BasePixel* res = NULL)
    { if (res == NULL) return new fRGBpixel(red, green, blue); 
      res->set_field(0,red);
      res->set_field(1,green);
      res->set_field(2,blue);
      return (res); }  
};


class RGBpixel : public BasePixel
{

public:

  int rgb[3];
  int& blue;
  int& green;
  int& red;

  PixelType pix_type() { return (RGB_PIX); }

  RGBpixel() : red(rgb[0]), green(rgb[1]), blue(rgb[2])
    { red = green = blue = 0; }
  RGBpixel(int r, int g, int b) :
  red(rgb[0]), green(rgb[1]), blue(rgb[2])
    { red = r; green = g; blue = b; }
  BasePixel* add(BasePixel* pix2, BasePixel* res = NULL)
    { 
      if (res == NULL)
	res = new RGBpixel(red + pix2->get_ifield(0),
			   green + pix2->get_ifield(1),
			   blue + pix2->get_ifield(2));
      else
	{
	  res->set_ifield(0, red + pix2->get_ifield(0));
	  res->set_ifield(1, green + pix2->get_ifield(1));
	  res->set_ifield(2, blue + pix2->get_ifield(2));
	}
      return (res);
    }


  BasePixel* subtract(BasePixel* pix2, BasePixel* res = NULL)
    { 
      if (res == NULL)
	res = new RGBpixel(red - pix2->get_ifield(0),
			   green - pix2->get_ifield(1),
			   blue - pix2->get_ifield(2));
      else 
	{
	  res->set_ifield(0, red - pix2->get_ifield(0));
	  res->set_ifield(1, green - pix2->get_ifield(1));
	  res->set_ifield(2, blue - pix2->get_ifield(2));
	}
      return (res);
    }

  realno dot(BasePixel* pix2)
    { return (blue * pix2->get_ifield(2)
	      + green * pix2->get_ifield(1) 
	      + red * pix2->get_ifield(0)); }

  BasePixel* imult(int a, BasePixel* res = NULL)
    { if (res == NULL)
	res = new RGBpixel(a * red, a * green, a * blue);
      else
	{
	  res->set_ifield(0, a * red);
	  res->set_ifield(1, a * green);
	  res->set_ifield(2, a * blue);
	}
      return (res);
    }
     
  BasePixel* fmult(realno a, BasePixel* res = NULL)
    {
      if (res == NULL)
	res = new fRGBpixel(a * red, a * green, a * blue);
      else
	{
	  res->set_field(0, a * red);
	  res->set_field(1, a * green);
	  res->set_field(2, a * blue);
	}
      return (res);
    } 

  BasePixel* idiv(int a, BasePixel* res = NULL)
    { 
      if (res == NULL)
	res = new RGBpixel(red / a, green / a, blue / a);
      else
	{
	  res->set_ifield(0, red / a);
	  res->set_ifield(1, green / a);
	  res->set_ifield(2, blue / a);
	}
      return (res);
    } 
  BasePixel* fdiv(realno a, BasePixel* res = NULL)
    { 
      if (res == NULL)
	res = new fRGBpixel(red / a, green / a, blue / a);
      else
	{
	  res->set_field(0, red / a);
	  res->set_field(1, green / a);
	  res->set_field(2, blue / a);
	}
      return (res);
    }  

  BasePixel* mult(BasePixel* pix2, BasePixel* res = NULL)
    {
      if (res == NULL)
	res = new RGBpixel(red * pix2->get_ifield(0),
			     green * pix2->get_ifield(1),
			     blue * pix2->get_ifield(2));

      else 
	{
	  res->set_ifield(0, red * pix2->get_ifield(0));
	  res->set_ifield(1, green * pix2->get_ifield(1));
	  res->set_ifield(2, blue * pix2->get_ifield(2));
	}
      return (res);
    }

  BasePixel* div(BasePixel* pix2, BasePixel* res = NULL)
    {
      if (res == NULL)
	res = new RGBpixel(red / pix2->get_ifield(0),
			    green / pix2->get_ifield(1),
			    blue / pix2->get_ifield(2));

      else 
	{
	  res->set_ifield(0, red / pix2->get_ifield(0));
	  res->set_ifield(1, green / pix2->get_ifield(1));
	  res->set_ifield(2, blue / pix2->get_ifield(2));
	}
      return (res);
    }
  
  BasePixel* copy_sign(BasePixel* res = NULL)
    {
      int rval1 = 0, rval2 = 0, rval3 = 0;
      if (red > 0) rval1 = 1;
      else if (red < 0) rval1 = -1;
      if (green > 0) rval2 = 1;
      else if (green < 0) rval2 = -1;
      if (blue > 0) rval3 = 1;
      else if (blue < 0) rval3 = -1;

      if (res == NULL)
	res = new RGBpixel(rval1, rval2, rval3);
      else 
	{
	  res->set_ifield(0, rval1);
	  res->set_ifield(1, rval2);
	  res->set_ifield(2, rval3);
	}
      return (res);
    }

  BasePixel* abs_pix(BasePixel* res = NULL)
    {
      if (res == NULL)
	res = new RGBpixel(abs(red), abs(green), abs(blue));
      else 
	{
	  res->set_ifield(0, abs(red));
	  res->set_ifield(1, abs(green));
	  res->set_ifield(2, abs(blue));
	}
      return (res);
    }

  BasePixel* fbinary_op(BasePixel* pix2, realno (*bin_fn) (realno,  realno),
			BasePixel* res = NULL)
    {
      if (res == NULL) res = 
	new fRGBpixel(bin_fn(red, pix2->get_field(0)),
		      bin_fn(green, pix2->get_field(1)),
		      bin_fn(blue, pix2->get_field(2)));
      else
	{
	  res->set_field(0, bin_fn(red, pix2->get_field(0)));
	  res->set_field(1, bin_fn(green, pix2->get_field(1)));
	  res->set_field(2, bin_fn(blue, pix2->get_field(2)));
	}
      return (res);
    }

 BasePixel* funitary_op(realno (*uni_fn) (realno), BasePixel* res = NULL)
    {
      if (res == NULL) res = 
	new fRGBpixel(uni_fn(red), uni_fn(green), uni_fn(blue));
      else
	{
	  res->set_field(0, uni_fn(red));
	  res->set_field(1, uni_fn(green));
	  res->set_field(2, uni_fn(blue));
	}
      return (res);
    }


  realno length2() 
    { return (red * red + green * green + blue * blue); }
  realno length()
    { return sqrt(length2()); }
  int ilength2()
    { return (red * red + green * green + blue * blue); }
  int ilength()
    { return (abs(red) + abs(green) + abs(blue)); }

  realno get_field(int i)
    { return rgb[i]; }
  int get_ifield(int i)
    { return rgb[i]; }
  void set_field(int i, realno x)
    { rgb[i] = x; }
  void set_ifield(int i, int x)
    { rgb[i] = x; }
      
  BasePixel* get_elt(int n)
    { return &(((RGBpixel*)this)[n]); }
  BasePixel* copy_type(realno n = 0)
    { return new RGBpixel(n,n,n); }
  
  BasePixel* clone(BasePixel* res = NULL)
    { if (res == NULL) return new RGBpixel(red, green, blue); 
      res->set_ifield(0,red);
      res->set_ifield(1,green);
      res->set_ifield(2,blue);
      return (res); }  
};


#endif


