/*
This source file is the copyright property of the University of Leeds
('The University').

Permission to use, copy, modify, and distribute this source file for
educational, research, and not-for-profit purposes, without fee and
without a signed licensing agreement, is hereby granted, provided that
the above copyright notice, this paragraph and the following three
paragraphs appear in all copies, modifications, and distributions.

In no event shall The University be liable to any party for direct,
indirect, special, incidental or consequential damages, including lost
profits, arising out of the use of this software and its documentation.

The software is provided without warranty. The University has no
obligation to provide maintenance, support, updates, enhancements, or
modifications.

This software was written by Adam Baumberg, Vision Group,
School of Computer Studies, University of Leeds, UK. The code and use
thereof should be attributed to the author where appropriate
(including demonstrations which rely on it's use).
*/

/***************************************************************
 * C - C++ Header
 *
 * File : 	SourceSplitter.h
 *
 * Module :	SourceSplitter
 *
 * Author : 	A M Baumberg (CoMIR)
 *
 * Creation Date : Mon Jun 17 12:15:18 1996 
 *
 * Comments : 	class for splitting image sources
 *
 ***************************************************************/

#ifndef __SOUURCE_SPLITTER_H__
#define __SOUURCE_SPLITTER_H__

#include <imgsrc.h>

// SourceSplitter connects an input source
// to multiple outputs sharing the same image data

class SSNode;

class SourceSplitter
{
protected:

  friend SSNode;

  SSNode* output_sources;
  ImageSource* input_source;

  bool* update_flags;
  int no_outputs;
  
  // called by each output_source from get_next
  virtual Image* request_new_frame(int id);

public:

  SourceSplitter(ImageSource* src_in, int no_output_sources);
  ~SourceSplitter();
  ImageSource* get_source(int n); // get source 0,1,...

};

// BufferSplitter
// split input and buffer to get 'n' output sources
// containing images from time (t-0), (t-1), ... (t-n)

class BufferSplitter: public SourceSplitter
{
protected:

  int count;
  int buff_size;
  Image** img_buffer;
  // in batch_mode each update refills the buffer
  // (i.e. n new frames are obtained)
  bool batch_mode;	

public:

  BufferSplitter(ImageSource* s1, int bufsize = 2, bool bmode = FALSE ) : 
    SourceSplitter(s1, bufsize)
  {
    count = 0;
    img_buffer = new Image*[no_outputs];
    update_flags = new bool[no_outputs];
    for (int i = 0; i < no_outputs; i++) img_buffer[i] = NULL;
    batch_mode = bmode;
  }

  // fill_buffer:
  // keep getting new frames until buffer is full
  // return (TRUE) if successful
  bool fill_buffer();

  Image* request_new_frame(int id);
};

#endif

