/*
This source file is the copyright property of the University of Leeds
('The University').

Permission to use, copy, modify, and distribute this source file for
educational, research, and not-for-profit purposes, without fee and
without a signed licensing agreement, is hereby granted, provided that
the above copyright notice, this paragraph and the following three
paragraphs appear in all copies, modifications, and distributions.

In no event shall The University be liable to any party for direct,
indirect, special, incidental or consequential damages, including lost
profits, arising out of the use of this software and its documentation.

The software is provided without warranty. The University has no
obligation to provide maintenance, support, updates, enhancements, or
modifications.

This software was written by Adam Baumberg, Vision Group,
School of Computer Studies, University of Leeds, UK. The code and use
thereof should be attributed to the author where appropriate
(including demonstrations which rely on it's use).
*/

/*
 * YUVImage.c++
 * general YUV image class
 * not fully implemented yet
 *
 * author : A M Baumberg
 */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>
#include <kernel.h>
#include <RGB32Image.h>
#include <YUVImage.h>
#include <BWImage.h>
#include <string.h>

#define MAX(a,b) (( (a) > (b)) ? (a) : (b))
#define MIN(a,b) (( (a) < (b)) ? (a) : (b))
#define INTL(x) ((int) ((65536 * (x)) + 0.5))

realno Y_WEIGHT = 0;
realno U_WEIGHT = 0;
realno V_WEIGHT = 0;

// use setenv DISPLAY_YUV_COLOUR ON
// to display YUV images as RGB
EnvParameter yuv_colour_display("DISPLAY_YUV_COLOUR", FALSE);

static const float UNDEFINED_HUE = 666;

DiffMode YUV_DIFF_MODE = DISCRETE;

static inline void yuv_to_rgb(unsigned char y, unsigned char u,
			      unsigned char v, int& r,
			      int& g,
			      int& b)
{
  /*r = (1.164 * (y - 16) + 1.596 * (v - 128));
  g = (1.164 * (y - 16) - 0.392 * (u - 128) - 0.813 * (v - 128));
  b = (1.164 * (y - 16) + 2.017 * (u - 128));*/
  int y1 = (y - 16) * 298;
  int u1 = u - 128;
  int v1 = v - 128;
  r = (y1 + 409 * v1);
  g = (y1 - 100 * u1 - 47 * v1);
  b = (y1 + 516 * u1);
  int max = MAX(MAX(MAX(r,g),b),1);
  r =  (r << 8) / max;
  g = (g << 8) / max;
  b = (b << 8) / max;

}

static inline void rgb_to_hsv(float r, float g, float b, float& h, float& s,
			      float& v)
{
  float min = MIN ( MIN(r,g) , b);
  float max = MAX ( MAX(r,g) , b);

  v =  max;
  if (max != 0) s = (max - min) / max;
  else s = 0;

  if (s == 0) h = UNDEFINED_HUE;
  else
    {
      float delta = max - min;
      if (r == max) h = (g - b) / delta;
      else if (g == max) h = 2 + (b - r) / delta;
      else if (b == max) h = 4 + (r - g) / delta;
      
      h *= 60;
      if (h < 0) h += 360;
      }
}


unsigned char 
weight_sum(int ydiff, int udiff, int vdiff, realno threshold)
{

  int temp = (int) 
    ((((ydiff * ydiff) * Y_WEIGHT) + ((udiff * udiff) * U_WEIGHT)
      + ((vdiff * vdiff) *  V_WEIGHT)) * (256.0 / 12.0));
  /*int temp = (abs(ydiff) + (abs(udiff) << 3) + (abs(vdiff) << 3));*/
  if (temp > 255) temp = 255;
  return ((unsigned char) temp);

}


inline static realno weight_sum(int ydiff, int udiff, int vdiff)
{
  return ((((ydiff * ydiff) * Y_WEIGHT) + ((udiff * udiff) * U_WEIGHT)
      + ((vdiff * vdiff) *  V_WEIGHT)));
}

void recalculate_weights(Image* img1, Image* img2)
{
  if (img1 == NULL || img2 == NULL || !(isYUV(img1->imgtype())) ||
      !(isYUV(img2->imgtype()))) return;
  YUVImage* im1 = (YUVImage*) img1;
  YUVImage* im2 = (YUVImage*) img2;
  realno mean, var1, var2, var3, sdi1, sdi2, sdi3;
  Image* temp1, *temp2, *temp3;
  temp1 = im1->diff_stats(im2, mean, var1);
  delete temp1;
  temp2 = im1->u_image()->diff_stats(im2->u_image(), mean, var2);
  delete temp2;
  temp3 = im1->v_image()->diff_stats(im2->v_image(), mean, var3);
  delete temp3;

  if ((var1 > 1e-10) && (var2 > 1e-10)  &&  (var3 > 1e-10))
    {
      Y_WEIGHT = 1.0 / var1;
      U_WEIGHT = 1.0 / var2;
      V_WEIGHT = 1.0 / var3;
    }
}

YUVImage::YUVImage(int w1, int h1, int w2, int h2, int w3, int h3) :
       Gray8Image(w1, h1)
{
  U_image = new Gray8Image(w2, h2);
  V_image = new Gray8Image(w3, h3);
  colour_image = NULL; 
  display_mode = 0;
}

YUVImage::YUVImage(int w1, int h1, unsigned char *y_data,
		   int w2, int h2, unsigned char *u_data,
		   int w3, int h3, unsigned char *v_data) :
  Gray8Image(w1,h1,y_data)
{
  U_image = new Gray8Image(w2, h2, u_data);
  V_image = new Gray8Image(w3, h3, v_data);
  colour_image = NULL;
  display_mode = 0;
}

YUVImage::~YUVImage()
{
  delete (U_image);
  delete (V_image);
  delete (colour_image);
}

void YUVImage::clear(unsigned char fill_char)
{ 
  this->Gray8Image::clear(fill_char); 
  U_image->clear(fill_char);
  V_image->clear(fill_char);
}
  
Image* YUVImage::resample(int xstep, int ystep, Image* r)
{
  int new_width = ((width / xstep) / 4) * 4;
  if (r == NULL) r = copy_type(new_width, height / ystep);
  if (r->imgtype() == GRAY8)
    return (this->Gray8Image::resample(xstep, ystep,r));

  this->Gray8Image::resample(xstep, ystep, ((YUVImage*) r)->y_image());
  U_image->Gray8Image::resample(xstep, ystep, ((YUVImage*) r)->u_image());
  V_image->Gray8Image::resample(xstep, ystep, ((YUVImage*) r)->v_image());
  return (r);
}


long YUVImage::display(long window)
{
  if (display_mode == 0)
    {
      if (yuv_colour_display.is_off())
	display_mode = 'b';
      else display_mode = 'c';
    }
  
  if (display_mode == 'b') 
    return(((Gray8Image*)this)->Gray8Image::display(window));  
  colour_image = to_RGB32(colour_image);
  return(colour_image->display(window));
}

int YUVImage::measure_contrast(void* p1, void* p2)
{
  unsigned char* y1 = (unsigned char*) p1;
  unsigned char* y2 = (unsigned char*) p2;
  int ratio = y_image()->get_width() / u_image()->get_width();
  int diff1 = (y1 - data) / ratio;
  int diff2 = (y2 - data) / ratio;
  unsigned char* u1 = get_udata() + diff1;
  unsigned char* v1 = get_vdata() + diff1;
  unsigned char* u2 = get_udata() + diff2;
  unsigned char* v2 = get_vdata() + diff2;

  realno contrast = Y_WEIGHT * abs(*y1 - *y2) +
    U_WEIGHT * abs(*u1 - *u2) +
      V_WEIGHT * abs(*v1 - *v2);
  return (256.0 * (contrast / (Y_WEIGHT + U_WEIGHT + V_WEIGHT)));
  
}
  
Image* YUVImage111::copy(Image* r)
{
  if (r == NULL) r = (Image *) new YUVImage111(width , height);
  this->Gray8Image::copy(r);
  U_image->copy(((YUVImage111*)r)->u_image());
  V_image->copy(((YUVImage111*)r)->v_image());
  return (r);
}

void YUVImage111::video_to_YUV(long* videodata) 
{
  
  unsigned char *y_odd;
  unsigned char *u_odd;
  unsigned char *v_odd;
  long l1, l2, l3, l4;
  int width4 = ((width+4)  << 2) + 16;
  for (int y = height - 1; y >= 0;y--)
    {
      y_odd = get_pixel(0,y);
      u_odd = U_image->get_pixel(0,y);
      v_odd = V_image->get_pixel(0,y);
      for (int x = 0; x < width; x ++)
	{
	  l1 = *videodata++;
	  l2 = *videodata++;
	  l3 = *videodata++;
	  l4 = *videodata++;
	  *y_odd++ = (unsigned char) (l1 >> 16) & 255;
	  *u_odd++ = (unsigned char) 
	    (((l1 & (3 << 14)) >> 8) + ((l2 & (3 << 14)) >> 10) + 
	     ((l3 & (3 << 14)) >> 12) + ((l4 & (3 << 14)) >> 14));
	  *v_odd++ = (unsigned char)
	    (((l1 & (3 << 12)) >> 6) + ((l2 & (3 << 12)) >> 8)
	    + ((l3 & (3 << 12)) >> 10) + ((l4 & (3 << 12)) >> 12));
	}
     videodata += width4; 
    }

}
	  
Image* YUVImage111::or_fields(Image* res)
{      
  if (res == NULL) res = new Gray8Image(width, height);
  unsigned char *ydat = data
    ,*udat = get_udata()
      ,*vdat = get_vdata()
	,*odat = res->get_data();
  while (ydat < end_data)
    {
      *odat++ = (*ydat++) | (*udat++) | (*vdat++);
    }
  return (res);
}
  
     
Image* YUVImage111::simple_difference(Image* image2, realno threshold,
				      int& no_marked, Image* res)
{
  no_marked = 0;
  if (this == NULL || image2 == NULL) return (NULL);
  if (image2->imgtype() != YUV111)
    {
      fprintf(stderr, " can't difference different image types! ");
      exit(1);
    }
  if (res == NULL) res = new BWImage(width, height);
  
  unsigned char* y1 = data;
  unsigned char* u1 = U_image->get_data();
  unsigned char* v1 = V_image->get_data();
  
  unsigned char* y2 = ((YUVImage *)image2)->get_ydata();
  unsigned char* u2 = ((YUVImage *)image2)->get_udata();
  unsigned char* v2 = ((YUVImage *)image2)->get_vdata();

  unsigned char* edat = endofdata();
  unsigned char* rdat = res->get_data();
  int ydiff, udiff, vdiff;
  while (y1 < edat)
    {
      ydiff = *y1++ -*y2++;
      udiff = *u1++ - *u2++;
      vdiff = *v1++ - *v2++;
      if ((ydiff * ydiff * Y_WEIGHT + udiff * udiff * U_WEIGHT +
	   vdiff * vdiff * V_WEIGHT) > threshold)
	{
	  *rdat++ = MARK;
	  no_marked++;
	}
      else *rdat++ = 0;
    }
  return (res);
}
 


Image* YUVImage211::copy(Image* r)
{
  if (r == NULL) r = (Image *) new YUVImage211(width, height);
  this->Gray8Image::copy(r);
  U_image->copy(((YUVImage211*)r)->u_image());
  V_image->copy(((YUVImage211*)r)->v_image());
  return (r);
}

void YUVImage211::video_to_YUV(long* videodata)
{
  unsigned char *y_odd;
  unsigned char *u_odd;
  unsigned char *v_odd;
  long l1, l2, l3, l4;
  for (int y = height - 1; y >= 0; y--)
    {
      y_odd = get_pixel(0,y);
      u_odd = U_image->get_pixel(0,y);
      v_odd = V_image->get_pixel(0,y);
      for (int x = 0; x < (width); x += 2)
	{
	  l1 = *videodata++;
	  l2 = *videodata++;
	  l3 = *videodata++;
	  l4 = *videodata++;
	  *y_odd++ = (unsigned char) (l1 >> 16) & 255;
	  *y_odd++ = (unsigned char) (l3 >> 16) & 255;
	  *u_odd++ = (unsigned char) 
	    (((l1 & (3 << 14)) >> 8) + ((l2 & (3 << 14)) >> 10) + 
	     ((l3 & (3 << 14)) >> 12) + ((l4 & (3 << 14)) >> 14));
	  *v_odd++ = (unsigned char)
	    (((l1 & (3 << 12)) >> 6) + ((l2 & (3 << 12)) >> 8)
	    + ((l3 & (3 << 12)) >> 10) + ((l4 & (3 << 12)) >> 12));
	}
      videodata += 16; 
    }

}

Image* YUVImage211::or_fields(Image* res)
{      
  if (res == NULL) res = new Gray8Image(width, height);
  unsigned char *ydat = data
    ,*udat = get_udata()
      ,*vdat = get_vdata()
	,*odat = res->get_data();
  while (ydat < end_data)
    {
      *odat++ = (*ydat++) | (*udat) | (*vdat);
      *odat++ = (*ydat++) | (*udat++) | (*vdat++);
    }
  return (res);
}

Image* YUVImage211::simple_difference(Image* image2, realno threshold,
				      int& no_marked, Image* res)

{
  no_marked = 0;
  if (this == NULL || image2 == NULL) return (NULL);
  if (image2->imgtype() != YUV211)
    {
      fprintf(stderr, " can't difference different image types! ");
      exit(1);
    }
  if (res == NULL) res = new BWImage(width, height);
  
  unsigned char* y1 = data;
  unsigned char* u1 = U_image->get_data();
  unsigned char* v1 = V_image->get_data();
  
  unsigned char* y2 = ((YUVImage *)image2)->get_ydata();
  unsigned char* u2 = ((YUVImage *)image2)->get_udata();
  unsigned char* v2 = ((YUVImage *)image2)->get_vdata();

  unsigned char* edat = endofdata();
  unsigned char* rdat = res->get_data();
  int udiff, vdiff, ydiff;
  while (y1 < edat)
    {
      udiff = (*u1++ - *u2++);
      vdiff = (*v1++ - *v2++);
      realno du = (udiff * udiff * U_WEIGHT);
      realno dv = (vdiff * vdiff * V_WEIGHT);

      ydiff = (*y1++ - *y2++); 
      if ((du + dv + (ydiff * ydiff * Y_WEIGHT)) > threshold)
	{ *rdat++ = MARK; no_marked++; }
      else *rdat++ = 0;

      ydiff = (*y1++ - *y2++);
      if ((du + dv + (ydiff * ydiff * Y_WEIGHT)) > threshold)
	{ *rdat++ = MARK; no_marked++; }
      else *rdat++ = 0;

    }
  return (res);
}

Image* YUVImage411::copy(Image* r)
{
  if (r == NULL) r = (Image *) new YUVImage411(width, height);
  this->Gray8Image::copy(r);
  U_image->copy(((YUVImage411*)r)->u_image());
  V_image->copy(((YUVImage411*)r)->v_image());
  return (r);
}

void YUVImage411::video_to_YUV(long* videodata) 
{
  
  unsigned char *y_even, *y_odd;
  unsigned char *u_even, *u_odd;
  unsigned char *v_even, *v_odd;
  long l1, l2, l3, l4;
  int wdth = width;
  for (int y = height - 1; y >= 0;y -=2)
    {
      y_even = get_pixel(0,y-1);
      y_odd = get_pixel(0,y);
      u_even = U_image->get_pixel(0,y-1);
      u_odd = U_image->get_pixel(0,y);
      v_even = V_image->get_pixel(0,y-1);
      v_odd = V_image->get_pixel(0,y);
      for (int x = 0; x < width; x += 4)
	{
	  l1 = *videodata++;
	  l2 = *videodata++;
	  l3 = *videodata++;
	  l4 = *videodata++;
	  *y_even++ = (unsigned char) (l1 >> 4) & 255;
	  *y_odd++ = (unsigned char) (l1 >> 16) & 255;
	  *y_even++ = (unsigned char) (l2 >> 4) & 255;
	  *y_odd++ = (unsigned char) (l2 >> 16) & 255;
	  *y_even++ = (unsigned char) (l3 >> 4) & 255;
	  *y_odd++ = (unsigned char) (l3 >> 16) & 255;
	  *y_even++ = (unsigned char) (l4 >> 4) & 255;
	  *y_odd++ = (unsigned char) (l4 >> 16) & 255;
	  *u_even++ = (unsigned char)
	    (((l1 & 12) << 4) + ((l2 & 12) << 2) +
	     (l3 & 12) + ((l4 & 12) >> 2));
	  *v_even++ = (unsigned char) 
	    (((l1 & 3) << 6) + ((l2 & 3) << 4) + ((l3 & 3) << 2) + (l4 & 3));
	  *u_odd++ = (unsigned char) 
	    (((l1 & (3 << 14)) >> 8) + ((l2 & (3 << 14)) >> 10) + 
	     ((l3 & (3 << 14)) >> 12) + ((l4 & (3 << 14)) >> 14));
	  *v_odd++ = (unsigned char)
	    (((l1 & (3 << 12)) >> 6) + ((l2 & (3 << 12)) >> 8)
	    + ((l3 & (3 << 12)) >> 10) + ((l4 & (3 << 12)) >> 12));
	}
      videodata += 12;
    }

}
	  
Image* YUVImage411::or_fields(Image* res)
{      
  if (res == NULL) res = new Gray8Image(width, height);
  unsigned char *ydat = data
    ,*udat = get_udata()
      ,*vdat = get_vdata()
	,*odat = res->get_data();
  while (ydat < end_data)
    {
      *odat++ = (*ydat++) | (*udat) | (*vdat);
      *odat++ = (*ydat++) | (*udat) | (*vdat);
      *odat++ = (*ydat++) | (*udat) | (*vdat);
      *odat++ = (*ydat++) | (*udat++) | (*vdat++);
    }
  return (res);
}

Image* YUVImage411::simple_difference(Image* image2, realno threshold,
				      int& no_marked, Image* res)
{
  no_marked = 0;
  if (this == NULL || image2 == NULL) return (NULL);
  if (image2->imgtype() != YUV411)
    {
      fprintf(stderr, " can't difference different image types! ");
      exit(1);
    }
  if (res == NULL) res = new BWImage(width, height);
  
  unsigned char* y1 = data;
  unsigned char* u1 = U_image->get_data();
  unsigned char* v1 = V_image->get_data();
  
  unsigned char* y2 = ((YUVImage *)image2)->get_ydata();
  unsigned char* u2 = ((YUVImage *)image2)->get_udata();
  unsigned char* v2 = ((YUVImage *)image2)->get_vdata();

  unsigned char* edat = endofdata();
  unsigned char* rdat = res->get_data();
  int ydiff, udiff, vdiff;
  while (y1 < edat)
    {
      udiff = (*u1++ - *u2++);
      vdiff = (*v1++ - *v2++);
      realno du = udiff * udiff * U_WEIGHT;
      realno dv = vdiff * vdiff * V_WEIGHT;

      ydiff = (*y1++ - *y2++);
      if ((du + dv + (ydiff * ydiff * Y_WEIGHT)) > threshold)
	{ *rdat++ = MARK; no_marked++; }
      else *rdat++ = 0;

      ydiff = (*y1++ - *y2++);
      if ((du + dv + (ydiff * ydiff * Y_WEIGHT)) > threshold)
	{ *rdat++ = MARK; no_marked++; }
      else *rdat++ = 0;

      ydiff = (*y1++ - *y2++);
      if ((du + dv + (ydiff * ydiff * Y_WEIGHT)) > threshold)
	{ *rdat++ = MARK; no_marked++; }
      else *rdat++ = 0;

      ydiff = (*y1++ - *y2++);
      if ((du + dv + (ydiff * ydiff * Y_WEIGHT)) > threshold)
	{ *rdat++ = MARK; no_marked++; }
      else *rdat++ = 0;

    }
  return (res);
}



static const int k1 = INTL(1.164);
static const int k2 = INTL(1.596);
static const int k3 = INTL(0.392);
static const int k4 = INTL(0.813);
static const int k5 = INTL(2.017);

Image* YUVImage::to_RGB32(Image* res)
{
  if (res == NULL) res = new RGB32Image(width, height);
  int xstep = 1;
  if (imgtype() == YUV211) xstep = 2;
  if (imgtype() == YUV411) xstep = 4;
  unsigned char *ydat = get_ydata()
    ,*udat = get_udata()
      ,*vdat = get_vdata()
	,*edat = endofdata();
  RGB32pixel* rgbdat = (RGB32pixel *) res->get_data();
  while (ydat < edat)
    {
      int u1 = *udat++ - 128;
      int v1 = *vdat++ - 128;
      for (int i = 0; i < xstep; i++)
	{
	  int y1 = k1 * (*ydat++ - 16);
	  int r = (y1 + k2 * v1) >> 16;
	  int g = (y1 - k3 * u1 - k4 * v1) >> 16;
	  int b = (y1 + k5 * u1) >> 16;
	  if (r > 255) r = 255; if (r < 0) r = 0;
	  if (g > 255) g = 255; if (g < 0) g = 0;
	  if (b > 255) b = 255; if (b < 0) b = 0;
	  rgbdat->alpha = 0;
	  rgbdat->red = r;
	  rgbdat->green = g;
	  rgbdat->blue = b;
	  rgbdat++;
	}
    }
  return (res); 
}
