/*
This source file is the copyright property of the University of Leeds
('The University').

Permission to use, copy, modify, and distribute this source file for
educational, research, and not-for-profit purposes, without fee and
without a signed licensing agreement, is hereby granted, provided that
the above copyright notice, this paragraph and the following three
paragraphs appear in all copies, modifications, and distributions.

In no event shall The University be liable to any party for direct,
indirect, special, incidental or consequential damages, including lost
profits, arising out of the use of this software and its documentation.

The software is provided without warranty. The University has no
obligation to provide maintenance, support, updates, enhancements, or
modifications.

This software was written by Adam Baumberg, Vision Group,
School of Computer Studies, University of Leeds, UK. The code and use
thereof should be attributed to the author where appropriate
(including demonstrations which rely on it's use).
*/

/*
 *  YUV image.h
 * YUV image class
 * not fully implemented yet
 *
 * author : A M Baumberg
 */

#ifndef __YUVIMAGE_H__
#define __YUVIMAGE_H__

enum DiffMode { DISCRETE, CONTINUOUS, NO_SHADOW };
extern DiffMode YUV_DIFF_MODE;

#include <image.h>
#include <Gray8Image.h>
#include <unistd.h>

void recalculate_weights(Image* img1, Image* img2);
unsigned char weight_sum(int ydiff, int udiff, int vdiff, realno
			 threshold);

class RGB32Image;
// use setenv DISPLAY_YUV_COLOUR ON
// to display YUV images as RGB

class YUVImage : public Gray8Image
{

 protected:
  
  Gray8Image* U_image;
  Gray8Image* V_image;

  Image* colour_image;
  char display_mode;

 public:

  YUVImage(int w1, int h1, int w2, int h2, int w3, int h3);
  YUVImage(int w1, int h1, unsigned char *y_data,
	   int w2, int h2, unsigned char *u_data,
	   int w3, int h3, unsigned char *v_data);

  ~YUVImage();


  void clear(unsigned char fill_char = CLEAR);
  Image* resample(int xstep, int ystep, Image* r = NULL);

  Image* y_image() { return ((Image*) this); }
  Image* u_image() { return ((Image*) U_image); }
  Image* v_image() { return ((Image*) V_image); }
  unsigned char* get_ydata() { return (data); }
  unsigned char* get_udata() { return (U_image->get_data()); }
  unsigned char* get_vdata() { return (V_image->get_data()); }

  virtual void video_to_YUV(long* /*videodata*/) {}
  virtual Image* or_fields(Image* /*res = NULL*/) { return NULL; }
  Image* to_RGB32(Image* res = NULL);
  long display(long window = NULLWIN);
  void save_pm(char *fname) {
    colour_image = to_RGB32(colour_image);
    colour_image->save_pm(fname); }
  Image* get_rgb32()
    {
      colour_image = to_RGB32(colour_image);
      return (colour_image);
    }
  Image* to_rgb(Image* res = NULL) { return (to_RGB32(res));}
  int measure_contrast(void* p1, void* p2);

};

class YUVImage111 : public YUVImage
{
 public:

  ImageType imgtype() { return (YUV111); }
  Image* copy (Image* r = NULL);
  Image* copy_type(int w, int h) 
    { 
      if (w == 0) w = width;
      if (h == 0) h = height;
      return (new YUVImage111(w, h));
    }
  /* constructor */ 
  YUVImage111(int w, int h) : YUVImage(w,h,w,h,w,h) {}
  YUVImage111(int w, int h, unsigned char *y_data, unsigned char
	      *u_data, unsigned char *v_data)
    : YUVImage(w,h,y_data, w, h, u_data, w, h, v_data) {}
  Image* virtual_copy()
  { return new YUVImage111(width, height, data, U_image->get_data(),
			   V_image->get_data()); }


  void video_to_YUV(long* videodata);
  virtual Image* or_fields(Image* res = NULL);
  Image* difference(Image* i2, realno threshold, Image* r = NULL);
  Image* simple_difference(Image* image2, realno threshold, 
			   int& no_marked, Image* r = NULL);

};

class YUVImage211 : public YUVImage
{
 public:

  ImageType imgtype() { return (YUV211); }
  Image* copy (Image* r = NULL);
  Image* copy_type(int w, int h)
    { 
      if (w == 0) w = width;
      if (h == 0) h = height;
      return (new YUVImage211(w, h));
    }

  /* constructor */ 
  YUVImage211(int w, int h) : YUVImage(w,h,w >> 1,h,w >> 1,h) {}
  YUVImage211(int w, int h, unsigned char *y_data,
	      unsigned char *u_data, unsigned char *v_data)
    : YUVImage(w,h,y_data,w >> 1,h,u_data,w >> 1,h,v_data) {}
  Image* virtual_copy()
  { return new YUVImage211(width, height, data, U_image->get_data(),
			   V_image->get_data()); }
  
  void video_to_YUV(long* videodata);
  virtual Image* or_fields(Image* res = NULL);
  Image* difference(Image* i2, realno threshold, Image* r = NULL);
  Image* simple_difference(Image* image2, realno threshold,
			   int& no_marked, Image* r = NULL);

};

class YUVImage411 : public YUVImage
{
 public:

  ImageType imgtype() { return (YUV411); }
  Image* copy (Image* r = NULL);  
  Image* copy_type(int w, int h) 
    {  
      if (w == 0) w = width;
      if (h == 0) h = height;
      return (new YUVImage411(w, h)); 
    }


  /* constructor */ 
  YUVImage411(int w, int h) : YUVImage(w,h,w >> 2,h,w >> 2,h) {}
  YUVImage411(int w, int h, unsigned char *y_data, 
	      unsigned char *u_data, unsigned char *v_data)
    : YUVImage(w,h,y_data, w >> 2, h, u_data, w >> 2, h, v_data) {}
  Image* virtual_copy()
  { return new YUVImage411(width, height, data, U_image->get_data(),
			   V_image->get_data()); }
  
  void video_to_YUV(long* videodata);
  virtual Image* or_fields(Image* res = NULL);
  Image* difference(Image* i2, realno threshold, Image* r = NULL);
  Image* simple_difference(Image* image2, realno threshold, 
			   int& no_marked, Image* r = NULL);

};

#endif




