/*
This source file is the copyright property of the University of Leeds
('The University').

Permission to use, copy, modify, and distribute this source file for
educational, research, and not-for-profit purposes, without fee and
without a signed licensing agreement, is hereby granted, provided that
the above copyright notice, this paragraph and the following three
paragraphs appear in all copies, modifications, and distributions.

In no event shall The University be liable to any party for direct,
indirect, special, incidental or consequential damages, including lost
profits, arising out of the use of this software and its documentation.

The software is provided without warranty. The University has no
obligation to provide maintenance, support, updates, enhancements, or
modifications.

This software was written by Adam Baumberg, Vision Group,
School of Computer Studies, University of Leeds, UK. The code and use
thereof should be attributed to the author where appropriate
(including demonstrations which rely on it's use).
*/

/***************************************************************
 * C - C++ Header
 *
 * File : 	base_pixel.h
 *
 * Module :	base_pixel
 *
 * Author : 	A M Baumberg (CoMIR)
 *
 * Creation Date : Tue Jun 28 12:16:47 1994 
 *
 * Comments : 	a pixel class for pixel operations
 *		for large images pixel by pixel operations are
 *		not advised 
 *	        -- use optimised image-based implementations
 *
 ***************************************************************/


#ifndef __BASE_PIXEL_H__
#define __BASE_PIXEL_H__
#include <realno.h>
#include <stdio.h>
enum PixelType { BASE_PIX, GRAY_PIX, fGRAY_PIX, RGB_PIX, fRGB_PIX,
		 YUV_PIX, fYUV_PIX };

class BasePixel
{
public:

  // virtual pixel handling functions -- do nothing for base class


  // pixel type

  virtual PixelType pix_type() { return (BASE_PIX); }

  // add 2 pixels

  virtual BasePixel* add(BasePixel* pix2, BasePixel* res = NULL)
    { fprintf(stderr,"%o %o", pix2, res); return (this); }

  // subtract 2 pixels
  virtual BasePixel* subtract(BasePixel* pix2, BasePixel* res = NULL)
    { fprintf(stderr,"%o %o", pix2, res); return (this); }

  // multiply (dot) 2 pixels
  virtual realno dot(BasePixel* pix2)
    { fprintf(stderr,"%o", &pix2); return (0); }

  // scale pixel
  virtual BasePixel* imult(int a, BasePixel* res = NULL)
    { fprintf(stderr,"%d %o", a, res); return (this); }

  virtual BasePixel* fmult(realno a, BasePixel* res = NULL)
    { fprintf(stderr,"%f %o", a, res); return (this); }
  
  virtual BasePixel* idiv(int a, BasePixel* res = NULL)
    { fprintf(stderr,"%d %o", a, res); return (this); }

  virtual BasePixel* fdiv(realno a, BasePixel* res = NULL)
    { fprintf(stderr,"%f %o", a, res); return (this); }

  virtual BasePixel* mult(BasePixel* pix2, BasePixel* res = NULL)
    { fprintf(stderr,"%o %o", pix2, res); return (this); }

  virtual BasePixel* div(BasePixel* pix2, BasePixel* res = NULL)
    { fprintf(stderr,"%o %o", pix2, res); return (this); }

  virtual BasePixel* copy_sign(BasePixel* res = NULL)
    { fprintf(stderr,"%o", res); return (this); }
  
  virtual BasePixel* abs_pix(BasePixel* res = NULL)
    { fprintf(stderr,"%o", res); return (this); }
  
  virtual BasePixel* get_elt(int n)
    { fprintf(stderr,"%d", n); return (this); }

  virtual BasePixel* copy_type(realno val = 0)
    { fprintf(stderr,"%f", val); return (this); }
  virtual BasePixel* clone(BasePixel* res = NULL)
    { fprintf(stderr,"%o", res); return (this); }

  // floating point binary operation on each element
  virtual BasePixel* fbinary_op(BasePixel* pix2, 
				realno (*bin_fn) (realno,  realno),
				BasePixel* res = NULL)
    { fprintf(stderr,"%o %o %o", pix2, res, bin_fn); return (NULL); }
  
  virtual BasePixel* funitary_op(realno (*uni_fn) (realno),
				 BasePixel* res = NULL)
    { fprintf(stderr,"%o %o", res, uni_fn); return (NULL); }

  // pixel length functions
  virtual realno length2() 
    { fprintf(stderr, "call to BasePixel::length2 \n"); return 0.0; }
  virtual realno length()
    { fprintf(stderr, "call to BasePixel::length  \n"); return 0.0; }
  virtual int ilength2()
    { fprintf(stderr, "call to BasePixel::ilength2\n"); return 0; }
  virtual int ilength()
    { fprintf(stderr, "call to BasePixel::ilength \n"); return 0; }

  // data access functions
  virtual realno get_field(int i)
    { fprintf(stderr, "call to BasePixel::get_field %d\n", i);
      return 0; }

  virtual int get_ifield(int i)
    { fprintf(stderr, "call to BasePixel::get_field %d\n", i);
      return 0; }

  virtual void set_field(int i, realno val)
    { fprintf(stderr, "call to BasePixel::get_field %d %f\n", 
	      i, val); }

   virtual void set_ifield(int i, int val)
    { fprintf(stderr, "call to BasePixel::get_field %d %d\n", 
	      i, val); }   
      

};


#endif
