/*
This source file is the copyright property of the University of Leeds
('The University').

Permission to use, copy, modify, and distribute this source file for
educational, research, and not-for-profit purposes, without fee and
without a signed licensing agreement, is hereby granted, provided that
the above copyright notice, this paragraph and the following three
paragraphs appear in all copies, modifications, and distributions.

In no event shall The University be liable to any party for direct,
indirect, special, incidental or consequential damages, including lost
profits, arising out of the use of this software and its documentation.

The software is provided without warranty. The University has no
obligation to provide maintenance, support, updates, enhancements, or
modifications.

This software was written by Adam Baumberg, Vision Group,
School of Computer Studies, University of Leeds, UK. The code and use
thereof should be attributed to the author where appropriate
(including demonstrations which rely on it's use).
*/

/*
 * sgmoviesrc.cc
 *  image source for movie files
 *
 */


#include <stdio.h>  /* For i/o */
#include <fcntl.h>  /* For open/read/close */
#include <stdlib.h> /* For malloc */
#include <unistd.h>
#include <sgmoviesrc.h>
#include <Gray8Image.h>
#include <RGB8Image.h>
#include <movie.h>
#include <dmedia/dmedia.h>


static struct mv_extras
{
public:
  
  MVid theMovie;
  MVid theImages;

};

SGmoviesrc::SGmoviesrc(char* mv_name)
{
  hidden = (void*) new (mv_extras);

  if (mvOpenFile(mv_name, O_RDONLY, &((mv_extras*)hidden)->theMovie) 
      != DM_SUCCESS)
    {
      fprintf(stderr, ": can't open %s\n", mv_name);
      exit(1);
    }    
  MVid theImages;
  mvFindTrackByMedium(((mv_extras*)hidden)->theMovie, DM_IMAGE, 
		      &((mv_extras*)hidden)->theImages);
  no_frames = (int) mvGetTrackLength(((mv_extras*)hidden)->theImages);
  width = mvGetImageWidth(((mv_extras*)hidden)->theImages);
  height = mvGetImageHeight(((mv_extras*)hidden)->theImages);

  DMpacking img_packing = mvGetImagePacking(((mv_extras*)hidden)->theImages);

  if (img_packing == DM_PACKING_RGBX) imgformat = RGB32;
  else
    {
      if (img_packing == DM_PACKING_GRAYSCALE) imgformat = GRAY8;
      else
	{
	  fprintf(stderr," unsupported packing type in movie \n");
	  exit(1);
	}
    }
} 
    

Image* SGmoviesrc::get_next()
{
  frame_number++;
  if (frame_number >= no_frames) return (NULL);
  if (current  == NULL)
    { 
      if (imgformat == GRAY8) current = new Gray8Image(width, height);
      else current = new RGB32Image(width, height);
    }    
  size_t img_size = (size_t)
    (current->endofdata() - current->get_data());
  
  if (mvReadFrames(((mv_extras*)hidden)->theImages,
		   (MVframe) frame_number, (MVframe) 1,
		   img_size, current->get_data()) != DM_SUCCESS)
    {
      fprintf(stderr," error in SGmoviesrc::get_next \n");
	fflush(stderr);
      abort();
    }
  return (current);
}    
   

SGmoviesrc::~SGmoviesrc()
{
  delete current;
  mvClose(((mv_extras*)hidden)->theMovie);
  delete (hidden);
}

