/*
This source file is the copyright property of the University of Leeds
('The University').

Permission to use, copy, modify, and distribute this source file for
educational, research, and not-for-profit purposes, without fee and
without a signed licensing agreement, is hereby granted, provided that
the above copyright notice, this paragraph and the following three
paragraphs appear in all copies, modifications, and distributions.

In no event shall The University be liable to any party for direct,
indirect, special, incidental or consequential damages, including lost
profits, arising out of the use of this software and its documentation.

The software is provided without warranty. The University has no
obligation to provide maintenance, support, updates, enhancements, or
modifications.

This software was written by Adam Baumberg, Vision Group,
School of Computer Studies, University of Leeds, UK. The code and use
thereof should be attributed to the author where appropriate
(including demonstrations which rely on it's use).
*/

/* diff2movie.cc
 * save differenced image in a movie
 * for processing later withoout camera!
 */

#include <image.h>
#include <sgimgsrc.h>
#include <miscsrc.h>
#include <moviestore.h>
#include <stdio.h>
#include <gl/gl.h>
#include "egetopt.cc"

main(int argc, char** argv)
{
  int data_format = YUV211 , rlength = 10 ,
    w = 320, h = 240, frate = 3, bufsize = 4, nframes = 100, c;

  float diff_thresh = 20;
  YUV_DIFF_MODE = CONTINUOUS;
  char moviename[50];
  sprintf(moviename,"/usr/tmp/amb/tempmovie");
  while ((c = egetopt(argc, argv, "f:l:t:w:h:r:b:n:o:")) != EOF)
    switch(c)
      {
      case 'f':
	if (optarg != NULL) sscanf(optarg, "%d", &data_format);
	break;
      case 'l':
	if (optarg != NULL) sscanf(optarg, "%d", &rlength);
	break;
      case 't':
	if (optarg != NULL) sscanf(optarg, "%f", &diff_thresh);
	break;
      case 'w':
	if (optarg != NULL) sscanf(optarg, "%d", &w);
	break;
      case 'h':
	if (optarg != NULL) sscanf(optarg, "%d", &h);
	break;
      case 'r':
	if (optarg != NULL) sscanf(optarg, "%d", &frate);
	break;
      case 'b':
	if (optarg != NULL) sscanf(optarg, "%d", &bufsize);
	break;
      case 'n':
	if (optarg != NULL) sscanf(optarg, "%d", &nframes);
	break;
      case 'o':
	if (optarg != NULL) sscanf(optarg, "%s", moviename);
	break;
      case '?':
      default:
	fprintf(stderr, " usage: %s", argv[0]);
	fprintf(stderr, "  videotest -f DataFormat -l runLength -t threshold -w width -h height -r frameRate -b BufferSize -n NoFrames \n");
	fprintf(stderr, " \n DataFormats (GRAY8, RGB8, YUV411, YUV211, YUV111) \n");
	exit(1);
      }

  MovieStore movieout(moviename, nframes);
  
  SGimgsrc vidsrc(bufsize,frate, (ImageType) data_format, w, h);
  Medsrc background(&vidsrc, rlength);
  DifferenceSource foreground1(&vidsrc, &background, diff_thresh);
  ThresholdSource foreground2(&foreground1,128);
  realno marked;

  for (int y = 0; y < (2 * rlength); y++)
    {
      background.get_next();
    }
  recalculate_weights(vidsrc.get_current(),
		      background.get_current());
  fprintf(stderr, " and ACTION! \n");
  for (int x=0; x >=0 ; x++)
    {
      if ((x % 4) == 0) background.get_next();
      else  vidsrc.get_next();
      foreground2.get_next();
      foreground1.display();
      marked = ((Gray8Image* ) (foreground2.get_current()))
	->get_marked_fraction();
      if (marked >= 0.013 && marked < 0.5)
	{
	  if (!movieout.add_a_frame(foreground1.get_current())) 
	    fprintf(stderr, " videotest: Finished Movie \n");
	  
	}
      else fprintf(stderr, " marked = %f \n", marked);
    }
}





