/*
This source file is the copyright property of the University of Leeds
('The University').

Permission to use, copy, modify, and distribute this source file for
educational, research, and not-for-profit purposes, without fee and
without a signed licensing agreement, is hereby granted, provided that
the above copyright notice, this paragraph and the following three
paragraphs appear in all copies, modifications, and distributions.

In no event shall The University be liable to any party for direct,
indirect, special, incidental or consequential damages, including lost
profits, arising out of the use of this software and its documentation.

The software is provided without warranty. The University has no
obligation to provide maintenance, support, updates, enhancements, or
modifications.

This software was written by Adam Baumberg, Vision Group,
School of Computer Studies, University of Leeds, UK. The code and use
thereof should be attributed to the author where appropriate
(including demonstrations which rely on it's use).
*/

/*
 * image.c++
 * general image class
 * not fully implemented yet
 *
 * author : A M Baumberg
 */

#include <point2.h>

#include <stdlib.h>
#include <image.h>
#include <kernel.h>
#include <stdio.h>
#include <string.h>
#include <time.h>
#include <math.h>
#include <EnvParameter.h>
#include <bstring.h>

#ifndef MAX
#define MAX(a,b) (( (a) > (b)) ? (a) : (b))
#endif

#ifndef MIN
#define MIN(a,b) (( (a) < (b)) ? (a) : (b))
#endif

#ifndef ABS
#define ABS(a)          (((a)<0) ? -(a) : (a))
#endif

#ifndef SGN
#define SGN(a)          (((a)<0) ? -1 : 1)
#endif

// Image addressing:
// default -- use environment variables
ImageAddressing DEFAULT_IMAGE_ADDRESSING = IA_UNDEFINED;

// use setenv IA_B_TO_T ON
// if image data is stored bottom to top 
EnvParameter image_class_use_bot_to_top("IA_B_TO_T", FALSE);

// use setenv IMAGE_ZOOM 2
// to display images magnified by a factor of 2
EnvIntParameter image_class_zoom("IMAGE_ZOOM", 1);

// currently available image types
void output_image_help_string(FILE* strm)
{
  fputs("----------------------------------------------\n", strm);
  fputs("Current image types :- \n", strm);
  fprintf(strm," (%d) GRAY8 - 8 bit Grayscale \n", (int) GRAY8);
  fprintf(strm," (%d) RGB8  - 8 bit (3:3:2) RGB \n", (int) RGB8);
  fprintf(strm," (%d) RGB32  - 32 bit (8:8:8:8) \
Indy Video format (A:B:G:R) \n", (int) RGB32);
  fprintf(strm," (%d) YUV411 - Y (Grayscale), U, V (Chroma) \
in ratio 4:1:1 \n", (int) YUV411);
  fprintf(strm," (%d) YUV211 - YUV in ratio 2:1:1 \n", (int) YUV211);
  fprintf(strm," (%d) YUV111 - YUV in ratio 1:1:1 \n", (int) YUV111);
  fputs("----------------------------------------------\n", strm);
}


#ifdef gl_interface
// use GL
#include <gl/gl.h>
#else
#include <X11/Xutil.h>
extern "C"
{
  void ginit();
  void vo_xt_window(Display*, Window, int, int);
  void vo_xt_drawable(Display*, Window, Drawable, GC, int, int);
  void vo_array(unsigned char*, int, int, int);
  typedef unsigned short  Colorindex;
  extern void color(Colorindex);
}
#endif

#ifndef VOGL
#include <gl/get.h>
#else
#endif

void set_color(int col)
{
#ifdef VOGL
  color(col);
#else
  long dmode = getdisplaymode();
  if (dmode == DMRGB || dmode == DMRGBDOUBLE)
    {
      short r = (col & 1) * 255;
      short g = (col & 2) * 255;
      short b = (col & 4) * 255;
      RGBcolor(r,g,b);
    }
  else color(col);
#endif
}


Image::Image(int w, int h, int bpp, unsigned char* img_data)
{
  width = w;
  height = h;
  bytes_per_pixel = bpp;
  if (bpp == 1) bpp_shift = 0;
  else if (bpp == 2) bpp_shift = 1;
  else if (bpp == 4) bpp_shift = 2;
  else 
    {
      fprintf(stderr," Bad call to image constructor \n");
	fflush(stderr);
      abort();
    }

  line_length = width * bytes_per_pixel;
  window_title = NULL;
  if (img_data == NULL)
    {
      if (((width * bytes_per_pixel) % 4) == 0)
	data = (unsigned char*) new long[(width * height *
					  bytes_per_pixel) / 4];
      else
	data = new unsigned char[width * height * bytes_per_pixel];
      new_data = TRUE;
    }
  else
    {
      data = img_data;
      new_data = FALSE;
    }

  end_data = (data + width * height * bytes_per_pixel);
  ypoints = new int[height];

  if (DEFAULT_IMAGE_ADDRESSING == IA_UNDEFINED)
    {
      if (image_class_use_bot_to_top.is_on())
	DEFAULT_IMAGE_ADDRESSING = IA_BOTTOM_TO_TOP;
      else DEFAULT_IMAGE_ADDRESSING = IA_TOP_TO_BOTTOM;

    }

  reset_image_addressing(DEFAULT_IMAGE_ADDRESSING);
#ifdef gl_interface
  glwin = NULLWIN;
#else
  mydisplay = NULL;
  mywindow = NULLWIN;
#endif
}

void Image::reset_image_addressing(ImageAddressing ia_mode)
{
  if (ypoints == NULL) ypoints = new int[height];
  if (ia_mode == IA_BOTTOM_TO_TOP)
    {
      for (int i = 0; i < height; i++)
	ypoints[i] = i * line_length;
    }
 else if (ia_mode == IA_TOP_TO_BOTTOM)
   {
     for (int i = 0; i < height; i++)
       ypoints[i] = (height - i - 1) * line_length;
   }
 else 
   {
     fprintf(stderr," undefined image addressing mode \n");
	fflush(stderr);
     abort();
   }
  access_mode = ia_mode;
}

Image::~Image()
{
  if (new_data) delete [] data;
  delete [] ypoints;

#ifdef gl_interface
// GL interface
  if (glwin != NULLWIN) winclose(glwin);
#else
// X interface
  if (mydisplay != NULL)
    {
      XFreeGC(mydisplay, mygc);
      XDestroyWindow(mydisplay, mywindow);
      if (myimage->data != (char *) data) delete [] (myimage->data);
      myimage->data = None;
      XFree((char *) myimage);
    }
#endif

}

void Image::clear(unsigned char fill_char)
{
  unsigned char* dat = data, *edat = endofdata();
  memset(data, (int) fill_char, (end_data - data));
  /*for (;dat < edat;dat ++)
    {
      *dat = fill_char;
    }*/
}

unsigned char* Image::check_pixel(int x, int y)
{ 
  if ((x >= 0) && (x < width) && (y >= 0) && (y < height)) 
    return (data + ypoints[y] + (x << bpp_shift));
  else return (NULL);
}

void Image::get_pixel_coords(unsigned char* pnt, int& x, int& y)
{
  int offset = pnt - data;
  y = offset / line_length;
  if (access_mode == IA_TOP_TO_BOTTOM) y = height - y - 1;
  x = offset % line_length;
  x = x >> bpp_shift;
}

static int vogl_init_flag = 0;

void Image::draw_in_image()
{

#ifdef gl_interface

if (glwin != NULLWIN) winset(glwin);

#else
if (vogl_init_flag == 0)
  {
    if (mydisplay == NULL) //  draw directly into image data
      {
	// use amb's extension to vogl
	vo_array(data, width, height, bytes_per_pixel);
	ginit();
	vogl_init_flag = 1;
      }
    else // draw into backing pixmap or into window
      {
	char *image_mode = getenv("IMAGE_DISPLAY");
	if ((image_mode != NULL) && strcasecmp(image_mode,"OFF") == 0)
	  {
	    vo_xt_drawable(mydisplay, mywindow, mypixmap, mygc, width,
			   height);
	    vogl_init_flag = 2;
	  }
	else
	  {
	    vo_xt_drawable(mydisplay, mywindow, mywindow, mygc, width,
			 height);
	    vogl_init_flag = 3;
	  }
	ginit();
      }
  }

#endif
}

void Image::read_image()
{

#ifdef gl_interface

winset(glwin);
lrectread(0,0,width-1,height-1, (unsigned long*) data);

#else

// do nothing if drawing directly into image data
if (vogl_init_flag == 1) return;

XImage* xi;
if (vogl_init_flag == 2)
  xi = XGetImage(mydisplay, mypixmap, 0,0, width, height, AllPlanes,
		 ZPixmap);
else if ((vogl_init_flag == 3) || (vogl_init_flag == 0))
  xi = XGetImage(mydisplay, mywindow, 0,0, width, height, AllPlanes,
		 ZPixmap);
else 
memcpy(data, xi->data, (end_data - data));


XDestroyImage(xi);
flip_vertically(this);

#endif
}

// the following VIRTUAL function implementations 
//  do nothing.

RGB32Image* Image::rgb_read_image(RGB32Image* res)
{
  fprintf(stderr, " illegal call to Image::rgb_read_image \n");
  fprintf(stderr, " res = %p \n",res);
  return (NULL);
}

BasePixel* Image::get_base_pixel(int x, int y)
{
  fprintf(stderr, " illegal call to Image::get_base_pixel \n");
  fprintf(stderr, " x = %d \n y = %d \n", x, y);
  return (NULL);
}


void Image::save_pm(char* filename)
{
  fprintf(stderr, " illegal call to Image::save_pbm \n");
  fprintf(stderr, " filename = %s \n", filename);
}

void Image::clear_border()
{
  fprintf(stderr, " illegal call to Image::clear_border \n");
}

Image* Image::resample(int xstep, int ystep, Image* r)
{
  fprintf(stderr, " illegal call to Image::resample \n");
  fprintf(stderr, " xstep = %d \n ystep = %d \n r = %p ", xstep,
	  ystep , r);
  return (NULL);
}


Image* Image::copy(Image* r) 
{ 
  fprintf(stderr, " illegal call to Image::copy \n");
  fprintf(stderr, " r = %p ", r);
  return NULL;
}

Image*  Image::flip_vertically(Image* r) 
{
  if (r == NULL) r = copy_type();
  unsigned char* tmp_store = new unsigned char[line_length];
  int endloop;
  if ((height % 2) == 0) endloop = height / 2;
  else
    {
      endloop = (height - 1) / 2;
      memcpy(r->get_pixel(0,endloop), get_pixel(0,endloop),
	     line_length);
    }
  for (int y1 = 0; y1 < endloop; y1++)
    {
      int y2 = height - y1 - 1;
      memcpy(tmp_store, get_pixel(0,y2), line_length);
      memcpy(r->get_pixel(0,y2), get_pixel(0,y1), line_length);
      memcpy(r->get_pixel(0,y1), tmp_store, line_length);
    }
  delete [] tmp_store;
  return (r);
}

Image* Image::fcombine(Image* image2, int (*func) (void*, void*),
			  Image* r) 
{
  fprintf(stderr, "illegal call to Image::fcombine \n");
  fprintf(stderr, " image2 = %p \n func = %p \n r = %p ",
	  image2, func, r);
  return NULL;
}

Image* Image::difference(Image* image2, realno threshold, Image* r)
{
  fprintf(stderr, "illegal call to Image::difference \n");
  fprintf(stderr, " image2 = %p \n threshold = %f \n r = %p ",
	  image2, (float) threshold, r);
  return NULL;
}


Image* Image::simple_difference(Image* image2, realno threshold,
				  int& no_marked, Image* r)
{
  fprintf(stderr, "illegal call to Image::difference \n");
  fprintf(stderr, " image2 = %p \n threshold = %f \n r = %p ",
	  image2, (float) threshold, r);
  return NULL;
}
  

Image* Image::difference(Image* image2, Image* r)
{
  fprintf(stderr, "illegal call to Image::difference \n");
  fprintf(stderr, " image2 = %p \n r = %p ",
	  image2, r);
  return NULL;
}

Image* Image::diff_stats(Image* img2, realno& mean, realno& variance,
			    Image* r) 
{ 
  fprintf(stderr, "illegal call to Image::diff_stats \n");
  fprintf(stderr, " img2 = %p \n r = %p ",
	  img2, r);
  mean = variance = 0;
  return NULL;
}

Image* Image::image_blend(Image* image2, int a, int b, Image* r)
{
  fprintf(stderr,"illegal call to Image::image_blend \n");
  fprintf(stderr, " a = %d, b = %d, r = %p \n", a, b, r);
  return (NULL);
}

Image* Image::map_intensities(PntGrayMap gmap, Image* r)
{
  fprintf(stderr,"illegal call to Image::gamma_correct \n");
  fprintf(stderr, " gmap = %p, r = %p \n", gmap, r);
  return (NULL);
}

Image* Image::convolve(Kernel k, Image* r) 
{ 
  fprintf(stderr, "illegal call to Image::convolve \n");
  fprintf(stderr, " Kernel = %p \n r = %p", &k, r);
  return NULL;
}
 
Image* Image::neighbour_order(int n, Image* r) 
{ 
  fprintf(stderr, "illegal call to Image::neighbour_order \n");
  fprintf(stderr, " n = %d \n r = %p", n, r);
  return NULL;
}
 
Image* Image::fmed(Image* r) 
{ 
  fprintf(stderr, "illegal call to Image::fmed \n");
  fprintf(stderr, " r = %p", r);
  return NULL; 
}

Image* Image::sobel(int threshold, Image* r)
{ 
  fprintf(stderr, "illegal call to Image::sobel \n");
  fprintf(stderr, " threshold = %d \n r = %p", threshold, r);
  return NULL;
} 
 
Image* Image::sobel(Image* r)
{ 
  fprintf(stderr, "illegal call to Image::sobel \n");
  fprintf(stderr, " r = %p", r);
  return NULL;
}

Image* Image::threshold(int threshold, Image* r, int* no_marked) 
{ 
  fprintf(stderr, "illegal call to Image::threshold \n");
  fprintf(stderr, " threshold = %d \n r = %p", threshold, r); 
  return NULL;
}

Image* Image::half_blur( Image* r) 
{ 
  fprintf(stderr, "illegal call to Image::half_blur \n");
  fprintf(stderr, " r = %p", r); 
  return NULL;
}

Image* Image::extract_subimage(int xmin, int xmax, int ymin,
				       int ymax, Image* r) 
{ 

  // new implementation for base class
  // should be reasonably efficient

  if (r == NULL) r = copy_type(xmax-xmin+1, ymax-ymin+1);
  for (int y = ymin; y <= ymax; y++)
    memcpy(r->get_pixel(0,y-ymin), get_pixel(xmin,y), r->line_length);

  return (r);


}

void Image::paste_subimage(int xmin, int xmax, int ymin, int ymax,
			    Image* subimg)
{
  if ((check_pixel(xmin,ymin) == NULL ) || 
      (check_pixel(xmax,ymax) == NULL) || 
      (xmin > xmax) || (ymin > ymax) || (imgtype() != subimg->imgtype()))
    {
      fprintf (stderr, " bad parameters to paste_subimage ");
      exit(1);
    }
  int nbytes = (xmax - xmin + 1) * bytes_per_pixel; 
  for (int y = ymin; y <= ymax; y++)
    memcpy(get_pixel(xmin,y),subimg->get_pixel(0,y-ymin), nbytes);
}

Image*  Image::get_subimage(int xmin, int xmax, int ymin, int ymax,
			    Image* subimg)
{
  if (subimg == NULL) subimg = copy_type(xmax - xmin +1, ymax - ymin + 1);
  if ((check_pixel(xmin,ymin) == NULL ) || 
      (check_pixel(xmax,ymax) == NULL) || 
      (xmin > xmax) || (ymin > ymax) || (imgtype() != subimg->imgtype()))
    {
      fprintf (stderr, " bad parameters to paste_subimage ");
      exit(1);
    }
  int nbytes = (xmax - xmin + 1) * bytes_per_pixel; 
  for (int y = ymin; y <= ymax; y++)
    memcpy(subimg->get_pixel(0,y-ymin),get_pixel(xmin,y), nbytes);
  return (subimg);
}

long Image::display(long window) 
{ 
  fprintf(stderr, "illegal call to Image::display \n");
  fprintf(stderr, " window = %d", (int) window);
  return NULLWIN;
}

Image* Image::mask(Image* mask, Image* r)
{
  fprintf(stderr, "illegal call to Image::mask \n");
  fprintf(stderr, " mask = %p, r = %p", mask, r);
  return NULL;
}

Image* Image::copy_type(int w, int h)
{
  fprintf(stderr, "illegal call to Image::copy_type \n");
  fprintf(stderr, "%d %d \n", w, h);
  return (NULL);
}

Image* Image::virtual_copy()
{
  fprintf(stderr, "illegal call to Image::copy_type \n");
  return (NULL);
}

int Image::measure_contrast(void* p1, void* p2)
{
  unsigned char *pix1 = (unsigned char*) p1;
  unsigned char *pix2 = (unsigned char*) p2;
  if ((pix1 == NULL) || (pix2 == NULL)) return (0);
  int val1 = (int) *pix1;  int val2 = (int) *pix2;
  return (abs(val1 - val2));
}
  

Image* Image::to_rgb(Image* r)
{
  fprintf(stderr, "illegal call to Image::to_rgb \n");
  fprintf(stderr, " r = %p", r);
  return NULL;
}


void Image::draw_rectangle(int xmin, int xmax, int ymin, int ymax, int
			   val)
{
  fprintf(stderr, "illegal call to Image::to_rgb \n");
  fprintf(stderr, " %d %d %d %d %d \n", xmin, xmax, ymin, ymax, val);
}


// Tony Heap's original implementation was using the base class
// I have reimplemented for RGB32 and GRAY8 images 
//  - amb 15/07/96
Image* Image::transform(int centre_x, int centre_y, realno angle,
                        realno scale, Image* r)
{
  fprintf(stderr, "illegal call to Image::transform \n");
  fprintf(stderr, " %d %d %f %f %p \n", centre_x, centre_y, angle,
	  scale,r);
}

// base class implementation assumes noise is to be added
// to each byte (i.e. each field is stored in a byte)


static int set_seed = 0;
static int iset = 0;
static realno gset;

Image* Image::add_gaussian_noise(realno sd, realno& noise, Image* res)
{
  if (res == NULL) res = copy_type();
  if (set_seed == 0)
    {
      long rseed = time(NULL);
      srand48(rseed);
      set_seed = 1;
    }
  noise = 0;
  realno bias = 0;
  unsigned char *in_pnt = data;
  unsigned char *out_pnt = res->data;
  while (in_pnt < end_data)
    {
      realno fac, rsq, v1, v2, corrupt;
      if (iset == 0)
	{
	  do
	    {
	      v1 = 2.0 * drand48() - 1.0;
	      v2 = 2.0 * drand48() - 1.0;
	      rsq = (v1 * v1) + (v2 * v2);
	    } while ((rsq >= 1.0 )|| (rsq == 0.0));
	  
	  fac = sqrt(-2.0 * log(rsq)/rsq);
	  gset = v1 * fac;
	  iset = 1;
	  corrupt = v2 * fac;
	}
      else
	{
	  iset = 0;
	  corrupt = gset;
	}
	  
      int icorrupt = (int) (sd * corrupt + 0.5);
      int ipix = (int) *in_pnt++;
      int new_ipix = ipix + icorrupt;
      if (new_ipix > 255) new_ipix = 255;
      if (new_ipix < 0) new_ipix = 0;
      *out_pnt++ = new_ipix;
      noise += (new_ipix - ipix) * (new_ipix - ipix);
    }
  return (res);
}
      


// sum pixel values 
// assume each field is stored in a byte
// all fields are set

realno Image::sum_pixels()
{
  realno total = 0;
  unsigned char *pix_field = data;
  while (pix_field < end_data)
    total += (realno) (*pix_field++);
  return (total);
}

// sum [I(x,y) - zero]^2

realno Image::sum_square_pixels(int zero)
{
  realno total = 0;
   unsigned char *pix_field = data;
   while (pix_field < end_data)
     {
       int ival = *pix_field++ - zero;
       total += (realno) (ival * ival);
     }

  return (total);
}


void Image:: median_img_array(Image** , int )
{
  fprintf(stderr, "illegal call to Image::median_img_array \n");
}


Image* Image::plane_project(realno& m00, realno& m01, realno& m02,
			    realno& m10, realno& m11, realno& m12,
			    realno& m20, realno& m21, realno& m22,
			    Image* res)
{
  if (res == NULL) res = copy_type();

  realno depth, back_x, back_y;

  int rw = res->get_width();
  int rh = res->get_height();
  for (int x = 0; x < rw; x++)
    for (int y = 0; y < rh; y++)
      {
	depth = m20 * x +  m21 * y + m22;
	back_x = (m00 * x + m01 * y + m02) / depth;
	back_y = (m10 * x + m11 * y + m12) / depth;

	unsigned char* p1 = nearest_pixel(back_x, back_y);
	if (p1 != NULL)
	  memcpy(res->get_pixel(x,y), p1, bytes_per_pixel);
	else
	  memset(res->get_pixel(x,y), 0, bytes_per_pixel);
      }
}

typedef struct {		
    realno x;	
    realno dx;	
    int i;	
} Edge;

/* comparison routines for qsort */
static Point2* pt;
static int compare_ind(int *u, int *v) {return pt[*u].y <= pt[*v].y ? -1 : 1;}
static int compare_active(Edge *u, Edge *v) {return u->x <= v->x ? -1 : 1;}

static poly_delete(int i, int& nact, Edge* active)
      
{
  for (int j=0; j<nact && active[j].i!=i; j++);
  if (j>=nact) return;
  nact--;
  bcopy(&active[j+1], &active[j], (nact-j)*sizeof(active[0]));
}

static poly_insert(int i, int y, int nvert, int& nact, Edge* active)         
{
    int j;
    double dx;
    Point2 *p, *q;

    j = i<nvert-1 ? i+1 : 0;
    if (pt[i].y < pt[j].y) {p = &pt[i]; q = &pt[j];}
    else		   {p = &pt[j]; q = &pt[i];}
    /* initialize x position at intersection of edge with scanline y */
    active[nact].dx = dx = (q->x-p->x)/(q->y-p->y);
    active[nact].x = dx*(y+.5-p->y)+p->x;
    active[nact].i = i;
    nact++;
}

void Image::draw_filled_polygon(int n, Point2* vertices)
{
  /*
   * Concave Polygon Scan Conversion
   * by Paul Heckbert
   * from "Graphics Gems", Academic Press, 1990
   */

  // modified by amb
  pt = vertices;

  Edge *active = new Edge[n];

  int win_x0 = 0, win_x1 = width-1;
  int win_y0 = 0, win_y1 = height-1;

  int y0, y1, i, j, xl, xr;
  int *ind = new int[n];

  if (n<=0) return;
  for (int k=0; k<n; k++)
    ind[k] = k;
  
  qsort(ind, n, sizeof(ind[0]), compare_ind);	/* sort ind by pt[ind[k]].y */

  int nact = 0;				
  k = 0;
  y0 = MAX(win_y0, ceil(pt[ind[0]].y-.5));
  y1 = MIN(win_y1, floor(pt[ind[n-1]].y-.5));
  
  for (int y=y0; y<=y1; y++) {		/* step through scanlines */
	/* scanline y is at y+.5 in continuous coordinates */

	/* check vertices between previous scanline and current one, if any */
	for (; k<n && pt[ind[k]].y<=y+.5; k++) {
	    /* to simplify, if pt.y=y+.5, pretend it's above */
	    /* invariant: y-.5 < pt[i].y <= y+.5 */
	    i = ind[k];	
	    /*
	     * insert or delete edges before and after vertex i (i-1 to i,
	     * and i to i+1) from active list if they cross scanline y
	     */
	    j = i>0 ? i-1 : n-1;	/* vertex previous to i */
	    if (pt[j].y <= y-.5)	/* old edge, remove from active list */
		poly_delete(j, nact, active);
	    else if (pt[j].y > y+.5)	/* new edge, add to active list */
		poly_insert(j, y, n, nact, active);
	    j = i<n-1 ? i+1 : 0;	/* vertex next after i */
	    if (pt[j].y <= y-.5)	/* old edge, remove from active list */
		poly_delete(i, nact, active);
	    else if (pt[j].y > y+.5)	/* new edge, add to active list */
		poly_insert(i, y, n, nact, active);
	}

	/* sort active edge list by active[j].x */
	qsort(active, nact, sizeof(active[0]), compare_active);

	/* draw horizontal segments for scanline y */
	for (j=0; j<nact; j+=2) {	/* draw horizontal segments */
	    /* span 'tween j & j+1 is inside, span tween j+1 & j+2 is outside */
	    xl = ceil(active[j].x-.5);		/* left end of span */
	    if (xl<win_x0) xl = win_x0;
	    xr = floor(active[j+1].x-.5);	/* right end of span */
	    if (xr>win_x1) xr = win_x1;
	    if (xl<=xr)
		draw_horizontal(y, xl, xr);
	    active[j].x += active[j].dx;	/* increment edge coords */
	    active[j+1].x += active[j+1].dx;
	}
    }
  delete [] ind;
  delete [] active;

}


void Image::draw_filled_circle(int x0, int y0, int r)
{
  register int x = 0;
  register int y = r;
  register int delta = ( 1 - r) << 1;
  int limit = 0;  
  int sigma;
  while (y >= limit)
    {
      if (delta < 0)
	{
	  sigma = ((delta + y) << 1) - 1;
	  if (sigma > 0)
	    {
	      draw_horizontal(y0+y, x0-x, x0+x);
	      draw_horizontal(y0-y, x0-x, x0+x);
	      x++;y--;
	      delta += (x - y + 1) << 1;
	    }
	  else
	    {
	      x++;
	      delta += (x << 1) + 1;
	    }
	}
      else if (delta > 0)
	{
	  sigma = ((delta - x) << 1) - 1;
	  if (sigma > 0)
	    {
	      draw_horizontal(y+y0, x0-x, x0+x);
	      draw_horizontal(y0-y, x0-x, x0+x);
	      y--;
	      delta += 1 - (y << 1);
	    }
	  else
	    {
	      draw_horizontal(y+y0, x0-x, x0+x);
	      draw_horizontal(y0-y, x0-x, x0+x);
	      x++; y--;
	      delta += (x - y + 1) << 1;
	    }
	}
      else
	{
	  draw_horizontal(y+y0, x0-x, x0+x);
	  draw_horizontal(y0-y, x0-x, x0+x);
	  x++;y--;
	  delta += (x - y + 1) << 1;
	}
	      
    }
}
    
void Image::draw_line(int x0, int y0, int x1, int y1)
{    
  if (DrawLineWidth > 1)
    {      
      Point2 plist[4];
      realno delta_x, delta_y, dlength;
      delta_x = (realno) (x1 - x0);
      delta_y = (realno) (y1 - y0);
      dlength = sqrt((delta_x * delta_x) + (delta_y * delta_y));
      int r = (DrawLineWidth - 0.99) / 2;
      if (dlength > 0)
        {
	  realno dx = delta_x / dlength;
	  realno dy = delta_y / dlength;
	  realno nx = dy;
	  realno ny = -dx;
          plist[0].x = x0 + 0.5 + ((r + 0.5) * nx) - (dx * 0.5);
          plist[0].y = y0 + 0.5 + ((r + 0.5) * ny) - (dy * 0.5);
          plist[1].x = x0 + 0.5 - ((r + 0.5) * nx) - (dx * 0.5);
          plist[1].y = y0 + 0.5 - ((r + 0.5) * ny) - (dy * 0.5);
          plist[2].x = x1 + 0.5 - ((r + 0.5) * nx) + (dx * 0.5);
          plist[2].y = y1 + 0.5 - ((r + 0.5) * ny) + (dy * 0.5);
          plist[3].x = x1 + 0.5 + ((r + 0.5) * nx) + (dx * 0.5);
          plist[3].y = y1 + 0.5 + ((r + 0.5) * ny) + (dy * 0.5);
          draw_filled_polygon(4, plist);
	}
      draw_filled_circle(x0,y0,r);	  
      draw_filled_circle(x1,y1,r);        
      return;
    }

  int d, x, y, ax, ay, sx, sy, dx, dy;

  dx = x1-x0;  ax = ABS(dx)<<1;  sx = SGN(dx);
  dy = y1-y0;  ay = ABS(dy)<<1;  sy = SGN(dy);

  if ((dx == 0) && (dy == 0)) return;
    x = x0;
    y = y0;
    if (ax>ay) {                /* x dominant */

        d = ay-(ax>>1);
        for (;;) {
	    plot_pixel(x,y);
            if (x==x1) return;
            if (d>=0) {
                y += sy;
                d -= ax;
            }
            x += sx;
            d += ay;
        }
    }
    else {                      /* y dominant */

        d = ax-(ay>>1);
        for (;;) {
	    plot_pixel(x,y);
            if (y==y1) return;
            if (d>=0) {
                x += sx;
                d -= ay;
            }
            y += sy;
            d += ax;
        }
    }
}

static const int CIRCLE_PRECISION = 8;

void Image::draw_circle(int x0, int y0, int r)
{
  static const realno cth = cos(M_PI_2 / ((realno) CIRCLE_PRECISION));
  static const realno sth = sin(M_PI_2 / ((realno) CIRCLE_PRECISION));

  if (DrawLineWidth > 1)
    {
      Point2* pnt_data = new Point2[CIRCLE_PRECISION*8+2];
      Point2* big_circle = pnt_data;
      Point2* small_circle = &pnt_data[CIRCLE_PRECISION*4+1];

      realno x_big = 0; realno y_big = r + 0.5;
      realno x_small = 0; realno y_small = (r  - DrawLineWidth);
      Point2 origin(x0+0.5, y0+0.5);
      for (int i = 0; i < CIRCLE_PRECISION; i++)
	{
	  big_circle[i] = Point2(x_big, y_big);
	  big_circle[i+CIRCLE_PRECISION] = Point2(y_big, -x_big);
	  big_circle[i+2*CIRCLE_PRECISION] = Point2(-x_big, -y_big);
	  big_circle[i+3*CIRCLE_PRECISION] = Point2(-y_big, x_big);
	  small_circle[i] = Point2(x_small, y_small);
	  small_circle[i+CIRCLE_PRECISION] = Point2(y_small, -x_small);
	  small_circle[i+2*CIRCLE_PRECISION] = Point2(-x_small, -y_small);
	  small_circle[i+3*CIRCLE_PRECISION] = Point2(-y_small, x_small);
	  realno new_x_big = cth * x_big + sth * y_big;
	  realno new_y_big = -sth * x_big + cth * y_big;
	  x_big = new_x_big; y_big = new_y_big;
	  realno new_x_small = cth * x_small + sth * y_small;
	  realno new_y_small = -sth * x_small + cth * y_small;
	  x_small = new_x_small; y_small = new_y_small;
	}
      big_circle[4*CIRCLE_PRECISION] = big_circle[0];
      small_circle[4*CIRCLE_PRECISION] = small_circle[0];
      for (i = 0; i < (8 * CIRCLE_PRECISION + 2); i++)
	pnt_data[i] = pnt_data[i] + origin;
      
      draw_filled_polygon(8*CIRCLE_PRECISION+2, pnt_data);
      delete [] pnt_data;
      return;
    }

  register int x = 0;
  register int y = r;
  register int delta = ( 1 - r) << 1;
  int limit = 0;  
  int sigma;
  while (y >= limit)
    {
      plot_pixel(x0-x,y0+y);
      plot_pixel(x0+x,y0+y);
      plot_pixel(x0-x,y0-y);
      plot_pixel(x0+x,y0-y);
      if (delta < 0)
	{
	  sigma = ((delta + y) << 1) - 1;
	  if (sigma > 0)
	    {
	      x++;y--;
	      delta += (x - y + 1) << 1;
	    }
	  else
	    {
	      x++;
	      delta += (x << 1) + 1;
	    }
	}
      else if (delta > 0)
	{
	  sigma = ((delta - x) << 1) - 1;
	  if (sigma > 0)
	    {
	      y--;
	      delta += 1 - (y << 1);
	    }
	  else
	    {
	      x++; y--;
	      delta += (x - y + 1) << 1;
	    }
	}
      else
	{
	  x++;y--;
	  delta += (x - y + 1) << 1;
	}
	      
    }
}

