/*
This source file is the copyright property of the University of Leeds
('The University').

Permission to use, copy, modify, and distribute this source file for
educational, research, and not-for-profit purposes, without fee and
without a signed licensing agreement, is hereby granted, provided that
the above copyright notice, this paragraph and the following three
paragraphs appear in all copies, modifications, and distributions.

In no event shall The University be liable to any party for direct,
indirect, special, incidental or consequential damages, including lost
profits, arising out of the use of this software and its documentation.

The software is provided without warranty. The University has no
obligation to provide maintenance, support, updates, enhancements, or
modifications.

This software was written by Adam Baumberg, Vision Group,
School of Computer Studies, University of Leeds, UK. The code and use
thereof should be attributed to the author where appropriate
(including demonstrations which rely on it's use).
*/

/*
 * imgsrc.cc
 * 
 * a general image source
 *
 */


#include <imgsrc.h>
#include <image.h>

// the default frame rate in frames per second
// (used by get_current_timestamp)
realno ImageSource::base_frame_rate = 0.04;

long ImageSource::display(long wndow)
{ 
  if ((display_title != NULL) && (current != NULL))
    current->set_title(display_title); 
  long res;
  if (current != NULL) res = (current->display(wndow));
  else res = (NULLWIN); 
  return (res); 
} 

// the default assumes the frames are
// captured at constant frame rate
// (live video overrides this for irix5)
timeval ImageSource::get_current_timestamp()
{
  timeval res;
  realno rtime = base_frame_rate * frame_number;
  res.tv_sec = (long) rtime;
  res.tv_usec = 1e6 * (rtime - res.tv_sec);

  return (res);
}

