/*
This source file is the copyright property of the University of Leeds
('The University').

Permission to use, copy, modify, and distribute this source file for
educational, research, and not-for-profit purposes, without fee and
without a signed licensing agreement, is hereby granted, provided that
the above copyright notice, this paragraph and the following three
paragraphs appear in all copies, modifications, and distributions.

In no event shall The University be liable to any party for direct,
indirect, special, incidental or consequential damages, including lost
profits, arising out of the use of this software and its documentation.

The software is provided without warranty. The University has no
obligation to provide maintenance, support, updates, enhancements, or
modifications.

This software was written by Adam Baumberg, Vision Group,
School of Computer Studies, University of Leeds, UK. The code and use
thereof should be attributed to the author where appropriate
(including demonstrations which rely on it's use).
*/

/*
 * imgsrc.h
 * base class for image sources
 *
 * author: A.M.Baumberg (amb)
 * creation date: 8/1/93
 */

#ifndef __IMGSRC_H__
#define __IMGSRC_H__

#include <stdio.h>
#include <image.h>
#include <Gray8Image.h>
#include <RGB8Image.h>
#include <RGB32Image.h>
#include <YUVImage.h>
#include <sys/time.h>

class ImageSource
{

public:

  Image* current;        /* current frame */
  int frame_number;      
  char* display_title;

  // the default frame rate in frames per second
  // (used by get_current_timestamp)
  static realno base_frame_rate; 

 public:

  Image* get_current() { return current; }

  // set_current should be used with caution 
  void set_current(Image* img1) { current = img1; }
  int get_frame_no() { return frame_number; }
  ImageSource(Image* c = NULL, int f = 0) 
    { current = c; frame_number = f; display_title = NULL; }
  virtual ~ImageSource() { delete current; }

  /* get_next is defined for each derived image source 
     hence the base method is virtual */

  virtual Image* get_next()
    { frame_number++; return current;  }

  virtual timeval get_current_timestamp();

  /* goto frame: assumes first frame is 0 
  	use get_next to actually read the frame */
  virtual void goto_frame(int n)
  { while (frame_number < n) get_next(); }
    
  virtual long display(long wndow = NULLWIN);
  
  void set_title(char* title) { display_title = title; }
};


#endif


  
