/*
This source file is the copyright property of the University of Leeds
('The University').

Permission to use, copy, modify, and distribute this source file for
educational, research, and not-for-profit purposes, without fee and
without a signed licensing agreement, is hereby granted, provided that
the above copyright notice, this paragraph and the following three
paragraphs appear in all copies, modifications, and distributions.

In no event shall The University be liable to any party for direct,
indirect, special, incidental or consequential damages, including lost
profits, arising out of the use of this software and its documentation.

The software is provided without warranty. The University has no
obligation to provide maintenance, support, updates, enhancements, or
modifications.

This software was written by Adam Baumberg, Vision Group,
School of Computer Studies, University of Leeds, UK. The code and use
thereof should be attributed to the author where appropriate
(including demonstrations which rely on it's use).
*/

/***************************************************************
 * C - C++ Header
 *
 * File : 	imgsrc_utils.h
 *
 * Module :	imgsrc_utils
 *
 * Author : 	A M Baumberg (CoMIR)
 *
 * Creation Date : Thu Jun 13 11:10:23 1996 
 *
 * Comments : 	
 *
 ***************************************************************/

#ifndef __IMGSRC_UTILS_H__
#define __IMGSRC_UTILS_H__

#include "imgsrc.h"

// get_live_or_movie_feed
// 
// parse command line arguments
// and either open a movie (SG movie or MPEG)
// or use live video images
// or use PNM images from stdin or files
//
// returns an appropriate ImageSource


/*
command line arguments :-

-i input_options 

input_options is a comma separated list of options :-

file=name	: input movie file (guess the type of file from the extension)
mpegfile=name	: input MPEG file
avifile=name	: input AVI file (or any Xanim compatible file) 
pnmfile=name	: set of PNM files -- this is the first (use '-' for stdin)
mvfile=name	: input Silicon Graphics Movie file
refimg=name	: PNM file containing first (reference) image		

RAmpeg		: Use Random Access MPEG library (caches file offsets)
gray		: force grayscale input for MPEG / live input
GRAY8		: force grayscale input for MPEG / live input
RGB8		: use 8bit RGB input for SG Indigo video
RGB32		: use 32bit RGB input for Indy video (default)
YUV411		: use YUV colour input ratio 4:1:1 for Indigo video
YUV211		: use YUV colour input ratio 2:1:1 for Indigo video
YUV111		: use YUV colour input ratio 1:1:1 for Indigo video

width=n		: set live video width
height=n	: set live video height
burst		: use Indigo video burst mode
bufsize=n	: live video buffer size in frames
rate=n		: live video sampling rate (1 - full, 2 - half etc)
skip=n		: use every n'th frame (for movies)

help		: output this help message
*/

ImageSource* get_live_or_movie_feed(int argc, char** argv);
ImageSource* get_live_or_movie_feed(char* option_string);

// get the reference image if set up
// by get_live_or_movie_feed
Image* get_reference_frame();

#endif
