/*
This source file is the copyright property of the University of Leeds
('The University').

Permission to use, copy, modify, and distribute this source file for
educational, research, and not-for-profit purposes, without fee and
without a signed licensing agreement, is hereby granted, provided that
the above copyright notice, this paragraph and the following three
paragraphs appear in all copies, modifications, and distributions.

In no event shall The University be liable to any party for direct,
indirect, special, incidental or consequential damages, including lost
profits, arising out of the use of this software and its documentation.

The software is provided without warranty. The University has no
obligation to provide maintenance, support, updates, enhancements, or
modifications.

This software was written by Adam Baumberg, Vision Group,
School of Computer Studies, University of Leeds, UK. The code and use
thereof should be attributed to the author where appropriate
(including demonstrations which rely on it's use).
*/

/*
 * cl.h - Compression Library typedefs and prototypes
 *
 *   02/18/92	Original Version by Brian Knittel
 */

#ifndef _INC_CL_H_
#define _INC_CL_H_ 1

#ifdef __cplusplus
extern "C" {
#endif

#if	!defined(TRUE) || ((TRUE) != 1)
#define	TRUE	(1)
#endif
#if	!defined(FALSE) || ((FALSE) != 0)
#define	FALSE	(0)
#endif
#if	!defined(SUCCESS) || ((SUCCESS) != 0)
#define	SUCCESS	(0)
#endif
#if	!defined(FAILURE) || ((FAILURE) != -1)
#define	FAILURE	(-1)
#endif
#if	!defined(byte)
typedef unsigned char	byte;
#endif
#if	!defined(FunctionPtr)
typedef long (*FunctionPtr)();
#endif

/*
 * originalFormat parameter values
 */
#define	CL_MAX_NUMBER_OF_ORIGINAL_FORMATS   (32)

/* Audio */
#define	CL_MONO		   	(0)
#define	CL_STEREO_INTERLEAVED	(1)

/* Video 
 * YUV is defined to be the same thing as YCrCb (luma and two chroma components).
 * 422 is appended to YUV (or YCrCb) if the chroma is sub-sampled by 2 
 *	horizontally, packed as U Y1 V Y2 (byte order).
 * 422HC is appended to YUV (or YCrCb) if the chroma is sub-sampled by 2 
 *	vertically in addition to horizontally, and is packed the same as 
 *	422 except that U & V are not valid on the second line.
 */
#define	CL_RGB			(0)
#define	CL_RGBX			(1)
#define	CL_RGBA			(2)
#define	CL_RGB332		(3)

#define	CL_GRAYSCALE		(4)
#define	  CL_Y			(4)
#define	CL_YUV			(5)	
#define	  CL_YCbCr		(5)	
#define CL_YUV422		(6)	/* 4:2:2 sampling */
#define   CL_YCbCr422		(6)	/* 4:2:2 sampling */
#define CL_YUV422HC		(7)	/* 4:1:1 sampling */
#define   CL_YCbCr422HC		(7)	/* 4:1:1 sampling */
#define   CL_YUV422DC		(7)	/* 4:1:1 sampling */
#define   CL_YCbCr422DC		(7)	/* 4:1:1 sampling */

#define	CL_BEST_FIT		(-1)	

#define clBytesPerPixel(f) \
(   /* if */ (((f) == CL_RGB) || ((f) == CL_YUV)) ? \
	/* return */ 3 : \
    /* else if */ (((f) == CL_RGBX) || ((f) == CL_RGBA)) ?  \
	/* return */ 4 : \
    /* else if */ (((f) == CL_RGB332) || ((f) == CL_GRAYSCALE)) ? \
	/* return */ 1 : \
    /* else */ \
	/* return */ 2 \
)

#define	CL_MAX_NUMBER_OF_AUDIO_ALGORITHMS	(32)
#define	CL_MAX_NUMBER_OF_VIDEO_ALGORITHMS	(32)

/*
 * "compressionScheme" argument values
 */
#define	CL_UNCOMPRESSED_AUDIO	(0)
#define	CL_G711_ULAW		(1)
#define	  CL_ULAW		(1)
#define	CL_G711_ALAW		(2)
#define	  CL_ALAW		(2)
#define	CL_G722			(3)
    
#define	CL_UNCOMPRESSED		(CL_MAX_NUMBER_OF_AUDIO_ALGORITHMS + 0)
#define	CL_UNCOMPRESSED_VIDEO	(CL_MAX_NUMBER_OF_AUDIO_ALGORITHMS + 0)
#define	CL_RLE			(CL_MAX_NUMBER_OF_AUDIO_ALGORITHMS + 1)
#define	CL_JPEG			(CL_MAX_NUMBER_OF_AUDIO_ALGORITHMS + 2)
#define	CL_MPEG_VIDEO		(CL_MAX_NUMBER_OF_AUDIO_ALGORITHMS + 3)
#define	CL_MVC1			(CL_MAX_NUMBER_OF_AUDIO_ALGORITHMS + 4)
#define CL_RTR			(CL_MAX_NUMBER_OF_AUDIO_ALGORITHMS + 5)
#define	    CL_RTR1		(CL_MAX_NUMBER_OF_AUDIO_ALGORITHMS + 5)

/*
 * Parameters
 */
#define	CL_MAX_NUMBER_OF_PARAMS	(256)
/* Default Parameters */
#define CL_IMAGE_WIDTH		    (0)
#define CL_IMAGE_HEIGHT		    (1) 
#define CL_ORIGINAL_FORMAT	    (2)
#define CL_INTERNAL_FORMAT	    (3)
#define CL_COMPONENTS		    (4)
#define CL_BITS_PER_COMPONENT	    (5)
#define CL_FRAME_RATE		    (6)
#define CL_COMPRESSION_RATIO	    (7)
#define CL_EXACT_COMPRESSION_RATIO  (8)
#define CL_FRAME_BUFFER_SIZE	    (9) 
#define CL_COMPRESSED_BUFFER_SIZE   (10)
#define CL_BLOCK_SIZE		    (11)
#define CL_PREROLL		    (12)
#define CL_UNIQUE		    (13)
#define CL_FRAME_TYPE		    (14)
#define CL_OVERWRITE_MODE	    (15)
#define	CL_NUMBER_OF_PARAMS	    (16)

/* JPEG Specific Parameters */
#define	CL_QUALITY_FACTOR	    (CL_NUMBER_OF_PARAMS + 0)

/* MPEG Specific Parameters */
#define	CL_SPEED		    (CL_NUMBER_OF_PARAMS + 0)
#define	CL_ACTUAL_FRAME_INDEX	    (CL_NUMBER_OF_PARAMS + 1)
#define	CL_END_OF_SEQUENCE	    (CL_NUMBER_OF_PARAMS + 2)

/* RTR Specific Parameters */
#define CL_QUALITY_LEVEL	    (CL_NUMBER_OF_PARAMS + 0)
#define CL_ZOOM_X		    (CL_NUMBER_OF_PARAMS + 1)
#define CL_ZOOM_Y		    (CL_NUMBER_OF_PARAMS + 2)

#define	clTypeIsFloat(v)	(*(float *)&(v))
#define	clTypeIsLong(v)		(*(long *)&(v))

/*
 * Parameter value types
 */
#define	CL_ENUM_VALUE		(0) /* only certain constant values are valid */
#define	CL_RANGE_VALUE		(1) /* any value in a given range is valid */
#define	CL_FLOATING_ENUM_VALUE	(2) /* only certain constant floating point values are valid */
#define	CL_FLOATING_RANGE_VALUE	(3) /* any value in a given floating point range is valid */
#define	CL_POINTER	 	(4) /* any legal pointer is valid */

/*
 * Algorithm types
 */
#define	CL_AUDIO		(0)
#define	CL_VIDEO		(1)

/*
 * Algorithm Functionality
 */
#define	CL_DECOMPRESSOR		(1)
#define	CL_COMPRESSOR		(2)
#define	CL_CODEC		(3)

/*
 * Buffer types
 */
#define	CL_NONE			(0)
#define	CL_FRAME		(1)
#define	CL_DATA			(2)

/*
 * Frame types
 */
#define	CL_NONE			(0)
#define	CL_KEYFRAME		(1)
#define	    CL_INTRA		(1)
#define	CL_PREDICTED		(2)
#define	CL_BIDIRECTIONAL	(3)

/*
 * error codes
 */
#define CL_BAD_NOT_IMPLEMENTED	    ( -1) /* not impimented yet */
#define CL_BAD_NO_BUFFERSPACE	    ( -2) /* no space for internal buffers */
#define CL_BAD_BUFFER_NULL	    ( -3) /* null buffer pointer */
#define CL_BAD_COUNT_NEG	    ( -4) /* negative count */
#define CL_BAD_PVBUFFER		    ( -5) /* param/val buffer doesn't make sense */
#define CL_BAD_BUFFERLENGTH_NEG	    ( -6) /* negative buffer length */
#define CL_BAD_BUFFERLENGTH_ODD	    ( -7) /* odd length parameter/value buffer */
#define CL_BAD_PARAM		    ( -8) /* invalid parameter */
#define CL_BAD_COMPRESSION_SCHEME   ( -9) /* compression scheme parameter invalid */
#define CL_BAD_COMPRESSOR_HANDLE    (-10) /* compression handle parameter invalid */
#define CL_BAD_COMPRESSOR_HANDLE_POINTER \
				    (-11) /* compression handle pointer invalid */
#define CL_BAD_BUFFER_HANDLE	    (-12) /* callback function invalid */
#define CL_BAD_ALGORITHM_INFO	    (-13) /* algorithm info invalid */
#define CL_BAD_CL_BAD_WIDTH_OR_HEIGHT \
				    (-14) /* compressor width or height invalid */
#define CL_BAD_POINTER_FROM_CALLBACK_FUNCTION \
				    (-15) /* pointer from callback invalid */
#define CL_JPEG_ERROR		    (-16) /* error from libjpeg */
#define CL_NO_SEMAPHORE		    (-17) /* could not get semaphore */
#define CL_BAD_WIDTH_OR_HEIGHT	    (-18) /* width or height invalid */
#define CL_BAD_FRAME_COUNT	    (-19) /* frame count invalid */
#define CL_BAD_FRAME_INDEX	    (-20) /* frame index invalid */
#define CL_BAD_FRAME_BUFFER	    (-21) /* frame buffer pointer invalid */
#define CL_BAD_FRAME_SIZE	    (-22) /* frame size invalid */
#define CL_BAD_DATA_BUFFER	    (-23) /* data buffer pointer invalid */
#define CL_BAD_DATA_SIZE	    (-24) /* data buffer size invalid */
#define CL_BAD_TOTAL_NUMBER_OF_FRAMES \
				    (-25) /* total number of frames invalid */
#define CL_BAD_IMAGE_FORMAT	    (-26) /* image format invalid */
#define CL_BAD_BITS_PER_COMPONENT   (-27) /* bits per component invalid */
#define CL_BAD_FRAME_RATE	    (-28) /* frame rate invalid */
#define CL_BAD_INSUFFICIENT_DATA_FROM_CALLBACK_FUNCTION \
				    (-29) /* insufficient data from callback invalid */
#define CL_PARAM_OUT_OF_RANGE	    (-30) /* parameter out of range */
#define CL_ADDED_ALGORITHM_ERROR    (-31) /* added algorithm had a unique error */
#define CL_BAD_ALGORITHM_TYPE       (-32) /* bad algorithm type */
#define CL_BAD_ALGORITHM_NAME       (-33) /* bad algorithm name */
#define CL_BAD_FRAME_INDEXING	    (-34) /* bad frame indexing */
#define CL_BAD_BUFFERING	    (-35) /* bad buffering calls */
#define CL_BUFFER_NOT_CREATED	    (-36) /* buffer not created */
#define CL_BAD_BUFFER_EXISTS	    (-37) /* buffer already created */
#define CL_BAD_INTERNAL_FORMAT	    (-38) /* buffer already created */

/*
 * CallBack function pointer definition
 */
typedef long	(*CL_FunctionPtr)();

struct CL_CompressorStorage;
typedef struct CL_CompressorStorage	*CL_Handle, *CL_CompressorHdl;
struct CL_BufferStorage;
typedef struct CL_BufferStorage	*CL_BufferHdl;


/* Public Routine Prototypes ********************************************/

long clDecompressImage(long decompressionScheme, 
    long width, long height, long originalFormat, 
    long compressedBufferSize, void *compressedBuffer, void *frameBuffer);
long clCompressImage(long compressionScheme, 
    long width, long height, long originalFormat, float compressionRatio, 
    void *frameBuffer, long *compressedBufferSize, void *compressedBuffer);

long clOpenCompressor(long scheme, CL_Handle *handle);
long clCompress(CL_Handle handle, long numberOfFrames, 
    void *frameBuffer, long *compressedDataSize, void *compressedData);
long clCloseCompressor(CL_Handle handle);

long clOpenDecompressor(long scheme, CL_Handle *handle);
long clDecompress(CL_Handle handle, long numberOfFrames, 
    long compressedDataSize, void *compressedData, void *frameBuffer);
long clCloseDecompressor(CL_Handle handle);
long clQueryAlgorithm(CL_Handle handle);

CL_BufferHdl clCreateBuf(CL_Handle handle, long bufferType, long blocks, 
    long blockSize, void **bufferPtr);
long clDestroyBuf(CL_BufferHdl bufferHdl);
CL_BufferHdl clQueryBufferHdl(CL_Handle handle, long bufferType, void **buffer);

long clQueryFree(CL_BufferHdl bufferHdl, long space, void **data, 
    long *wrap);
long clUpdateHead(CL_BufferHdl bufferHdl, long size);
long clQueryValid(CL_BufferHdl bufferHdl, long numberOfFrames, 
		void **frameBuffer, long *wrap);
long clUpdateTail(CL_BufferHdl bufferHdl, long numberOfFrames);

long clScanData(CL_Handle handle, long frameType, long numberOfFrames, 
    long maxBytesToScan);
long clReadHeader(CL_Handle handle);

/*
 * Parameter querying, getting, setting, etc.
 */
long clQueryParams(CL_Handle handle, long *parameterValuebuffer, long maxLength);
void clGetParams(CL_Handle handle, long *parameterValuebuffer, long bufferLength);
void clSetParams(CL_Handle handle, long *parameterValuebuffer, long bufferLength);
void clGetMinMax(CL_Handle handle, long param,long* minParam, long* maxParam);
long clGetDefault(CL_Handle handle, long param);
char* clGetName(CL_Handle handle, long param);
long clGetParamID(CL_Handle handle, char *name);

/*
 * Routines for querying the list of compression algorithms, adding a new 
 *  algorithm, and adding any extra parameters it may require.
 */
long clQueryAlgorithms(long algorithmType, long *PVbuffer, long bufferlength);
long clGetAlgorithmID(long algorithmType, char *name);
long clAddAlgorithm(char *name, long type, FunctionPtr openCompressor, 
    FunctionPtr compress, FunctionPtr closeCompressor, 
    FunctionPtr openDecompressor, FunctionPtr decompress, 
    FunctionPtr closeDecompressor, FunctionPtr scanData, 
    FunctionPtr getParams, FunctionPtr setParams, long *compressionScheme);
long clAddParam(long compressionScheme, char *name, long type, long min, 
    long max, long initial, long *paramID);
long clSetDefault(long scheme, long paramID, long value);

extern void (*clError)(long code, const char *fmt, ...);

void clReadSetup(CL_BufferHdl bufferHdl, long maxReadAhead);
long clReadData(CL_BufferHdl bufferHdl, long requestedDataSize, void **compressedData);
long clReadDataRange(CL_BufferHdl bufferHdl, long minSize, long maxSize, 
    void **compressedData);
long clDone(CL_BufferHdl bufferHdl, long size);

#define CL_ReadData(bufferHdl, size, data, nextData, dataEnd, dataPtr) \
( \
    /* if */ ((size) <= (long)((dataEnd) - (data))) ? ( \
	/* The request is within the current block of data */ \
	(data) = (nextData), \
	(nextData) += (size), \
	/* return */ (size) \
    ) \
    /* else */ : ( \
	clDone(bufferHdl, (long)((nextData) - (dataPtr))), \
	(dataEnd) = (byte *)clReadData(bufferHdl, size, dataPtr), \
	(data) = *(dataPtr), \
	(dataEnd) += (long)(data), \
	(nextData) = (data) + (size), \
	/* return */ (size) \
    ) \
)

#define CL_ReadMax(bufferHdl, max) bufferHdl->maxReadAhead = max; 

#define CL_ReadSetup(bufferHdl, data, dataEnd, dataPtr) \
    data = (byte *)bufferHdl->dataBlock; \
    dataEnd = (byte *)bufferHdl->dataBlock + bufferHdl->dataBlockSize;

#define CL_ReadByte(bufferHdl, data, dataEnd, dataPtr) \
( \
    /* if */ ((data) < (dataEnd)) ? ( \
	/* The request is within the current block of data */ \
	/* return */ (*(data)++) \
    ) \
    /* else */ : ( \
	clReadData(bufferHdl, 1, (void **)dataPtr), \
	(data) = (byte *)bufferHdl->dataBlock - 1, \
	(dataEnd) = (byte *)(bufferHdl->dataBlock + bufferHdl->dataBlockSize + 1), \
	/* return */ (*(data)++) \
    ) \
)

#define CL_Done2(bufferHdl, data, dataEnd) \
    bufferHdl->dataBlock = (data); \
    bufferHdl->dataBlockSize = (dataEnd) - (data); \

/*
 * error handler: you can replace the default error handler with your
 *     own routine by calling clSetErrorHandler
 */
typedef void (*CL_ErrFunc)(long code, const char *fmt, ...);
extern CL_ErrFunc clSetErrorHandler(CL_ErrFunc efunc);

#ifdef __cplusplus
}
#endif

#endif
