/*
This source file is the copyright property of the University of Leeds
('The University').

Permission to use, copy, modify, and distribute this source file for
educational, research, and not-for-profit purposes, without fee and
without a signed licensing agreement, is hereby granted, provided that
the above copyright notice, this paragraph and the following three
paragraphs appear in all copies, modifications, and distributions.

In no event shall The University be liable to any party for direct,
indirect, special, incidental or consequential damages, including lost
profits, arising out of the use of this software and its documentation.

The software is provided without warranty. The University has no
obligation to provide maintenance, support, updates, enhancements, or
modifications.

This software was written by Adam Baumberg, Vision Group,
School of Computer Studies, University of Leeds, UK. The code and use
thereof should be attributed to the author where appropriate
(including demonstrations which rely on it's use).
*/

/*
This source file is the copyright property of the University of Leeds
('The University').

Permission to use, copy, modify, and distribute this source file for
educational, research, and not-for-profit purposes, without fee and
without a signed licensing agreement, is hereby granted, provided that
the above copyright notice, this paragraph and the following three
paragraphs appear in all copies, modifications, and distributions.

In no event shall The University be liable to any party for direct,
indirect, special, incidental or consequential damages, including lost
profits, arising out of the use of this software and its documentation.

The software is provided without warranty. The University has no
obligation to provide maintenance, support, updates, enhancements, or
modifications.

This software was written by Adam Baumberg, Vision Group,
School of Computer Studies, University of Leeds, UK. The code and use
thereof should be attributed to the author where appropriate
(including demonstrations which rely on it's use).
*/

/***************************************************************
 * C++ source
 *
 * File : 	sgimgsrc.cc
 *
 * Module :	sgimgsrc
 *
 * Author : 	A M Baumberg (CoMIR)
 *
 * Creation Date : Thu Jun 16 12:45:12 1994 
 *
 * Comments : 	video interface class
 * 		- indy video board implementation
 *
 *
 ***************************************************************/


/* include files */

#include <sgimgsrc.h>
#include <vl/vl.h>

/* global functions */


/* local functions */


static struct vl_data
{
  VLServer svr;
  VLPath path;
  VLNode src, drn;
  VLBuffer buffer;
  VLInfoPtr info;
  timeval timestamp;
};


SGimgsrc::SGimgsrc(int bufsize, int rate, ImageType img_format, int w,
		   int h, int DO_BURST)
{
  extra_data = (void*) new vl_data;
  VLServer& svr = ((vl_data*) extra_data)->svr;
  VLPath& path = ((vl_data*) extra_data)->path;
  VLNode& src = ((vl_data*) extra_data)->src;
  VLNode& drn = ((vl_data*) extra_data)->drn;
  VLBuffer& buffer = ((vl_data*) extra_data)->buffer;
  VLInfoPtr& info = ((vl_data*) extra_data)->info;

  
  if (!(svr = vlOpenVideo("")))
    {
      vlPerror("Can't open Video");
      exit(1);
    }

  /* Set up a drain node in memory */
  drn = vlGetNode(svr, VL_DRN, VL_MEM, VL_ANY);
  
  /* Set up a source node on any video source  */
  src = vlGetNode(svr, VL_SRC, VL_VIDEO, VL_ANY);
  
  /* Create a path using the first device that will support it */
  path = vlCreatePath(svr, VL_ANY, src, drn);
  
  /* Set up the hardware for and define the usage of the path */
  if ((vlSetupPaths(svr, (VLPathList)&path, 1, VL_SHARE, VL_SHARE)) < 0)
    {
      vlPerror("can't setup hardware for video");
      exit(1);
    }
  
  format = img_format;
  buffer_size = bufsize;
  samplingrate = rate;
  frame_number = 0;
  USE_BURST = 0;
  VLControlValue val;
  if (w != 0) 
    {
      vlGetControl(svr, path, drn, VL_SIZE, &val);
      realno ratio = val.xyVal.x / w;
      val.xyVal.x = w;
      val.xyVal.y = ((realno) val.xyVal.y) / ratio;
      vlSetControl(svr, path, drn, VL_SIZE, &val);
      val.fractVal.numerator = 1;
      val.fractVal.denominator = ratio;
      vlSetControl(svr, path, drn, VL_ZOOM, &val);
    }
  
  if (format == RGB8) val.intVal = VL_PACKING_RBG_323;
  else if (format == GRAY8) val.intVal = VL_PACKING_Y_8_P;
  else if (format == RGB32) val.intVal = VL_PACKING_RGBA_8;
  else if (isYUV(format)) val.intVal = VL_PACKING_VUY_411_SV;

  if (isYUV(format))
    {
      vlPerror("Warning: Indy Video board does not support");
      vlPerror("         YUV Image formats -- try RGB32 or GRAY8");
    }

  vlSetControl(svr, path, drn, VL_PACKING, &val);
  int packing = val.intVal;
  vlGetControl(svr, path, drn, VL_PACKING, &val);
  if (packing != val.intVal)
    {
      vlPerror("Packing mode not supported !?");
      exit(1);
    }
  if (samplingrate != 0)
    {
      vlGetControl(svr, path, drn, VL_RATE, &val);      
      val.fractVal.numerator /= samplingrate;
      vlSetControl(svr, path, drn, VL_RATE, &val);
    }

  buffer = vlCreateBuffer(svr, path, drn, buffer_size);
  if (buffer == NULL)
    {
      vlPerror("can't create vl buffer");
      exit(1);
    }
  vlRegisterBuffer(svr, path, drn, buffer);

}
  
  
void SGimgsrc::start()
{
  VLServer& svr = ((vl_data*) extra_data)->svr;
  VLPath& path = ((vl_data*) extra_data)->path;
  VLNode& src = ((vl_data*) extra_data)->src;
  VLNode& drn = ((vl_data*) extra_data)->drn;
  VLBuffer& buffer = ((vl_data*) extra_data)->buffer;
  VLInfoPtr& info = ((vl_data*) extra_data)->info;  

  if (vlBeginTransfer(svr, path, 0, NULL))
    {
      vlPerror("can't start vl transfer");
      exit(1);
    }

}


void SGimgsrc::stop()
{
  VLServer& svr = ((vl_data*) extra_data)->svr;
  VLPath& path = ((vl_data*) extra_data)->path;
  VLNode& src = ((vl_data*) extra_data)->src;
  VLNode& drn = ((vl_data*) extra_data)->drn;
  VLBuffer& buffer = ((vl_data*) extra_data)->buffer;
  VLInfoPtr& info = ((vl_data*) extra_data)->info;  

  vlEndTransfer(svr, path);
}

Image* SGimgsrc::get_next(void)
{
  VLServer& svr = ((vl_data*) extra_data)->svr;
  VLPath& path = ((vl_data*) extra_data)->path;
  VLNode& src = ((vl_data*) extra_data)->src;
  VLNode& drn = ((vl_data*) extra_data)->drn;
  VLBuffer& buffer = ((vl_data*) extra_data)->buffer;
  VLInfoPtr& info = ((vl_data*) extra_data)->info;  
  timeval& timestamp = ((vl_data*) extra_data)->timestamp;

  if (frame_number == 0)
    start();
  vlPutFree(svr,buffer);
  do
    {
      info = vlGetNextValid(svr, buffer);
      if (info == NULL) sginap(1);
    } while (!info);

  DMediaInfo* media_info = vlGetDMediaInfo(svr, buffer, info);
  timestamp = media_info->time;

  videoframe = (unsigned char*) vlGetActiveRegion(svr, buffer, info);
  int w,h;
  VLControlValue val;
  vlGetControl(svr, path, drn, VL_SIZE, &val);  
  w = val.xyVal.x;
  h = val.xyVal.y;
  if (frame_number == 0)
    {
      switch (format)
	{
	case RGB8:
	  current = new RGB8Image(w, h, videoframe);
	  break;

	case GRAY8:
	  current = new Gray8Image(w, h, videoframe);
	  break;

	case RGB32:
	  current = new RGB32Image(w, h, videoframe);
	  break;

	case YUV411:
	  current = new YUVImage411(w,h);
	  ((YUVImage*)current)->video_to_YUV((long*) videoframe);
	  break;

	case YUV211:
	  current = new YUVImage211(w >> 1, h >> 1);
	  ((YUVImage*)current)->video_to_YUV((long*) videoframe);
	  break;

	case YUV111:
	  current = new YUVImage111(w >> 2, h >> 2);
	  ((YUVImage*)current)->video_to_YUV((long*) videoframe);
	  break;

	}

    }

  else 
    {
      if (isYUV(format)) 
	((YUVImage*)current)->video_to_YUV((long*) videoframe);

      else current->set_data(videoframe);

    }

  frame_number++;
  return (current);
}


timeval SGimgsrc::get_current_timestamp()
{
  return (((vl_data*) extra_data)->timestamp);
}


long SGimgsrc::display(long win)
{
  return (glwin = current->display(win));
}


SGimgsrc::~SGimgsrc()
{ 
  VLServer& svr = ((vl_data*) extra_data)->svr;
  VLPath& path = ((vl_data*) extra_data)->path;
  VLNode& src = ((vl_data*) extra_data)->src;
  VLNode& drn = ((vl_data*) extra_data)->drn;
  VLBuffer& buffer = ((vl_data*) extra_data)->buffer;
  VLInfoPtr& info = ((vl_data*) extra_data)->info;  

  stop();
  
  vlDeregisterBuffer(svr, path, drn, buffer);
  vlDestroyBuffer(svr, buffer);
  vlDestroyPath(svr, path);
  vlCloseVideo(svr);

  delete (current);
}

