/*
This source file is the copyright property of the University of Leeds
('The University').

Permission to use, copy, modify, and distribute this source file for
educational, research, and not-for-profit purposes, without fee and
without a signed licensing agreement, is hereby granted, provided that
the above copyright notice, this paragraph and the following three
paragraphs appear in all copies, modifications, and distributions.

In no event shall The University be liable to any party for direct,
indirect, special, incidental or consequential damages, including lost
profits, arising out of the use of this software and its documentation.

The software is provided without warranty. The University has no
obligation to provide maintenance, support, updates, enhancements, or
modifications.

This software was written by Adam Baumberg, Vision Group,
School of Computer Studies, University of Leeds, UK. The code and use
thereof should be attributed to the author where appropriate
(including demonstrations which rely on it's use).
*/
/*
This source file is the copyright property of the University of Leeds
('The University').

Permission to use, copy, modify, and distribute this source file for
educational, research, and not-for-profit purposes, without fee and
without a signed licensing agreement, is hereby granted, provided that
the above copyright notice, this paragraph and the following three
paragraphs appear in all copies, modifications, and distributions.

In no event shall The University be liable to any party for direct,
indirect, special, incidental or consequential damages, including lost
profits, arising out of the use of this software and its documentation.

The software is provided without warranty. The University has no
obligation to provide maintenance, support, updates, enhancements, or
modifications.

This software was written by Adam Baumberg, Vision Group,
School of Computer Studies, University of Leeds, UK. The code and use
thereof should be attributed to the author where appropriate
(including demonstrations which rely on it's use).
*/
/*
 * sgmoviestore.cc
 * class stores images in a moviefile format
 *
 * author: A.M.Baumberg (amb)
 * creation date: 5/2/93
 */

#include <sgmoviestore.h>
#include <image.h>
#include <YUVImage.h>
#include <movie.h>
/*
#include <dmedia/moviefile.h>
*/

struct NewMovieExtras
{
  MVid theMovie;
  MVid theTrack;
};



MovieStore::MovieStore(char* filename, int n_frames, bool mcompress)
{

  compress = mcompress;
  max_frames = n_frames;
  movie_name = filename;
  frame_no = 0;

  NewMovieExtras* mv_data = new NewMovieExtras;
  hidden = (void*) mv_data;

  // if file exists then append to it ! 
  if ((n_frames == 0) && 
      (mvOpenFile(filename, O_RDWR, &mv_data->theMovie) == DM_SUCCESS))
    {
      if (mvFindTrackByMedium(mv_data->theMovie, DM_IMAGE,
			      &mv_data->theTrack) != DM_SUCCESS)
		{
			fprintf(stderr,
				"MovieStore::MovieStore err with mvFindTrackByMedium\n");
	fflush(stderr);
			abort();
		}
      
      // start adding frames at end of track
      frame_no = mvGetTrackLength(mv_data->theTrack);
    }
  else
    {
      DMparams* params;
      
      if (dmParamsCreate(&params) != DM_SUCCESS)
		{
			fprintf(stderr,
				"MovieStore::MovieStore err with dmParamsCreate\n");
	fflush(stderr);
			abort();
		}
      if ( mvSetMovieDefaults( params, MV_FORMAT_SGI_3) != DM_SUCCESS)
		{
			fprintf(stderr,
				"MovieStore::MovieStore err with  mvSetMovieDefaults\n");
	fflush(stderr);
			abort();
		}
      if (mvCreateFile(filename,params, NULL, &mv_data->theMovie)
	  				!= DM_SUCCESS)
		{
			fprintf(stderr,
				"MovieStore::MovieStore err with  mvCreateFile\n");
	fflush(stderr);
			abort();
		}
      dmParamsDestroy(params);
    }
} 


int  MovieStore::add_a_frame(Image* new_frame)
{
  if (new_frame == NULL) return (TRUE);
  if ((max_frames != 0) && (frame_no >= max_frames)) return (FALSE);
  NewMovieExtras* mv_data = (NewMovieExtras*) hidden;
  if (frame_no == 0)
    {
      DMparams* params;
      if (dmParamsCreate(&params) != DM_SUCCESS)
		{
			fprintf(stderr,
				"MovieStore::add_a_frame err with dmParamsCreate\n");
	fflush(stderr);
			abort();
		}

      if (dmSetImageDefaults(params, new_frame->get_width(),
			     new_frame->get_height(), DM_PACKING_RGB) != DM_SUCCESS)
		{
			fprintf(stderr,
				"MovieStore::add_a_frame err with dmSetImageDefaults\n");
	fflush(stderr);
			abort();
		}
      
      if (new_frame->get_access_mode() == IA_TOP_TO_BOTTOM)
	dmParamsSetEnum(params, DM_IMAGE_ORIENTATION,
			DM_TOP_TO_BOTTOM);
      if (!compress)
	dmParamsSetString(params, DM_IMAGE_COMPRESSION,
			    DM_IMAGE_UNCOMPRESSED);

      switch (new_frame->imgtype())
      {
      case RGB8:
	dmParamsSetEnum(params, DM_IMAGE_PACKING, DM_PACKING_RGB332);
	if (compress)
	  dmParamsSetString(params, DM_IMAGE_COMPRESSION,
			    DM_IMAGE_MVC1);
	break;

      case GRAY8:
	dmParamsSetEnum(params, DM_IMAGE_PACKING, DM_PACKING_GRAYSCALE);
	if (compress)
	  dmParamsSetString(params, DM_IMAGE_COMPRESSION,
			    DM_IMAGE_MVC1);
	break;

      case RGB32:
      case YUV411:
      case YUV211:
      case YUV111:
	dmParamsSetEnum(params, DM_IMAGE_PACKING, DM_PACKING_RGBX);
	if (compress)
	  dmParamsSetString(params, DM_IMAGE_COMPRESSION,
			    DM_IMAGE_MVC1);

		
      }
      if (mvAddTrack(mv_data->theMovie, DM_IMAGE, params, NULL,
		     &mv_data->theTrack) != DM_SUCCESS) 
		{
			fprintf(stderr,
				"MovieStore::add_a_frame err with mvAddTrack\n");
	fflush(stderr);
			abort();
		}

      dmParamsDestroy(params);

    }
  if (isYUV(new_frame->imgtype()))
    new_frame = ((YUVImage*) new_frame)->get_rgb32();


  if (mvInsertFrames(mv_data->theTrack, frame_no++, 1, 
		     new_frame->endofdata() - new_frame->get_data(),
		     new_frame->get_data()) != DM_SUCCESS)
		{
			fprintf(stderr,
				"MovieStore::add_a_frame err with mvInsertFrames\n");
	fflush(stderr);
			abort();
		}

  if (max_frames == 0)
    {
      // always flush the header
      mvWrite(mv_data->theMovie);
    }
  else
    {
      if (frame_no >= max_frames)
	{
	  mvWrite(mv_data->theMovie);
	  return (FALSE);
	}
    }
  return (TRUE);

}

MovieStore::~MovieStore()
{
  NewMovieExtras* mv_data = (NewMovieExtras*) hidden;
  mvClose(mv_data->theMovie);
  delete (mv_data);
}

