/*
From T.D.Grove@reading.ac.uk Fri Feb 27 15:55:24 1998
Date: Fri, 27 Feb 1998 15:53:27 GMT
From: T.D.Grove@reading.ac.uk (Thomas Grove)
To: rjm
Subject: Re:  Adam's code

> From rjm@dcre.leeds.ac.uk Fri Feb 27 13:08 GMT 1998
> Date: Fri, 27 Feb 1998 12:54:05 GMT
> To: T.D.Grove@reading.ac.uk (Thomas Grove)
> Subject: Re:  Adam's code
> 
> Tom,
> 	Just wondering what you have managed to do with Adams code.
> Have you A) got it running on an O2, or which versions of Irix?
> B) Managed to get it to read the folkstone .mov files (which my system
> says are QuickTime). If so, how? 
> 
> 		Rich

Irix 6.3 

If you recompile the system under irix 6.3 you should be 
able to read quicktime movies, since it will statically 
link with the newer versions of the video library. However,
Adam's code doesn't accept the  jpeg format (CbYCrY) used
in the folkstone sequence, so you will need to add this in
around line 135 in sgmoviesrc.cc. I.e. 

if ((img_packing == DM_PACKING_RGBX) ||
     (img_packing == DM_PACKING_CbYCrY)) imgformat = RGB32;
else
  :
  etc

The behaviour of mvReadFrames also seems to have changed, so you will 
need to comment out a certain amount of code. It's probably better
if I send you my version of sgmoviesrc.cc - I don't think I've
changed anything apart from minor bugs and the above, but run diff
on it with your version to check that there are no drastic differences.

This file is normally found in /imgsrc/irix6/


Mail me if you need further help.

Tom

*/

/*
This source file is the copyright property of the University of Leeds
('The University').

Permission to use, copy, modify, and distribute this source file for
educational, research, and not-for-profit purposes, without fee and
without a signed licensing agreement, is hereby granted, provided that
the above copyright notice, this paragraph and the following three
paragraphs appear in all copies, modifications, and distributions.

In no event shall The University be liable to any party for direct,
indirect, special, incidental or consequential damages, including lost
profits, arising out of the use of this software and its documentation.

The software is provided without warranty. The University has no
obligation to provide maintenance, support, updates, enhancements, or
modifications.

This software was written by Adam Baumberg, Vision Group,
School of Computer Studies, University of Leeds, UK. The code and use
thereof should be attributed to the author where appropriate
(including demonstrations which rely on it's use).
*/
/*
This source file is the copyright property of the University of Leeds
('The University').

Permission to use, copy, modify, and distribute this source file for
educational, research, and not-for-profit purposes, without fee and
without a signed licensing agreement, is hereby granted, provided that
the above copyright notice, this paragraph and the following three
paragraphs appear in all copies, modifications, and distributions.

In no event shall The University be liable to any party for direct,
indirect, special, incidental or consequential damages, including lost
profits, arising out of the use of this software and its documentation.

The software is provided without warranty. The University has no
obligation to provide maintenance, support, updates, enhancements, or
modifications.

This software was written by Adam Baumberg, Vision Group,
School of Computer Studies, University of Leeds, UK. The code and use
thereof should be attributed to the author where appropriate
(including demonstrations which rely on it's use).
*/
/*
 * sgmoviesrc.cc
 *  image source for movie files
 *
 */


#include <stdio.h>  /* For i/o */
#include <fcntl.h>  /* For open/read/close */
#include <stdlib.h> /* For malloc */
#include <unistd.h>
#include <sgmoviesrc.h>
#include <Gray8Image.h>
#include <RGB8Image.h>
#include <movie.h>
#include <dmedia/dmedia.h>
#include <cl.h>

extern "C" 
{ 
  #include "Movie.h" 
}


static void old_constructor(SGmoviesrc* mv, char* mv_name);
static void old_destructor(SGmoviesrc* mv);
static Image* old_get_next(SGmoviesrc* mv);






#define IMAGE_FORMAT_MVC1	    1
#define IMAGE_FORMAT_8RGB	    5
#define IMAGE_FORMAT_JPEG	    6
#define IMAGE_FORMAT_RLE	    7
#define IMAGE_FORMAT_UNCOMPRESSED   8
#define IMAGE_FORMAT_RTR	    9
#define IMAGE_FORMAT_GRAYSCALE	    16
#define IMAGE_FORMAT_YUV	    32



static struct mv_extras
{
public:
  
  // new stuff
  MVid theMovie;
  MVid theImages;
  int is_new;

  // old stuff
  long decompressionScheme;
  long *rawdata; 
  long frameSize;
  DirectoryEntry *directory;
  CL_Handle decompressorHdl;
  int inFile;
};

static bool is_old_format(char* mv_name)
{
  int inFile;
  if ((inFile = open(mv_name,O_RDONLY)) == -1) {
      fprintf(stderr, ": can't open %s\n", mv_name);
      exit(FAILURE);
    }    
  long mv_id;
  short version;
  read(inFile, &mv_id, sizeof(long));
  read(inFile, &version, sizeof(short));
  close(inFile);
  if (version == 2) return (TRUE);
  return (FALSE);
}
  

SGmoviesrc::SGmoviesrc(char* mv_name)
{
  hidden = (void*) new (mv_extras);

  if (is_old_format(mv_name))
    {
      old_constructor(this, mv_name);
      ((mv_extras*)hidden)->is_new = FALSE;
    }  
  else
    {
      if (mvOpenFile(mv_name, O_RDONLY, &((mv_extras*)hidden)->theMovie) 
	  != DM_SUCCESS)
	{
	  fprintf(stderr, ": can't open %s\n", mv_name);
	  exit(1);
	}    
      
      ((mv_extras*)hidden)->is_new = TRUE;
      
      MVid theImages;
      mvFindTrackByMedium(((mv_extras*)hidden)->theMovie, DM_IMAGE, 
			  &((mv_extras*)hidden)->theImages);
      no_frames = (int) mvGetTrackLength(((mv_extras*)hidden)->theImages);
      width = mvGetImageWidth(((mv_extras*)hidden)->theImages);
      height = mvGetImageHeight(((mv_extras*)hidden)->theImages);
      
      DMpacking img_packing = mvGetImagePacking(((mv_extras*)hidden)->theImages);
      
      if ((img_packing == DM_PACKING_RGBX) ||
          (img_packing == DM_IMAGE_PACKING_CbYCrY)) imgformat = RGB32;
      else
	{
	  if (img_packing == DM_PACKING_GRAYSCALE) imgformat = GRAY8;
	  else
	    {
	      fprintf(stderr,
		" unsupported packing type of type %d in movie \n",
	(int)img_packing);
	      exit(1);
	    }
	}
    }
} 


Image* SGmoviesrc::get_next()
{
  if (!((mv_extras*)hidden)->is_new)
    return (old_get_next(this));

  frame_number++;
  if (frame_number >= no_frames) return (NULL);
  if (current  == NULL)
    { 
      if (imgformat == GRAY8) current = new Gray8Image(width, height);
      else current = new RGB32Image(width, height);

// tdg 9/2/98 - obselete; mvReadFrames always returns images
// B_TO_T by default

/*
	if ((current->get_access_mode() == IA_TOP_TO_BOTTOM) !=
	  (mvGetImageOrientation(((mv_extras*)hidden)->theImages)
	   == DM_TOP_TO_BOTTOM))
	{
	  if (current->get_access_mode() == IA_TOP_TO_BOTTOM)
	    current->reset_image_addressing(IA_BOTTOM_TO_TOP);
	  else
	    current->reset_image_addressing(IA_TOP_TO_BOTTOM);
	}
*/	 

    }    
  size_t img_size = (size_t)
    (current->endofdata() - current->get_data());
  
  if (mvReadFrames(((mv_extras*)hidden)->theImages,
		   (MVframe) frame_number - 1, (MVframe) 1,
		   img_size, current->get_data()) != DM_SUCCESS)
    {
      fprintf(stderr," error in SGmoviesrc::get_next \n");
	fflush(stderr);
      abort();
   } 
// tdg 9/2/98 mvReadFrames always returns images b_to_t
	  current->reset_image_addressing(IA_BOTTOM_TO_TOP);

  return (current);
}    
   

SGmoviesrc::~SGmoviesrc()
{
  if (!((mv_extras*)hidden)->is_new)
    old_destructor(this);
  else
    {
      delete current;
      mvClose(((mv_extras*)hidden)->theMovie);
      delete (hidden);
    }
}

// old movie class





/*
 * Function for reading compressed video data.
 * 
 * The movie file has a "directory" that stores the location and sizes
 * of the audio and video data. This routine reads the directory entry
 * then the video data for the specified frame. The data is read into
 * the specified buffer along with the size of the frames.
 */

static long GetCompressedVideo(long frameIndex, long size, void *data, 
			DirectoryEntry *directory, int inFile)
{
    long actualSize;
    
    /* Read the video data which is after the audio data */
    lseek(inFile, directory[frameIndex].location + directory[frameIndex].asize, 
	SEEK_SET);
    actualSize = directory[frameIndex].vsize;
    if(actualSize <= size)
        read(inFile, data, (unsigned) actualSize);
    else
      {
	fprintf(stderr, " Frame too big ");
	exit(FAILURE);
      }
    
    return(actualSize);
}
/*
 *  image source initialisation  
 */

static void old_constructor(SGmoviesrc* mv, char* mv_name)
{
    MovieHeader	tmvhdr;
    long    grayscale, yuv;
    static int    paramBuf[][2] = {
	CL_IMAGE_WIDTH, 0, 
	CL_IMAGE_HEIGHT, 0, 
	CL_ORIGINAL_FORMAT, 0, 
	CL_BITS_PER_COMPONENT, 0, 
	CL_FRAME_RATE, 0, 
	CL_INTERNAL_FORMAT, 0, 
      };
    mv->hidden = (void*) new (mv_extras);

    int& inFile = ((mv_extras*)(mv->hidden))->inFile;

    if ((inFile = open(mv_name,O_RDONLY)) == -1) {
      fprintf(stderr, ": can't open %s\n", mv_name);
      exit(FAILURE);
    }    
    
    
    /* Read movie file header */
    read(inFile, &tmvhdr, (unsigned) sizeof(MovieHeader));
    ((mv_extras*)(mv->hidden))->directory = (DirectoryEntry *) 
      malloc((unsigned) (sizeof(DirectoryEntry) * tmvhdr.frameCount));
    read(inFile, ((mv_extras*)(mv->hidden))->directory, 
	 (unsigned) (sizeof(DirectoryEntry) * tmvhdr.frameCount));
    
    mv->width = tmvhdr.frameWidth;
    mv->height = tmvhdr.frameHeight;
    grayscale = FALSE;
    yuv = FALSE;
    if((tmvhdr.image_format & IMAGE_FORMAT_GRAYSCALE) != 0) {
	grayscale = TRUE;
	tmvhdr.image_format &= ~IMAGE_FORMAT_GRAYSCALE;
      }
    else if((tmvhdr.image_format & IMAGE_FORMAT_YUV) != 0) {
      yuv = TRUE;
      tmvhdr.image_format &= ~IMAGE_FORMAT_YUV;
    }
    
    long &decompressionScheme =
      ((mv_extras*)(mv->hidden))->decompressionScheme;

    /* Open a Decompressor */
    if(tmvhdr.image_format == IMAGE_FORMAT_MVC1)
      decompressionScheme = CL_MVC1;
    else if((tmvhdr.image_format == IMAGE_FORMAT_MPEG) || 
	    (tmvhdr.image_format == 0))
      decompressionScheme = CL_MPEG_VIDEO;
    else if(tmvhdr.image_format == IMAGE_FORMAT_JPEG)
       decompressionScheme = CL_JPEG;
    else if(tmvhdr.image_format == IMAGE_FORMAT_RLE)
      decompressionScheme = CL_RLE;
    else if(tmvhdr.image_format == IMAGE_FORMAT_UNCOMPRESSED ||
	    tmvhdr.image_format == IMAGE_FORMAT_8RGB)
      decompressionScheme = CL_UNCOMPRESSED;
    else if(tmvhdr.image_format == IMAGE_FORMAT_RTR)
       decompressionScheme = CL_RTR;
    
    clOpenDecompressor(decompressionScheme,
		       &((mv_extras*)(mv->hidden))->decompressorHdl);
    
    paramBuf[0][1] = tmvhdr.frameWidth;
    paramBuf[1][1] = tmvhdr.frameHeight;
    if(grayscale) {
      paramBuf[2][1] = CL_GRAYSCALE;
    }
    else {
      paramBuf[2][1] = CL_RGBX;
    }
    paramBuf[3][1] = 8;
    paramBuf[4][1] = 30;
    if(yuv | grayscale) {
      if (yuv) paramBuf[5][1] = CL_YUV422HC;
      if (grayscale) paramBuf[5][1] = CL_GRAYSCALE;
	clSetParams(((mv_extras*)(mv->hidden))->decompressorHdl,
		    (int *)paramBuf, sizeof(paramBuf) / sizeof(int));
    }
    else
      clSetParams(((mv_extras*)(mv->hidden))->decompressorHdl, 
		  (int *)paramBuf, sizeof(paramBuf) / sizeof(int) - 2);
    
    ((mv_extras*)(mv->hidden))->frameSize = (long) mv->width * mv->height * sizeof(long);
    ((mv_extras*)(mv->hidden))->rawdata 
      = (long *) malloc((unsigned) ((mv_extras*)(mv->hidden))->frameSize);
    mv->frame_number = 0;
    mv->no_frames = (int) tmvhdr.frameCount;
    mv->current =  NULL;
    if (grayscale) mv->imgformat = GRAY8;
    else mv->imgformat = RGB32;
} 
    

static Image* old_get_next(SGmoviesrc* mv)
{
  long dataSize;
  mv->frame_number++;
  int& inFile = ((mv_extras*)(mv->hidden))->inFile;

  if (mv->frame_number > mv->no_frames) return (NULL);
  if (mv->current  == NULL)
    { 
      if (mv->imgformat == GRAY8) mv->current = new Gray8Image(mv->width, mv->height);
      else mv->current = new RGB32Image(mv->width, mv->height);
    }    

  long &decompressionScheme =
      ((mv_extras*)(mv->hidden))->decompressionScheme;
 
  if (decompressionScheme == CL_UNCOMPRESSED)
    {
      dataSize = GetCompressedVideo(mv->frame_number-1,
				    ((mv_extras*)(mv->hidden))->frameSize,
				    mv->current->get_data(), 
				    ((mv_extras*)(mv->hidden))->directory, 
				    inFile);
      mv->current->set_enddata(mv->current->get_data() + dataSize);
    }
  else 
    {
      dataSize = GetCompressedVideo(mv->frame_number-1,
				    ((mv_extras*)(mv->hidden))->frameSize,
				    ((mv_extras*)(mv->hidden))->rawdata,
				    ((mv_extras*)(mv->hidden))->directory,
				    inFile);
      clDecompress(((mv_extras*)(mv->hidden))->decompressorHdl, 1, dataSize, 
		   ((mv_extras*)(mv->hidden))->rawdata, 
		   (unsigned char*) (mv->current->get_data()));
      //mv->current->flip_vertically(mv->current);

    }
  return (mv->current);
}    
   

static void old_destructor(SGmoviesrc* mv)
{
  free(((mv_extras*)(mv->hidden))->rawdata);
  free(((mv_extras*)(mv->hidden))->directory);
  delete mv->current;
  clCloseDecompressor(((mv_extras*)(mv->hidden))->decompressorHdl);
  close(((mv_extras*)(mv->hidden))->inFile);
  delete (mv->hidden);
}

