/*
This source file is the copyright property of the University of Leeds
('The University').

Permission to use, copy, modify, and distribute this source file for
educational, research, and not-for-profit purposes, without fee and
without a signed licensing agreement, is hereby granted, provided that
the above copyright notice, this paragraph and the following three
paragraphs appear in all copies, modifications, and distributions.

In no event shall The University be liable to any party for direct,
indirect, special, incidental or consequential damages, including lost
profits, arising out of the use of this software and its documentation.

The software is provided without warranty. The University has no
obligation to provide maintenance, support, updates, enhancements, or
modifications.

This software was written by Adam Baumberg, Vision Group,
School of Computer Studies, University of Leeds, UK. The code and use
thereof should be attributed to the author where appropriate
(including demonstrations which rely on it's use).
*/

/*
 * kernel.h
 * 3x3 kernel class
 * for image processing
 */

#ifndef __KERNEL_H__
#define __KERNEL_H__

#ifndef TRUE
#define TRUE 1
#endif

#ifndef FALSE
#define FALSE 0
#endif

typedef int bool;


class Kernel 
{
 public:
  
  /* the kernel is simply a 3x3 integer array */
  int coeff[3][3];

  /* along with some constructors */
  Kernel() { coeff[0][0] = coeff[0][1] = coeff[0][2] = coeff[1][0] = coeff[1][1]
	     = coeff[1][2] = coeff[2][0] = coeff[2][1] = coeff[2][2] = 0;}
  Kernel( int row1[3], int row2[3], int row3[3], bool by_row = TRUE );
  Kernel( int data[9] , bool by_row  = TRUE);
  Kernel(int d1,int d2, int d3, int d4, int d5, int d6, int d7, 
	       int d8, int d9, bool by_row = TRUE);

};

Kernel operator+ (Kernel& K1 , Kernel& K2);
Kernel operator* (Kernel& K1, Kernel& K2);

const Kernel SOBELV_KER = Kernel(1,0,-1,2,0,-2,1,0,-1,TRUE);
const Kernel SOBELH_KER = Kernel(1,0,-1,2,0,-2,1,0,-1,FALSE);
const Kernel BLUR_KER = Kernel(1,1,1,1,1,1,1,1,1,TRUE);

#endif





