/*
This source file is the copyright property of the University of Leeds
('The University').

Permission to use, copy, modify, and distribute this source file for
educational, research, and not-for-profit purposes, without fee and
without a signed licensing agreement, is hereby granted, provided that
the above copyright notice, this paragraph and the following three
paragraphs appear in all copies, modifications, and distributions.

In no event shall The University be liable to any party for direct,
indirect, special, incidental or consequential damages, including lost
profits, arising out of the use of this software and its documentation.

The software is provided without warranty. The University has no
obligation to provide maintenance, support, updates, enhancements, or
modifications.

This software was written by Adam Baumberg, Vision Group,
School of Computer Studies, University of Leeds, UK. The code and use
thereof should be attributed to the author where appropriate
(including demonstrations which rely on it's use).
*/

/*
 * medsrc.h
 *
 * median filter (over time) to extract background
 * 
 * defined as an ImageSource
 */

#ifndef __MEDSRC_H__
#define __MEDSRC_H__

#include <imgsrc.h>
#include <Gray8Image.h>
#include <YUVImage.h>
#include <miscsrc.h>


static const DEF_RUN_LENGTH = 20;

class Medsrc : public PipeSource

{
 protected:
  /* for gray images only the red field is used */
  Image* counts_red; 
  Image* counts_green;
  Image* counts_blue;
  Image* nearest;
 
 public:

  // ImageSource* in_source;
  Image* motion_mask;

  int run_length;

  // input_images -- the source of images
  // run_length	  -- the (temporal) window used for update
  // background_image -- optional starting estimate for background
  Medsrc(ImageSource* input_images, int run_length = DEF_RUN_LENGTH, 
	 Image* background_image = NULL);


  void set_motion_mask(Image* img) { motion_mask = img; }

  //Image* get_next();
  //Image* refresh();
  Image* recalc();
  void restart();
		   

  ~Medsrc();
};

#endif
