/*
This source file is the copyright property of the University of Leeds
('The University').

Permission to use, copy, modify, and distribute this source file for
educational, research, and not-for-profit purposes, without fee and
without a signed licensing agreement, is hereby granted, provided that
the above copyright notice, this paragraph and the following three
paragraphs appear in all copies, modifications, and distributions.

In no event shall The University be liable to any party for direct,
indirect, special, incidental or consequential damages, including lost
profits, arising out of the use of this software and its documentation.

The software is provided without warranty. The University has no
obligation to provide maintenance, support, updates, enhancements, or
modifications.

This software was written by Adam Baumberg, Vision Group,
School of Computer Studies, University of Leeds, UK. The code and use
thereof should be attributed to the author where appropriate
(including demonstrations which rely on it's use).
*/

/*
 * moviestore.cc
 * class stores images in a moviefile format
 *
 * author: A.M.Baumberg (amb)
 * creation date: 5/2/93
 */

#include <moviestore.h>
#include <unistd.h>
#include <stdio.h>
#include <fcntl.h>  /* For open/read/close */
#include <stdlib.h>
#include <strings.h>


static int movie_formats[10];

MovieStore::MovieStore(char* filename, int n_frames)
{
  movie_formats[RGB8] = IMAGE_FORMAT_8RGB;
  movie_formats[GRAY8] = IMAGE_FORMAT_GRAYSCALE | IMAGE_FORMAT_UNCOMPRESSED;
  movie_formats[YUV411] = movie_formats[YUV211] = movie_formats[YUV111]
    = IMAGE_FORMAT_GRAYSCALE | IMAGE_FORMAT_UNCOMPRESSED;
  mh.frameCount = n_frames;
  mh.framesPerSec = 15;
  if ((outfd = open(filename, O_CREAT|O_RDWR|O_TRUNC, 0644)) < 0) 
    {
      fprintf(stderr, " moviestore: couldn't open file %s", filename);
      exit(1);
    }
  mh.movie_id = MOVIE_ID;
  mh.version = MOVIE_VERSION_2;
  mh.movieType = SILENT;
  mh.showType = MOVIE;
  mh.imageCsize = 4;
  mh.imageDatatype = 2;
  mh.loop_mode = LOOP_CONTINUOUSLY;
  strcpy(mh.title, filename );
  strcpy(mh.comments, " created using MovieStore class ");
  strcpy(mh.field_extensions, "field_extensions is the next area");
  strcpy(&mh.field_extensions[248], "the end");
  
  direntry = new DirectoryEntry[n_frames];
  frame_no = 0;
  if ((write(outfd,(char*) &mh, sizeof(MovieHeader))) != sizeof(MovieHeader)) 
    {
        fprintf(stderr, "moviestore: Can't write file %s", filename);
        exit(1);
    }
  if (write(outfd, (char*) direntry, (unsigned int) mh.frameCount*sizeof(DirectoryEntry))
      < mh.frameCount*sizeof(DirectoryEntry))
    {
      fprintf(stderr, "moviestore: can't write file %s", filename);
      exit(1);
    }
} 


int  MovieStore::add_a_frame(Image* new_frame)
{
  if (new_frame == NULL) return (TRUE);
  if (frame_no >= mh.frameCount) return (FALSE);
  if (frame_no == 0)
    {
      mh.image_format = movie_formats[new_frame->imgtype()];
      mh.imageXsize = (long) (new_frame->get_width());
      mh.imageYsize = (long) (new_frame->get_height());
      mh.frameWidth = (short) (mh.imageXsize);
      mh.frameHeight = (short) (mh.imageYsize);
      direntry[frame_no].location = sizeof(MovieHeader) + 
	mh.frameCount*sizeof(DirectoryEntry);
    }
  int videosize = (int) (new_frame->endofdata() - new_frame->get_data());
  direntry[frame_no].asize = 0;
  direntry[frame_no].vsize = videosize;
  
  /* Write video data out to movie file */
  if (write(outfd, (char*)  (new_frame->get_data()), videosize) < videosize)
    {
      fprintf(stderr, "moviestore:bad write frame %d in movie %s\n",
	     frame_no, mh.title);
      exit(1);
    }
  frame_no++;
  if (frame_no < mh.frameCount)
    {
      direntry[frame_no].location = direntry[frame_no-1].location +
	direntry[frame_no-1].vsize;
      return (TRUE);
    }
  else 
    {
      int j;
      /* Seek back to beginning of file */
      if ((j = (int) lseek(outfd, 0, SEEK_SET)) != 0)
	{
	  fprintf(stderr, "moviestore: lseek failed %s, %d not 0\n", mh.title, j);
	  exit(1);
	}
      /* Rewrite movie header and directory */
      if ((write(outfd, (char*) &mh, sizeof(MovieHeader))) != sizeof(MovieHeader)) 
	{
	  fprintf(stderr, "moviestore: Can't write file %s", mh.title);
	  exit(1);
	}
      if (write(outfd, (char*) direntry, (unsigned) mh.frameCount*sizeof(DirectoryEntry))
	  < mh.frameCount*sizeof(DirectoryEntry)) 
	{
	  fprintf(stderr, "moviestore: Can't write file %s", mh.title);
	  exit(1);
	}
      close(outfd);
      return (FALSE);
    }
}

MovieStore::~MovieStore()
{
  delete [] direntry;
  close(outfd);
}

