/*
This source file is the copyright property of the University of Leeds
('The University').

Permission to use, copy, modify, and distribute this source file for
educational, research, and not-for-profit purposes, without fee and
without a signed licensing agreement, is hereby granted, provided that
the above copyright notice, this paragraph and the following three
paragraphs appear in all copies, modifications, and distributions.

In no event shall The University be liable to any party for direct,
indirect, special, incidental or consequential damages, including lost
profits, arising out of the use of this software and its documentation.

The software is provided without warranty. The University has no
obligation to provide maintenance, support, updates, enhancements, or
modifications.

This software was written by Adam Baumberg, Vision Group,
School of Computer Studies, University of Leeds, UK. The code and use
thereof should be attributed to the author where appropriate
(including demonstrations which rely on it's use).
*/

/***************************************************************
 * C - C++ Header
 *
 * File : 	PolyFilter.h
 *
 * Module :	PolyFilter
 *
 * Author : 	A M Baumberg
 *
 * Creation Date : Mon Nov  8 13:08:53 1993 
 *
 * Comments : 	An object for prediting the next value in a sequence
 *	        of equally spaced data values using a polynomial fit.
 *		(Each data value is a vector.)
 *
 ***************************************************************/


/* include files */

#include <NagMatrix.h>
#include <NagVector.h>

class PolyFilter 
{
 protected:

  int curve_order;
  int no_eqns;
  int vector_size;

  int t_stamp;
  NagMatrix past_data;
  NagMatrix poly_coefficients;
  NagMatrix pseudo_inverse;

  NagMatrix extra_row;


 public:

  PolyFilter(int order, int no_eqns, int vector_size);
  void reset() { t_stamp = 0; }
  void add_observation(NagVector& observed);
  void get_prediction(NagVector& predicted);


};

