/*
This source file is the copyright property of the University of Leeds
('The University').

Permission to use, copy, modify, and distribute this source file for
educational, research, and not-for-profit purposes, without fee and
without a signed licensing agreement, is hereby granted, provided that
the above copyright notice, this paragraph and the following three
paragraphs appear in all copies, modifications, and distributions.

In no event shall The University be liable to any party for direct,
indirect, special, incidental or consequential damages, including lost
profits, arising out of the use of this software and its documentation.

The software is provided without warranty. The University has no
obligation to provide maintenance, support, updates, enhancements, or
modifications.

This software was written by Adam Baumberg, Vision Group,
School of Computer Studies, University of Leeds, UK. The code and use
thereof should be attributed to the author where appropriate
(including demonstrations which rely on it's use).
*/

/*
 * Point2 a simple 2D integer point class
 * 
 */

#ifndef __POINT2_H__
#define __POINT2_H__

#include <realno.h>
#include <math.h>
#include <stream.h>

class Point2
{
 public:

  realno x;
  realno y;
  
  Point2(realno x1, realno y1) { x = x1; y = y1; }
  Point2() { x = 0; y = 0; }
  inline realno length() { return (sqrt(x*x + y*y)); }
  inline realno length2() { return (x*x + y*y); }
  inline realno block_length() { return (fabs(x) + fabs(y)); }
  inline Point2 map(realno ax, realno ay) 
  { return (Point2(ax * x - ay  * y,
		   ay * x + ax * y)); }


  // operators
  inline Point2& operator+=(Point2 v) { x += v.x; y += v.y; return (*this); }
  inline Point2& operator-=(Point2 v) { x -= v.x; y -= v.y; return (*this); }
  inline Point2& operator*=(realno a) { x *= a; y *= a; return (*this); }
  inline Point2& operator/=(realno a) { x /= a; y /= a; return (*this); }

  inline void normalise()
  {
    if (length2() > 1e-6)
      {	*this /= length(); }
  }

  
};

inline Point2 operator-(Point2 u)
{ return Point2(-u.x, -u.y); }

inline Point2 operator*(realno a, Point2 v)
{ return Point2(a*v.x,a*v.y); }

inline Point2 operator*(Point2 v, realno a)
{ return Point2(a*v.x,a*v.y); }

inline Point2 operator/(Point2 v, realno a)
{ return Point2(v.x/a,v.y/a); }

inline Point2 operator+(Point2 u, Point2 v) 
{ return Point2(u.x+v.x, u.y+v.y); }

inline Point2 operator-(Point2 u, Point2 v)
{return Point2(u.x-v.x, u.y-v.y); }

inline realno operator*(Point2 u, Point2 v)
{return (u.x * v.x + u.y * v.y); }

inline realno operator^(Point2 u, Point2 v)
{return (u.x * v.y - u.y * v.x); }

inline int operator==(Point2 p1, Point2 p2)
{ return ((p1.x == p2.x) && (p1.y == p2.y)); }

inline int operator!=(Point2 p1, Point2 p2)
{ return ((p1.x != p2.x) || (p1.y != p2.y)); }

inline ostream& operator<<(ostream& out_strm, Point2& pnt)
{ out_strm << "(" << pnt.x << "," << pnt.y << ")"; 
  return (out_strm); }

inline istream& operator>>(istream& in_strm, Point2& pnt)
{ 
  char dummy;
  in_strm >> dummy >> pnt.x >> dummy >> pnt.y >> dummy;
  return (in_strm);
}


#endif
