/*
This source file is the copyright property of the University of Leeds
('The University').

Permission to use, copy, modify, and distribute this source file for
educational, research, and not-for-profit purposes, without fee and
without a signed licensing agreement, is hereby granted, provided that
the above copyright notice, this paragraph and the following three
paragraphs appear in all copies, modifications, and distributions.

In no event shall The University be liable to any party for direct,
indirect, special, incidental or consequential damages, including lost
profits, arising out of the use of this software and its documentation.

The software is provided without warranty. The University has no
obligation to provide maintenance, support, updates, enhancements, or
modifications.

This software was written by Adam Baumberg, Vision Group,
School of Computer Studies, University of Leeds, UK. The code and use
thereof should be attributed to the author where appropriate
(including demonstrations which rely on it's use).
*/

// testdrive matrices, vectors

#include "NagMatrix.h"
#include <../imgsrc/realno.h>

#include <strstream.h>
realno test_fn(int i)
{
  return (i);
}

static void find_ip_to_gp(NagMatrix& x_ip, NagMatrix& x_gp, NagMatrix& res)
{
  NagVector tmp_v;
  NagMatrix tmp_inv;

  NagMatrix m1;
  x_ip.get_block(0,2,0,2, m1);
  NagVector x4;
  x_ip.get_column(3, x4);
  m1.invert(tmp_inv);
  tmp_inv.multiply(x4, tmp_v);
  m1.column(0) *= (tmp_v[0]);
  m1.column(1) *= (tmp_v[1]);
  m1.column(2) *= (tmp_v[2]);
  
  NagMatrix m2;
  x_gp.get_block(0,2,0,2, m2);
  x_gp.get_column(3, x4);
  m2.invert(tmp_inv);
  tmp_inv.multiply(x4, tmp_v);
  m2.column(0) *= (tmp_v[0]);
  m2.column(1) *= (tmp_v[1]);
  m2.column(2) *= (tmp_v[2]);
  
  m1.invert(tmp_inv);
  m2.multiply(tmp_inv, res);
}
  

main(int argc, char** argv)
{

  realno reading_points_data[] = { 385,413, 1,
				   191, 372, 1,
				   535, 377, 1,
				   339, 339, 1};

  realno world_points_data[] = { -737.62051, -1966.03289,1,
				 -47.2213, 2183.79352,1,
				 2472.37189, -2807.04155,1,
				 3244.53764, 1448.36952, 1 };

  realno leeds_to_rdg_data[] = { 2.165644, 0.0, 0.0,
				 0.0, -2.179263, 0.0,
				 -1.869625, 600.258972, 1.0 };
  
  // premultiply by Leeds image to Reading Image
  NagMatrix LtoR(3,3,leeds_to_rdg_data);
  NagMatrix RtoL;
  LtoR.invert(RtoL);

  NagMatrix reading_points(3,4, reading_points_data);
  NagMatrix world_points(3,4, world_points_data);

  
  NagMatrix world_to_reading;
  find_ip_to_gp(world_points, reading_points, world_to_reading);
  cout << "World to Reading " << endl;
  world_to_reading.scale(1.0 / world_to_reading.read(2,2), world_to_reading);
  world_to_reading.output();

  NagMatrix world_to_leeds;
  RtoL.multiply(world_to_reading, world_to_leeds);

  NagMatrix reading_to_world;
  find_ip_to_gp(reading_points, world_points, reading_to_world);
  reading_to_world.scale(1.0 / reading_to_world.read(2,2), reading_to_world);
  cout << "Reading to World " << endl;
  reading_to_world.output();

  
  NagVector test_point(3); test_point[0] = 30311.3;
  test_point[1] = 21160.7; test_point[2] = 1.0;
  NagVector res1;
  NagVector res2;
  world_to_leeds.multiply(test_point, res1);
  world_to_reading.multiply(test_point, res2);

  cout << "test world point " << endl;
  test_point.output();
  cout << "Leeds coords " << endl;
  res1.scale(1.0 / res1[2], res1);
  res2.scale(1.0 / res2[2], res2);
  res1.output();
  cout << "Reading coords " << endl;
  res2.output();
}


