/***************************************************************************
 * C source
 *
 * File : 	random.c
 *
 * Module :	random
 *
 * Author : 	A. Baumberg
 *
 * Creation Date : Wed Sep 25 11:17:15 1996 
 *
 * Comments : 	random access for mpeg streams
 *
 ***************************************************************************/


/* include files */
#include <config.h>
#include <stdio.h>
#include <stdlib.h>
#include <assert.h>

#ifndef MIPS
#include <sys/time.h>
#else
#include <sys/types.h>
#include <sys/system.h>
#endif

#include "globals.h"
#include "decoders.h"
#include "video.h"
#include "util.h"
#include "proto.h"
#include "random.h"

/* global functions */
/* local functions */

static TableEntry old_node;
static TableEntry backup_node;

static void backup_globals()
{
  backup_node.streamPos = ftell(input) - (4 * bufLength);
  backup_node.bitOffset = bitOffset;
  backup_node.frame_no = 0;
}

static void push_backup_globals()
{
  old_node.streamPos = backup_node.streamPos;
  old_node.bitOffset = backup_node.bitOffset;
  old_node.frame_no = 0;
}

static void pull_globals()
{
  long curStreamPos, startPos, bufOffset;
  curStreamPos = ftell(input);
  startPos = curStreamPos - (4 * curVidStream->buf_length);
  if ((old_node.streamPos > startPos) &&
      (old_node.streamPos < curStreamPos))
    {
      /* current buffer contains access point */
      bitOffset = old_node.bitOffset;
      bufOffset = (old_node.streamPos - startPos) / 4;
      bitBuffer += bufOffset;
      bufLength -= bufOffset;
    }
  else
    {
      /* need to refill bitstream buffer */
      EOF_flag = FALSE;
      fseek(input, old_node.streamPos, SEEK_SET);
      
      bufLength = 0;
      bitBuffer = curVidStream->buf_start;
      get_more_data(bitBuffer, curVidStream->max_buf_length, 
		    &bufLength, &bitBuffer);
      bitOffset = old_node.bitOffset;
    }
#ifdef UTIL2
  curBits = *bitBuffer << bitOffset;
#else
  curBits = *bitBuffer;
#endif
}
      


static VidStream *
count_frames(no_frames, no_GOPs, vid_stream)
  int *no_frames, *no_GOPs;
  VidStream *vid_stream;
{
  unsigned int data;
  int i;
  Boolean MovieDone = FALSE;

  /* If vid_stream is null, create new VidStream structure. */

  if (vid_stream == NULL) {
    return NULL;
  }
  
  curVidStream = vid_stream;
  bitOffset = curVidStream->bit_offset;
#ifdef UTIL2
  curBits = *curVidStream->buffer << bitOffset;
#else
  curBits = *curVidStream->buffer;
#endif
  bufLength = curVidStream->buf_length;
  bitBuffer = curVidStream->buffer;

  backup_globals(); push_backup_globals();

  *no_GOPs = 0;
  *no_frames = 0;
  next_start_code();
  while (!MovieDone)
    {
      backup_globals();
      show_bits32(data);
      switch (data) {
	
      case SEQ_END_CODE:
	MovieDone = TRUE;
	break;

      case SEQ_START_CODE:
	
	/* Sequence start code. Parse sequence header. */
	flush_bits32;
	dprintf ("SEQ_START_CODE\n");	
	next_start_code();
	break;

      case GOP_START_CODE:

	dprintf ("GOP_START_CODE\n");
	flush_bits32;
	next_start_code();
	(*no_GOPs)++;
	break;
 
     case PICTURE_START_CODE:
	flush_bits32;
	next_start_code();
	(*no_frames)++;
	break;
	
      default:
	flush_bits(24);
	next_start_code();
	break;
      }
    }
  /* Return pointer to video stream structure. */
  
  pull_globals();
  vid_stream->buffer = bitBuffer;
  vid_stream->buf_length = bufLength;
  vid_stream->bit_offset = bitOffset;
/*   vid_stream->current = vid_stream->past = vid_stream->future = NULL; */
/*   for (i = 0; i < RING_BUF_SIZE; i++) */
/*     vid_stream->ring[i]->locked = 0;   */

  return vid_stream;

}


static TableEntry*
build_table(no_GOPs, vid_stream)
  int no_GOPs;
  VidStream *vid_stream;
{
  unsigned int data;
  int i;
  Boolean MovieDone = FALSE;
  TableEntry *the_table;
  int curr_frame, curr_GOP;

  curr_frame = curr_GOP = 0;
  the_table = (TableEntry*) malloc((no_GOPs+1)*sizeof(TableEntry));

  curVidStream = vid_stream;
  bitOffset = curVidStream->bit_offset;
#ifdef UTIL2
  curBits = *curVidStream->buffer << bitOffset;
#else
  curBits = *curVidStream->buffer;
#endif
  bufLength = curVidStream->buf_length;
  bitBuffer = curVidStream->buffer;

  backup_globals(); push_backup_globals();

  next_start_code();
  while (!MovieDone)
    {
      backup_globals();
      show_bits32(data);
      switch (data) {
	
      case SEQ_END_CODE:
	MovieDone = TRUE;
	break;

      case SEQ_START_CODE:
	
	/* Sequence start code. Parse sequence header. */
	flush_bits32;
	dprintf ("SEQ_START_CODE\n");
	next_start_code();
	break;

      case GOP_START_CODE:

	dprintf ("GOP_START_CODE\n");
	flush_bits32;
	next_start_code();
	the_table[curr_GOP] = backup_node;
	the_table[curr_GOP++].frame_no = curr_frame;
	break;
	
      case PICTURE_START_CODE:
	flush_bits32;
	next_start_code();
	curr_frame++;
	break;
	
      default:
	flush_bits(24);
	next_start_code();
	break;
      }
    }
  /* flag end of table */
  the_table[curr_GOP].frame_no = -1;
  the_table[curr_GOP].bitOffset = 0;
  the_table[curr_GOP].streamPos = 0;

  pull_globals();
  vid_stream->buffer = bitBuffer;
  vid_stream->buf_length = bufLength;
  vid_stream->bit_offset = bitOffset;
/*   vid_stream->current = vid_stream->past = vid_stream->future = NULL; */
/*   for (i = 0; i < RING_BUF_SIZE; i++) */
/*     vid_stream->ring[i]->locked = 0;   */

  return the_table;

}


VidHeader* 
build_header(vid_header, vid_stream)
     VidHeader* vid_header;
     VidStream* vid_stream;
{
  int no_frames, no_gops;
  
  printf("building MPEG header - please be patient \n");

  if (vid_header == NULL)
    vid_header = (VidHeader*) malloc(sizeof(VidHeader));
  
  count_frames(&no_frames, &no_gops, vid_stream);
  vid_header->table = build_table(no_gops, vid_stream);
  vid_header->no_GOPs = no_gops;
  vid_header->no_frames = no_frames;

  return (vid_header);
}


Boolean
save_header(vid_header, hdr_strm)
     VidHeader *vid_header;
     FILE *hdr_strm;
{
  int i;
  
  i = vid_header->no_GOPs + 1;
  if (fwrite(vid_header, sizeof(VidHeader), 1, hdr_strm) != 1)
    return (FALSE);
  if (fwrite(vid_header->table, sizeof(TableEntry) * i,
	     1, hdr_strm) != 1)
    return (FALSE);
  
  return (TRUE);
}

VidHeader*
load_header(vid_header, hdr_strm)
     VidHeader *vid_header;
     FILE *hdr_strm;
{
  int i;


  if (vid_header == NULL)
    vid_header = (VidHeader*) malloc(sizeof(VidHeader));
  if (fread(vid_header, sizeof(VidHeader), 1, hdr_strm) != 1)
    {
      fprintf(stderr,"Error loading header \n");
      exit(1);
    }
  i = vid_header->no_GOPs + 1;
  vid_header->table = (TableEntry*)  malloc(sizeof(TableEntry) * i);
  if (fread(vid_header->table, sizeof(TableEntry) * i,
	    1, hdr_strm) != 1)
    {
      fprintf(stderr,"Error loading header \n");
      exit(1);
    }
  return (vid_header);
}


int 
find_table_entry(vid_header, frame)
     VidHeader* vid_header;
     int frame;
{
  int a,b, c;
  TableEntry *theTable;
  theTable = vid_header->table;
  a = 0; b = vid_header->no_GOPs - 1;
  if (theTable[a].frame_no >= frame) return (a);
  if (theTable[b].frame_no <= frame) return (b);
  while ((b-a) > 1)
    {
      c = (a + b) / 2;
      if (theTable[c].frame_no == frame) return (c);
      if (theTable[c].frame_no > frame) b = c;
      else a = c;
    }
  /* return closest I_frame < frame */
  return (a);
}

Boolean
jump_to_entry(vid_header, vid_stream,entry)
     VidHeader* vid_header;
     VidStream* vid_stream;
     int entry;
{
  int i;
  if (entry >= vid_header->no_GOPs) return (FALSE);
  old_node = vid_header->table[entry];
  curVidStream = vid_stream;
  bitOffset = curVidStream->bit_offset;
#ifdef UTIL2
  curBits = *curVidStream->buffer << bitOffset;
#else
  curBits = *curVidStream->buffer;
#endif
  bufLength = curVidStream->buf_length;
  bitBuffer = curVidStream->buffer;
  pull_globals();  
  vid_stream->buffer = bitBuffer;
  vid_stream->buf_length = bufLength;
  vid_stream->bit_offset = bitOffset;
  vid_stream->current = vid_stream->past = vid_stream->future = NULL;
  for (i = 0; i < RING_BUF_SIZE; i++)
    vid_stream->ring[i]->locked = 0;  
  return (TRUE);
}
