/* Copyright (c) 1994 The Geometry Center; University of Minnesota
   1300 South Second Street;  Minneapolis, MN  55454, USA;

This file is free software; you can redistribute it and/or modify it only
under the the terms of the GNU GENERAL PUBLIC LICENSE which should be
included along with this file.  This software may be obtained via anonymous
ftp from geom.umn.edu; email: software@geom.umn.edu. */

/* Author: Daeron Meyer */

#include "mibload.h"
#include "interface/rewind.xbm"
#include "interface/play.xbm"
#include "interface/step.xbm"
#include "interface/stop.xbm"
#include "interface/loop.xbm"
#include "interface/ff.xbm"
#include "interface/fback.xbm"
#include "interface/playback.xbm"
#include "interface/stepback.xbm"
#include "interface/fwind.xbm"
#include "interface/MainWindow.mib"
#include "interface/Info.mib"
#include <setjmp.h>
#include <info.h>

/* Declarataion of global display pointer. */
extern Display *display;
extern jmp_buf env;

int loopFlag = 0;
extern int argc_share;
extern char **argv_share;
extern char infomessage[];
extern Colormap cmap;
extern int curr_frame;

extern char* image_data;

/*****************************************************************************

/* Private Variables and Methods */

/* Intrinsics Application Context */
static  XtAppContext		App;
static  Window			monitorwindow = NULL;
static  mib_Widget		*mainload, *infoload;
static  Widget			shell = NULL, infoshell = NULL;
static  Widget			frametext = NULL;
static  Widget			framebar = NULL;
static  int			frameshown = 0;
static  XtWorkProcId dovid      = NULL;
static  int rewound		= 0;
static  int at_end		= 0;
static  Window displaywin       = NULL;
static  int minx		= 400;
static  int miny		= 300;
static  int winx		= 0;
static  int winy		= 0;
static  int sizelock		= 0;
static  int framelock		= 0;
static  int privcolormap	= 0;
static  int currentframe	= 0;
static int drag_mode = 0;
static int play_speed = 1;
static int play_forward_slow = 1;
static int play_backward_slow = -1;
static int play_forward_fast = 2;
static int play_backward_fast = -2;



static void exit_callback(Widget, XtPointer, XmAnyCallbackStruct *);
static void stop_callback(Widget, XtPointer, XmAnyCallbackStruct *);
static void step_callback(Widget, XtPointer, XmAnyCallbackStruct *);
static void stepback_callback(Widget, XtPointer, XmAnyCallbackStruct *);
static void play_callback(Widget, XtPointer, XmPushButtonCallbackStruct *);
static void loop_callback(Widget, XtPointer, XmAnyCallbackStruct *);
static void rewind_callback(Widget, XtPointer, XmAnyCallbackStruct *);
static void fwind_callback(Widget, XtPointer, XmAnyCallbackStruct *);
static void info_callback(Widget, XtPointer, XmAnyCallbackStruct *);
static void close_callback(Widget, XtPointer, XmAnyCallbackStruct *);
static void monitor_expose(Widget, XtPointer, XmDrawingAreaCallbackStruct *);
static void change_frame_no(Widget, XtPointer, XmAnyCallbackStruct *);
static void drag_callback(Widget, XtPointer, XmAnyCallbackStruct *);
static Boolean do_video(XtPointer);

/*****************************************************************************

  UICreate: Create/load in a motif interface.

 *****************************************************************************/

void UICreate()
{
  Widget		toplevel,
			mainwindow,
			temp;

  Pixel			fg, bg;
  Pixmap		button_pix;

  Arg			args[20];
  int			n, depth;
  Dimension		tmp_dimension[4];

  static String		fallbacks[] = {
  "*Foreground:  gray20",
  "*BorderWidth: 0",
  "*Background:  gray70",
  "*XmToggleButton.selectColor:         yellow",
  "*XmToggleButton.indicatorSize:       16",
  "*XmToggleButtonGadget.selectColor:   yellow",
  "*XmToggleButtonGadget.indicatorSize: 16",
  "*fontList: -adobe-helvetica-medium-r-normal--14-*-*-*-p-76-iso8859-1",
  "*XmText*fontList: -adobe-courier-medium-r-normal--12-*-*-*-m-70-iso8859-1",
  NULL
  };


/*****************************************************************************/

  n = 0;

  toplevel = XtAppInitialize(&App, "mpeg_play", NULL, 0, &argc_share,
				argv_share, fallbacks, args, n);

  display = XtDisplay(toplevel);

/*****************************************************************************/

  n = 0;
  XtSetArg(args[n], XmNtitle, "mpeg player"); n++;
  XtSetArg(args[n], XmNdeleteResponse, XmDO_NOTHING);n++;
  XtSetArg(args[n], XmNkeyboardFocusPolicy, XmPOINTER); n++;
  XtSetArg(args[n], XmNwidth, minx); n++;
  XtSetArg(args[n], XmNminWidth, minx); n++;
  XtSetArg(args[n], XmNheight, miny); n++;
  XtSetArg(args[n], XmNminHeight, miny); n++;

  shell = XtAppCreateShell("mpeg player", "mpeg_play",
				topLevelShellWidgetClass, display, args, n);

/*****************************************************************************/

  n = 0;
  XtSetArg(args[n], XmNtitle, "mpeg player"); n++;

  mainwindow = XmCreateForm(shell, "mpeg player", args, n);
  mainload = mib_load_interface(mainwindow, MainMib,
				MI_FROMSTRING);

  XtManageChild(mainwindow);
  XtSetMappedWhenManaged(shell, False);
  XtRealizeWidget(shell);

  frametext = mib_find_name(mainload, "FrameText")->me;
/*   XtVaSetValues(frametext, XmNcursorPositionVisible, False, */
/* 			   XmNeditable, False, NULL); */
  XtAddCallback(frametext, XmNactivateCallback,
		(XtCallbackProc) change_frame_no, (XtPointer) NULL);
  XtAddCallback(frametext, XmNmotionVerifyCallback,
		(XtCallbackProc) stop_callback, (XtPointer) NULL);

  temp = mib_find_name(mainload, "Monitor")->me;
  XtAddCallback(temp, XmNexposeCallback,
			(XtCallbackProc) monitor_expose, (XtPointer) NULL);
  XtVaSetValues(temp,   XmNforeground,
			BlackPixel(display, DefaultScreen(display)),
			XmNbackground,
			BlackPixel(display, DefaultScreen(display)),
                        NULL);
  monitorwindow = XtWindow(temp);

  temp = mib_find_name(mainload, "Stop")->me;
  XtVaGetValues(temp, XmNforeground, &fg,
                        XmNbackground, &bg, XmNdepth, &depth, NULL);
  button_pix = XCreatePixmapFromBitmapData(display, DefaultRootWindow(display),
                stop_bits, stop_width, stop_height, fg, bg, depth);
  XtVaSetValues(temp, XmNlabelType, XmPIXMAP,
                        XmNlabelPixmap, button_pix, NULL);
  XtAddCallback(temp, XmNactivateCallback, (XtCallbackProc) stop_callback,
			(XtPointer) NULL);

  temp = mib_find_name(mainload, "Step")->me;
  XtVaGetValues(temp, XmNforeground, &fg,
                        XmNbackground, &bg, XmNdepth, &depth, NULL);
  button_pix = XCreatePixmapFromBitmapData(display, DefaultRootWindow(display),
                step_bits, step_width, step_height, fg, bg, depth);
  XtVaSetValues(temp, XmNlabelType, XmPIXMAP,
                        XmNlabelPixmap, button_pix, NULL);
  XtAddCallback(temp, XmNactivateCallback, (XtCallbackProc) step_callback,
			(XtPointer) NULL);

  temp = mib_find_name(mainload, "Play")->me;
  XtVaGetValues(temp, XmNforeground, &fg,
                        XmNbackground, &bg, XmNdepth, &depth, NULL);
  button_pix = XCreatePixmapFromBitmapData(display, DefaultRootWindow(display),
                play_bits, play_width, play_height, fg, bg, depth);
  XtVaSetValues(temp, XmNlabelType, XmPIXMAP,
                        XmNlabelPixmap, button_pix, NULL);
  XtAddCallback(temp, XmNactivateCallback, (XtCallbackProc) play_callback,
			(XtPointer) &play_forward_slow);

  XtVaGetValues(temp, XmNshadowThickness, &tmp_dimension[0],
		XmNmarginLeft, &tmp_dimension[1],
		XmNmarginRight, &tmp_dimension[2],
		XmNmarginWidth, &tmp_dimension[3],NULL);

  temp = mib_find_name(mainload, "Loop")->me;
  XtVaGetValues(temp, XmNforeground, &fg,
                        XmNbackground, &bg, XmNdepth, &depth, NULL);
  button_pix = XCreatePixmapFromBitmapData(display, DefaultRootWindow(display),
                loop_bits, loop_width, loop_height, fg, bg, depth);
  XtVaSetValues(temp, XmNlabelType, XmPIXMAP,
		XmNlabelPixmap, button_pix, 
		XmNindicatorOn, False, XmNset, False, 
		XmNshadowThickness, tmp_dimension[0],
		XmNmarginTop, 0,
		XmNmarginBottom, 0,
		XmNmarginLeft, tmp_dimension[1],
		XmNmarginRight, tmp_dimension[2],
		XmNmarginHeight, 0,
		XmNmarginWidth, tmp_dimension[3],NULL);

  XtAddCallback(temp, XmNvalueChangedCallback, (XtCallbackProc) loop_callback,
			(XtPointer) NULL);

  temp = mib_find_name(mainload, "Fastforward")->me;
  XtVaGetValues(temp, XmNforeground, &fg,
                        XmNbackground, &bg, XmNdepth, &depth, NULL);
  button_pix = XCreatePixmapFromBitmapData(display, DefaultRootWindow(display),
                ff_bits, ff_width, ff_height, fg, bg, depth);
  XtVaSetValues(temp, XmNlabelType, XmPIXMAP,
                        XmNlabelPixmap, button_pix, NULL);
  XtAddCallback(temp, XmNactivateCallback, (XtCallbackProc) play_callback,
			(XtPointer) &play_forward_fast);

  temp = mib_find_name(mainload, "Stepback")->me;
  XtVaGetValues(temp, XmNforeground, &fg,
                        XmNbackground, &bg, XmNdepth, &depth, NULL);
  button_pix = XCreatePixmapFromBitmapData(display, DefaultRootWindow(display),
                stepback_bits, stepback_width, stepback_height, fg, bg, depth);
  XtVaSetValues(temp, XmNlabelType, XmPIXMAP,
                        XmNlabelPixmap, button_pix, NULL);
  XtAddCallback(temp, XmNactivateCallback, (XtCallbackProc) stepback_callback,
			(XtPointer) NULL);

  temp = mib_find_name(mainload, "Fwind")->me;
  XtVaGetValues(temp, XmNforeground, &fg,
                        XmNbackground, &bg, XmNdepth, &depth, NULL);
  button_pix = XCreatePixmapFromBitmapData(display, DefaultRootWindow(display),
                fwind_bits, fwind_width, fwind_height, fg, bg, depth);
  XtVaSetValues(temp, XmNlabelType, XmPIXMAP,
		XmNlabelPixmap, button_pix, NULL);
  XtAddCallback(temp, XmNactivateCallback, (XtCallbackProc) fwind_callback,
			(XtPointer) NULL);

  temp = mib_find_name(mainload, "Fastback")->me;
  XtVaGetValues(temp, XmNforeground, &fg,
                        XmNbackground, &bg, XmNdepth, &depth, NULL);
  button_pix = XCreatePixmapFromBitmapData(display, DefaultRootWindow(display),
                fback_bits, fback_width, fback_height, fg, bg, depth);
  XtVaSetValues(temp, XmNlabelType, XmPIXMAP,
                        XmNlabelPixmap, button_pix, NULL);
  XtAddCallback(temp, XmNactivateCallback, (XtCallbackProc) play_callback,
			(XtPointer) &play_backward_fast);

  temp = mib_find_name(mainload, "Playback")->me;
  XtVaGetValues(temp, XmNforeground, &fg,
                        XmNbackground, &bg, XmNdepth, &depth, NULL);
  button_pix = XCreatePixmapFromBitmapData(display, DefaultRootWindow(display),
                playback_bits, playback_width, playback_height, fg, bg, depth);
  XtVaSetValues(temp, XmNlabelType, XmPIXMAP,
                        XmNlabelPixmap, button_pix, NULL);
  XtAddCallback(temp, XmNactivateCallback, (XtCallbackProc) play_callback,
			(XtPointer) &play_backward_slow);

  temp = mib_find_name(mainload, "Info")->me;
  XtAddCallback(temp, XmNactivateCallback, (XtCallbackProc) info_callback,
			(XtPointer) NULL);

  temp = mib_find_name(mainload, "Rewind")->me;
  XtVaGetValues(temp, XmNforeground, &fg,
                        XmNbackground, &bg, XmNdepth, &depth, NULL);
  button_pix = XCreatePixmapFromBitmapData(display, DefaultRootWindow(display),
                rewind_bits, rewind_width, rewind_height, fg, bg, depth);
  XtVaSetValues(temp, XmNlabelType, XmPIXMAP,
                        XmNlabelPixmap, button_pix, NULL);
  XtAddCallback(temp, XmNactivateCallback, (XtCallbackProc) rewind_callback,
			(XtPointer) NULL);

  framebar = mib_find_name(mainload, "FrameBar")->me;
  XtAddCallback(framebar, XmNdragCallback, (XtCallbackProc) drag_callback,
		(XtPointer) NULL);
  XtAddCallback(framebar, XmNvalueChangedCallback, 
		(XtCallbackProc) change_frame_no,
		(XtPointer) NULL);
  XtVaSetValues(framebar, XmNmaximum, RAget_no_frames(), XmNscaleMultiple,
		RAget_no_frames() / 10, NULL);

  temp = mib_find_name(mainload, "Exit")->me;
  XtAddCallback(temp, XmNactivateCallback, (XtCallbackProc) exit_callback,
			(XtPointer) NULL);

/*****************************************************************************/

  n = 0;
  XtSetArg(args[n], XmNtitle, "Info"); n++;
  XtSetArg(args[n], XmNdeleteResponse, XmDO_NOTHING);n++;
  XtSetArg(args[n], XmNkeyboardFocusPolicy, XmPOINTER); n++;
  XtSetArg(args[n], XmNwidth, minx+200); n++;
  XtSetArg(args[n], XmNminWidth, minx); n++;
  XtSetArg(args[n], XmNheight, miny+100); n++;
  XtSetArg(args[n], XmNminHeight, miny); n++;

  infoshell = XtAppCreateShell("Info", "mpeg_play",
				topLevelShellWidgetClass, display, args, n);

/*****************************************************************************/

  n = 0;
  XtSetArg(args[n], XmNtitle, "Info"); n++;

  temp  = XmCreateForm(infoshell, "Info", args, n);
  infoload = mib_load_interface(temp, InfoMib,
                                MI_FROMSTRING);
  XtManageChild(temp);

  temp = mib_find_name(infoload, "Close")->me;
  XtAddCallback(temp, XmNactivateCallback, (XtCallbackProc) close_callback,
			(XtPointer) NULL);

  temp = mib_find_name(infoload, "TextBig")->me;
  XtVaSetValues(temp, XmNcursorPositionVisible, False,
			   XmNeditable, False, NULL);
  XmTextSetString(temp, infomessage);

}

/*****************************************************************************/

void UIMainLoop()
{
  int frame;

  if (loopFlag && !rewound)
    dovid = XtAppAddWorkProc(App, do_video, NULL);

  at_end = 0;

  if ((curr_frame == 0) && (!at_end))
    {
      if (GetMPEGFrame(image_data)) curr_frame++;      
      else at_end = TRUE;
      ShowXImage();
    }
  
  frameshown = 1;
  rewound = 0;
  at_end = 0;

  XtAppMainLoop(App);
}

/*****************************************************************************/

Window MonitorWindow()
{
  return monitorwindow;
}

/*****************************************************************************/

void UISetColormap()
{
  privcolormap = 1;
}

/*****************************************************************************/

void UISetwin(Window win)
{
  displaywin = win;
}

/*****************************************************************************/

void UISetFrame()
{
  char framestr[100];

  sprintf(framestr, "%d", curr_frame);
  XmTextFieldSetString(frametext, framestr);
  if (!drag_mode) XmScaleSetValue(framebar, curr_frame);
}

/*****************************************************************************/

void UIMinsize(int x, int y)
{
  winx = x;
  winy = y;

  if (sizelock)
    return;

  x += 34;
  y += 97;

  if (x < minx)
    x = minx;
  if (y < miny)
    y = miny;

  minx = x; miny = y;

  if (privcolormap)
  {
    XtVaSetValues(shell, XmNcolormap, cmap, NULL);
  }
  if (shell)
  {
    XtResizeWidget(shell, (Dimension)minx, (Dimension)miny, (Dimension)0);
    XtVaSetValues(shell, XmNminWidth, 400,
			 XmNmaxWidth, minx,
			 XmNminHeight, 300,
			 XmNmaxHeight, miny, NULL);

    XtMapWidget(shell); /* Map the shell now to avoid disappearing buttons! */
    sizelock = 1;
    if (displaywin)
    {
      x = winx; y = winy;
      winx = minx - 34; winy = miny - 97;
      XMoveWindow(display, displaywin, (winx - x)/2, (winy - y)/2);
    }
  }
}

/*****************************************************************************/

void UIEnd()
{

  if (dovid)
    XtRemoveWorkProc(dovid);

  XmTextFieldSetString(frametext, "end");
  dovid = NULL;
  at_end = 1;

  if (loopFlag)
  {
    at_end = 0;
    rewound = 0;
    longjmp(env, 1);
  }
}

/*****************************************************************************

  From here on we have private methods:

 *****************************************************************************/

static void exit_callback(Widget w, XtPointer data, XmAnyCallbackStruct *cbs)
{
  exit(0);
}

/*****************************************************************************/

static void stop_callback(Widget w, XtPointer data, XmAnyCallbackStruct *cbs)
{

  if (dovid)
    XtRemoveWorkProc(dovid);
  dovid = NULL;

}

/*****************************************************************************/

static void play_callback(Widget w, XtPointer data, 
			  XmPushButtonCallbackStruct *cbs)
{
  int *new_speed;
  new_speed = (int *) data;
  play_speed = *new_speed * cbs->click_count;
  if (!dovid)
    dovid = XtAppAddWorkProc(App, do_video, NULL);

}

/*****************************************************************************/

static void step_callback(Widget w, XtPointer data, XmAnyCallbackStruct *cbs)
{
  if (dovid)
    XtRemoveWorkProc(dovid);

  dovid = NULL;

  drag_mode = FALSE;
  play_speed = 1;
  do_video(NULL);
}

static void stepback_callback(Widget w, XtPointer data, 
			      XmAnyCallbackStruct *cbs)
{
  if (dovid)
    XtRemoveWorkProc(dovid);

  dovid = NULL;

  drag_mode = FALSE;
  play_speed = -1;
  do_video(NULL);
}

/*****************************************************************************/

static void loop_callback(Widget w, XtPointer data, XmAnyCallbackStruct *cbs)
{
/*   if (!dovid) */
/*     dovid = XtAppAddWorkProc(App, do_video, NULL); */

  loopFlag = !loopFlag;
}

/*****************************************************************************/

static void info_callback(Widget w, XtPointer data, XmAnyCallbackStruct *cbs)
{
  XtRealizeWidget(infoshell);
}

static void close_callback(Widget w, XtPointer data, XmAnyCallbackStruct *cbs)
{
  XtUnrealizeWidget(infoshell);
}

/*****************************************************************************/

static void rewind_callback(Widget w, XtPointer data, XmAnyCallbackStruct *cbs)
{
  if (dovid) XtRemoveWorkProc(dovid);
  dovid = NULL;
  while (framelock);
  framelock = 1;
  curr_frame = RAgoto_ref_frame(0);
  framelock = 0;
  play_speed = 1;
  drag_mode = FALSE;
  at_end = FALSE;
  do_video(NULL);
}

static void fwind_callback(Widget w, XtPointer data, XmAnyCallbackStruct *cbs)
{
  if (dovid) XtRemoveWorkProc(dovid);
  dovid = NULL;
  while (framelock);
  play_speed = 1;
  drag_mode = FALSE;
  curr_frame = RAget_no_frames() - 1;
  curr_frame = RAgoto_ref_frame(curr_frame);
  while (GetMPEGFrame(image_data)) curr_frame++;
  at_end = TRUE;
  ShowXImage();
  /* do_video(NULL);*/
}


/*****************************************************************************/

static Boolean do_video( XtPointer data)
{
  int desired_frame, old_frame, i;
  if (!framelock)
    {
      framelock = 1;
      if (drag_mode)
	{
	  XmScaleGetValue(framebar, &desired_frame);
	  if (desired_frame != (curr_frame))
	    {
	      curr_frame = RAgoto_ref_frame(desired_frame-1);
	      if(GetMPEGFrame(image_data))
		{ curr_frame++; at_end = FALSE; }
	      else at_end = TRUE;
	      ShowXImage();
	    }
	}
      else
	{
	  if (abs(play_speed) == 1)
	    {
	      desired_frame = curr_frame + play_speed - 1;
	      if (desired_frame < 0) desired_frame = 0;
	      if (play_speed < 0) at_end = FALSE;
	      if (at_end || (desired_frame >= RAget_no_frames()))
		{
		  if (loopFlag) { desired_frame = 0; at_end = FALSE; }
		  else desired_frame = RAget_no_frames() - 1;
		}
	      if (curr_frame != desired_frame)
		{
		  RAgoto_frame(desired_frame);
		  curr_frame = desired_frame;
		}
	    }	  
	  else 
	    {
	      for (i = 0; i < abs(play_speed); i += play_forward_fast)
		{	
		  old_frame = curr_frame;
		  if (play_speed > 0)
		    {
		      curr_frame = RAgoto_next_ref_frame(curr_frame);
		      if ((old_frame == curr_frame) && (loopFlag))
			curr_frame = RAgoto_ref_frame(0);
		    }
		  else
		      curr_frame = RAgoto_ref_frame(curr_frame-2);
		}
	    }
	  if(GetMPEGFrame(image_data))
	    curr_frame++;
	  else at_end = TRUE;
	  ShowXImage();
	}
      framelock = 0;
    }
  return False;
}

/*****************************************************************************/

static void monitor_expose(Widget w, XtPointer data,
				XmDrawingAreaCallbackStruct *cbs)
{
  Window		t_root;
  int			toss;
  unsigned int		toss2, rx, ry;
  int			nx, ny;
  
  framelock = 1;
  if (!dovid && frameshown)
  {
    ShowXImage();
  }
  framelock = 0;
}

/*****************************************************************************/

/*****************************************************************************/

static void change_frame_no(Widget w, XtPointer data,
			    XmAnyCallbackStruct *cbs)
{
  char *text_field;
  int desired_frame;
  int use_exact_frame = 1;

  if (dovid) 
    {
      XtRemoveWorkProc(dovid);
      dovid = NULL;
      while (framelock);
    }

  if (w == frametext)
    {
      text_field = XmTextFieldGetString(w); 
      desired_frame = atoi(text_field);
      XtFree (text_field);
    }
  else
    {
      XmScaleGetValue(framebar, &desired_frame);
      if (!drag_mode) use_exact_frame = 0;
      drag_mode = 0;
    }
  
  if (!framelock && (desired_frame != curr_frame))
    {
      framelock = 1;
      if (!use_exact_frame)
	curr_frame = RAgoto_ref_frame(desired_frame-1);
      else 
	{
	  if (RAgoto_frame(desired_frame-1)) 
	    curr_frame = desired_frame-1;
	  else curr_frame--;
	}
      if (!GetMPEGFrame(image_data)) at_end = TRUE; 
      else { curr_frame++; ShowXImage();  at_end = FALSE;}
/*       XmScaleSetValue(framebar, curr_frame); */
      framelock = 0;
    }
}

/*****************************************************************************/

static void drag_callback(Widget w, XtPointer data,
			  XmAnyCallbackStruct *cbs)
{
  drag_mode = TRUE;
  if (!dovid)
    dovid = XtAppAddWorkProc(App, do_video, NULL);
}

