/*
This source file is the copyright property of the University of Leeds
('The University').

Permission to use, copy, modify, and distribute this source file for
educational, research, and not-for-profit purposes, without fee and
without a signed licensing agreement, is hereby granted, provided that
the above copyright notice, this paragraph and the following three
paragraphs appear in all copies, modifications, and distributions.

In no event shall The University be liable to any party for direct,
indirect, special, incidental or consequential damages, including lost
profits, arising out of the use of this software and its documentation.

The software is provided without warranty. The University has no
obligation to provide maintenance, support, updates, enhancements, or
modifications.

This software was written by Adam Baumberg, Vision Group,
School of Computer Studies, University of Leeds, UK. The code and use
thereof should be attributed to the author where appropriate
(including demonstrations which rely on it's use).
*/

/* test program */

#include <image.h>
#include <imgsrc.h>
#include <stdio.h>
#include <getopt.h>
#include <ProfileSet.h>
#include <sgmoviestore.h>
#include <RGB8Image.h>
#include <utils.h>
#include <imgsrc_utils.h>
#include <GroundPeopleTracker.h>


#define USAGE "quick_ground_track -i options [-p file] [-q] \n\
-i input image options (-i help for details) \n\
-p parameter file \n\
-q quiet mode -- display no images \n"

 

int MAX_OBJECTS = 
AmbGlobals.register_int("MAX_OBJECTS", 8,
			&MAX_OBJECTS,  FALSE,
			"main", 
			"The maximum number of silhouettes to track");



main(int argc, char** argv)
{
  Profile::draw_colour = TRUE;
  Profile::draw_boxes = FALSE;

  // remove the following 2 lines to draw the silouhette as splines 
  // rather than just bounding rectangles
  Profile::draw_spline = FALSE;
  Profile::draw_rectangle = TRUE;

  bool show_images = TRUE;
  char *parameter_file = NULL;
  char* output_image = NULL;
  ImageSource* input_source = NULL;

  int c;
  while ((c = getopt(argc, argv, "i:p:q")) != EOF)
    switch(c)
      {
      case 'i':
	input_source = get_live_or_movie_feed(optarg);
	break;

      case 'p':
	parameter_file = optarg;
	break;

      case 'q':
	show_images = FALSE;
	break;

      case '?':
      default:
	fprintf(stderr, USAGE);
	exit(1);
      }

  AmbGlobals.parse_parameter_file(parameter_file);
  AmbGlobals.create_interface(argc, argv);

  ImageSource& vidsrc = *input_source;
  TrackedProfileSet curr_profiles; // pedestrian silhouettes


  GroundPeopleTracker* people_tracker = new
    GroundPeopleTracker(&vidsrc, NULL, &curr_profiles,
			NULL);

  if (show_images) vidsrc.display();

  /********************************************************************/
  /*                                                                  */
  /*                      Main Loop                                   */
  /*                                                                  */
  /********************************************************************/

  while (vidsrc.get_next() != NULL) 
    {
      people_tracker->MainLoop(MAX_OBJECTS);
      if (show_images)  
	{
	  vidsrc.display();
	  curr_profiles.draw();
	}
	  
      else fprintf(stderr," frame %d \n", vidsrc.get_frame_no());

      TrackedProfileSet filtered = 
	curr_profiles.filter_out_short(GroundPeopleTracker::SMALL_NO_FRAMES);

      cout << filtered;
      cout << "*** " << vidsrc.get_current_timestamp().tv_sec << " : "
	   << vidsrc.get_current_timestamp().tv_usec << " ****" <<
	endl;
      
      filtered.delete_all();     						  
    }

  /********************************************************************/
}




