/*
This source file is the copyright property of the University of Leeds
('The University').

Permission to use, copy, modify, and distribute this source file for
educational, research, and not-for-profit purposes, without fee and
without a signed licensing agreement, is hereby granted, provided that
the above copyright notice, this paragraph and the following three
paragraphs appear in all copies, modifications, and distributions.

In no event shall The University be liable to any party for direct,
indirect, special, incidental or consequential damages, including lost
profits, arising out of the use of this software and its documentation.

The software is provided without warranty. The University has no
obligation to provide maintenance, support, updates, enhancements, or
modifications.

This software was written by Adam Baumberg, Vision Group,
School of Computer Studies, University of Leeds, UK. The code and use
thereof should be attributed to the author where appropriate
(including demonstrations which rely on it's use).
*/

/***************************************************************
 * C++ source
 *
 * File : 	viz_tracking_results.cc
 *
 * Module :	viz_tracking_results
 *
 * Author : 	A M Baumberg (CoMIR)
 *
 * Creation Date : Tue Sep  3 16:26:16 1996 
 *
 * Comments : 	
 *
 ***************************************************************/


#define USAGE "viz_tracking_results -i i_options -z pgm_file \n\
[-o mv_out] [-c colourmap] [-s n] [-q] \n\n\
-i i_options : input image options (-i help for info) \n\
-z pgm_file : an input pgm filename \n\
-o mv_out : output sent to a SG movie file mv_out \n\
-c colourmap : a ppm image file to use as a colour map \n\
-s n : only show every n'th frame \n\
-q : quiet mode (no image display) \n"


#include <profile.h>
#include <PgmSource.h>
#include <imgsrc_utils.h>
#include <stdio.h>
#include <RGB32Image.cc>
#include <sgmoviestore.h>
#include <EnvParameter.h>

#define CMAP_FILE "/home/dream/imv/data/cmap.ppm"

int NO_CONTROL_POINTS = 32;

main(int argc, char** argv)
{  
  char *cmap_file = CMAP_FILE;

  bool show_camera_view = TRUE;
  char *pgm_name = NULL;
  char *output_movie = NULL;

  int c;
  int skip = 1;
  ImageSource* input_source = NULL;
  while ((c = getopt(argc, argv, "hqo:z:i:s:c:")) != EOF)
    switch(c)
      {
      case 'q':
	show_camera_view = FALSE;
	break;

      case 'z':
	if (optarg != NULL) 
	  pgm_name = optarg;
	break;
      case 's':
	if (optarg != NULL) skip = atoi(optarg);
	break;
      case 'o':
	if (optarg != NULL) 
	  output_movie = optarg;
	break;	
      case 'i':
	input_source = get_live_or_movie_feed(optarg);
	break;
      case 'c':
	cmap_file = optarg;
	break;

      case 'h':
      case '?':
      default:
	fprintf(stderr,USAGE);
	exit(1);
      }
  RGB32Image* cmap = (RGB32Image*) read_pnm(CMAP_FILE);
  if (cmap->get_access_mode() == IA_BOTTOM_TO_TOP)
    cmap->flip_vertically(cmap);
  RGB32pixel* cmap_data = (RGB32pixel*) cmap->get_data();

  PgmSource pgm_input(pgm_name);
//   if (input_source != NULL)
//     while (input_source->get_frame_no() < pgm_input.PgmSource::start_frame)
//       input_source->get_next();
  char *tmp_buffer1 = new char[strlen(pgm_name) + 50];
  char *tmp_buffer2 = new char[strlen(pgm_name) + 50];
  
  Image* tmp_img = NULL;
  Image* output_image = NULL;
  Image* camera_view = NULL;

  MovieStore* mv_out = NULL;
  if (output_movie != NULL) 
    mv_out = new MovieStore(output_movie, 0, TRUE);

  pgm_input.get_next();
  pgm_input.restart();
  skip--;
  bool fin = FALSE;
  while (!fin)
    {     
      if (input_source == NULL)
	{
	  tmp_img = new  
	    RGB32Image(pgm_input.get_current()->get_width(),
		       pgm_input.get_current()->get_height());
	  input_source = new ImageSource(tmp_img);
	}
      if (tmp_img != NULL) tmp_img->clear();
      sprintf(tmp_buffer1,pgm_input.format_string, 
	      pgm_input.file_prefix, pgm_input.curr_frame,".dat.gz");
      sprintf(tmp_buffer2,"zcat %s > /usr/tmp/vtr", tmp_buffer1);
      system(tmp_buffer2);
      if (pgm_input.get_next() == NULL) fin = TRUE;
      if (input_source->get_frame_no() < (pgm_input.curr_frame-1))
	input_source->goto_frame(pgm_input.curr_frame-1);
      if (input_source->get_next() == NULL) fin = TRUE;

      ifstream strm_in("/usr/tmp/vtr");
      
      while (strm_in.peek() != EOF)
	{
	  Profile curr_prf;
	  strm_in >> curr_prf;
	  input_source->get_current()->set_line_width(3);
	  RGB32pixel* prf_color = &cmap_data[128 + (curr_prf.ref_no & 127)];
 	  input_source->get_current()->set_draw_colour(prf_color->red,
						prf_color->green,
						prf_color->blue);
	  curr_prf.draw_in_image(input_source->get_current(), FALSE);
	}
      camera_view =
	((Gray8Image*)pgm_input.get_current())->to_rgb(cmap_data,
						       camera_view);
      if (show_camera_view) 
	{ 
	  input_source->display();
	  camera_view->display();
	}

      if (mv_out != NULL)
	{
	  Image* img1 = input_source->get_current();
	  Image* img2 = camera_view;
	  int w = img1->get_width();
	  int h = img1->get_height();
	  if (output_image == NULL)
	    output_image = new RGB32Image(2 * w, h);
	  output_image->paste_subimage(0, w-1, 0, h-1, img1);
	  output_image->paste_subimage(w, (2 * w)-1, 0, h-1,
				       img2);
	  mv_out->add_a_frame(output_image);
	}
      pgm_input.curr_frame += skip;
    }
}

