/*
This source file is the copyright property of the University of Leeds
('The University').

Permission to use, copy, modify, and distribute this source file for
educational, research, and not-for-profit purposes, without fee and
without a signed licensing agreement, is hereby granted, provided that
the above copyright notice, this paragraph and the following three
paragraphs appear in all copies, modifications, and distributions.

In no event shall The University be liable to any party for direct,
indirect, special, incidental or consequential damages, including lost
profits, arising out of the use of this software and its documentation.

The software is provided without warranty. The University has no
obligation to provide maintenance, support, updates, enhancements, or
modifications.

This software was written by Adam Baumberg, Vision Group,
School of Computer Studies, University of Leeds, UK. The code and use
thereof should be attributed to the author where appropriate
(including demonstrations which rely on it's use).
*/

/*
 * Boundary Points.h
 * 
 * the ordered set of points on the boundary of an object
 *
 */

#ifndef __BOUNDARY_POINTS_H__
#define __BOUNDARY_POINTS_H__

#include<list.h>
#include<point2.h>
#include<boolean.h>
#include<PointVector.h>
#include<NagMatrix.h>

class Profile;

extern int NO_CONTROL_POINTS;

class BoundaryPoints : public PointVector
{

 public:
  
  int anchor_point;

  BoundaryPoints() : PointVector() {}

  BoundaryPoints(int n) : PointVector(n) {}

  /* smooth the curve by convolving with a Gaussian
     standard deviation sd, window_size for convolving */
  void gauss_smooth(realno sd, int window_size = 5);

  /* find_best_line - finds best fit line 
     returns - (1/gradient) , origin_x, origin_y */
  void find_best_line(realno& inv_grad, realno& ox, realno& oy);
  void find_robust_best_line(realno& inv_grad, realno& ox, realno& oy);
					   
  void find_end(Point2& n, Point2& origin);
  void find_ends(realno inv_grad, realno ox, realno oy);

  void draw_points(int xlo = 0, int ylo = 0);
  void draw_curve(int ox = 0, int oy = 0, int step = 1);

  bool filter();
  bool convert_to_spline(PointVector *res); // filter and spline
  void to_spline(PointVector* res); // just spline , no filter 
  bool find_optimal_spline(Profile* result); // non-uniform optimal
					      // spline


  void recenter(realno, realno);

};

#endif
