/*
This source file is the copyright property of the University of Leeds
('The University').

Permission to use, copy, modify, and distribute this source file for
educational, research, and not-for-profit purposes, without fee and
without a signed licensing agreement, is hereby granted, provided that
the above copyright notice, this paragraph and the following three
paragraphs appear in all copies, modifications, and distributions.

In no event shall The University be liable to any party for direct,
indirect, special, incidental or consequential damages, including lost
profits, arising out of the use of this software and its documentation.

The software is provided without warranty. The University has no
obligation to provide maintenance, support, updates, enhancements, or
modifications.

This software was written by Adam Baumberg, Vision Group,
School of Computer Studies, University of Leeds, UK. The code and use
thereof should be attributed to the author where appropriate
(including demonstrations which rely on it's use).
*/

// PointVector.h
// 
// A Class for manipulating a vector of n Points
// stored as a vector of 2n realnos
//

#ifndef __POINT_VECTOR_H__
#define __POINT_VECTOR_H__

#include <point2.h>
#include <NagVector.h>

class PointVector : public NagVector
{
protected:
  
  int no_points;
  
public:
  
  PointVector() : NagVector(0,1024) { no_points = 0; }
  PointVector(int n) : NagVector(2 *n) { no_points = n; }

  void add_point(Point2& pnt)
    { add(pnt.x); add(pnt.y); no_points++; }
  
  Point2* pop() { size -=2; return &(point_data()[no_points--]); }
  Point2* last() { return &(point_data()[no_points-1]); }
  Point2* first() { return (point_data()); }

  inline Point2* point_data() { return ((Point2*) data); }
  inline Point2* end_data() { return &(point_data()[no_points]); }

  int get_no_points() { return no_points; }

  void operator=(PointVector& pv2)
    { no_points = pv2.no_points; NagVector::operator=(pv2); }

  // utility functions

  void map(realno ax, realno ay, PointVector& result);

  
};

#endif
