/*
This source file is the copyright property of the University of Leeds
('The University').

Permission to use, copy, modify, and distribute this source file for
educational, research, and not-for-profit purposes, without fee and
without a signed licensing agreement, is hereby granted, provided that
the above copyright notice, this paragraph and the following three
paragraphs appear in all copies, modifications, and distributions.

In no event shall The University be liable to any party for direct,
indirect, special, incidental or consequential damages, including lost
profits, arising out of the use of this software and its documentation.

The software is provided without warranty. The University has no
obligation to provide maintenance, support, updates, enhancements, or
modifications.

This software was written by Adam Baumberg, Vision Group,
School of Computer Studies, University of Leeds, UK. The code and use
thereof should be attributed to the author where appropriate
(including demonstrations which rely on it's use).
*/

/*
 * ProfileSet -
 * a class consisting of a list of Profiles
 *
 */

#include <ProfileSet.h>

listimplement(Profile)
void ProfileSet::draw(bool flag)
{ 
  for (start(); current_ok(); forward())
    get_current().draw(flag);

//   for (item(Profile)* curr = first; curr != NULL; curr = curr->next)
// 	curr->dat->draw(flag); 

}

void ProfileSet::recenter(realno* wghts)
{ 
  for (item(Profile)* curr = first; curr != NULL; curr = curr->next)
	curr->dat->recenter(wghts); 
}

void ProfileSet::draw_filled()
{ 
  for (item(Profile)* curr = first; curr != NULL; curr = curr->next)
	curr->dat->draw_filled(); 
}

void ProfileSet::normalise()
{ 
  for (item(Profile)* curr = first; curr != NULL; curr = curr->next)
	curr->dat->normalise(); 
}

void ProfileSet::add_origins(Point2 new_origin)
{
  for (item(Profile)* curr = first; curr != NULL; curr = curr->next)
    {
      for (int i = 0; i < NO_CONTROL_POINTS; i++)
	curr->dat->point_data()[i] += curr->dat->origin - new_origin;
    }
}


void ProfileSet::reflect(ProfileSet& result)
{
  item(Profile)* lastp = last;
  for (item(Profile)* curr = first; curr != NULL; curr = curr->next)
    {
      result.add(curr->dat->reflect());
      if (curr == lastp) break;
    }
}

void ProfileSet::transform(NagMatrix& H)
{
  if (H.get_data() == NULL) return;
  for (item(Profile)* curr = first; curr != NULL; curr = curr->next)
    H.multiply(*curr->dat);
}

void ProfileSet::keep_ok()
{
  for (item(Profile)* curr = first; curr != NULL;)
    if (!(curr->dat->is_ok()))
      {
	item(Profile)* bad_profile = curr;
	curr = curr->next;
	destroy(bad_profile);
      }
  else curr = curr->next;
}

void ProfileSet::remove_ok()
{
  for (item(Profile)* curr = first; curr != NULL; )
    if (curr->dat->is_ok())
      {
	item(Profile)* tmp_profile = curr;
	curr = curr->next;
	destroy(tmp_profile);
      }
  else curr = curr->next;
}

Profile* ProfileSet::find_labelled(int n)
{
  for (start(); current_ok(); forward())
    if (get_current().ref_no == n) return (&get_current());
  return (NULL);
}
	
void ProfileSet::display_shapes(realno ox, realno oy)
{ 
  for (item(Profile)* curr = first; curr != NULL; curr = curr->next)
    curr->dat->display_shape(ox,oy); 
}

void ProfileSet::gauss_smooth(realno sd)
{
  int win = (int) (2.0 * sd + 0.5) + 1;
  realno denom = 2 * sd * sd;
  NagVector coeff(win);
  realno sum = 1.0;
  coeff[0] = 1.0;
  for (int n = 1; n < win; n++)
    sum += 2.0 * (coeff[n] = exp(-(realno) (n*n) / denom));
  
  coeff.scale(1.0 / sum, coeff);
  ProfileSet res;
  for (n = 0; n < no_items; n++)
    {
      Profile res_prf;
      res_prf.clear();
      for (int m = -win + 1; m < win; m++)
	{
	  int k = abs(n+m);
	  if (k >= no_items) k = abs(2 * (no_items-1) - k);
	  NagVector tmp;
	  (*this)[k]->dat->NagVector::scale(coeff[abs(m)], tmp);
	  res_prf.add(tmp, res_prf);
	}
      res.add(res_prf);
    }
  this->destroy_all();
  first = res.first;
  last = res.last;
  no_items = res.no_items;
}

void ProfileSet::fit_to_line()
{
  NagVector t(no_items);
  NagVector x(no_items);
  NagVector y(no_items);

  int i = 0;
  for (item(Profile)* curr = first; curr != NULL; curr = curr->next)
    {
      t[i] = i;
      x[i] = curr->dat->origin.x;
      y[i] = curr->dat->origin.y;
      i++;
    }

  realno denom = no_items * t.length2() - (t.sum())*(t.sum());
  realno m_x = (no_items * t.dot(x) - t.sum() * x.sum()) / denom;
  realno c_x = (t.length2() * x.sum() - t.sum() * t.dot(x)) / denom;

  realno m_y = (no_items * t.dot(y) - t.sum() * y.sum()) / denom;
  realno c_y = (t.length2() * y.sum() - t.sum() * t.dot(y)) / denom;
  i = 0;
  for (curr = first; curr != NULL; curr = curr->next)
    {
      Point2 new_origin(i * m_x + c_x, i * m_y + c_y);
      Point2 adjust(new_origin - curr->dat->origin);
      i++;
      for (int k = 0; k < NO_CONTROL_POINTS; k++)
	curr->dat->spline[k] -= adjust;

      curr->dat->origin = new_origin;
    }
}  

void ProfileSet::scale_coords(realno scale_fac)
{ 
  if (scale_fac != 1.0)
    {
      for (item(Profile)* curr = first; curr != NULL; curr = curr->next)
	curr->dat->scale_coords(scale_fac);
    }
}
