/*
This source file is the copyright property of the University of Leeds
('The University').

Permission to use, copy, modify, and distribute this source file for
educational, research, and not-for-profit purposes, without fee and
without a signed licensing agreement, is hereby granted, provided that
the above copyright notice, this paragraph and the following three
paragraphs appear in all copies, modifications, and distributions.

In no event shall The University be liable to any party for direct,
indirect, special, incidental or consequential damages, including lost
profits, arising out of the use of this software and its documentation.

The software is provided without warranty. The University has no
obligation to provide maintenance, support, updates, enhancements, or
modifications.

This software was written by Adam Baumberg, Vision Group,
School of Computer Studies, University of Leeds, UK. The code and use
thereof should be attributed to the author where appropriate
(including demonstrations which rely on it's use).
*/

/* RegionSet.h
 * 
 * an object consisting of a set of regions
 *
 * 
 */

#ifndef __REGION_SET_H__
#define __REGION_SET_H__

#include <region.h>
#include <ProfileSet.h>

static const int DEFAULT_MIN_REGION_SIZE = 20;
static const int DEFAULT_MAX_REGION_SIZE = 100;
  
listdeclare(Region);
class RegionSet : public list(Region)
{
 public:

  long gl_win;

  RegionSet() { first = last = NULL; no_items = 0; gl_win = NULLWIN; }
  ~RegionSet() { destroy_all(); }

  /* constructs the set of regions in an image */  
  void grab_regions(Image* , int min_size = DEFAULT_MIN_REGION_SIZE,
		    int max_size = DEFAULT_MAX_REGION_SIZE,
		    bool keep_edge_regions = FALSE);

  // grab all regions but don't grab region image
  void grab_all_regions(Image* img, int min_size, int max_size);

  void label_regions(int& next_ref);

  //void merge_regions(int dthld = 5);
  void merge_regions(realno dthld = 0.1);
  
  // remove small regions
  void filter(realno min_region_size);


  void draw_boxes();

  void trace_regions(); 
  
  void draw_regions(); /* draw the regions */
  void clear_window();

  void smooth(realno sd = 1.0, int win_sz = 5);

  ProfileSet to_profiles();
};


#endif
