/*
This source file is the copyright property of the University of Leeds
('The University').

Permission to use, copy, modify, and distribute this source file for
educational, research, and not-for-profit purposes, without fee and
without a signed licensing agreement, is hereby granted, provided that
the above copyright notice, this paragraph and the following three
paragraphs appear in all copies, modifications, and distributions.

In no event shall The University be liable to any party for direct,
indirect, special, incidental or consequential damages, including lost
profits, arising out of the use of this software and its documentation.

The software is provided without warranty. The University has no
obligation to provide maintenance, support, updates, enhancements, or
modifications.

This software was written by Adam Baumberg, Vision Group,
School of Computer Studies, University of Leeds, UK. The code and use
thereof should be attributed to the author where appropriate
(including demonstrations which rely on it's use).
*/

/***************************************************************
 * C - C++ Header
 *
 * File : 	SplineWeights.h
 *
 * Module :	SplineWeights
 *
 * Author : 	A M Baumberg
 *
 * Creation Date : Tue Jan 11 16:30:09 1994 
 *
 * Comments : 	
 *
 ***************************************************************/


#ifndef __SPLINE_WEIGHTS_H__
#define __SPLINE_WEIGHTS_H__

/* include files */

#include <realno.h>

/* global functions */


const int NO_SUBDIVISIONS = 2;

class SplineWeights
{
public:

  realno *w0, *w1, *w2, *w3;
  realno *dw0, *dw1, *dw2, *dw3;
  int nsubs;
  SplineWeights(int n = NO_SUBDIVISIONS);
  ~SplineWeights() { delete [] w0; delete [] w1; delete [] w2; 
		     delete [] w3; delete [] dw0; delete [] dw1;
		     delete [] dw2; delete [] dw3; }
};

extern SplineWeights* spline_weights;
extern int NO_CONTROL_POINTS;

void setup_spline_weights();
realno B_func(int i, realno u);
realno dB_func(int i, realno u);
realno ddB_func(int i, realno u);

/* local functions */



#endif
