/*
This source file is the copyright property of the University of Leeds
('The University').

Permission to use, copy, modify, and distribute this source file for
educational, research, and not-for-profit purposes, without fee and
without a signed licensing agreement, is hereby granted, provided that
the above copyright notice, this paragraph and the following three
paragraphs appear in all copies, modifications, and distributions.

In no event shall The University be liable to any party for direct,
indirect, special, incidental or consequential damages, including lost
profits, arising out of the use of this software and its documentation.

The software is provided without warranty. The University has no
obligation to provide maintenance, support, updates, enhancements, or
modifications.

This software was written by Adam Baumberg, Vision Group,
School of Computer Studies, University of Leeds, UK. The code and use
thereof should be attributed to the author where appropriate
(including demonstrations which rely on it's use).
*/

/*
 * profile.h
 * 
 * a record class for storing the profile data
 * of an object
 *
 * AMB 27/4/93
 */

#ifndef __PROFILE_H__
#define __PROFILE_H__

#include <shape_constants.h>
#include <BoundaryPoints.h>
#include <boolean.h>
#include <EnvParameter.h>

class Image;



#define spline point_data()

enum ProfType { BASE_PROFILE, TRACKED_PROFILE };
class Profile : public PointVector
{
public:
  

  Point2 origin;
  realno width, height;
  realno inv_grad;
  // inverse of the gradient of line of best fit

  Point2& direction;
  // used in tracking module

  int ref_no;
  // reference for blob tracking


// member functions


  virtual ProfType get_type() { return (BASE_PROFILE); }
// constructor / destructors

  Profile();
  Profile(realno ox, realno oy, realno wdth, realno hgt);
  Profile(Profile& original);
  Profile& operator=(Profile &);
  Profile(Point2& p_o, realno& p_w, realno& p_h, realno& p_grad,
	  PointVector& pnts , Point2& direction);
  virtual ~Profile();

// data access functions

  realno* get_shape(int i);
  realno get_data(int i); 
  void set_data(int i, realno val); 
  int get_data_size();


// utility functions

  // get_size calculates height, width
  // returns 0.5 * (height + width)
  realno get_size();

  void get_y_bounds(realno& ymin, realno& ymax);
  void get_box(int& ixmin, int& ixmax, int& iymin, int& iymax);

  virtual void get_enclosing_box(int& ixmin, int& ixmax,
				 int& iymin, int& iymax)
  // { return (get_box(ixmin, ixmax, iymin, iymax)); }
  { get_box(ixmin, ixmax, iymin, iymax); }  // RJM change to avoid compile error

  void draw(int col1, int col2, int col3);
  void draw_filled();
  virtual void draw(bool flag = TRUE);
  void normalise(bool flag = TRUE);
  // scale shape
  void scale(realno );
  // scale_coords -- scale up coordinates
  void scale_coords(realno );

  void undraw() { draw(FALSE); }
  void unnormalise() { normalise(FALSE); }
  
  void map(realno& ax, realno& ay, Profile* result = NULL);

  void align_to(Profile* mean, Profile* result, realno& s, realno& th,
		realno* weights = NULL);

  void recenter(realno* weights = NULL);

  BoundaryPoints* to_points(BoundaryPoints* res = NULL);
  void get_normals(Profile& res);
  BoundaryPoints* sample_normals(BoundaryPoints* res = NULL);

  // draw at given origin
  void display_shape(realno ox, realno oy, bool flag = TRUE);
  // draw into an Image structure
  void draw_in_image(Image* canvas, Point2 origin, bool filled =
		     TRUE);
  void draw_in_image(Image* canvas, bool filled = TRUE)
  { draw_in_image(canvas, origin, filled); }

  void grab_data(Profile& original);
  
  // does profile satisfy shape constraints ?
  bool is_ok();

  // reflect about y-axis
  Profile* reflect(Profile* result = NULL);
  void points_to_spline();

  virtual void output_to_stream(ostream& );

  // static variables for drawing/ output routines
  
  static bool draw_boxes;	// draw control points ?
  static bool draw_arrow;	// draw direction arrow ?
  static bool draw_colour;	// use colour ?
  static bool draw_spline;	// draw boundary with a curve ?
  static bool draw_rectangle;	// draw with a rectangle ?
  static bool draw_fill;	// draw filled 2D shape ?

  static bool output_points;	// output control point data ?
  
  static int draw_linewidth;	// line width for drawing 
  static realno draw_box_size;	// size for control point boxes
};

ostream& operator<<(ostream&, Profile&);
istream& operator>>(istream&, Profile&);  


#endif
