/*
This source file is the copyright property of the University of Leeds
('The University').

Permission to use, copy, modify, and distribute this source file for
educational, research, and not-for-profit purposes, without fee and
without a signed licensing agreement, is hereby granted, provided that
the above copyright notice, this paragraph and the following three
paragraphs appear in all copies, modifications, and distributions.

In no event shall The University be liable to any party for direct,
indirect, special, incidental or consequential damages, including lost
profits, arising out of the use of this software and its documentation.

The software is provided without warranty. The University has no
obligation to provide maintenance, support, updates, enhancements, or
modifications.

This software was written by Adam Baumberg, Vision Group,
School of Computer Studies, University of Leeds, UK. The code and use
thereof should be attributed to the author where appropriate
(including demonstrations which rely on it's use).
*/

/***************************************************************
 * C - C++ Header
 *
 * File : 	GroundPeopleTracker.h
 *
 * Module :	GroundPeopleTracker
 *
 * Author : 	A M Baumberg (CoMIR)
 *
 * Creation Date : Mon Oct 21 11:15:12 1996 
 *
 * Comments : 	High level tracking on the ground plane
 *
 ***************************************************************/


/* include files */

#ifndef __GROUND_PEOPLE_TRACKER_H__
#define __GROUND_PEOPLE_TRACKER_H__

#include <WorldOcclusionHandler.h>
#include <ActiveModel.h>
#include <MotionDetector.h>
#include <TrackedProfileSet.h>

class GroundPeopleTracker
{
protected:
  Image* previous_frame;

public:
  
  // used to handle occlusions
  WorldOcclusionHandler* occ_handler;

  // local image fitting
  ActiveModel* local_optimiser;


  // current tracked people
  ProfileSet* profiles;

  MotionDetector* detect_motion;

  // globals

  static realno FITNESS_THRESHOLD;
  static realno VERY_POOR_FIT;
  static int SMALL_NO_FRAMES;
  static realno MIN_SIGNIFICANCE;
  static realno terminate_thresh;

  static realno init_pos_sd;	// initial GP position std deviation
  static realno acc_sd;		// GP random acceleration std deviation
  static realno model_err_sd;	// modeling error term (GP coords)

  static realno init_scale_sd;	// the std dev of scale estimate
  static realno rel_scale_sd;	// the std dev of scale noise term


  static int TEMPORAL_SKIP;

public:

  GroundPeopleTracker(WorldOcclusionHandler*,   ActiveModel*,
		      MotionDetector*, ProfileSet* = NULL);
  
  // construct a complete ground plane tracking system 
  // using input image source and PCA model file
  // and optional external data, optional feature detctor

  GroundPeopleTracker(ImageSource*, char* pca_file = NULL,
		      ProfileSet* = NULL,
		      ExternalData* = NULL,
		      FeatureDetector* = NULL);
		     
  void predict_profiles();

  void track_all();

  void track_new_objects();

  void initialise_track(TrackedProfile* );

  // tracking loop for each new image
  void MainLoop(int max_objects = 10);


  // generate visualisation of tracked objects
  Image* get_viz_image();

  // get the tracked profiles
  TrackedProfileSet* get_tracked_profiles()
  { return ((TrackedProfileSet*) profiles); }

};

#endif
