/*
This source file is the copyright property of the University of Leeds
('The University').

Permission to use, copy, modify, and distribute this source file for
educational, research, and not-for-profit purposes, without fee and
without a signed licensing agreement, is hereby granted, provided that
the above copyright notice, this paragraph and the following three
paragraphs appear in all copies, modifications, and distributions.

In no event shall The University be liable to any party for direct,
indirect, special, incidental or consequential damages, including lost
profits, arising out of the use of this software and its documentation.

The software is provided without warranty. The University has no
obligation to provide maintenance, support, updates, enhancements, or
modifications.

This software was written by Adam Baumberg, Vision Group,
School of Computer Studies, University of Leeds, UK. The code and use
thereof should be attributed to the author where appropriate
(including demonstrations which rely on it's use).
*/

/***************************************************************
 * C++ source
 *
 * File : 	GroundPlaneToImagePlane.cc
 *
 * Module :	GroundPlaneToImagePlane
 *
 * Author : 	A M Baumberg (CoMIR)
 *
 * Creation Date : Fri Aug  9 12:55:56 1996 
 *
 * Comments : 	
 *
 ***************************************************************/


/* include files */

#include "GroundPlaneToImagePlane.h"
#include <utils.h>
#include <math.h>
#include <ieeefp.h>
#define UNSET_REAL  (sqrt(-1.0))
#define IS_SET_REAL(x) (!real_isnan(x))

NagMatrix GroundPlaneToImagePlane(3,3);
NagMatrix ImagePlaneToGroundPlane(3,3);
NagVector ImagePlaneToDepth(3);

static Point2 GP_camera_position;
static bool know_camera_position = FALSE;

static realno gptoip_m00, gptoip_m01, gptoip_m02, gptoip_m10,
  gptoip_m11, gptoip_m12, gptoip_m20, gptoip_m21, gptoip_m22,
  iptogp_m20, iptogp_m21, iptogp_m22, camera_pos_u, camera_pos_v;


			 
static void find_ip_to_gp(NagMatrix& x_ip, NagMatrix& x_gp, NagMatrix& res)
{
  NagVector tmp_v;
  NagMatrix tmp_inv;

  NagMatrix m1;
  x_ip.get_block(0,2,0,2, m1);
  NagVector x4;
  x_ip.get_column(3, x4);
  m1.invert(tmp_inv);
  tmp_inv.multiply(x4, tmp_v);
  m1.column(0) *= (tmp_v[0]);
  m1.column(1) *= (tmp_v[1]);
  m1.column(2) *= (tmp_v[2]);
  
  NagMatrix m2;
  x_gp.get_block(0,2,0,2, m2);
  x_gp.get_column(3, x4);
  m2.invert(tmp_inv);
  tmp_inv.multiply(x4, tmp_v);
  m2.column(0) *= (tmp_v[0]);
  m2.column(1) *= (tmp_v[1]);
  m2.column(2) *= (tmp_v[2]);
  
  m1.invert(tmp_inv);
  m2.multiply(tmp_inv, res);
}
  
static void scale_world_depth()
{
  // rescales ImagePlaneToDepth so that
  // image_size (in pixels) = (world_object_size) * (world_depth)
  
  // e.g. image_height = world_height * world_depth
  NagMatrix& m = ImagePlaneToGroundPlane;
  realno m20 = ImagePlaneToDepth[0];
  realno m21 = ImagePlaneToDepth[1];

  realno dx = m.read(0,0) * m21 - m.read(0,1) * m20;
  realno dy = m.read(1,0) * m21 - m.read(1,1) * m20;
  
  realno ds2 = (dx * dx + dy * dy) / (m20 * m20 + m21 * m21);
  realno ds = sqrt(ds2);
  ImagePlaneToDepth.scale(1.0 / ds, ImagePlaneToDepth);
}

  
void setup_default_projection(Point2* IP_pnts, Point2* GP_pnts)
{
  NagMatrix x_ip(3,4);
  x_ip.clear(1.0);
  NagMatrix x_gp(3,4);
  x_gp.clear(1.0);
  for (int i = 0; i < 4; i++)
    {
      x_ip.set(0,i, IP_pnts[i].x);
      x_ip.set(1,i, IP_pnts[i].y);
      x_gp.set(0,i, GP_pnts[i].x);
      x_gp.set(1,i, GP_pnts[i].y);
    }
  find_ip_to_gp(x_ip, x_gp, ImagePlaneToGroundPlane);
  set_image_to_world_trans(ImagePlaneToGroundPlane);
}

static void setup_Reading_projection()
{
  // reading image points
  Point2 image_points[4] = { Point2(385,413),
			     Point2(191,372),
			     Point2 (535,377),
			     Point2 (339,339) };
  
  Point2 world_points[4] = { Point2(-737.62051, -1966.03289),
			     Point2(-47.2213, 2183.79352),
			     Point2(2472.37189, -2807.04155),
			     Point2(3244.53764, 1448.36952) };

  // calculate Reading Image Plane to Ground Plane
  setup_default_projection(image_points, world_points);
  
  realno leeds_to_rdg_data[] = { 2.165644, 0.0, 0.0,
				 0.0, -2.179263, 0.0,
				 -1.869625, 600.258972, 1.0 };

  // post-multiply by 'Leeds image to Reading Image'
  NagMatrix corrected;
  NagMatrix LtoR(3,3,leeds_to_rdg_data);
  ImagePlaneToGroundPlane.multiply(LtoR, corrected);

				 
  set_image_to_world_trans(corrected);
  cout << GroundPlaneToImagePlane << endl;

  //  ImagePlaneToGroundPlane.get_row(2, ImagePlaneToDepth);
  //  hack to correct for poor depth mapping
  ImagePlaneToDepth.reconstruct(3);
  ImagePlaneToDepth[0] = 0.0;
  ImagePlaneToDepth[1] = -0.00525;
  ImagePlaneToDepth[2] = 1.67;
  scale_world_depth();
  set_GP_camera_pos(Point2(-30877.54196,-20801.77570));
}


void setup_default_projection()
{
  
  if (IS_SET_REAL(gptoip_m00) && IS_SET_REAL(gptoip_m01) && 
      IS_SET_REAL(gptoip_m02) && IS_SET_REAL(gptoip_m10) &&
      IS_SET_REAL(gptoip_m11) && IS_SET_REAL(gptoip_m12) &&
      IS_SET_REAL(gptoip_m20) && IS_SET_REAL(gptoip_m21) &&
      IS_SET_REAL(gptoip_m22))
    {
      GroundPlaneToImagePlane.set(0,0,gptoip_m00);
      GroundPlaneToImagePlane.set(0,1,gptoip_m01);
      GroundPlaneToImagePlane.set(0,2,gptoip_m02);
      GroundPlaneToImagePlane.set(1,0,gptoip_m10);
      GroundPlaneToImagePlane.set(1,1,gptoip_m11);
      GroundPlaneToImagePlane.set(1,2,gptoip_m12);
      GroundPlaneToImagePlane.set(2,0,gptoip_m20);
      GroundPlaneToImagePlane.set(2,1,gptoip_m21);
      GroundPlaneToImagePlane.set(2,2,gptoip_m22);
      set_world_to_image_trans(GroundPlaneToImagePlane);
      if (IS_SET_REAL(iptogp_m20) && IS_SET_REAL(iptogp_m21)
	  && IS_SET_REAL(iptogp_m22))
	{
	  ImagePlaneToDepth[0] = iptogp_m20;
	  ImagePlaneToDepth[1] = iptogp_m21;
	  ImagePlaneToDepth[2] = iptogp_m22;
	}

      if (IS_SET_REAL(camera_pos_u) && IS_SET_REAL(camera_pos_v))
	set_GP_camera_pos(Point2(camera_pos_u, camera_pos_v));
    }

  else setup_Reading_projection();
}
    
static void update_constants()
{
  gptoip_m00 = GroundPlaneToImagePlane.read(0,0);
  gptoip_m01 = GroundPlaneToImagePlane.read(0,1);
  gptoip_m02 = GroundPlaneToImagePlane.read(0,2);
  gptoip_m10 = GroundPlaneToImagePlane.read(1,0);
  gptoip_m11 = GroundPlaneToImagePlane.read(1,1);
  gptoip_m12 = GroundPlaneToImagePlane.read(1,2);
  gptoip_m20 = GroundPlaneToImagePlane.read(2,0);
  gptoip_m21 = GroundPlaneToImagePlane.read(2,1);
  gptoip_m22 = GroundPlaneToImagePlane.read(2,2);

  iptogp_m20 = ImagePlaneToDepth[0];
  iptogp_m21 = ImagePlaneToDepth[1];
  iptogp_m22 = ImagePlaneToDepth[2];
}


void set_image_to_world_trans(NagMatrix& img_to_world_trans)
{
  if (&img_to_world_trans != &ImagePlaneToGroundPlane)
    img_to_world_trans.copy(ImagePlaneToGroundPlane);
  ImagePlaneToGroundPlane.get_row(2, ImagePlaneToDepth);
  ImagePlaneToGroundPlane.invert(GroundPlaneToImagePlane);
  scale_world_depth();
  update_constants();
}

void set_world_to_image_trans(NagMatrix& world_to_img_trans)
{
  if (&world_to_img_trans != &GroundPlaneToImagePlane)
    world_to_img_trans.copy(GroundPlaneToImagePlane);
  GroundPlaneToImagePlane.invert(ImagePlaneToGroundPlane);
  ImagePlaneToGroundPlane.get_row(2, ImagePlaneToDepth);
  scale_world_depth();
  update_constants();
}

realno world_depth_at_image_coords(realno x, realno y)
{
  return (ImagePlaneToDepth[0] * x + ImagePlaneToDepth[1] * y +
	  ImagePlaneToDepth[2]);
}

bool get_GP_camera_pos(Point2& result)
{ 
  if (know_camera_position) 
    {
      result = GP_camera_position;
      return (TRUE);
    }
  return (FALSE);
}

void set_GP_camera_pos(Point2 cam_pos)
{
  GP_camera_position = cam_pos;
  know_camera_position = TRUE;
  camera_pos_u = cam_pos.x;
  camera_pos_v = cam_pos.y;
}

void setup_default_projection(Point2* icoords, NagVector& hghts, realno
			      fixed_height)
{
  int n = hghts.get_size();
  NagMatrix lin_eqns(n, 3);
  for (int i = 0; i < n; i++)
    {
      lin_eqns.set(i,0, icoords[i].x);
      lin_eqns.set(i,1, icoords[i].y);
      lin_eqns.set(i,2, 1.0);
    }

  NagMatrix lsquares;
  lin_eqns.pseudo_invert(lsquares);

  NagVector res(3);
  lsquares.multiply(hghts, res);
  res.scale(1.0 / fixed_height, res);
  ImagePlaneToGroundPlane.clear(0.0);
  ImagePlaneToGroundPlane.set(0,0,1);
  ImagePlaneToGroundPlane.set(1,1,1);
  ImagePlaneToGroundPlane.set(2,0,res[0]);
  ImagePlaneToGroundPlane.set(2,1,res[1]);
  ImagePlaneToGroundPlane.set(2,2,res[2]);
  set_image_to_world_trans(ImagePlaneToGroundPlane);
}
  

static int have_registered = 0;
  
int gp_to_ip_register_globals()
{
  if (have_registered++ > 0) return (0);

  gptoip_m00 =
    AmbGlobals.register_real("GP_TO_IP_0_0", UNSET_REAL, 
			   &gptoip_m00, TRUE, "GroundPlaneToImagePlane",
			   "Row 0 Coloumn 0 element of Ground Plane to \
Image plane transformation");

  gptoip_m01 = 
AmbGlobals.register_real("GP_TO_IP_0_1", UNSET_REAL, 
			 &gptoip_m01, TRUE, "GroundPlaneToImagePlane",
			 "Row 0 Coloumn 1 element of Ground Plane to \
Image plane transformation");
  
  gptoip_m02 = 
AmbGlobals.register_real("GP_TO_IP_0_2", UNSET_REAL, 
			 &gptoip_m02, TRUE, "GroundPlaneToImagePlane",
			 "Row 0 Coloumn 2 element of Ground Plane to \
Image plane transformation");

  gptoip_m10 = 
AmbGlobals.register_real("GP_TO_IP_1_0", UNSET_REAL, 
			 &gptoip_m10, TRUE, "GroundPlaneToImagePlane",
			 "Row 1 Coloumn 0 element of Ground Plane to \
Image plane transformation");

  gptoip_m11 = 
AmbGlobals.register_real("GP_TO_IP_1_1", UNSET_REAL, 
			 &gptoip_m11, TRUE, "GroundPlaneToImagePlane",
			 "Row 1 Coloumn 1 element of Ground Plane to \
Image plane transformation");

  gptoip_m12 = 
AmbGlobals.register_real("GP_TO_IP_1_2", UNSET_REAL, 
			 &gptoip_m12, TRUE, "GroundPlaneToImagePlane",
			 "Row 1 Coloumn 2 element of Ground Plane to \
Image plane transformation");

  gptoip_m20 = 
AmbGlobals.register_real("GP_TO_IP_2_0", UNSET_REAL, 
			 &gptoip_m20, TRUE, "GroundPlaneToImagePlane",
			 "Row 2 Coloumn 0 element of Ground Plane to \
Image plane transformation");

  gptoip_m21 = 
AmbGlobals.register_real("GP_TO_IP_2_1", UNSET_REAL, 
			 &gptoip_m21, TRUE, "GroundPlaneToImagePlane",
			 "Row 2 Coloumn 1 element of Ground Plane to \
Image plane transformation");

  gptoip_m22 = 
AmbGlobals.register_real("GP_TO_IP_2_2", UNSET_REAL, 
			 &gptoip_m22, TRUE, "GroundPlaneToImagePlane",
			 "Row 2 Coloumn 2 element of Ground Plane to \
Image plane transformation");


  iptogp_m20 = 
AmbGlobals.register_real("IP_TO_DEPTH_0", UNSET_REAL, 
			 &iptogp_m20, TRUE, "GroundPlaneToImagePlane",
			 "Optional, corrected value of element \
(2,0) of Image Plane to Ground Plane matrix. Patch due to Reading ground \
plane calibration error");


  iptogp_m21 = 
AmbGlobals.register_real("IP_TO_DEPTH_1", UNSET_REAL, 
			 &iptogp_m21, TRUE, "GroundPlaneToImagePlane",
			 "Optional, corrected value of element \
(2,1) of Image Plane to Ground Plane matrix. Patch due to Reading ground \
plane calibration error");

  
  iptogp_m22 = 
AmbGlobals.register_real("IP_TO_DEPTH_2", UNSET_REAL, 
			 &iptogp_m22, TRUE, "GroundPlaneToImagePlane",
			 "Optional, corrected value of element \
(2,2) of Image Plane to Ground Plane matrix. Patch due to Reading ground \
plane calibration error");


  camera_pos_u = 
AmbGlobals.register_real("CAMERA_U", UNSET_REAL,
			 &camera_pos_u, TRUE, 
			 "GroundPlaneToImagePlane",
			 "Optional ground plane u-coordinate of the \
camera");


  camera_pos_v = 
AmbGlobals.register_real("CAMERA_V", UNSET_REAL,
			 &camera_pos_v, TRUE, 
			 "GroundPlaneToImagePlane",
			 "Optional ground plane v-coordinate of the \
camera");

  return (1);
}
