/*
This source file is the copyright property of the University of Leeds
('The University').

Permission to use, copy, modify, and distribute this source file for
educational, research, and not-for-profit purposes, without fee and
without a signed licensing agreement, is hereby granted, provided that
the above copyright notice, this paragraph and the following three
paragraphs appear in all copies, modifications, and distributions.

In no event shall The University be liable to any party for direct,
indirect, special, incidental or consequential damages, including lost
profits, arising out of the use of this software and its documentation.

The software is provided without warranty. The University has no
obligation to provide maintenance, support, updates, enhancements, or
modifications.

This software was written by Adam Baumberg, Vision Group,
School of Computer Studies, University of Leeds, UK. The code and use
thereof should be attributed to the author where appropriate
(including demonstrations which rely on it's use).
*/

/***************************************************************
 * C - C++ Header
 *
 * File : 	GroundPlaneToImagePlane.h
 *
 * Module :	GroundPlaneToImagePlane
 *
 * Author : 	A M Baumberg (CoMIR)
 *
 * Creation Date : Fri Aug  9 12:52:56 1996 
 *
 * Comments : 	
 *
 ***************************************************************/



#ifndef __GROUND_PLANE_TO_IMAGE_PLANE_H__
#define __GROUND_PLANE_TO_IMAGE_PLANE_H__

#include <NagMatrix.h>
#include <point2.h>
#include <boolean.h>

extern NagMatrix GroundPlaneToImagePlane; // maps homogeneous world
					  // coordinates to image

extern NagMatrix ImagePlaneToGroundPlane; // and vv.

extern NagVector ImagePlaneToDepth;	// for mapping image plane
                                        // coordinates to depth value

					
// setup projection using 4 correspondences
void setup_default_projection(Point2* IP_pnts, Point2* GP_pnts);

// setup projection using globals
void setup_default_projection();

// setup projection using heights at image positions
// icoords : array of image points
// heights : vector of image heights
// fixed_height : the average world height
void setup_default_projection(Point2* icoords, NagVector& heights, 
			      realno fixed_height);

void set_image_to_world_trans(NagMatrix& img_to_world_trans);

void set_world_to_image_trans(NagMatrix& world_to_img_trans);

realno world_depth_at_image_coords(realno x, realno y);

// get Ground Plane coordinates of camera if known
// return FALSE otherwise

bool get_GP_camera_pos(Point2& result);

void set_GP_camera_pos(Point2 cam_pos);


// register all the default matrix elements with the
// default GlobalsManager 
int gp_to_ip_register_globals();

#endif
