/*
This source file is the copyright property of the University of Leeds
('The University').

Permission to use, copy, modify, and distribute this source file for
educational, research, and not-for-profit purposes, without fee and
without a signed licensing agreement, is hereby granted, provided that
the above copyright notice, this paragraph and the following three
paragraphs appear in all copies, modifications, and distributions.

In no event shall The University be liable to any party for direct,
indirect, special, incidental or consequential damages, including lost
profits, arising out of the use of this software and its documentation.

The software is provided without warranty. The University has no
obligation to provide maintenance, support, updates, enhancements, or
modifications.

This software was written by Adam Baumberg, Vision Group,
School of Computer Studies, University of Leeds, UK. The code and use
thereof should be attributed to the author where appropriate
(including demonstrations which rely on it's use).
*/

/***************************************************************
 * C - C++ Header
 *
 * File : 	MotionDetector.h
 *
 * Module :	MotionDetector
 *
 * Author : 	A M Baumberg
 *
 * Creation Date : Mon Jun 13 11:25:41 1994 
 *
 * Comments : 	A class to detect moving objects for subsequent 
 *		identification/ tracking.
 *
 ***************************************************************/


/* include files */

#ifndef __MOTION_DETECTOR_H__
#define __MOTION_DETECTOR_H__

#include <miscsrc.h>
#include <boolean.h>
#include <medsrc.h>

class RegionSet;
class ProfileSet;

class MotionDetector
{

public:

  ImageSource* input_frames;
  PipeSource* small_input;		// small subsampled image
  PipeSource* small_background;
  SimpleDiffSource* small_differenced;
  //FMedSource* small_foreground;
  NeighbourSource* small_foreground;
  ImageSource* large_background;
  Medsrc* background;

  Image* tracked_mask;
  BlurSource* blur1;
  BlurSource* blur2;
  int frame_count;
  bool have_differenced;	// TRUE if up to date
  bool finished;		// TRUE if no more images

  int sample_skip; 	// for image subsampling
  int run_length; 	// for median filtering
  int time_skip;	// for temporal subsampling

  static realno diff_threshold; 	// threshold for image subtraction


  static realno max_region_ratio;
  static realno min_region_ratio;

  static realno camera_moved_threshold;

  int total_image_size;		// number of pixels in subsampled image
  static bool show_images;	// viusalise the subtracted images ?


  bool merge_regions; 		// true if fragments are merged
  static realno merge_threshold;

  bool do_blur;			// true if images are blured before
		      		// subtraction

public:

  MotionDetector(ImageSource* input, int sample_skip, int run_length,
		 realno diff_threshold, realno max_region_ratio,
		 realno min_region_ratio,  Image* reference_image = NULL,
		 bool use_small_background = TRUE,
		 bool merge_fragments = FALSE,
		 bool do_blur = FALSE);

  MotionDetector(ImageSource* input, Image* reference_image = NULL,
		 bool use_small_background = FALSE);


  void process_next_frame();		// get next frame and process
  void process_current_frame();		
  
  // draw tracked_objects into motion mask;
  void draw_old_objects(ProfileSet* old_set);

  // extract moving objects as Profile list
  void extract_new_objects(ProfileSet* current_set);

  // extract moving regions as Region list
  // if grab_boxes is TRUE only extract the bounding box
  // otherwise extract the shape
  void extract_new_regions(RegionSet* region_set, bool grab_boxes = TRUE);
  
 
};

#endif
