/*
This source file is the copyright property of the University of Leeds
('The University').

Permission to use, copy, modify, and distribute this source file for
educational, research, and not-for-profit purposes, without fee and
without a signed licensing agreement, is hereby granted, provided that
the above copyright notice, this paragraph and the following three
paragraphs appear in all copies, modifications, and distributions.

In no event shall The University be liable to any party for direct,
indirect, special, incidental or consequential damages, including lost
profits, arising out of the use of this software and its documentation.

The software is provided without warranty. The University has no
obligation to provide maintenance, support, updates, enhancements, or
modifications.

This software was written by Adam Baumberg, Vision Group,
School of Computer Studies, University of Leeds, UK. The code and use
thereof should be attributed to the author where appropriate
(including demonstrations which rely on it's use).
*/

/***************************************************************
 * C++ source
 *
 * File : 	OcclusionImage.cc
 *
 * Module :	OcclusionImage
 *
 * Author : 	A M Baumberg
 *
 * Creation Date : Wed May 25 15:41:40 1994 
 *
 * Comments : 	
 *
 ***************************************************************/


/* include files */

#include "OcclusionImage.h"
#include <TrackedProfile.h>
#include <utils.h>

int OcclusionImage::RESAMPLE_SHIFT = 
AmbGlobals.register_int("OCCLUSION_RESAMPLE_SHIFT", 2,
			&OcclusionImage::RESAMPLE_SHIFT,
			TRUE, "OcclusionImage", 
			"Resample image dimensions by 2^n \
for occlusion handling");

void OcclusionImage::mark_rect(TrackedProfile* prf)
{
  setup_occlusion_map(prf);

  int xmin, xmax, ymin, ymax;
  prf->get_box(xmin, xmax, ymin, ymax);
  xmin = (xmin >> RESAMPLE_SHIFT);
  xmax = (xmax >> RESAMPLE_SHIFT);
  ymin = (ymin >> RESAMPLE_SHIFT);
  ymax = (ymax >> RESAMPLE_SHIFT);

  if (xmin < 0) xmin = 0;
  if (xmax >= width) xmax = width - 1;
  if (ymin < 0) ymin = 0;
  if (ymax >= height) ymax = height - 1;
  
  for (int y = ymin; y <= ymax; y++)
    {
      unsigned char* start_pnt = Image::get_pixel(xmin,y);
      unsigned char* end_pnt = Image::get_pixel(xmax,y);
      while (start_pnt <= end_pnt)
	{
	  if (*start_pnt == 0) *start_pnt = curr_id;
	  else *start_pnt = 255;
	  start_pnt++;
	}
    }
}
	
void OcclusionImage::setup_occlusion_map(TrackedProfile* prf)
{
  curr_id = (prf->ref_no & 127);
}

bool OcclusionImage::is_off_image(int x, int y)
{
  
  if ((x >= big_width) || (x < 0) || (y < 0) || (y >= big_height))
    return (TRUE);

  return (FALSE);

}


bool OcclusionImage::is_occlusion(int x, int y)
{
  unsigned char* pnt = check_pixel(x >> RESAMPLE_SHIFT, y >> RESAMPLE_SHIFT);
  if (pnt == NULL) return (FALSE);
  if ((*pnt != curr_id) && (*pnt != 0)) return (TRUE);
  return (FALSE);
}



