/*
This source file is the copyright property of the University of Leeds
('The University').

Permission to use, copy, modify, and distribute this source file for
educational, research, and not-for-profit purposes, without fee and
without a signed licensing agreement, is hereby granted, provided that
the above copyright notice, this paragraph and the following three
paragraphs appear in all copies, modifications, and distributions.

In no event shall The University be liable to any party for direct,
indirect, special, incidental or consequential damages, including lost
profits, arising out of the use of this software and its documentation.

The software is provided without warranty. The University has no
obligation to provide maintenance, support, updates, enhancements, or
modifications.

This software was written by Adam Baumberg, Vision Group,
School of Computer Studies, University of Leeds, UK. The code and use
thereof should be attributed to the author where appropriate
(including demonstrations which rely on it's use).
*/

/***************************************************************
 * C - C++ Header
 *
 * File : 	OcclusionImage.h
 *
 * Module :	OcclusionImage
 *
 * Author : 	A M Baumberg
 *
 * Creation Date : Wed May 25 15:36:16 1994 
 *
 * Comments : 	a simple 2D occlusion handler without depth 
 *		ordering
 *
 ***************************************************************/


/* include files */

#ifndef __OCCLUSION_IMAGE_H__
#define __OCCLUSION_IMAGE_H__

#include <OcclusionHandler.h>
#include <RGB8Image.h>
#include <boolean.h>


#define OCCL_SHIFT 2
 
class TrackedProfile;

class OcclusionImage : public OcclusionHandler, public RGB8Image
{
  
public:

  char curr_id;
  int big_width, big_height;
  static int RESAMPLE_SHIFT;

  OcclusionImage(int w, int h) : RGB8Image(w >> RESAMPLE_SHIFT, 
					   h >> RESAMPLE_SHIFT) 
  { big_width = w; big_height = h; }

  void mark_rect(TrackedProfile* prf);

  void setup_occlusion_map(TrackedProfile* );
  bool is_occlusion(int x, int y);
  bool is_off_image(int x, int y);

  
};



#endif
