/*
This source file is the copyright property of the University of Leeds
('The University').

Permission to use, copy, modify, and distribute this source file for
educational, research, and not-for-profit purposes, without fee and
without a signed licensing agreement, is hereby granted, provided that
the above copyright notice, this paragraph and the following three
paragraphs appear in all copies, modifications, and distributions.

In no event shall The University be liable to any party for direct,
indirect, special, incidental or consequential damages, including lost
profits, arising out of the use of this software and its documentation.

The software is provided without warranty. The University has no
obligation to provide maintenance, support, updates, enhancements, or
modifications.

This software was written by Adam Baumberg, Vision Group,
School of Computer Studies, University of Leeds, UK. The code and use
thereof should be attributed to the author where appropriate
(including demonstrations which rely on it's use).
*/

/***************************************************************
 * C++ source
 *
 * File : 	ReadingExternalData.cc
 *
 * Module :	ReadingExternalData
 *
 * Author : 	A M Baumberg (CoMIR)
 *
 * Creation Date : Fri Oct 25 10:29:34 1996 
 *
 * Comments : 	
 *
 ***************************************************************/


/* include files */

#include "ReadingExternalData.h"

// read in data from a file
static bool parse_data_file(char* filename, 
			    TrackedObjectList* new_objs)
{
  ifstream data_in(filename);
  if (!data_in) return (FALSE);
  new_objs->destroy_all();
  char dummy[100];	// input buffer
  while (!data_in.eof())
    {
      strcpy(dummy,"    ");	// clear dummy buffer string
      data_in >> dummy;
      if (strcmp(dummy,"label") == 0)
	{
	  int id; 
	  data_in >> id;
	  Point2 p1,p2,p3,p4;
	  data_in >> p1.x >> p1.y >> p2.x >> p2.y >> p3.x >> p3.y 
		  >> p4.x >> p4.y;
	  new_objs->add(new TrackedObject(p1,p2,p3,p4,id, id,
					  TrackedObject::CAR));
	}
    }
  return (TRUE);
}


  
// patch to re-label car id's
// due to Reading bug !
//
// leave id_char's to ensure compatibility with 
// associated image

static void patch_reading_data(TrackedObjectList* prev_cars,
			       TrackedObjectList* new_cars)
{
  if (prev_cars == NULL) return;


  for (new_cars->start(); new_cars->current_ok(); new_cars->forward())
    {
      if (new_cars->get_current().type != TrackedObject::CAR)
	continue;

      realno closest_dist = 1e6;
      TrackedObject* closest = NULL;
      for (prev_cars->start(); prev_cars->current_ok(); prev_cars->forward())
	{
	  realno dist = 
	    (new_cars->get_current().GroundPlaneOrigin - 
	     prev_cars->get_current().GroundPlaneOrigin).length2();
	  if (dist < closest_dist)
	    {
	      closest = &prev_cars->get_current();
	      closest_dist = dist;
	    }
	}
      if (closest_dist < (0.2 * 0.2 * new_cars->get_current().sq_size()))
	{
	  // relabel the new object
	  new_cars->get_current().obj_id = closest->obj_id;
	}
    }
}


void ReadingExternalData::get_next_frame()
{
  int next_frame = views_images->get_frame_no() + frame_skip;

  if (views_images->get_next() != NULL)
    {
      char *tmp_buffer = new char[strlen(data_file_prefix) +
				 views_images->no_digits + 
				 strlen(data_file_suffix) + 1];
      sprintf(tmp_buffer,views_images->format_string,data_file_prefix,
	      views_images->curr_frame-1,data_file_suffix);

      TrackedObjectList* new_objs = new TrackedObjectList;
      if (parse_data_file(tmp_buffer, new_objs))
	{
	  patch_reading_data(footprint_data, new_objs);
	  if (footprint_data != NULL)
	    {
	      footprint_data->destroy_all();
	      delete (footprint_data);
	    }
	  footprint_data = new_objs;
	}
      else delete (new_objs);
      delete [] tmp_buffer;
    }

  CamView = views_images->get_current();
  views_images->goto_frame(next_frame);
}

      
