/*
This source file is the copyright property of the University of Leeds
('The University').

Permission to use, copy, modify, and distribute this source file for
educational, research, and not-for-profit purposes, without fee and
without a signed licensing agreement, is hereby granted, provided that
the above copyright notice, this paragraph and the following three
paragraphs appear in all copies, modifications, and distributions.

In no event shall The University be liable to any party for direct,
indirect, special, incidental or consequential damages, including lost
profits, arising out of the use of this software and its documentation.

The software is provided without warranty. The University has no
obligation to provide maintenance, support, updates, enhancements, or
modifications.

This software was written by Adam Baumberg, Vision Group,
School of Computer Studies, University of Leeds, UK. The code and use
thereof should be attributed to the author where appropriate
(including demonstrations which rely on it's use).
*/

/***************************************************************
 * C - C++ Header
 *
 * File : 	ReadingExternalData.h
 *
 * Module :	ReadingExternalData
 *
 * Author : 	A M Baumberg (CoMIR)
 *
 * Creation Date : Fri Oct 25 10:11:24 1996 
 *
 * Comments : 	Reading as in the place
 *
 ***************************************************************/


/* include files */

#ifndef __READING_EXTERNAL_DATA_H__
#define __READING_EXTERNAL_DATA_H__

#include <ExternalData.h>
#include <PgmSource.h>
#include <utils.h>

class ReadingExternalData : public ExternalData
{  
protected:

  PgmSource* views_images;
  char* data_file_prefix;
  char* data_file_suffix;

  int frame_skip;

public:

  ReadingExternalData(char* pgm_filename, char* dat_file, 
		      int first_frame = -1, int fskip = 1)
  : ExternalData()
  {
    views_images = new PgmSource(pgm_filename);
    if (first_frame != -1) views_images->goto_frame(first_frame);
    char *tmp;
    if (split_num_filename(dat_file, data_file_prefix, 
			   tmp, data_file_suffix))
      delete [] tmp;
    else
      {
	data_file_prefix = name_mangle(dat_file, NULL);
	data_file_suffix = ".dat";
      }
    frame_skip = fskip;
  }

  void get_next_frame();

};    
    


#endif
