/*
This source file is the copyright property of the University of Leeds
('The University').

Permission to use, copy, modify, and distribute this source file for
educational, research, and not-for-profit purposes, without fee and
without a signed licensing agreement, is hereby granted, provided that
the above copyright notice, this paragraph and the following three
paragraphs appear in all copies, modifications, and distributions.

In no event shall The University be liable to any party for direct,
indirect, special, incidental or consequential damages, including lost
profits, arising out of the use of this software and its documentation.

The software is provided without warranty. The University has no
obligation to provide maintenance, support, updates, enhancements, or
modifications.

This software was written by Adam Baumberg, Vision Group,
School of Computer Studies, University of Leeds, UK. The code and use
thereof should be attributed to the author where appropriate
(including demonstrations which rely on it's use).
*/

/***************************************************************
 * C - C++ Header
 *
 * File : 	SimplePeopleTracker.h
 *
 * Module :	SimplePeopleTracker
 *
 * Author : 	A M Baumberg (CoMIR)
 *
 * Creation Date : Tue Oct 22 13:32:53 1996 
 *
 * Comments : 	High level people tracker without ground plane info
 *
 ***************************************************************/


#ifndef __SIMPLE_PEOPLE_TRACKER_H__
#define __SIMPLE_PEOPLE_TRACKER_H__

#include <OcclusionImage.h>
#include <ActiveModel.h>
#include <MotionDetector.h>
#include <TrackedProfileSet.h>

class SimplePeopleTracker
{
protected:

  Image *previous_frame;

public:

  // used to handle occlusions
  OcclusionImage* occ_handler;

  // local image fitting
  ActiveModel* local_optimiser;


  // current tracked people
  ProfileSet* profiles;

  MotionDetector* detect_motion;

  // globals

  static realno FITNESS_THRESHOLD;
  static realno VERY_POOR_FIT;
  static int SMALL_NO_FRAMES;
  static realno MIN_SIGNIFICANCE;
  static realno terminate_thresh;

  // uncertainties for Kalman filtering
  // relative to the initial object height
  static realno init_pos_sd;
  static realno init_vel_sd;
  static realno noise_vel_sd;
  static realno noise_acc_sd;


  static realno init_scale_sd;	// the std dev of initial scale estimate
  static realno noise_scale_sd;	// the std dev of scale noise term
  static realno init_theta_sd;	// the std dev of initial theta estimate
  static realno noise_theta_sd; // the std dev of theta noise term

  static int TEMPORAL_SKIP;

public:

  SimplePeopleTracker(OcclusionImage*,   ActiveModel*,
		      MotionDetector*, ProfileSet* = NULL);
  
  
  SimplePeopleTracker(ImageSource*, char* pca_file = NULL,
		      ProfileSet* = NULL,
		      FeatureDetector* = NULL);

  void predict_profiles();

  void track_all();

  void track_new_objects();

  void initialise_track(TrackedProfile* );

  // tracking loop for each new image
  void MainLoop(int max_objects = 10);

  // get the tracked profiles
  TrackedProfileSet* get_tracked_profiles()
  { return ((TrackedProfileSet*) profiles); }      
};

#endif


