/*
This source file is the copyright property of the University of Leeds
('The University').

Permission to use, copy, modify, and distribute this source file for
educational, research, and not-for-profit purposes, without fee and
without a signed licensing agreement, is hereby granted, provided that
the above copyright notice, this paragraph and the following three
paragraphs appear in all copies, modifications, and distributions.

In no event shall The University be liable to any party for direct,
indirect, special, incidental or consequential damages, including lost
profits, arising out of the use of this software and its documentation.

The software is provided without warranty. The University has no
obligation to provide maintenance, support, updates, enhancements, or
modifications.

This software was written by Adam Baumberg, Vision Group,
School of Computer Studies, University of Leeds, UK. The code and use
thereof should be attributed to the author where appropriate
(including demonstrations which rely on it's use).
*/

/***************************************************************
 * C - C++ Header
 *
 * File : 	TrackedObject.h
 *
 * Module :	TrackedObject
 *
 * Author : 	A M Baumberg (CoMIR)
 *
 * Creation Date : Thu Aug  8 16:51:46 1996 
 *
 * Comments : 	for handling cars/ tracked people etc
 *
 ***************************************************************/


/* include files */

#ifndef __TRACKED_OBJECT_H__
#define __TRACKED_OBJECT_H__

#include <point2.h>
#include <GroundPlaneToImagePlane.h>

#define MAX_OBJECT_ID 256

class TrackedObject
{
public:

  enum ObjectType {CAR, PERSON};
  ObjectType type;
  int obj_id;		// object reference number
  char id_char;		// object image greylevel

  Point2 GroundPlaneBB[4];
  Point2 GroundPlaneOrigin;

  int no_active_frames;	// number of frames object has been tracked
  Point2 BB_velocity[4]; // running average of the bbox displacements
  Point2 velocity;	// running average of the origin displacement

  realno depth;		// depth wrt image plane
  bool is_stationary;	// TRUE if object has stopped
  int no_stationary_frames; // how long has object been stopped

  // constructors
  TrackedObject(Point2 p1, Point2 p2, Point2 p3, Point2 p4,
		int id, char ch_id, ObjectType type = TrackedObject::CAR);
  TrackedObject(Point2 pos, int id, char ch_id, ObjectType type =
		TrackedObject::PERSON);

  // null constructor
  TrackedObject() {}

  realno sq_size() // square length of bbox diagonal 
  { return ((GroundPlaneBB[0] - GroundPlaneBB[2]).length2()); }

  realno size() { return sqrt(sq_size()); }

  // is_point_hidden:
  // returns TRUE if pnt is behind TrackedObject
  bool is_point_hidden(Point2& pnt);

  // occludes:
  // return TRUE if obj2 is behind TrackedObject
  bool occludes(TrackedObject& obj2);

  // update:
  // update filters etc using values from previous frame
  void update(TrackedObject& prev_state);

  // encloses:
  // return TRUE if the bounding box encloses the point
  bool encloses(Point2 pnt);

  // return the nearest point on the bounding box lines
  Point2 nearest_boundary_point(Point2 pnt);
    
  Point2 constrain_movement(Point2 old_pos, Point2 new_pos);

};

ostream& operator<<(ostream&, TrackedObject&);
istream& operator>>(istream&, TrackedObject&);  

#endif
