/*
This source file is the copyright property of the University of Leeds
('The University').

Permission to use, copy, modify, and distribute this source file for
educational, research, and not-for-profit purposes, without fee and
without a signed licensing agreement, is hereby granted, provided that
the above copyright notice, this paragraph and the following three
paragraphs appear in all copies, modifications, and distributions.

In no event shall The University be liable to any party for direct,
indirect, special, incidental or consequential damages, including lost
profits, arising out of the use of this software and its documentation.

The software is provided without warranty. The University has no
obligation to provide maintenance, support, updates, enhancements, or
modifications.

This software was written by Adam Baumberg, Vision Group,
School of Computer Studies, University of Leeds, UK. The code and use
thereof should be attributed to the author where appropriate
(including demonstrations which rely on it's use).
*/

/***************************************************************
 * C++ source
 *
 * File : 	TrackedProfile.h
 *
 * Module :	TrackedProfile
 *
 * Author : 	A M Baumberg
 *
 * Creation Date : Thu Apr 21 14:48:21 1994 
 *
 * Comments : 	
 *
 ***************************************************************/


#ifndef __TRACKED_PROFILE_H__
#define __TRACKED_PROFILE_H__

#include <KalmanFilter.h>
#include <profile.h>

class ColourBlob;
class GPfilter;


class TrackedProfile : public Profile
{
public:
  

  realno a_x, a_y;			// alignment parameters
  int no_shape_parameters;		// equivalent to model depth!
  StaticKalmanTwoD* a_filter;   // alignment (scale and rotation) filter
  FilterOneD** filters;		// shape filters

  FilterTwoD* pos_filter; // origin filter

  StaticKalmanOneD* s_filter; // scale filter or NULL
  StaticKalmanOneD* th_filter; // rotation filter or NULL
  GPfilter* gp_filter; // (Ground plane position filter or NULL)

  int time_alive;		// no of frames object has been
				// tracked

  realno frame_error;		// the error on last active fit

  ColourBlob* colour_info;	// for tracking coloured edges

  TrackedProfile(Profile& original, int ref) : Profile(original)
    { filters = NULL;
      pos_filter = NULL; 
      a_filter = NULL;      
      s_filter = NULL;
      th_filter = NULL;
      ref_no = ref;
      colour_info = NULL;
      gp_filter = NULL;
      time_alive = 0;
      a_x = a_y = 0;
      frame_error = 0;
    }

  TrackedProfile(TrackedProfile& original);

  TrackedProfile(int ref = 0) 
  {   
    filters = NULL;
    pos_filter = NULL; 
    a_filter = NULL;      
    s_filter = NULL;
    th_filter = NULL;
    ref_no = ref;
    colour_info = NULL;
    gp_filter = NULL;
    time_alive = 0;
    a_x = a_y = 0;
    frame_error = 0;
  }

  ~TrackedProfile()
    {
      delete a_filter;
      if (filters != NULL) 
	{
	  for (int i = 0; i < no_shape_parameters; i++)
	    delete filters[i];
	  delete [] filters;
	}
      delete pos_filter;
      if (colour_info != NULL) delete [] colour_info;
      delete gp_filter;
    }
  void draw(bool flag = TRUE);

  ProfType get_type() { return (TRACKED_PROFILE); }
  void map_to_model_frame()
    {
      realno det = a_x * a_x + a_y * a_y;
      this->map(a_x / det, -a_y / det);
    }

  // get enclosing box
  // return a box that encloses the current shape 
  // taking into account the positional uncertainty
  void get_enclosing_box(int& xmin, int& xmax, int& ymin, int& ymax);

  void dump_reading_info(ostream& strm_out);
  void output_to_stream(ostream& strm_out);

  void update_shape_filters();

};


#endif
