/*
This source file is the copyright property of the University of Leeds
('The University').

Permission to use, copy, modify, and distribute this source file for
educational, research, and not-for-profit purposes, without fee and
without a signed licensing agreement, is hereby granted, provided that
the above copyright notice, this paragraph and the following three
paragraphs appear in all copies, modifications, and distributions.

In no event shall The University be liable to any party for direct,
indirect, special, incidental or consequential damages, including lost
profits, arising out of the use of this software and its documentation.

The software is provided without warranty. The University has no
obligation to provide maintenance, support, updates, enhancements, or
modifications.

This software was written by Adam Baumberg, Vision Group,
School of Computer Studies, University of Leeds, UK. The code and use
thereof should be attributed to the author where appropriate
(including demonstrations which rely on it's use).
*/

/***************************************************************
 * C - C++ Header
 *
 * File : 	TrackedProfileSet.h
 *
 * Module :	TrackedProfileSet
 *
 * Author : 	A M Baumberg (CoMIR)
 *
 * Creation Date : Wed Nov  6 13:47:25 1996 
 *
 * Comments : 	a ProfileSet with added functionality
 *
 ***************************************************************/


/* include files */

#ifndef __TRACKED_PROFILE_SET_H__
#define __TRACKED_PROFILE_SET_H__

#include <ProfileSet.h>
#include <TrackedProfile.h>
#include <AmbErrorHandler.h>

class TrackedProfileSet : public ProfileSet
{

public:

  TrackedProfile& get_current() 
  { 
    if (ProfileSet::get_current().get_type() != TRACKED_PROFILE)
      {
	// AmbErrorHandler.handle_error("can not cast a Profile to \
	AmbErrorHandler::handle_error("can not cast a Profile to \
a TrackedProfile in TrackedProfileSet::get_current");
	return (NULL);
      }
    return *((TrackedProfile*) &ProfileSet::get_current());
  }

  map_to_model_frame()
  {
    for (start(); current_ok(); forward())
      get_current().map_to_model_frame();
  }

  // draw the outlines onto the image data
  draw_in_image(Image* img, int short_time = 0)
  {
    for (start(); current_ok(); forward())
      {
	if (get_current().time_alive > short_time)
	  {
	    img->set_line_width(3.0);
	    // pick a colour for this object
	    int r = (get_current().ref_no & 1) * 255;
	    int g = (get_current().ref_no & 2) * 255;
	    int b = (get_current().ref_no & 4) * 255;
	    img->set_draw_colour(r,g,b);
	    // draw into image data
	    get_current().draw_in_image(img, FALSE);
	  }
      }
  }

  // return a list of TrackedProfile's which have
  // been alive for more than "short_time" frames
  // list nodes should be removed with delete_all
  TrackedProfileSet filter_out_short(int short_time)
  {
    TrackedProfileSet res;
    for (start(); current_ok(); forward())
      {
	if (get_current().time_alive > short_time)
	  res.add(&get_current());
      }
    // result should be cleaned up using delete_all()
    return (res);
  }

};

#endif
