/*
This source file is the copyright property of the University of Leeds
('The University').

Permission to use, copy, modify, and distribute this source file for
educational, research, and not-for-profit purposes, without fee and
without a signed licensing agreement, is hereby granted, provided that
the above copyright notice, this paragraph and the following three
paragraphs appear in all copies, modifications, and distributions.

In no event shall The University be liable to any party for direct,
indirect, special, incidental or consequential damages, including lost
profits, arising out of the use of this software and its documentation.

The software is provided without warranty. The University has no
obligation to provide maintenance, support, updates, enhancements, or
modifications.

This software was written by Adam Baumberg, Vision Group,
School of Computer Studies, University of Leeds, UK. The code and use
thereof should be attributed to the author where appropriate
(including demonstrations which rely on it's use).
*/

/***************************************************************
 * C - C++ Header
 *
 * File : 	WorldOcclusionHandler.h
 *
 * Module :	WorldOcclusionHandler
 *
 * Author : 	A M Baumberg (CoMIR)
 *
 * Creation Date : Fri Aug  9 15:21:49 1996 
 *
 * Comments : 	Class to handle occlusion using world 
 *		ground plane depth ordering
 *
 ***************************************************************/


// This class ...

// (i) sets up a *mask* image for the Motion Detector 

// (ii) Draws (predicted) pedestrian silhouettes in to an Image Plane
// view

// (iii) Sets up an OcclusionImage for each tracked pedestrian

// (iv) handles input of Reading data


#ifndef __WORLD_OCCLUSION_HANDLER_H__
#define __WORLD_OCCLUSION_HANDLER_H__

#include <OcclusionHandler.h>
#include <TrackedObjectList.h>
#include <ProfileSet.h>
#include <imgsrc.h>
#include <PgmSource.h>
#include <MotionDetector.h>
#include <ProfileSet.h>
#include <ExternalData.h>

class TrackedProfile;

class WorldOcclusionHandler : public OcclusionHandler
{
public:

  TrackedObjectList* curr_objects; // current frame
  TrackedObjectList* prev_objects; // previous frame

  Image* CameraView;	// Image Plane view
  Image* backup_view;  
  Image* viz_image;	// for visualisations

  // Occlusion data from an external object tracker 
  ExternalData* car_data;

  int stop_time;
  int sample_skip;		// spatial resampling

  static bool quick_mode;	// if TRUE then people are boxes for
			        // occlusion reasoning

  unsigned char flags[MAX_OBJECT_ID];

  ImageSource* video_images;	// raw video images

  MotionDetector* movement_detector;
  ProfileSet* tracked_people;

  WorldOcclusionHandler(ImageSource* input_images, 
			MotionDetector* mdetect, ProfileSet* people, 
			ExternalData* cars = NULL);

		   
  void update_frame(); // read in next frame 

  void setup_motion_mask(); // setup mask of regions to ignore

  // setup flags for objects that might occlude 
  void setup_occlusion_map(TrackedProfile* prf);

  // draw rectangles into camera view for occlusion
  void update_profiles();

  // setup_viz_image
  // draw the full sized accurate camera view image
  Image* setup_viz_image();

  // draw accurate shapes or boxes in to a resampled image
  void render_profiles(ProfileSet& profiles, bool use_boxes,Image* theView,
		       int skip, bool viz_mode = FALSE);

  void clear_flags();

  bool is_occlusion(int x, int y);

  // return TRUE if pixel is not in image
  bool is_off_image(int x, int y);

  bool merge_tracks();

  void constrain_position(TrackedProfile* currx);
  void constrain_motion(TrackedProfile* currx);   
};

#endif


