/*
This source file is the copyright property of the University of Leeds
('The University').

Permission to use, copy, modify, and distribute this source file for
educational, research, and not-for-profit purposes, without fee and
without a signed licensing agreement, is hereby granted, provided that
the above copyright notice, this paragraph and the following three
paragraphs appear in all copies, modifications, and distributions.

In no event shall The University be liable to any party for direct,
indirect, special, incidental or consequential damages, including lost
profits, arising out of the use of this software and its documentation.

The software is provided without warranty. The University has no
obligation to provide maintenance, support, updates, enhancements, or
modifications.

This software was written by Adam Baumberg, Vision Group,
School of Computer Studies, University of Leeds, UK. The code and use
thereof should be attributed to the author where appropriate
(including demonstrations which rely on it's use).
*/

/***************************************************************
 * C++ source
 *
 * File : 	AmbErrorHandler.cc
 *
 * Module :	AmbErrorHandler
 *
 * Author : 	A M Baumberg (CoMIR)
 *
 * Creation Date : Wed Oct 16 16:08:09 1996 
 *
 * Comments : 	
 *
 ***************************************************************/


/* include files */

#include "AmbErrorHandler.h"
#include "stdio.h"

// default behaviour determined by environment variables
// at run time
EnvParameter AmbErrorHandler::quiet_mode("AMB_QUIET_MODE", FALSE);
EnvParameter AmbErrorHandler::exit_on_error("AMB_EXIT_ON_ERROR",
					    TRUE);


char AmbErrorHandler::buffer[255] = "\0";

void AmbErrorHandler::set_quiet_mode(bool flag)
{
  if (flag) quiet_mode.set_on();
  else quiet_mode.set_off();
}


void AmbErrorHandler::set_exit_on_error(bool flag)
{
  if (flag) exit_on_error.set_on();
  else exit_on_error.set_off();
}

  
void AmbErrorHandler::ignore_all()
{
  set_quiet_mode(TRUE); set_exit_on_error(FALSE);
}


void AmbErrorHandler::handle_warning(char* message)
{
  if (quiet_mode.is_off())
    {
      fprintf(stderr,"warning: %s", message);
      fprintf(stderr, buffer);
      fprintf(stderr,"\n");
      buffer[0] = '\0';
    }
  // set breakpoint here !
}


void AmbErrorHandler::handle_error(char* message)
{
  if (quiet_mode.is_off())
    {
      fprintf(stderr,"error: %s", message);
      fprintf(stderr, buffer);
      fprintf(stderr,"\n");
      buffer[0] = '\0';
    }

  if (exit_on_error.is_off()) return;

  if (quiet_mode.is_off())
    fprintf(stderr,"exiting \n use setenv AMB_EXIT_ON_ERROR 0 to \
ignore errors\n");
  exit(1);
}
