/*
This source file is the copyright property of the University of Leeds
('The University').

Permission to use, copy, modify, and distribute this source file for
educational, research, and not-for-profit purposes, without fee and
without a signed licensing agreement, is hereby granted, provided that
the above copyright notice, this paragraph and the following three
paragraphs appear in all copies, modifications, and distributions.

In no event shall The University be liable to any party for direct,
indirect, special, incidental or consequential damages, including lost
profits, arising out of the use of this software and its documentation.

The software is provided without warranty. The University has no
obligation to provide maintenance, support, updates, enhancements, or
modifications.

This software was written by Adam Baumberg, Vision Group,
School of Computer Studies, University of Leeds, UK. The code and use
thereof should be attributed to the author where appropriate
(including demonstrations which rely on it's use).
*/

/***************************************************************
 * C - C++ Header
 *
 * File : 	AmbErrorHandler.h
 *
 * Module :	AmbErrorHandler
 *
 * Author : 	A M Baumberg (CoMIR)
 *
 * Creation Date : Wed Oct 16 15:49:11 1996 
 *
 * Comments : 	
 *
 ***************************************************************/


/* include files */

#ifndef __ERROR_HANDLER_H__
#define __ERROR_HANDLER_H__

// My global error handler routine
// either print message / exit or ignore
// depending on switches which can be set from within
// the top-level program or using environment variables
// (see AmbErrorHandler.cc)

// e.g. to ignore all errors use ...
// AmbErrorHandler::ignore_all();

#include "EnvParameter.h"

// this is a "Pure Static Class"
// (no constructor)

class AmbErrorHandler
{
protected:

  static EnvParameter quiet_mode;
  static EnvParameter exit_on_error;
	
public:
  
  // sprintf additional messages into this buffer
  // for outputing in warning/ error methods
  static char buffer[];

  // these static functions are invoked using
  // AmbErrorHandler::function

  static void set_quiet_mode(bool flag = TRUE) ;

  static void set_exit_on_error(bool flag = TRUE) ;
  
  static void ignore_all();

  static void handle_warning(char* message);
  static void handle_error(char* message);

  static bool is_quiet_mode() { return (quiet_mode.is_on()); }

};

#endif

