/*
This source file is the copyright property of the University of Leeds
('The University').

Permission to use, copy, modify, and distribute this source file for
educational, research, and not-for-profit purposes, without fee and
without a signed licensing agreement, is hereby granted, provided that
the above copyright notice, this paragraph and the following three
paragraphs appear in all copies, modifications, and distributions.

In no event shall The University be liable to any party for direct,
indirect, special, incidental or consequential damages, including lost
profits, arising out of the use of this software and its documentation.

The software is provided without warranty. The University has no
obligation to provide maintenance, support, updates, enhancements, or
modifications.

This software was written by Adam Baumberg, Vision Group,
School of Computer Studies, University of Leeds, UK. The code and use
thereof should be attributed to the author where appropriate
(including demonstrations which rely on it's use).
*/

/***************************************************************
 * C - C++ Header
 *
 * File : 	EnvParameter.h
 *
 * Module :	
 *
 * Author : 	A M Baumberg (CoMIR)
 *
 * Creation Date : Tue Feb 28 09:22:37 1995 
 *
 * Comments : 	
 *
 ***************************************************************/


// An Enviroment Parameter is a boolean switch which may
// be set on or off by use of a shell variable

#ifndef __ENV_PARAMETER_H__
#define __ENV_PARAMETER_H__

#include <stdlib.h>
#include "boolean.h"
#include <string.h>
#include <stdio.h>
#include "realno.h"

#define ENV_VERBOSE "ENV_VERBOSE"


// should be EnvBoolParam
class EnvParameter
{
private:

  bool flag;

public:

  EnvParameter(char* shell_string, bool def = TRUE)
    {
      char* s = getenv(shell_string);
      if (s == NULL) // undefined set flag to default
	flag = def;
      else
	{
	  flag = !def;
	  if (strcasecmp(s, "OFF") == 0) flag = FALSE;
	  if (strcasecmp(s, "ON") == 0)  flag = TRUE;

	  if (strcmp(s, "0") == 0) flag = FALSE;
	  if (strcmp(s, "1") == 0)  flag = TRUE;

	}
      if (getenv(ENV_VERBOSE) != NULL)
	{
	  if (flag) fprintf(stderr,"%s: enabled\n", shell_string);
	  else fprintf(stderr,"%s: disabled\n", shell_string);
	}
    }

  void set_on() { flag = TRUE; }
  void set_off() { flag = FALSE; }
  void change_state() { flag = !flag; }
  bool get_state() { return (flag); }
  bool is_on() { return (flag == TRUE); }
  bool is_off() { return (flag == FALSE); }
};
	  
class EnvIntParameter
{
public:
  
  int value;

public:

  EnvIntParameter(char* shell_string, int def = 0)
    {
      char* s = getenv(shell_string);
      if (s == NULL) value = def;
      else value = atoi(s);
      if (getenv(ENV_VERBOSE) != NULL)
	fprintf(stderr,"%s = %d\n", shell_string, value);
    }
};


class EnvRealParameter
{
public:
  
  realno value;

public:

  EnvRealParameter(char* shell_string, realno def = 0)
    {
      char* s = getenv(shell_string);
      if (s == NULL) value = def;
      else value = atof(s);
      if (getenv(ENV_VERBOSE) != NULL)
	fprintf(stderr,"%s = %f\n", shell_string, value);
    }
};

class EnvStringParameter
{
public:

  char* value;

  EnvStringParameter(char* var_name, char* default_str = "")
  {
    char* s = getenv(var_name);
    if (s == NULL) value = default_str;
    else value = s;
    if (getenv(ENV_VERBOSE) != NULL)
      fprintf(stderr,"%s = %s\n", var_name, value);
  }
};


#endif
