/*
This source file is the copyright property of the University of Leeds
('The University').

Permission to use, copy, modify, and distribute this source file for
educational, research, and not-for-profit purposes, without fee and
without a signed licensing agreement, is hereby granted, provided that
the above copyright notice, this paragraph and the following three
paragraphs appear in all copies, modifications, and distributions.

In no event shall The University be liable to any party for direct,
indirect, special, incidental or consequential damages, including lost
profits, arising out of the use of this software and its documentation.

The software is provided without warranty. The University has no
obligation to provide maintenance, support, updates, enhancements, or
modifications.

This software was written by Adam Baumberg, Vision Group,
School of Computer Studies, University of Leeds, UK. The code and use
thereof should be attributed to the author where appropriate
(including demonstrations which rely on it's use).
*/

/***************************************************************
 * C++ source
 *
 * File : 	GlobalsManager.cc
 *
 * Module :	GlobalsManager
 *
 * Author : 	A M Baumberg (CoMIR)
 *
 * Creation Date : Mon Oct 21 13:33:42 1996 
 *
 * Comments : 	
 *
 ***************************************************************/


/* include files */

#include "GlobalsManager.h"
#include "AmbErrorHandler.h"
#include <strstream.h>
#include <math.h>
#include <ieeefp.h>

#include <X11/X.h>
#include <X11/Xlib.h>
#include <X11/Intrinsic.h>
#include <Xm/RowColumn.h>
#include <Xm/ToggleB.h>
#include <Xm/PushB.h>
#include <Xm/MessageB.h>
#include <X11/cursorfont.h>
#include <Xm/TextF.h>
#include <Xm/Label.h>
#include <Xm/Text.h>
#include <Xm/Scale.h>
#include <Xm/Form.h>
#include <Xm/Separator.h>
#include <Xm/CascadeB.h>
#include <Xm/FileSB.h>
#include <Xm/SelectioB.h>
#include <Xm/DialogS.h>
#include <sys/types.h>
#include <sys/prctl.h>
#include <sys/types.h>
#include <unistd.h>
#include <sys/types.h>
#include <signal.h>

#include <strstream.h>

// this object should be initialize in compile-time
// (before run-time initialization using the register_*** functions)
GlobalsManager AmbGlobals = {NULL, 0, 0};

void GlobalVariable::full_printout(ostream& strm_out )
{
  strm_out << "Tag: " << tag << endl;
  if (fixed) strm_out << "constant " << endl;
  else strm_out << "variable " << endl;
  if (owner_class != NULL)
    strm_out << "object class: " << owner_class << endl;
  if (help_string != NULL)
    strm_out << "help info: " << help_string << endl;
}
 
bool RealGlobalVariable::read_value(istream& strm_in)
{
  realno new_value;
  strm_in >> new_value;
  if (constrained)
    {
      if ((new_value > max) || (new_value < min))
	return (FALSE);
    }
  value = new_value;
  return (TRUE);
}

void RealGlobalVariable::output_value(ostream& strm_out)
{
  if (real_isnan(value)) strm_out << "Undefined" << endl;
  else strm_out << value << endl;
}

void RealGlobalVariable::full_printout(ostream& strm_out )
{
  GlobalVariable::full_printout(strm_out);
  strm_out << "current value "; output_value(strm_out);
  if (constrained)
    {
      strm_out << "( min: " << min << " max: "
	   << max << ")";
    }
  strm_out << endl;
}


bool IntGlobalVariable::read_value(istream& strm_in)
{
  int new_value;
  strm_in >> new_value;
  if (constrained)
    {
      if ((new_value > max) || (new_value < min))
	return (FALSE);
    }
  value = new_value;
  return (TRUE);
}

void IntGlobalVariable::output_value(ostream& strm_out)
{
  strm_out << value << endl;
}

void IntGlobalVariable::full_printout(ostream& strm_out )
{
  GlobalVariable::full_printout(strm_out);
  strm_out << "current value " << value;
  if (constrained)
    {
      strm_out << "( min: " << min << " max: "
	   << max << ")";
    }
  strm_out << endl;
}

bool BoolGlobalVariable::read_value(istream& strm_in)
{
  strm_in >> value;
  return (TRUE);
}

void BoolGlobalVariable::output_value(ostream& strm_out)
{
  strm_out << value << endl; 
}

void BoolGlobalVariable::full_printout(ostream& strm_out )
{
  GlobalVariable::full_printout(strm_out);
  strm_out << "current value ";
  if (value) strm_out << "TRUE" << endl;
  else strm_out << "FALSE" << endl;
}

bool StringGlobalVariable::read_value(istream& strm_in)
{
  static char buffer[255];
  strm_in >> buffer;
  char* new_str = new char[strlen(buffer)+1];
  strcpy(new_str, buffer);
  value = new_str;
  return (TRUE);
}

void StringGlobalVariable::output_value(ostream& strm_out)
{
  if (value != NULL)
    strm_out << value << endl;
}

void StringGlobalVariable::full_printout(ostream& strm_out )
{
  GlobalVariable::full_printout(strm_out);
  
  if (value != NULL) strm_out << "current value: " << value << endl;
  else strm_out << "not set to any value" << endl;
}


// GlobalsManager::GlobalsManager()
// {
//   max_globals = 100;
//   no_globals = 0;
//   // set up array of pointers
//   table = new GlobalVariable*[max_globals];
// }

GlobalsManager::~GlobalsManager()
{
  for (int i = 0; i < no_globals; i++)
    delete table[i];

  delete [] table;
}

void GlobalsManager::register_global_variable(GlobalVariable* var)
{
  if (table == NULL)
    { 
      max_globals = 100;
      no_globals = 0;
      // set up array of pointers
      table = new GlobalVariable*[max_globals];
      flags = NULL;
    }  

  if ((no_globals+1) < max_globals)
    table[no_globals++] = var;
  else
    {
      int max_globals2 = 2 * max_globals;
      GlobalVariable** table2 = new GlobalVariable*[max_globals2];
      memcpy(table2, table, sizeof(table));
      delete [] table;
      table = table2;
      max_globals = max_globals2;
      register_global_variable(var);
    }
}


realno& GlobalsManager::register_real(char* tag, realno default_value, 
				      realno* holder, bool fixed,
				      char* owner, char* help)
{
  if (holder == NULL) holder = new realno;
  *holder = default_value;

  RealGlobalVariable* tmp = 
    new RealGlobalVariable(tag, fixed, owner, help, *holder);

  register_global_variable(tmp);

  return (tmp->value);
}

realno& GlobalsManager::register_real(char *tag,
				      realno default_value,
				      realno min,
				      realno max, realno* holder,
				      bool fixed,
				      char* owner, char* help)
{
  if (holder == NULL) holder = new realno;
  *holder = default_value;

  RealGlobalVariable* tmp = 
    new RealGlobalVariable(tag, fixed, owner, help, *holder, min, max);

  register_global_variable(tmp);

  return (tmp->value);
}


int& GlobalsManager::register_int(char* tag,
				  int default_value, int* holder,
				  bool fixed,
				  char* owner, char* help)
{
  if (holder == NULL) holder = new int;
  *holder = default_value;

  IntGlobalVariable* tmp = 
    new IntGlobalVariable(tag, fixed, owner, help, *holder);

  register_global_variable(tmp);

  return (tmp->value);
}

int& GlobalsManager::register_int(char* tag,
				  int default_value,
				  int min,
				  int max, int* holder,
				  bool fixed,
				  char* owner, char* help)
{
  if (holder == NULL) holder = new int;
  *holder = default_value;

  IntGlobalVariable* tmp = 
    new IntGlobalVariable(tag, fixed, owner, help, *holder, min, max);

  register_global_variable(tmp);

  return (tmp->value);
}



bool& GlobalsManager::register_bool(char* tag,
				    bool default_value, bool* holder,
				    bool fixed,
				    char* owner, char* help)
{
  if (holder == NULL) holder = new bool;
  *holder = default_value;

  BoolGlobalVariable* tmp = 
    new BoolGlobalVariable(tag, fixed, owner, help, *holder);

  register_global_variable(tmp);

  return (tmp->value);
}


  
char*& GlobalsManager::register_string(char* tag,
				       char *default_value,  
				       char **holder, bool fixed,
				       char* owner, char* help,
				       char** valid_strings)
{
  if (holder == NULL) holder = new char*;
  *holder = default_value;

  StringGlobalVariable* tmp = 
    new StringGlobalVariable(tag, fixed, owner, help, *holder);
  tmp->valid_strings = valid_strings;

  register_global_variable(tmp);

  return (tmp->value);
}


char*& GlobalsManager::register_filename(char* tag,
					 char *default_value,  
					 char **holder,
					 char* owner, char* help)
{
  if (holder == NULL) holder = new char*;
  *holder = default_value;

  StringGlobalVariable* tmp = 
    new StringGlobalVariable(tag, TRUE, owner, help, *holder);
  tmp->is_filename = TRUE;

  register_global_variable(tmp);

  return (tmp->value);
}

  
void GlobalsManager::parse_parameter_file(char* param_file)
{
  if (param_file == NULL) return;
  ifstream strm_in(param_file);
  if (!strm_in)
    {
      sprintf(AmbErrorHandler::buffer,"Can't open %s", param_file);
      AmbErrorHandler::handle_warning("in \
GlobalsManager::parse_parameter_file\n");
      return;
    }
  parse_parameter_stream(strm_in);
}


// read in global values from a file or stream
void GlobalsManager::parse_parameter_stream(istream& input_stream)
{

  char tag_name[100];
  while (input_stream.peek() != EOF)
    {
      // skip out commented lines
      while (input_stream.peek() == '#') 
	input_stream.ignore(200, '\n');
      tag_name[0] = '\0';
      input_stream >> tag_name;
      for (int i = 0; i < no_globals; i++)
	{
	  if ((strlen(tag_name) == strlen(table[i]->tag)) &&
	      (strcasecmp(table[i]->tag, tag_name) == 0))
	    {
	      if(!table[i]->read_value(input_stream))
		{
		  AmbErrorHandler::handle_warning("Value out of\
range");
		  table[i]->full_printout(cerr);
		}
	    }
	}
    }
}

      
void GlobalsManager::dump_parameters(char* param_file)
{
  ofstream strm_out(param_file);
  
  if (!strm_out)
    { 
      sprintf(AmbErrorHandler::buffer,"Can't open %s", param_file);
      AmbErrorHandler::handle_warning("in \
GlobalsManager::dump_parameters\n");
      return;
    }
  
  dump_parameters(strm_out);
}


// save parameters in parseable format
void GlobalsManager::dump_parameters(ostream& output_stream)
{
  output_stream << "#\n# Global Parameters\n#" << endl;

  for (int i = 0; i < no_globals; i++)
    {
      if ((table[i]->type == GlobalVariable::REAL)
	  && (real_isnan(((RealGlobalVariable*)table[i])->value)))
	continue;
      output_stream << table[i]->tag << "  ";
      table[i]->output_value(output_stream);
    }
}

void GlobalsManager::output_info()
{
  cout << "Global variables" << endl;
  cout << "****************" << endl;
  for (int i = 0; i < no_globals; i++)
    {
      table[i]->full_printout(cout);
      cout << endl;
    }
}

void GlobalsManager::set_value(char* tag, realno new_value)
{
  for (int i = 0; i < no_globals; i++)    
    {
      if ((strcmp(table[i]->tag, tag) == 0)
	&& (table[i]->type == GlobalVariable::REAL))
	
	((RealGlobalVariable*) table[i])->value = new_value;
    }
}

void GlobalsManager::set_value(char* tag, int new_value)
{
  for (int i = 0; i < no_globals; i++)    
    {
      if (strcmp(table[i]->tag, tag) == 0)
	{

	  switch (table[i]->type)
	    {
	    case GlobalVariable::INTEGER:
	      ((IntGlobalVariable*) table[i])->value = new_value;
	      break;
	      
	    case GlobalVariable::BOOLEAN:
	      ((BoolGlobalVariable*) table[i])->value = new_value;
	      break;
	    }      
	}
    }
}

void GlobalsManager::set_value(char* tag, char *new_value)
{
  for (int i = 0; i < no_globals; i++)    
    {
      if ((strcmp(table[i]->tag, tag) == 0)
	&&  (table[i]->type == GlobalVariable::STRING))
	((StringGlobalVariable*) table[i])->value = new_value;
    }
}

void GlobalsManager::unmanage_global(char* tag)
{
  GlobalVariable** new_table = new GlobalVariable*[max_globals];
  int j = 0;
  for (int i = 0; i < no_globals; i++)    
    {
      if (strcmp(table[i]->tag, tag) != 0)
	new_table[j++] = table[i];
    }
  delete [] table;
  table = new_table;
  no_globals = j;
}


/************************ MOTIF INTERFACE ***************************/

Widget toplevel = NULL;

Widget GetTopShell(Widget w)
{
  while (w && !XtIsTopLevelShell(w))
    w = XtParent(w);
  return (w);
}


static void toggled(Widget widget, BoolGlobalVariable *client_data,
	     XmToggleButtonCallbackStruct *call_data)
{
  client_data->get_value() = (bool) call_data->set;
}

static void help_done(Widget dialog, XtPointer, XtPointer)
{
  XtDestroyWidget(dialog);
}


static char *word_wrap_string(char *str)
{
  static const int MAX_LINES = 100;
  static const int MAX_LINE_LENGTH = 100;
  char *new_str = new char[strlen(str)+ MAX_LINES];
  istrstream data_in(str, strlen(str));
  ostrstream data_out(new_str, strlen(str) + MAX_LINES);
  char aline[MAX_LINE_LENGTH+1];

  while(data_in.good())
    {
      aline[0] = '\0';
      data_in.getline(aline, MAX_LINE_LENGTH);
      int line = strlen(aline);
      if (line >= (MAX_LINE_LENGTH-1))
	{
	  while (strchr(" .:-;!?", aline[--line]) == 0)
	    data_in.putback(aline[line]);
	  aline[line+1] = '\0';
	}
      data_out << aline << endl;;
    }
  data_out << '\0' << flush;
  return (new_str);
}
    

static void help_callback(Widget parent, char* help_text, 
		   XtPointer call_data)
{
  Widget dialog;
  XmString text;
  Arg args[5];
  int n = 0;
  char* word_wrap_text = word_wrap_string(help_text);
  text = XmStringCreateLtoR(word_wrap_text, XmFONTLIST_DEFAULT_TAG);
  delete [] word_wrap_text;
  XtSetArg(args[n], XmNmessageString, text); n++;
  XtSetArg(args[n], XmNautoUnmanage, False); n++;
  dialog = XmCreateInformationDialog(parent,"help",args,n);
  XmStringFree(text);

  XtUnmanageChild(XmMessageBoxGetChild(dialog,
				       XmDIALOG_CANCEL_BUTTON));
  XtSetSensitive(XmMessageBoxGetChild(dialog, XmDIALOG_HELP_BUTTON),
		 False);
  XtAddCallback(dialog, XmNokCallback, help_done, NULL);
  
  XtManageChild(dialog);
  XtPopup(XtParent (dialog), XtGrabNone);
}

  
void* BoolGlobalVariable::create_widget(void* parent)
{
  Widget toggle = 
    XtVaCreateManagedWidget(tag, xmToggleButtonWidgetClass,
			    (Widget) parent, XmNset, (Boolean) value, NULL);

  XtAddCallback(toggle, XmNvalueChangedCallback,
		(XtCallbackProc) toggled, this);

  XtAddCallback(toggle, XmNhelpCallback, help_callback, help_string);
  return (toggle);
}


static void real_changed_callback(Widget widget, RealGlobalVariable *var,
			   XmAnyCallbackStruct *call_data)
{

  char *text = XmTextGetString(widget);
  float new_value;
  if (sscanf(text,"%f", &new_value) == 1)
    var->get_value() = atof(text);

  static char buffer[100];
  if (real_isnan(var->get_value()))
    sprintf(buffer,"undefined");
  else sprintf(buffer,"%f", var->get_value());
  XtRemoveCallback(widget, XmNactivateCallback, (XtCallbackProc)
		   real_changed_callback, var);
		   
  XmTextSetString(widget, buffer);

  XtAddCallback(widget, XmNactivateCallback, (XtCallbackProc)
		real_changed_callback, var);
  XtFree(text);
}

static void real_scale_callback(Widget widget, RealGlobalVariable *var,
			 XmScaleCallbackStruct *call_data)
{
  short dpoints;
  XtVaGetValues(widget, XmNdecimalPoints, &dpoints, NULL);
  realno sfac = pow(10.0, dpoints);
  var->get_value() = ((realno) call_data->value) / sfac;
}
  

void* RealGlobalVariable::create_widget(void* parent)
{
  if (!constrained)
    {
      Widget container = 
	XtVaCreateManagedWidget("container", xmRowColumnWidgetClass,
				(Widget) parent,
				XmNorientation, XmHORIZONTAL,
				NULL);
      Widget wfield = 
	XtVaCreateManagedWidget("field", xmTextFieldWidgetClass, container,
				XmNcolumns, 9, NULL);
      
      XtAddCallback(wfield, XmNactivateCallback, 
		    (XtCallbackProc) real_changed_callback, this);
      
      XtAddCallback(wfield, XmNlosingFocusCallback,
		    (XtCallbackProc) real_changed_callback, this);

      XtAddCallback(container, XmNhelpCallback, help_callback,
		    help_string);
      real_changed_callback(wfield, this, NULL);
      XtVaCreateManagedWidget(tag, xmLabelWidgetClass, container,
			      NULL);
      return (container);
    }
  else
    {
      realno fstep = (max - min);
      int ndigits = 0;
      while (fstep >= 1) { fstep /= 10; ndigits++; }
      while ((ndigits <= 0) && (fstep < 0.1))
	{ fstep *= 10; ndigits--; }

      // we would like 4 digits so scale numbers appropriately
      int dpoints = 4 - ndigits;
      realno sfac = pow(10.0, dpoints);
      int imax = max * sfac;
      int imin = min * sfac;
      int ivalue = value * sfac;

      Widget wscale = 
	XtVaCreateManagedWidget(tag, xmScaleWidgetClass,(Widget)
				parent, XtVaTypedArg, XmNtitleString,
				XmRString, tag, strlen(tag),
				XmNscaleWidth, 180,
				XmNmaximum, imax,
				XmNminimum, imin,
				XmNdecimalPoints, dpoints,
				XmNvalue, ivalue,
				XmNshowValue, True, 
				XmNorientation, XmHORIZONTAL, NULL);
      XtAddCallback(wscale, XmNvalueChangedCallback, 
		    (XtCallbackProc) real_scale_callback, this);
      XtAddCallback(wscale, XmNhelpCallback, help_callback,
		    help_string);			       
      return (wscale);
    }
      
}



static void int_changed_callback(Widget widget, IntGlobalVariable *var,
			   XmAnyCallbackStruct *call_data)
{

  char *text = XmTextGetString(widget);
  int new_value;
  if (sscanf(text,"%d", &new_value) == 1)
    var->get_value() = atoi(text);

  static char buffer[100];
  sprintf(buffer,"%d", var->get_value());
  XtRemoveCallback(widget, XmNactivateCallback, (XtCallbackProc)
		   int_changed_callback, var);
		   
  XmTextSetString(widget, buffer);

  XtAddCallback(widget, XmNactivateCallback, (XtCallbackProc)
		int_changed_callback, var);
  XtFree(text);
}

static void int_scale_callback(Widget widget, IntGlobalVariable *var,
			 XmScaleCallbackStruct *call_data)
{
  var->get_value() = (int) call_data->value;
}
  

void* IntGlobalVariable::create_widget(void* parent)
{
  if (!constrained)
    {
      Widget container = 
	XtVaCreateManagedWidget("container", xmRowColumnWidgetClass,
				(Widget) parent,
				XmNorientation, XmHORIZONTAL,
				NULL);
      Widget wfield = 
	XtVaCreateManagedWidget("field", xmTextFieldWidgetClass, container,
				XmNcolumns, 6, NULL);
      XtAddCallback(wfield, XmNactivateCallback, 
		    (XtCallbackProc) int_changed_callback, this);
      XtAddCallback(wfield, XmNlosingFocusCallback,
		    (XtCallbackProc) int_changed_callback, this);
      XtAddCallback(container, XmNhelpCallback, help_callback,
		    help_string);
      int_changed_callback(wfield, this, NULL);
      XtVaCreateManagedWidget(tag, xmLabelWidgetClass, container, NULL);
      return (container);
    }
  else
    {
      Widget wscale = 
	XtVaCreateManagedWidget(tag, xmScaleWidgetClass,(Widget)
				parent, XtVaTypedArg, XmNtitleString,
				XmRString, tag, strlen(tag),
				XmNscaleWidth, 180,
				XmNmaximum, max,
				XmNminimum, min,
				XmNvalue, value,
				XmNshowValue, True, 
				XmNorientation, XmHORIZONTAL, NULL);
      XtAddCallback(wscale, XmNvalueChangedCallback, 
		    (XtCallbackProc) int_scale_callback, this);
      XtAddCallback(wscale, XmNhelpCallback, help_callback,
		    help_string);			       
      return (wscale);
    }
      
}

static void string_changed_callback(Widget w, StringGlobalVariable *var,
			     XmToggleButtonCallbackStruct* call_data)
{
  if (call_data->set == True)
    {
      // the selected string is the name of the widget !
      var->get_value() = XtName(w);	
    }
}

static XmString create_directory_string(char* full_filename)
{
  static char buffer[256];
  strncpy(buffer, full_filename, 255);
  char *pnt = strrchr(buffer, '/');
  if (pnt != NULL) pnt[1] = '\0';
  XmString motif_str = XmStringCreateLocalized(buffer);
  return (motif_str);
}

static void file_select_callback(Widget w, StringGlobalVariable *var,
				 XmFileSelectionBoxCallbackStruct 
				 *call_data)
{
  char *new_value = NULL;
  if (!XmStringGetLtoR(call_data->value, XmFONTLIST_DEFAULT_TAG,
		       &new_value)) return;

  var->get_value() = new_value;
  
  XtDestroyWidget(w);
}

static void get_file_callback(Widget w, StringGlobalVariable *var,
			      XmPushButtonCallbackStruct *call_data)
{
  Widget dialog = 
    XmCreateFileSelectionDialog(GetTopShell(w), "_file", NULL, 0);
  XtAddCallback(dialog, XmNcancelCallback, (XtCallbackProc) help_done,
		NULL);
  XtAddCallback(dialog, XmNhelpCallback, (XtCallbackProc)
		help_callback, var->help_string);
  XtAddCallback(dialog, XmNokCallback, (XtCallbackProc)
		file_select_callback, var);

  XmString dirname = create_directory_string(var->get_value());
  XmString dirspec = XmStringCreateLocalized(var->get_value());
  XmString tag_str = XmStringCreateLocalized(var->tag);
  XmString tmp_str = XmStringCreateLocalized("Selection for ");
  XmString select_str = XmStringConcat(tmp_str, tag_str);

  XtVaSetValues(dialog, XmNdirectory, dirname,
		XmNdirSpec, dirspec, NULL);

  Widget flabel =
    XmFileSelectionBoxGetChild(dialog, XmDIALOG_SELECTION_LABEL);
  XtVaSetValues(flabel, XmNlabelString, select_str, NULL);

  XmStringFree(dirname);
  XmStringFree(dirspec);
  XmStringFree(tag_str);
  XmStringFree(tmp_str);
  XmStringFree(select_str);

  XtManageChild(dialog);
}
		

static void string_select_callback(Widget w, 
				   StringGlobalVariable   *var,
				   XmSelectionBoxCallbackStruct 
				   *call_data)
{

  char *new_value = NULL;
  if (!XmStringGetLtoR(call_data->value, XmFONTLIST_DEFAULT_TAG,
		       &new_value)) return;
  if (strlen(new_value) == 0) new_value = NULL;
  var->get_value() = new_value;
  
  XtDestroyWidget(w);

}
static void get_string_callback(Widget w, StringGlobalVariable *var,
				XmPushButtonCallbackStruct *call_data)
{
  Widget dialog =
    XmCreatePromptDialog(w, var->tag, NULL, 0);
  XmString tmp1 = XmStringCreateLocalized(var->tag);
  XmString tmp2 = XmStringCreateLocalized(var->get_value());
  XtVaSetValues(dialog, XmNselectionLabelString, tmp1, 
		XmNtextString, tmp2, NULL);
  XtAddCallback(dialog, XmNcancelCallback, (XtCallbackProc) help_done,
		NULL);
  XtAddCallback(dialog, XmNhelpCallback, (XtCallbackProc)
		help_callback, var->help_string);
  XtAddCallback(dialog, XmNokCallback, (XtCallbackProc)
		string_select_callback, var);
  XmStringFree(tmp1);
  XmStringFree(tmp2);
  XtManageChild(dialog);
  XtPopup(XtParent(dialog), XtGrabNone);
}

void* StringGlobalVariable::create_widget(void* vparent)
{
  Widget parent = (Widget) vparent;
  if (is_filename)
    {
      Widget get_file = 
	XtVaCreateManagedWidget(tag, xmPushButtonWidgetClass,
				parent, NULL);
      XtAddCallback(get_file, XmNactivateCallback,
		    (XtCallbackProc) get_file_callback, this);
      XtAddCallback(get_file, XmNhelpCallback,
		    (XtCallbackProc) help_callback, help_string);
      return (get_file);
    }
  
  if (valid_strings == NULL) 
    {
      Widget get_string = 
	XtVaCreateManagedWidget(tag, xmPushButtonWidgetClass,
				parent, NULL);
      XtAddCallback(get_string, XmNactivateCallback,
		    (XtCallbackProc) get_string_callback, this);
      XtAddCallback(get_string, XmNhelpCallback,
		    (XtCallbackProc) help_callback, help_string);
      return (get_string);
    }


  XmString tag_str = XmStringCreateLocalized(tag);
  Widget menubar = 
    XmVaCreateSimpleMenuBar(parent, "_menubar", XmVaCASCADEBUTTON,
			    tag_str, tag[0], NULL);
  XmStringFree(tag_str);

  Widget PullDown =
    XmVaCreateSimplePulldownMenu(menubar, "_pulldown", 0,
				 (XtCallbackProc) NULL, NULL);
  XtVaSetValues(PullDown, XmNradioBehavior, True,
		XmNradioAlwaysOne, True, NULL);
//   Widget cascade = 
//     XtVaCreateManagedWidget(tag, xmCascadeButtonWidgetClass,
// 			    parent, XmNsubMenuId, PullDown, NULL);
  for (int i = 0; valid_strings[i] != NULL; i++)
    {
      Boolean is_set = False;
      if (strcmp(valid_strings[i], value) == 0)
	is_set = True;
      Widget option_i = 
	XtVaCreateManagedWidget(valid_strings[i],
				xmToggleButtonWidgetClass,
				PullDown, XmNset, is_set,
				XmNhighlightOnEnter, True, NULL);
      XtAddCallback(option_i, XmNvalueChangedCallback,
		    (XtCallbackProc) string_changed_callback,
		    this);
    }
  XtAddCallback(menubar, XmNhelpCallback, help_callback, help_string);
  XtManageChild(menubar);
  return (menubar);
}
  

void query_for_help(Widget widget, Widget topwidget,
		    XtPointer call_data)
{
  Cursor cursor;
  Display *display;
  Widget help_widget;
  XmAnyCallbackStruct *cbs, newcbs;
  XEvent event;
  
  cbs = (XmAnyCallbackStruct *) call_data;
  display = XtDisplay(topwidget);
  cursor = XCreateFontCursor(display, XC_hand2);
  
  help_widget = XmTrackingEvent(topwidget, cursor, True, &event);
  while (help_widget != NULL)
    {
      if (XtHasCallbacks(help_widget, XmNhelpCallback) == 
	  XtCallbackHasSome) 
	{
	  newcbs.reason = XmCR_HELP;
	  newcbs.event = &event;
	  XtCallCallbacks(help_widget, XmNhelpCallback,
			  (XtPointer) &newcbs);
	  help_widget = NULL;
	}
      else help_widget = XtParent(help_widget);
    }
  XFreeCursor(display, cursor);
}


static void show_widget_cb(Widget, Widget shell, XtPointer)
{

  XtRealizeWidget(shell);
  XMapRaised(XtDisplay(shell), XtWindow(shell));

}

static void hide_widget_cb(Widget, Widget shell, XtPointer)
{
  XtUnmapWidget(shell);
}

static void destroy_shell_cb(Widget, Widget shell, XtPointer)
{
  XtUnmapWidget(shell);
  XtDestroyWidget(shell);
}

static void do_save_as_cb(Widget w, GlobalsManager* gm,  
			  XmFileSelectionBoxCallbackStruct *call_data)
{

  char *new_value = NULL;
  if (!XmStringGetLtoR(call_data->value, XmFONTLIST_DEFAULT_TAG,
		       &new_value)) return;

  gm->dump_parameters(new_value);

  // XmStringFree(new_value);

  XtDestroyWidget(w);
}

static void save_as_cb(Widget w, GlobalsManager* gm, XtPointer)
{
  Widget dialog = 
    XmCreateFileSelectionDialog(GetTopShell(w), "_file", NULL, 0);

  XtAddCallback(dialog, XmNcancelCallback, (XtCallbackProc) help_done,
		NULL);
  XtAddCallback(dialog, XmNokCallback, (XtCallbackProc)
		do_save_as_cb, gm);

  XtManageChild(dialog);
}


static int back_argc;
static char** back_argv;
static char** back_owners;
static char** back_tags;
GlobalsManager* back_this;
static pid_t back_pid = getpid();

static void bye_bye(Widget w, XtPointer, XtPointer)
{
  kill(back_pid, SIGKILL);
  exit(1); 
}


void entry_fn(void* )
{
  signal(SIGHUP, SIG_DFL);
  prctl(PR_TERMCHILD);
  back_this->create_interface(back_argc, back_argv,
			      back_owners, back_tags, FALSE);
}

void GlobalsManager::create_interface(int argc, char** argv, 
				      char** owners,
				      char** tags, bool use_constants )
{
  XtAppContext app;
  static String		fallbacks[] = {
    "*Foreground:  gray20",
    "*BorderWidth: 0",
    "*Background:  gray70",
    "*XmToggleButton.selectColor:         yellow",
    "*XmToggleButton.indicatorSize:       16",
    "*XmToggleButtonGadget.selectColor:   yellow",
    "*XmToggleButtonGadget.indicatorSize: 16",
    "*fontList: 6x13",
    "*XmText*fontList: -adobe-courier-medium-r-normal--12-*-*-*-m-70-iso8859-1",
    NULL
  };

  static bool have_done = 0;

  if (use_constants) back_pid = getpid();

  
  XtSetLanguageProc(NULL, NULL, NULL);
  
  toplevel = XtVaAppInitialize(&app,"GUI", NULL, 0, &argc, argv, fallbacks,
			       NULL);

  static char * main_list[] = {"main", NULL};

  Widget mainpanel = (Widget)
    create_partial_interface(toplevel, use_constants, main_list, tags);
  
  Widget extra_shell = 
    XtVaCreatePopupShell("Options", topLevelShellWidgetClass,
			 toplevel, NULL);
					  
  Widget extra_panel = (Widget)
    create_partial_interface(extra_shell, use_constants, owners, tags);
  
  XtVaCreateManagedWidget("sep", xmSeparatorWidgetClass,
			  extra_panel, XmNorientation, XmHORIZONTAL, NULL);

  Widget more_buttons = 
    XtVaCreateManagedWidget("more_buttons", xmRowColumnWidgetClass,
			    extra_panel,
			    XmNorientation, XmHORIZONTAL,
			    XmNpacking, XmPACK_COLUMN, NULL);
  Widget hide_button =
     XtVaCreateManagedWidget("Hide", xmPushButtonWidgetClass,
			     more_buttons, NULL);
  XtAddCallback(hide_button, XmNactivateCallback, (XtCallbackProc)
		hide_widget_cb, extra_shell);

  Widget helpb2 = 
    XtVaCreateManagedWidget("Help", xmPushButtonWidgetClass, more_buttons,
			    NULL);
  XtAddCallback(helpb2, XmNactivateCallback, (XtCallbackProc)
		query_for_help, extra_shell);

  Widget savebutton = 
    XtVaCreateManagedWidget("Save parameters",
			    xmPushButtonWidgetClass, more_buttons,
			    NULL);
  XtAddCallback(savebutton, XmNactivateCallback, (XtCallbackProc)
		save_as_cb, this);

			    

  XtVaCreateManagedWidget("sep", xmSeparatorWidgetClass,
			  mainpanel, XmNorientation, XmHORIZONTAL, NULL);

  Widget buttons = 
    XtVaCreateManagedWidget("buttons", xmRowColumnWidgetClass,
			    mainpanel,
			    XmNorientation, XmHORIZONTAL,
			    XmNpacking, XmPACK_COLUMN, NULL);

  Widget extras_button = 
    XtVaCreateManagedWidget("More options ...", xmPushButtonWidgetClass,
			    buttons, NULL);
  XtAddCallback(extras_button, XmNactivateCallback, (XtCallbackProc)
		show_widget_cb, extra_shell);

  Widget help_button = 
    XtVaCreateManagedWidget("Help", xmPushButtonWidgetClass, buttons,
			    NULL);
  XtAddCallback(help_button, XmNactivateCallback, (XtCallbackProc)
		query_for_help, toplevel);

  
  if (use_constants)
    {
      BoolGlobalVariable var_done("GO", FALSE, "", "", have_done);
      Widget end_button =
	XtVaCreateManagedWidget("GO!", xmPushButtonWidgetClass, buttons,
				NULL);
      XtAddCallback(end_button, XmNactivateCallback,
		(XtCallbackProc) toggled, &var_done);
      Widget quit_button =
	XtVaCreateManagedWidget("Cancel", xmPushButtonWidgetClass, buttons,
				NULL);
      XtAddCallback(quit_button, XmNactivateCallback,
		    (XtCallbackProc) bye_bye, NULL);
    }
  else
    {
      Widget end_button =
	XtVaCreateManagedWidget("Hide", xmPushButtonWidgetClass, buttons,
				NULL);
      XtAddCallback(end_button, XmNactivateCallback,
		(XtCallbackProc) destroy_shell_cb, toplevel);
      Widget quit_button =
	XtVaCreateManagedWidget("Quit", xmPushButtonWidgetClass, buttons,
				NULL);
      XtAddCallback(quit_button, XmNactivateCallback,
		    (XtCallbackProc) bye_bye, NULL);
    }

  XtRealizeWidget(toplevel);  
  XMapRaised(XtDisplay(toplevel), XtWindow(toplevel));

  if (!use_constants) 
    {
      have_done = TRUE;
      XtAppMainLoop(app);
      exit(1);
    }
  while (!have_done)
    {
      XEvent next_event;
      XtAppNextEvent(app, &next_event);
      XtDispatchEvent(&next_event);
    }

  XtUnmanageChild(extra_shell); XtDestroyWidget(extra_shell);
  Display *the_display = XtDisplay(toplevel);
  XtDestroyWidget(toplevel);
  XFlush(the_display);
  toplevel = NULL;
  if (use_constants) 
    {
      // now generate background interface
      delete [] flags; flags = NULL;
      back_argc = argc; back_argv = argv;
      back_this = this; back_owners = owners;
      back_tags = tags;
      have_done = FALSE;      
fprintf(stderr,"start sproc\n");
      sproc(entry_fn, PR_SALL);
fprintf(stderr,"end sproc have_done %d\n",have_done);
      // while (!have_done);	// RJM why on earth? Something to 
				// do with sproc forking
    }
      
//  XtAppMainLoop(app);
}

// create_partial_interface:
// creates a row-column widget containing all the desired
// buttons etc 
// owners : list of owner strings to match on 
// tags: list of tag strings to match on
// use_constants : match constants as well as variables
void* GlobalsManager::create_partial_interface(void *parent,
					       bool use_constants,
					       char** owners,
					       char** tags )
{
  Widget rowcol = 
    XtVaCreateManagedWidget("rowcolumn",
			    xmRowColumnWidgetClass,	   
			    (Widget) parent, 
			    XmNorientation, XmVERTICAL,
			    NULL);
  Widget row1 =
    XtVaCreateManagedWidget("row1", xmRowColumnWidgetClass,
			    rowcol,
			    XmNorientation, XmHORIZONTAL,
			    XmNpacking, XmPACK_COLUMN, NULL);
  int n1 = 
    add_global_widgets(row1, use_constants, FALSE,
		       GlobalVariable::BOOLEAN,  owners, tags);
  int r1 = 1 + (n1 / 5);
  XtVaSetValues(row1, XmNnumColumns, r1, NULL);
  if (n1 == 0) 
    { XtUnmanageChild(row1); XtDestroyWidget(row1); }


  Widget row2 =
    XtVaCreateManagedWidget("row2", xmRowColumnWidgetClass,
			    rowcol,
			    XmNorientation, XmHORIZONTAL,
			    XmNpacking, XmPACK_COLUMN, NULL);
  
  int n2 = 
    add_global_widgets(row2, use_constants, FALSE, GlobalVariable::REAL,
		       owners, tags);

  int r2 = 1 + (n2 / 4);
  XtVaSetValues(row2, XmNnumColumns, r2, NULL);
  if (n2 == 0)
    {  XtUnmanageChild(row2); XtDestroyWidget(row2); }

  Widget row3 = XtVaCreateManagedWidget("row3", xmRowColumnWidgetClass,
					rowcol,
					XmNorientation, XmHORIZONTAL,
					XmNpacking, XmPACK_COLUMN, NULL);

  int n3 = 
    add_global_widgets(row3, use_constants, FALSE, GlobalVariable::INTEGER,
		       owners, tags);
  int r3 = 1 + (n3 / 4);
  XtVaSetValues(row3, XmNnumColumns, r3, NULL);
  if (n3 == 0)
    {  XtUnmanageChild(row3); XtDestroyWidget(row3); }


  Widget row4 = XtVaCreateManagedWidget("row4", xmRowColumnWidgetClass,
					rowcol,
					XmNorientation, XmHORIZONTAL,
					XmNpacking, XmPACK_COLUMN, NULL);

  int n4 = add_global_widgets(row4, use_constants,
			      TRUE, GlobalVariable::INTEGER,
    			      owners, tags);
  n4 += add_global_widgets(row4, use_constants, TRUE, GlobalVariable::REAL,
			   owners, tags);
  int r4 = 1 + (n4 / 5);
  XtVaSetValues(row4, XmNnumColumns, r4, NULL);
  if (n4 == 0)
    {  XtUnmanageChild(row4); XtDestroyWidget(row4); }

  Widget sep =
    XtVaCreateManagedWidget("sep", xmSeparatorWidgetClass,
			    rowcol, XmNorientation, XmHORIZONTAL, NULL);

  Widget row5 = XtVaCreateManagedWidget("row5", xmRowColumnWidgetClass,
					rowcol, 
					XmNorientation, XmHORIZONTAL,
					XmNpacking, XmPACK_COLUMN, NULL);
  int n5 = add_global_widgets(row5, use_constants, FALSE,
			      GlobalVariable::STRING,
			      owners, tags);
  int r5 = 1 + (n5 / 5);
  XtVaSetValues(row5, XmNnumColumns, r5, NULL);
  if (n5 == 0)
    { 
      XtUnmanageChild(row5); XtDestroyWidget(row5); 
      XtUnmanageChild(sep); XtDestroyWidget(sep); 
    }

  return (rowcol);
}

void GlobalsManager::setup_flags()
{
  if (flags != NULL) delete [] flags;
  flags = new bool[no_globals];
  for (int i = 0; i < no_globals; i++)
    flags[i] = FALSE;
}

      
int GlobalsManager::add_global_widgets(void* parent, bool allow_fixed,
					bool constrained, 
					GlobalVariable::Type type,
					char** owner_list,
					char** tag_list)
{
  if (flags == NULL) setup_flags();

  int nmatched = 0;
  for (int i = 0; i < no_globals; i++)
    {
      if (flags[i]) continue;
      GlobalVariable* curr = table[i];
      if (!allow_fixed && curr->fixed) continue;
      if (curr->type != type) continue;
      if ((type == GlobalVariable::REAL) &&
	  (constrained !=
	   ((RealGlobalVariable*)curr)->is_constrained()))
	continue;
      if ((type == GlobalVariable::INTEGER) &&
	  (constrained != 
	   ((IntGlobalVariable*)curr)->is_constrained()))
	continue;
      
      if (owner_list != NULL)
	{
	  int i = 0;
	  bool found_match = FALSE;
	  while ((owner_list[i] != NULL) && (!found_match))
	    {
	      if (strcmp(owner_list[i], curr->owner_class) == 0)
		found_match = TRUE;
	      else i++;
	    }
	  if (!found_match) continue;
	}

      if (tag_list != NULL)
	{
	  int i = 0;
	  bool found_match = FALSE;
	  while ((tag_list[i] != NULL) && (!found_match))
	    {
	      if (strcmp(tag_list[i], curr->tag) == 0)
		found_match = TRUE;
	      else i++;
	    }
	  if (!found_match) continue;
	}

      // add this global
      nmatched++;
      Widget myWidget = (Widget) curr->create_widget(parent);
      // flag so that we don't match this one again
      flags[i] = TRUE;
    }
  return (nmatched);
}					     
	
