/*
This source file is the copyright property of the University of Leeds
('The University').

Permission to use, copy, modify, and distribute this source file for
educational, research, and not-for-profit purposes, without fee and
without a signed licensing agreement, is hereby granted, provided that
the above copyright notice, this paragraph and the following three
paragraphs appear in all copies, modifications, and distributions.

In no event shall The University be liable to any party for direct,
indirect, special, incidental or consequential damages, including lost
profits, arising out of the use of this software and its documentation.

The software is provided without warranty. The University has no
obligation to provide maintenance, support, updates, enhancements, or
modifications.

This software was written by Adam Baumberg, Vision Group,
School of Computer Studies, University of Leeds, UK. The code and use
thereof should be attributed to the author where appropriate
(including demonstrations which rely on it's use).
*/

/***************************************************************
 * C++ source
 *
 * File : 	db_printf.cc
 *
 * Module :	db_printf
 *
 * Author : 	A M Baumberg (CoMIR)
 *
 * Creation Date : Thu Oct 17 15:28:03 1996 
 *
 * Comments : 	
 *
 ***************************************************************/

#include <stdarg.h>
#include <stdio.h>
#include <string.h>
#include "EnvParameter.h"

static EnvParameter show_debug_messages("DEBUG_MESSAGES", FALSE);

void turn_on_debug_messages()
{ show_debug_messages.set_on(); }

void turn_off_debug_messages()
{ show_debug_messages.set_off(); }

bool is_debug_on()
{ return (show_debug_messages.is_on()); }


int db_printf(char* format, ...)
{
  // supports "%s" "%f" and "%d" formats
  // (without optional formatting tokens)

  if (show_debug_messages.is_off()) return (0);
  va_list ap;
  va_start(ap,format);
  
  char* curr_pos = format;

  for (;;)
    {
      char* new_pos = strchr(curr_pos,'%');
      if (new_pos == NULL)
	{
	  fprintf(stderr, curr_pos);
	  break;
	}
      *new_pos = '\0';
      fprintf(stderr,curr_pos);
      *new_pos = '%';

      switch (new_pos[1])
	{
	case 'd':
	  int n =  va_arg(ap,int);
	  fprintf(stderr,"%d", n);
	  break;

	case 'f':
	  double x = va_arg(ap,double);
	  fprintf(stderr,"%f", x);
	  break;

	case 's':
	  char* s = va_arg(ap,char*);
	  fprintf(stderr,s);
	  break;
	}

      curr_pos = new_pos+2;
    } 

  va_end(ap);
  return (1);
}
  
   
      
